package com.tztzf.network.cosign;

import com.tztzf.baseos.tztZFBaseDeviceInfo;
import com.tztzf.baseos.tztZFNullable;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.keychain.tztZFKeyChain;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.protocol.core.tztZFCosignSSL;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.core.tztZFVPNSSL;
import com.tztzf.protocol.define.tztZFEncryptType;
import com.tztzf.protocol.define.tztZFOperType;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrDevice;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOK;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrOther;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrParam;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrPin;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrUnfind;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Cosign;

/**
 * 协同签名证书
 */
public class tztZFCosignCert {
    private static final String tztZFDeviceID_Cosign = "com.tzt.deviceid.z";
    private static final String tztCert_UserID = "USERID";
    private static final String tztCert_UserKey = "USERKEY";
    private static final String tztCert_UserCert = "USERCERT";
    private static final String tztCert_CertInfo = "CERTINFO";
    public static final String tztCert_SystemSafetyCode = "USEREX";

    /// 用户ID
    private String userID;
    /// 协同类别 0-中焯 1-智慧
    private int cosignType;
    /// PIN码
    private String pin;
    private byte[] dPin;

    private byte[] D;
    private byte[] P1;
    private byte[] P;
    private String subjectDN;

    /// 证书类别
    private int certType;
    /// 加密证书格式
    private int encType;
    /// 加密证书
    private byte[] encCert;
    /// 加密key
    private byte[] encKey;
    /// 签名证书格式
    private int signType;
    /// 签名证书
    private byte[] signCert;
    /// 签名key
    private byte[] signKey;
    ///使用自定义安全码 (YES使用自定义安全码, NO使用系统安全码) 默认使用自定义安全码
    private boolean bUseCustomSafetyCode = true;
    /// 服务端部分证书状态
    private int nHave;
    /// 证书信息(签名证书)
    private tztZFCertificateInfo signInfo;
    /// 密钥分量标志
    private String DMark;
    //region get/set

    public int getCosignType() {
        return cosignType;
    }

    public String getPin() {
        return pin;
    }

    public int getCertType() {
        return certType;
    }

    public int getEncType() {
        return encType;
    }

    public byte[] getEncCert() {
        return encCert;
    }

    public byte[] getEncKey() {
        return encKey;
    }

    public int getSignType() {
        return signType;
    }

    public byte[] getSignCert() {
        return signCert;
    }

    public byte[] getSignKey() {
        return signKey;
    }

    public int getnHave() {
        return nHave;
    }

    public void setnHave(int nHave) {
        this.nHave = nHave;
    }

    public tztZFCertificateInfo getSignInfo() {
        return signInfo;
    }

    public void setSignInfo(tztZFCertificateInfo signInfo) {
        this.signInfo = signInfo;
    }

    public String getDMark() {
        return DMark;
    }

    public void setDMark(String DMark) {
        this.DMark = DMark;
    }
    //endregion

    private tztZFCosignCert() {
    }

    public static boolean verifyUserID(String userID, String pin) {
        if (tztZFTextUtils.isEmpty(userID) || tztZFTextUtils.isEmpty(pin))
            return false;
        byte[] dPin = tztZFNetCosignEx.SM3(pin.getBytes());
        if (dPin == null || dPin.length != 32)
            return false;
        String strUserKey = userCertKey(userID);
        if (tztZFTextUtils.isEmpty(strUserKey))
            return false;

        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1)
            return false;
        tztZFDataMapBase<String, Object> values = dict.getDataMap(strUserKey);
        if (values == null || values.getCount() < 1)
            return false;

        byte[] encryptUserID = values.getBytes(tztCert_UserID);
        byte[] encryptCosignKey = values.getBytes(tztCert_UserKey);
        if (encryptUserID == null || encryptUserID.length < 1 || encryptCosignKey == null || encryptCosignKey.length < 1)
            return false;

        byte[] orgUserID = userID.getBytes();
        byte[] userEncKey = encCosignKey(null);
        byte[] valUserID = decryptData(encryptUserID, userEncKey);
        if (valUserID == null || valUserID.length < 1 || !Arrays.equals(valUserID, orgUserID))
            return false;
        byte[] cosignKey = decryptData(encryptCosignKey, dPin);
        if (cosignKey == null || cosignKey.length < orgUserID.length + 32)
            return false;
        byte[] orgData = new byte[orgUserID.length];
        System.arraycopy(cosignKey, 0, orgData, 0, orgUserID.length);
        if (!Arrays.equals(orgData, orgUserID))
            return false;
        return true;
    }

    /**
     * 根据用户和PIN码加载
     *
     * @param userID 用户ID
     * @param pin    pin码
     * @param error  错误码
     * @return null=加载失败，并回调errorCallBack；否则返回对应数据对象
     */
    public static tztZFCosignCert loadWithUserID(String userID, String pin, List<tztZFCosignLoginError> error) {
        if (userID == null || userID.length() < 1 || pin == null || pin.length() < 1) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            error.add(new tztZFCosignLoginError(tztZFCosignErrParam, String.format(Locale.getDefault(), "用户ID或者%s为空!", pinLable)));
            return null;
        }
        byte[] dPin = tztZFNetCosignEx.SM3(pin.getBytes());
        if (dPin == null || dPin.length != 32) {
            error.add(new tztZFCosignLoginError(tztZFCosignErrOther, ""));
            return null;
        }
        tztZFDataMapBase<String, Object> ret = loadCertKeyChain(userID, dPin);
        if (ret.getErrorNo() != tztZFCosignErrOK) {
            error.add(new tztZFCosignLoginError(ret.getErrorNo(), ret.getErrorMessage()));
            return null;
        }

        tztZFCosignCert cosignCert = new tztZFCosignCert();
        cosignCert.userID = userID;
        cosignCert.pin = pin;
        cosignCert.dPin = dPin;
        cosignCert.nHave = 0;
        byte[] certData = ret.getBytes(tztCert_UserCert);
        cosignCert.loadCertData(certData);
        byte[] certInfo = ret.getBytes(tztCert_CertInfo);
        cosignCert.signInfo = new tztZFCertificateInfo();
        loadCertInfo(cosignCert.signInfo, certInfo);
        return cosignCert;
    }

    /**
     * 初始化
     *
     * @param userID 用户ID
     * @param pin    pin码
     * @return null=初始化失败
     */
    public static tztZFCosignCert initWithUserID(String userID, String pin) {
        if (userID == null || userID.length() < 1 || pin == null || pin.length() < 1)
            return null;
        byte[] dPin = tztZFNetCosignEx.SM3(pin.getBytes());
        if (dPin == null || dPin.length != 32)
            return null;
        tztZFCosignCert cosignCert = new tztZFCosignCert();
        cosignCert.userID = userID;
        cosignCert.cosignType = 0;
        cosignCert.pin = pin;
        cosignCert.dPin = dPin;
        cosignCert.nHave = 0;
        cosignCert.signInfo = new tztZFCertificateInfo();
        return cosignCert;
    }

    /**
     * 是否可签名
     */
    public boolean isCosigned() {
        if (cosignType == 0) {
            if (this.D == null || this.D.length < 1)
                return false;
        }

        return userID != null && userID.length() > 0 && pin != null && pin.length() > 0 && dPin != null && dPin.length == 32;
    }

    /**
     * 是否有证书
     */
    public boolean haveCert() {
        if (cosignType == 0) {
            if (this.P == null || this.P.length < 1 || this.D == null || this.D.length < 1)
                return false;
        }
        return userID != null && userID.length() > 0
                && pin != null && pin.length() > 0
                && dPin != null && dPin.length == 32
                && encCert != null && encCert.length > 0
                && encKey != null && encKey.length > 0
                && signCert != null && signCert.length > 0;
    }

    /**
     * 证书有效期判断
     *
     * @return true=有效
     */
    public boolean isValidity() {
        if (!haveCert() || nHave == -1 || nHave == -2 || nHave == -3)
            return false;
        Date serverDate = new Date();
        String strServerDate = tztZFNetCosignEx.getExStrForKey(tztZFNetCosignEx.tztCosignEx_CurrentTime);
        if (strServerDate != null && strServerDate.length() > 0) {
            serverDate = tztZFCertificateInfo.dateFromString(strServerDate, "yyyyMMddHHmmss");
        }
        if (signInfo != null && signInfo.getNotAfter() != null && serverDate != null) {
            if (signInfo.getNotAfter().after(serverDate))
                return true;
        } else {
            if (tztZFVPNSSL.validityCert(signCert, signType)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 清除证书信息
     */
    private void clearCert() {
        nHave = 0;
        certType = 0;
        encType = 1;
        encCert = null;
        encKey = null;
        signType = 1;
        signCert = null;
        signKey = null;
        subjectDN = "";
        if (signInfo != null) {
            signInfo.clear();
        } else {
            signInfo = new tztZFCertificateInfo();
        }
    }

    /**
     * 保存证书信息
     */
    private boolean saveCertKeyChain() {
        if (userID == null || userID.length() < 1 || pin == null || pin.length() < 1)
            return false;
        String strUserKey = userCertKey(userID);
        if (strUserKey.length() < 1) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("ueridNil:%s", this.userID);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return false;
        }

        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1)
            dict = new tztZFDataMapBase<>();

        byte[] cosignKey = encCosignKey(this.P1);
        byte[] dataUserID = this.userID.getBytes();

        //加密出错
        if (cosignKey == null || cosignKey.length != 32) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("cosignKeyLength:%s", cosignKey == null ? 0 : cosignKey.length);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return false;
        }

        int nLen = cosignKey.length + dataUserID.length;
        byte[] orgData = new byte[nLen];
        System.arraycopy(dataUserID, 0, orgData, 0, dataUserID.length);
        System.arraycopy(cosignKey, 0, orgData, dataUserID.length, cosignKey.length);

        byte[] encryptCosignKey = encryptData(orgData, this.dPin);
        if (encryptCosignKey == null || encryptCosignKey.length < 1) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("encryptCosignKeyNil,DPin:%s, orgData:%s", this.dPin, orgData);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return false;
        }

        byte[] userEncKey = encCosignKey(null);
        byte[] encryptUser = encryptData(this.userID.getBytes(), userEncKey);
        if (encryptUser == null || encryptUser.length < 1) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("encryptUserNil,userID:%s, userEncKey:%s", this.userID, encryptUser == null ? "" : userEncKey);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return false;
        }

        tztZFDataMapBase<String, Object> userDict = dict.getObj(strUserKey);
        if (userDict == null) {
            userDict = new tztZFDataMapBase<>();
        }
        if (!bUseCustomSafetyCode) {
            byte[] safetyEncKey = encCosignKey(dataUserID);
            byte[] encryptSafety = encryptData(this.pin.getBytes(), safetyEncKey);
            if (encryptSafety == null || encryptSafety.length < 1) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("encryptSafetyNil,userID:%s, dataUserID:%s, pin:%s, safetyEncKey:%s", this.userID, dataUserID, this.pin, encryptSafety == null ? "" : encryptSafety);
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                            tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                }
                return false;
            }
            userDict.setObj(tztCert_SystemSafetyCode, encryptSafety);
        }
        userDict.setObj(tztCert_UserID, encryptUser);
        userDict.setObj(tztCert_UserKey, encryptCosignKey);
        byte[] encryptCertInfo = encryptData(toCertInfo(), userEncKey);
        if (encryptCertInfo != null && encryptCertInfo.length > 0) {
            userDict.setObj(tztCert_CertInfo, encryptCertInfo);
        }

        byte[] encryptCosignData = encryptData(toCertData(), cosignKey);
        if (encryptCosignData != null && encryptCosignData.length > 0) {
            userDict.setObj(tztCert_UserCert, encryptCosignData);
        }
        dict.setObj(strUserKey, userDict);
        tztZFKeyChain.save(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign, dict);
        return true;
    }

    /**
     * 从数据流读取信息
     *
     * @param data 数据流
     */
    private void loadCertData(byte[] data) {
        if (data == null || data.length < 1)
            return;
        tztZFOutputStream outputStream = tztZFOutputStream.stream(data);
        this.userID = outputStream.readLenStr();
        this.cosignType = outputStream.readInt();

        this.P = outputStream.readLenBytes();
        this.D = outputStream.readLenBytes();
        this.P1 = outputStream.readLenBytes();
        this.subjectDN = outputStream.readLenStr();

        this.certType = outputStream.readInt();

        this.encType = outputStream.readInt();
        this.encCert = outputStream.readLenBytes();
        this.encKey = outputStream.readLenBytes();

        this.signType = outputStream.readInt();
        this.signCert = outputStream.readLenBytes();
        this.signKey = outputStream.readLenBytes();

        this.DMark = outputStream.readLenStr();
    }

    /**
     * 解析数据流为签名数据对象
     *
     * @param signInfo 签名证书信息对象
     * @param data     数据流
     */
    public static void loadCertInfo(tztZFCertificateInfo signInfo, byte[] data) {
        if (signInfo == null || data == null || data.length < 1) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams("readDataNil");
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return;
        }
        tztZFOutputStream outputStream = tztZFOutputStream.stream(data);
        signInfo.setUserID(outputStream.readLenStr());
        signInfo.setSerialNumber(outputStream.readLenStr());
        signInfo.setSubject(outputStream.readLenStr());
        signInfo.setNotBefore(tztZFCertificateInfo.dateFromString(outputStream.readLenStr()));
        signInfo.setNotAfter(tztZFCertificateInfo.dateFromString(outputStream.readLenStr()));
        signInfo.setSignDate(tztZFCertificateInfo.dateFromString(outputStream.readLenStr()));
        signInfo.setStatus(outputStream.readLenStr());
        signInfo.setDeviceInfo(outputStream.readLenStr());
        signInfo.setDeviceID(outputStream.readLenStr());
        signInfo.setLocal(true);
    }

    /**
     * 转换成数据流
     */
    private byte[] toCertData() {
        tztZFInputStream inputStream = new tztZFInputStream();
        inputStream.appendLenStr(userID);
        inputStream.appendInt(cosignType);

        inputStream.appendLenByte(this.P);
        inputStream.appendLenByte(this.D);
        inputStream.appendLenByte(this.P1);
        inputStream.appendLenStr(subjectDN);

        inputStream.appendInt(certType);

        inputStream.appendInt(encType);
        inputStream.appendLenByte(encCert);
        inputStream.appendLenByte(encKey);

        inputStream.appendInt(signType);
        inputStream.appendLenByte(signCert);
        inputStream.appendLenByte(signKey);

        inputStream.appendLenStr(DMark);

        return inputStream.getInputData();
    }

    /**
     * 签名证书转换字节流
     */
    private byte[] toCertInfo() {
        if (signInfo == null)
            return null;
        tztZFInputStream inputStream = new tztZFInputStream();
        inputStream.appendLenStr(signInfo.getUserID());
        inputStream.appendLenStr(signInfo.getSerialNumber());
        inputStream.appendLenStr(signInfo.getSubject());
        inputStream.appendLenStr(tztZFCertificateInfo.dateFormatter(signInfo.getNotBefore()));
        inputStream.appendLenStr(tztZFCertificateInfo.dateFormatter(signInfo.getNotAfter()));
        inputStream.appendLenStr(tztZFCertificateInfo.dateFormatter(signInfo.getSignDate()));
        inputStream.appendLenStr(signInfo.getStatus());
        inputStream.appendLenStr(signInfo.getDeviceInfo());
        inputStream.appendLenStr(signInfo.getDeviceID());
        return inputStream.getInputData();
    }

    private static tztZFDataMapBase<String, Object> loadCertKeyChain(String userID, byte[] pin) {
        tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
        String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
        if (tztZFTextUtils.isEmpty(userID) || pin == null || pin.length != 32) {
            ret.setErrorNo(tztZFCosignErrParam);
            ret.setErrorMessage(String.format(Locale.getDefault(), "用户号或%s错误", pinLable));
            return ret;
        }

        String strUserKey = userCertKey(userID);
        if (tztZFTextUtils.isEmpty(strUserKey)) {
            ret.setErrorNo(tztZFCosignErrDevice);
            ret.setErrorMessage("获取设备号失败");
            return ret;
        }

        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        tztZFDataMapBase<String, Object> values = null;
        if (dict != null) {
            values = dict.getDataMap(strUserKey);
        }

        if (values == null || values.getCount() < 1) {
            ret.setStr(tztCert_UserID, userID);
            ret.setErrorNo(tztZFCosignErrOK);
            ret.setErrorMessage("成功");
            return ret;
        }

        byte[] encryptUserID = values.getBytes(tztCert_UserID);
        byte[] encryptCosignKey = values.getBytes(tztCert_UserKey);
        if (encryptUserID == null || encryptUserID.length < 1 || encryptCosignKey == null || encryptCosignKey.length < 1) {
            ret.setStr(tztCert_UserID, userID);
            ret.setErrorNo(tztZFCosignErrOK);
            ret.setErrorMessage("成功");
            return ret;
        }

        byte[] orgUserID = userID.getBytes();
        byte[] userEncKey = encCosignKey(null);
        byte[] valUserID = decryptData(encryptUserID, userEncKey);
        if (valUserID == null || valUserID.length < 1 || !Arrays.equals(orgUserID, valUserID)) {
            ret.setErrorNo(tztZFCosignErrDevice);
            ret.setErrorMessage("获取设备号失败");
            return ret;
        }

        byte[] cosignKey = decryptData(encryptCosignKey, pin);
        if (cosignKey == null || cosignKey.length < orgUserID.length + 32) {
            ret.setErrorNo(tztZFCosignErrPin);
            ret.setErrorMessage(String.format(Locale.getDefault(), "%s错误", pinLable));
            return ret;
        }
        byte[] tempData = new byte[orgUserID.length];
        System.arraycopy(cosignKey, 0, tempData, 0, orgUserID.length);
        if (!Arrays.equals(orgUserID, tempData)) {
            ret.setErrorNo(tztZFCosignErrPin);
            ret.setErrorMessage(String.format(Locale.getDefault(), "%s错误", pinLable));
            return ret;
        }

        byte[] rCosignKey = new byte[32];
        System.arraycopy(cosignKey, orgUserID.length, rCosignKey, 0, 32);
        ret.setStr(tztCert_UserID, userID);

        byte[] encryptCertInfo = values.getBytes(tztCert_CertInfo);
        if (encryptCertInfo != null && encryptCertInfo.length > 0) {
            byte[] certInfo = decryptData(encryptCertInfo, userEncKey);
            if (certInfo != null && certInfo.length > 0) {
                ret.setBytes(tztCert_CertInfo, certInfo);
            }
        }
        byte[] encryptCosignCert = values.getBytes(tztCert_UserCert);
        if (encryptCosignCert == null || encryptCosignCert.length < 1) {
            ret.setErrorNo(tztZFCosignErrOK);
            ret.setErrorMessage("成功");
            return ret;
        }

        byte[] cosignCert = decryptData(encryptCosignCert, rCosignKey);
        if (cosignCert == null || cosignCert.length < 1) {
            ret.setErrorNo(tztZFCosignErrPin);
            ret.setErrorMessage(String.format(Locale.getDefault(), "%s错误", pinLable));
            return ret;
        }

        ret.setBytes(tztCert_UserCert, cosignCert);
        ret.setErrorNo(tztZFCosignErrOK);
        ret.setErrorMessage("成功");
        return ret;
    }

    private static tztZFDataMapBase<String, Object> loadCertKeyChainP1(String userID, byte[] p1) {
        tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
        String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
        if (p1 == null || p1.length < 1 || tztZFTextUtils.isEmpty(userID)) {
            ret.setErrorNo(tztZFCosignErrParam);
            ret.setErrorMessage(String.format(Locale.getDefault(), "用户号或%s错误", pinLable));
            return ret;
        }

        String strUserKey = userCertKey(userID);
        if (tztZFTextUtils.isEmpty(strUserKey)) {
            ret.setErrorNo(tztZFCosignErrDevice);
            ret.setErrorMessage("获取设备号失败");
            return ret;
        }

        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1) {
            ret.setErrorNo(tztZFCosignErrDevice);
            ret.setErrorMessage("获取设备号失败");
            return ret;
        }

        tztZFDataMapBase<String, Object> values = dict.getDataMap(strUserKey);
        if (values == null || values.getCount() < 1) {
            ret.setStr(tztCert_UserID, userID);
            ret.setErrorNo(tztZFCosignErrUnfind);
            ret.setErrorMessage(String.format(Locale.getDefault(), "未找到对应%s", pinLable));
            return ret;
        }

        byte[] encryptUserID = values.getBytes(tztCert_UserID);
        byte[] encryptCosignKey = values.getBytes(tztCert_UserKey);
        if (encryptUserID == null || encryptUserID.length < 1 || encryptCosignKey == null || encryptCosignKey.length < 1) {
            ret.setStr(tztCert_UserID, userID);
            ret.setErrorNo(tztZFCosignErrUnfind);
            ret.setErrorMessage(String.format(Locale.getDefault(), "未找到对应%s", pinLable));
            return ret;
        }

        byte[] orgUserID = userID.getBytes();
        byte[] userEncKey = encCosignKey(null);
        byte[] valUserID = decryptData(encryptUserID, userEncKey);
        if (valUserID == null || !Arrays.equals(orgUserID, valUserID)) {
            ret.setErrorNo(tztZFCosignErrUnfind);
            ret.setErrorMessage(String.format(Locale.getDefault(), "未找到对应%s", pinLable));
            return ret;
        }

        byte[] cosignKey = encCosignKey(p1);
        if (cosignKey == null || cosignKey.length != 32) {
            ret.setErrorNo(tztZFCosignErrParam);
            ret.setErrorMessage("P码错误");
            return ret;
        }
        ret.setStr(tztCert_UserID, userID);
        byte[] encryptCertInfo = values.getBytes(tztCert_CertInfo);
        if (encryptCertInfo != null && encryptCertInfo.length > 0) {
            byte[] certInfo = decryptData(encryptCertInfo, userEncKey);
            if (certInfo != null && certInfo.length > 0)
                ret.setBytes(tztCert_CertInfo, certInfo);
        }
        byte[] encryptCosignCert = values.getBytes(tztCert_UserCert);
        if (encryptCosignCert == null || encryptCosignCert.length < 1) {
            ret.setErrorNo(tztZFCosignErrUnfind);
            ret.setErrorMessage(String.format(Locale.getDefault(), "未找到对应%s", pinLable));
            return ret;
        }
        byte[] cosignCert = decryptData(encryptCosignCert, cosignKey);
        if (cosignCert == null || cosignCert.length < 1) {
            ret.setErrorNo(tztZFCosignErrParam);
            ret.setErrorMessage("P码错误");
            return ret;
        }
        ret.setBytes(tztCert_UserCert, cosignCert);
        ret.setErrorNo(tztZFCosignErrOK);
        ret.setErrorMessage("成功");

        return ret;
    }

    /**
     * 中焯协同签名初始化
     *
     * @param D  D
     * @param P1 P1
     * @param P  P
     * @return true=初始化成功
     */
    public boolean cosignInitZZ(byte[] D, byte[] P1, byte[] P) {
        return cosignInitZZ(D, P1, P, null);
    }

    /**
     * 中焯协同签名初始化
     *
     * @param D     D
     * @param P1    P1
     * @param P     P
     * @param DMark DMark
     * @return true=初始化成功
     */
    public boolean cosignInitZZ(byte[] D, byte[] P1, byte[] P, String DMark) {
        if (userID == null || userID.length() < 1 || dPin == null || dPin.length != 32)
            return false;

        cosignType = 0;
        this.P1 = P1;
        this.P = P;
        this.D = D;
        this.DMark = DMark;

        clearCert();
        saveCertKeyChain();
        return true;
    }

    public byte[] cosignSignZZ(byte[] k, byte[] r, byte[] s2, byte[] s3) {
        if (k == null || k.length < 1
                || r == null || r.length < 1
                || s2 == null || s2.length < 1
                || s3 == null || s3.length < 1) {
            return null;
        }
        if (this.D == null || this.D.length < 1)
            return null;

        byte[] s = tztZFCosignSSL.cosignStep3(this.D, k, r, s2, s3);
        if (s == null || s.length < 1)
            return null;
        byte[] c_R = tztZFDataEncrypt.encryptOper(tztZFOperType.tztZFDecrypt, tztZFEncryptType.tztZFHEX, null, r, 0);
        int rLen = c_R.length;
        byte[] c_S = tztZFDataEncrypt.encryptOper(tztZFOperType.tztZFDecrypt, tztZFEncryptType.tztZFHEX, null, s, 0);
        int sLen = c_S.length;

        //使用实际长度进行创建,否则存在签名失败的情况
        byte[] sign = new byte[rLen + sLen + 4 + 4];
        byte[] cRLen = intToByteArray(rLen);
        System.arraycopy(cRLen, 0, sign, 0, 4);
        System.arraycopy(c_R, 0, sign, 4, c_R.length);
        byte[] cSLen = intToByteArray(sLen);
        System.arraycopy(cSLen, 0, sign, c_R.length + 4, 4);
        System.arraycopy(c_S, 0, sign, c_R.length + 4 + 4, c_S.length);
        return sign;
    }

    /**
     * 设置使用安全码类型
     *
     * @param bUseCustomSafetyCode 是否使用自定义安全码 YES使用自定义， NO使用系统安全码
     */
    public void setBUseCustomSafetyCode(boolean bUseCustomSafetyCode) {
        this.bUseCustomSafetyCode = bUseCustomSafetyCode;
    }

    /**
     * int值转byte数组，低位在前
     *
     * @param value int数据
     * @return 转换后的byte数组
     */
    private static byte[] intToByteArray(int value) {
        return new byte[]{
                (byte) (value & 0xFF),
                (byte) ((value >> 8) & 0xFF),
                (byte) ((value >> 16) & 0xFF),
                (byte) ((value >> 24) & 0xFF)
        };
    }

    public int cosignStepZZ(byte[] P) {
        if (this.D == null || this.D.length < 1)
            return -1;//未初始化
        if (this.P == null || this.P.length < 1 || !Arrays.equals(this.P, P))
            return 0;//初始化、但未生成P10、或者P10已经更改
        if (this.signCert == null || this.signCert.length < 1
                || this.encCert == null || this.encCert.length < 1
                || this.encKey == null || this.encKey.length < 1)
            return 1;//有P10，但无证书导入
        return 2;//已有证书导入
    }

    /**
     * 导入证书
     *
     * @param applyCert  证书应答信息
     * @param cosignType 类型
     * @param appKey     appKey
     * @return true=导入成功
     */
    public boolean cosignImportZZ(tztZFNetCosignHead.tztZFNetCosignRespApplyCert applyCert, int cosignType, String appKey) {
        String userID = null;
        int certType = 0;
        byte[] encCert = null;
        byte[] encKey = null;
        String serialNumber = null;
        String subject = null;
        byte[] signCert = null;
        Date notBefore = null;
        Date notAfter = null;
        Date signDate = null;
        String status = null;
        String deviceInfo = null;
        String deviceID = null;

        if (applyCert != null) {
            userID = applyCert.getUserID();
            certType = applyCert.getCertType();
            encCert = tztZFCosignSSL.decryptCosignCert(applyCert.getCertSignBufKmc(), appKey, 1);
            encKey = tztZFCosignSSL.decryptCosignCert(applyCert.getEncPrivateKeyUser(), appKey, 1);

            serialNumber = applyCert.getCertSerialNumber();
            subject = applyCert.getCertSubjectDN();

            signCert = applyCert.getCertSignBuf().getBytes();

            notBefore = tztZFCertificateInfo.dateFromString(applyCert.getCertNotBefore());
            notAfter = tztZFCertificateInfo.dateFromString(applyCert.getCertNotAfter());
            signDate = tztZFCertificateInfo.dateFromString(applyCert.getCertSignDate());

            status = applyCert.getCertStatus();
            deviceInfo = applyCert.getDeviceInfo();
            deviceID = applyCert.getDeviceID();
        }
        this.userID = userID;
        this.cosignType = cosignType;
        this.encType = 1;
        this.certType = certType;
        this.encCert = encCert;
        this.encKey = encKey;
        if (this.encCert == null || this.encCert.length < 1 || this.encKey == null || this.encKey.length < 1)
            return false;

        this.signInfo.setUserID(userID);
        this.signInfo.setSerialNumber(serialNumber);
        this.signInfo.setSubject(subject);
        this.subjectDN = subject;
        this.signType = 1;
        this.signCert = signCert;
        this.signKey = null;

        this.signInfo.setNotBefore(notBefore);
        this.signInfo.setNotAfter(notAfter);
        this.signInfo.setSignDate(signDate);

        this.signInfo.setStatus(status);
        this.signInfo.setDeviceInfo(deviceInfo);
        this.signInfo.setDeviceID(deviceID);
        this.signInfo.setLocal(true);
        return saveCertKeyChain();
    }

    private static String userCertKey(String userID) {
        if (userID == null || userID.length() < 1)
            return "";
        String deviceID = getDeviceID();
        if (tztZFTextUtils.isEmpty(deviceID))
            return "";
        String fileName = String.format(Locale.getDefault(), "tztZFUserCertData_%s_%s.data", userID, deviceID);
        byte[] dData = tztZFDataEncrypt.encryptMD5_32(fileName.getBytes());
        if (dData == null)
            return "";
        return new String(dData);
    }

    /**
     * P进行比较 YES 与本地一致， NO与本地不一致
     * 如果P为空，不做处理，默认相等
     *
     * @param P P
     * @return 是否和本地的P相同
     */
    public boolean compareP(byte[] P) {
        return P == null || Arrays.equals(this.P, P);
    }

    /**
     * 获取设备唯一号
     *
     * @return 设备唯一号
     */
    public static String getDeviceID() {
        tztZFDataMapBase<String, Object> dataMap = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dataMap == null) {
            dataMap = new tztZFDataMapBase<>();
        }
        String deviceID = dataMap.getStr(tztZFDeviceID_Cosign);
        if (deviceID == null || deviceID.length() <= 0) {
            //FIXME: 生成唯一号
            deviceID = tztZFBaseDeviceInfo.getAndroidId(tztZFNetCosign.getContext());
            if (tztZFTextUtils.isEmpty(deviceID)) {
                deviceID = UUID.randomUUID().toString();
            }
            if (tztZFTextUtils.isEmpty(deviceID)) {
                return "";
            }
            deviceID = deviceID.replaceAll("-", "");
            dataMap.setStr(tztZFDeviceID_Cosign, deviceID);
            tztZFKeyChain.save(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign, dataMap);
        }
        return deviceID;
    }

    /**
     * 获取设备信息，以(,)分隔
     *
     * @return 设备信息
     */
    public static String getDeviceInfo() {
        return tztZFBaseDeviceInfo.getSysDeviceInfo();
    }

    private static String caCertKey() {
        String fileName = "tztZFCACertData.data";
        byte[] md5 = tztZFDataEncrypt.encryptMD5_32(fileName.getBytes());
        if (md5 == null || md5.length < 1)
            return "";
        return new String(md5);
    }

    /**
     * 读取CA证书信息
     *
     * @return CA证书信息字典
     */
    @tztZFNullable
    public static tztZFDataMapBase<String, Object> readCACert() {
        String key = caCertKey();
        if (tztZFTextUtils.isEmpty(key))
            return null;

        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1)
            return null;
        return dict.getObj(key);
    }

    /**
     * 更新CA信息数据
     *
     * @param caCert ca信息字典
     * @return true=成功
     */
    public static boolean writeCACert(tztZFDataMapBase<String, Object> caCert) {
        String key = caCertKey();
        if (tztZFTextUtils.isEmpty(key))
            return false;
        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1)
            return false;
        if (caCert == null || caCert.getCount() < 1) {
            dict.removeByKey(key);
        } else {
            dict.setObj(key, caCert);
        }
        tztZFKeyChain.save(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign, dict);
        return true;
    }

    private static byte[] decryptData(byte[] data, byte[] key) {
        if (data == null || data.length < 1 || key == null || key.length != 32)
            return null;
        //先base64解码
        byte[] base64 = tztZFDataEncrypt.decryptBase64(data);
        return tztZFDataEncrypt.encryptOper(tztZFOperType.tztZFDecrypt, tztZFEncryptType.tztZFSM4, key, base64, 0);
    }

    private static byte[] encryptData(byte[] data, byte[] key) {
        if (data == null || data.length < 1 || key == null || key.length != 32)
            return null;
        byte[] encData = tztZFDataEncrypt.encryptOper(tztZFOperType.tztZFEncrypt, tztZFEncryptType.tztZFSM4, key, data, 0);
        //base64编码
        return tztZFDataEncrypt.encryptBase64(encData);
    }

    private static byte[] encCosignKey(byte[] key) {
        return tztZFNetCosignEx.cosignKey(key);
    }

    private static tztZFCertificateInfo readCertInfo(tztZFDataMapBase<String, Object> dict) {
        if (dict == null || dict.getCount() < 1)
            return null;
        byte[] encryptUser = dict.getBytes(tztCert_UserID);
        if (encryptUser == null || encryptUser.length < 1)
            return null;
        byte[] encryptCertInfo = dict.getBytes(tztCert_CertInfo);
        if (encryptCertInfo == null || encryptCertInfo.length < 1)
            return null;
        byte[] userEncKey = encCosignKey(null);
        byte[] certInfo = decryptData(encryptCertInfo, userEncKey);
        if (certInfo == null || certInfo.length < 1)
            return null;
        tztZFCertificateInfo signInfo = new tztZFCertificateInfo();
        loadCertInfo(signInfo, certInfo);
        return signInfo;
    }

    /**
     * 获取本地证书列表
     *
     * @return 本地证书列表
     */
    @tztZFNullable
    public static ArrayList<tztZFCertificateInfo> localCertListZZ() {
        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null)
            return null;
        final ArrayList<tztZFCertificateInfo> ayList = new ArrayList<>();
        dict.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                if (value instanceof tztZFDataMapBase) {
                    tztZFCertificateInfo certInfo = readCertInfo((tztZFDataMapBase<String, Object>) value);
                    if (certInfo != null) {
                        ayList.add(certInfo);
                    }
                }
                return false;
            }
        });
        return ayList;
    }

    /**
     * 清除证书
     *
     * @param userID 客户号
     * @return true=成功
     */
    public static boolean removeLocalCertZZ(String userID) {
        if (tztZFTextUtils.isEmpty(userID))
            return false;
        String strUserKey = userCertKey(userID);
        if (tztZFTextUtils.isEmpty(strUserKey))
            return false;
        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict != null) {
            dict.removeByKey(strUserKey);
            tztZFKeyChain.save(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign, dict);
        }
        return true;
    }

    /**
     * 根据userid获取系统安全码
     *
     * @param userID userID
     * @return 系统安全码
     */
    public static String loadSystemSafetyCode(String userID) {
        if (tztZFTextUtils.isEmpty(userID))
            return "";
        String strUserKey = userCertKey(userID);
        if (tztZFTextUtils.isEmpty(strUserKey)) {
            return "";
        }
        tztZFDataMapBase<String, Object> dict = tztZFKeyChain.load(tztZFNetCosign.getContext(), tztZFDeviceID_Cosign);
        if (dict == null || dict.getCount() < 1) {
            return "";
        }
        tztZFDataMapBase<String, Object> userDic = dict.getObj(strUserKey);
        if (userDic == null || dict.getCount() < 1) {
            return "";
        }
        String safetyCode = userDic.getStr(tztCert_SystemSafetyCode);
        if (tztZFTextUtils.isEmpty(safetyCode)) {
            return "";
        }
        byte[] encryptSafety = safetyCode.getBytes();
        if (encryptSafety == null || encryptSafety.length < 1) {
            return "";
        }
        byte[] dataUserID = userID.getBytes();
        byte[] safetyEncKey = encCosignKey(dataUserID);
        byte[] safetyDecData = decryptData(encryptSafety, safetyEncKey);
        return new String(safetyDecData);
    }

    /**
     * 根据P1重置安全码
     *
     * @param userID 客户号
     * @param newPin 新的Pin码
     * @param P1     P1
     * @return 结果数据字典
     */
    public static tztZFDataMapBase<String, Object> resetCertPin(String userID, String newPin, byte[] P1, boolean bUseCustomSafetyCode) {
        byte[] dPin = tztZFNetCosignEx.SM3(newPin.getBytes());
        if (dPin.length != 32) {
            tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
            ret.setErrorNo(tztZFCosignErrParam);
            ret.setErrorMessage("参数错误!");
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("参数错误DPin:%s leng:%s, newPin:%s", dPin, dPin.length, newPin);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_ResetCert,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return ret;
        }
        tztZFDataMapBase<String, Object> ret = loadCertKeyChainP1(userID, P1);
        if (ret.getErrorNo() != tztZFCosignErrOK)
            return ret;

        tztZFCosignCert certObj = new tztZFCosignCert();
        certObj.bUseCustomSafetyCode = bUseCustomSafetyCode;
        certObj.userID = userID;
        certObj.pin = newPin;
        certObj.dPin = dPin;
        certObj.P1 = P1;
        certObj.nHave = 0;
        byte[] certData = ret.getBytes(tztCert_UserCert);
        certObj.loadCertData(certData);
        byte[] certInfo = ret.getBytes(tztCert_CertInfo);
        certObj.signInfo = new tztZFCertificateInfo();
        loadCertInfo(certObj.signInfo, certInfo);
        //判断保存是否成功
        boolean result = certObj.saveCertKeyChain();
        if (!result) {
            ret.setErrorNo(tztZFCosignErrOther);
        }
        return ret;
    }

    @Override
    public String toString() {
        return "tztZFCosignCert{" +
                "userID='" + userID + '\n' +
                "cosignType=" + cosignType + "\n" +
                "certType=" + certType + "\n" +
                "subjectDN='" + subjectDN + '\n' +
                "D=" + (D != null ? tztZFDataEncrypt.encryptBase64Str(D) : "null") + '\n' +
                "P1=" + (P1 != null ? tztZFDataEncrypt.encryptBase64Str(P1) : "null") + '\n' +
                "P=" + (P != null ? tztZFDataEncrypt.encryptBase64Str(P) : "null") + '\n' +
                "encType=" + encType + '\n' +
                "encCert=" + (encCert != null ? tztZFDataEncrypt.encryptBase64Str(encCert) : "null") + '\n' +
                "encKey=" + (encKey != null ? tztZFDataEncrypt.encryptBase64Str(encKey) : "null") + '\n' +
                "signType=" + signType + '\n' +
                "signCert=" + (signCert != null ? tztZFDataEncrypt.encryptBase64Str(signCert) : "null") + '\n' +
                "signKey=" + (signKey != null ? tztZFDataEncrypt.encryptBase64Str(signKey) : "null") + '\n' +
                '}';
    }

    interface tztZFCosignCertError {
        void onCallBack(int errorNo, String errorMessage);
    }

}
