package com.tztzf.network.cosign;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.tztZFNetWork;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.core.tztZFCosignSSL;
import com.tztzf.protocol.core.tztZFDataEncrypt;
import com.tztzf.protocol.define.tztZFEncryptType;
import com.tztzf.protocol.define.tztZFOperType;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Cosign;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFHEX;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFSM4;
import static com.tztzf.protocol.define.tztZFOperType.tztZFDecrypt;
import static com.tztzf.protocol.define.tztZFOperType.tztZFEncrypt;

/**
 * 协同签名扩展
 */
public class tztZFNetCosignEx {

    /// key 定义
    /** CA文件版本 */
    private static final String tztCosignEx_CAFileVer = "Ver";
    /** 版本 */
    private static final String tztCosignEx_Ver = "Ver";
    /** 服务器时间 */
    public static final String tztCosignEx_CurrentTime = "CurrentTime";
    /** 是否验签标志 */
    public static final String tztCosignEx_NoSignFlag = "NoSignFlag";
    /** CA证书信息字典 */
    private static final String tztCosignEx_CACertDict = "CACertDict";
    /** CA证书 */
    public static final String tztCosignEx_CACert = "CACert";
    /** 证书类别 */
    public static final String tztCosignEx_ActiveCertType = "ActiveCertType";
    /** 协同签名类别 */
    public static final String tztCosignEx_ActiveCosignType = "ActiveCosignType";
    /** 状态 */
    public static final String tztCosignEx_Status = "Status";
    /** 协议 */
    public static final String tztCosignEx_Protocol = "Protocol";
    /** 加密证书 */
    public static final String tztCosignEx_EncCert = "EncCert";
    /** 加密key */
    public static final String tztCosignEx_EncCertKey = "EncCertKey";
    /** 服务PubKey */
    public static final String tztCosignEx_ServerPubKey = "ServerPKey";
    /** 服务PubKey序号 */
    public static final String tztCosignEx_ServerPubKeyIndex = "ServerPKeyIndex";
    /** iDA */
    public static final String tztCosignEx_IDA = "IDA";
    /** iDB */
    public static final String tztCosignEx_IDB = "IDB";
    /** 客户端PubKey */
    public static final String tztCosignEx_ClientPubKey = "ClientPKey";
    /** 客户端PubKey序号 */
    public static final String tztCosignEx_ClientPubKeyIndex = "ClientPKeyIndex";
    /** 协商类别 */
    public static final String tztCosignEx_Handtype = "Handtype";
    /** 加密类别 */
    public static final String tztCosignEx_Encrypttype = "Encrypttype";
    /** 加密pub键值 */
    public static final String tztCosignEnc_KeyIndex = "keyindex";
    /** 加密pub */
    public static final String tztCosignEnc_Enc = "enc";
    ///即将过期前提醒天数
    public static final String tztCosignEx_AlertBeforeExpire = "AlertBeforeDays";

    private static final String tztCosignCert_Pub = "C=CN,ST=江苏,L=南京,O=天威诚信测试系统,OU=SM2KMC";

    //CA证书类型
    private static final String tztCosignCA_certType_Idx = "certType_Idx";
    //CA名称
    private static final String tztCosignCA_CAName_idx = "CAName_idx";
    //CA证书
    private static final String tztCosignCA_Cert_idx = "Cert_idx";
    //部门名OU
    private static final String tztCosignCA_OU_idx = "OU_idx";
    //组织O
    private static final String tztCosignCA_O_idx = "O_idx";
    //过期日期
    private static final String tztCosignCA_EndDate_idx = "EndDate_idx";

    private static final String tztCosignEnc_Key = "3OuwIgoi9auuMWXum";
    ///
    private static tztZFNetCosignEx g_netCosignExIns = null;

    private String appKey;
    private String cosignPub;
    private tztZFDataMapBase<String, Object> cosignExMap;

    public static synchronized tztZFNetCosignEx getShareInstance() {
        if (g_netCosignExIns == null) {
            g_netCosignExIns = new tztZFNetCosignEx();
        }
        return g_netCosignExIns;
    }

    private tztZFNetCosignEx() {
        initdata();
    }

    private void initdata() {
        cosignExMap = new tztZFDataMapBase<>();
        appKey = "";
        cosignPub = "";
        cosignExMap.setStr(tztCosignEx_Ver, "1.01.001");
        cosignExMap.setInt(tztCosignEx_Status, 0);
        cosignExMap.setInt(tztCosignEx_Protocol, 2013);
    }

    private void readCAData() {
        tztZFDataMapBase<String, Object> fileCA = tztZFCosignCert.readCACert();
        if (fileCA != null && fileCA.getCount() > 0) {
            /// 初始值
            tztZFDataMapBase<String, Object> importCA = cosignExMap.getObj(tztCosignEx_CACertDict);
            String importVer = "";
            if (importCA != null && importCA.getCount() > 0) {
                importVer = importCA.getStr(tztCosignEx_CAFileVer);
                String fileVer = fileCA.getStr(tztCosignEx_CAFileVer);
                /// 导入和写入的版本不一致，写入的丢弃，不处理
                if (!tztZFTextUtils.isEmpty(importVer) && !tztZFTextUtils.isEmpty(fileVer) && !importVer.equals(fileVer)) {
                    return;
                }
            }

            tztZFDataMapBase<String, Object> dictCA = fileCA;
            if (!tztZFTextUtils.isEmpty(importVer)) {
                dictCA.setStr(tztCosignEx_CAFileVer, importVer);
            }
            if (importCA != null && importCA.getCount() > 0) {
                Set<String> ayKeys = importCA.getAllLowKeys();
                for (String strKey : ayKeys) {
                    if (strKey.equals(tztCosignEx_CAFileVer))
                        continue;
                    Object dictVal = importCA.getObj(strKey);
                    if (dictVal == null || !(dictVal instanceof tztZFDataMapBase)) {
                        continue;
                    }
                    if (((tztZFDataMapBase<String, Object>) dictVal).getCount() < 1)
                        continue;
                    tztZFDataMapBase<String, Object> fileVal = dictCA.getObj(strKey);
                    if (fileVal == null || fileVal.getCount() > 0)
                        continue;
                    dictCA.setObj(strKey, dictVal);
                }
            }
            setCACertDataWithDict(fileCA);
        }
    }

    private void writeCAData() {
        tztZFDataMapBase<String, Object> dictCA = cosignExMap.getObj(tztCosignEx_CACertDict);
        if (dictCA != null && dictCA.getCount() > 0)
            tztZFCosignCert.writeCACert(dictCA);
    }

    public static boolean setAppKey(String appKey, tztZFDataMapBase<String, Object> cert) {
        return getShareInstance()._setAppKey(appKey, cert);
    }

    private boolean _setAppKey(String appKey, tztZFDataMapBase<String, Object> cert) {
        this.appKey = appKey;
        if (cert != null && cert.getCount() > 0) {
            tztZFDataMapBase<String, Object> dictCA = dictCADataMap(cert, false);
            if (dictCA != null && dictCA.getCount() > 0) {
                setCACertDataWithDict(dictCA);
            }
        }
        readCAData();
        return true;
    }

    //导入客户端关键加密密钥证书
    //1.业务包协议  2013，2021（即国密 SM4SM3）
    //2.客户端加密证书文件内容
    //3.客户端加密私钥文件内容
    //4.SM2握手服务端公钥ServerPubKey
    //5.SM2握手服务端公钥(密码卡或密码机）中的序号 KeyIndex
    //6.IDA(TztClient)
    //7.IDB(TztServer)
    //8.SM2协商时客户端初始ClientKey
    //9.SM2协商时客户端初始公钥ClientPx
    //10.SM2协商时客户端初始公钥ClientPy
    private boolean dealCosignData(byte[] data) {
        boolean bRet = false;
        if (this.appKey == null || this.appKey.length() < 1)
            return bRet;

        byte[] orgData = tztZFCosignSSL.decryptCosignCert(data, this.appKey, 1);
        if (orgData == null || orgData.length < 1)
            return bRet;
        int nMax = orgData.length;
        int nLen = 0;
        int nPos = 0;
        ArrayList<String> ayKeys = new ArrayList<>();
        ayKeys.add(tztCosignEx_Protocol);
        ayKeys.add(tztCosignEx_EncCert);
        ayKeys.add(tztCosignEx_EncCertKey);
        ayKeys.add(tztCosignEx_ServerPubKey);
        ayKeys.add(tztCosignEx_ServerPubKeyIndex);
        ayKeys.add(tztCosignEx_IDA);
        ayKeys.add(tztCosignEx_IDB);
        ayKeys.add(tztCosignEx_ClientPubKey);
        ayKeys.add(tztCosignEx_ClientPubKeyIndex);
        ayKeys.add(tztCosignEx_Handtype);
        ayKeys.add(tztCosignEx_Encrypttype);

        for (String strKey : ayKeys) {
            if (nMax < nPos + 2)
                break;
            nLen = 0;
            nLen = tztZFPublicFun.getInt(orgData, nPos, 2);
            nPos += 2;
            if (nLen < 0 || nLen > nMax - nPos) {
                tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Warn, "tztZFNetCosign", "解析错误[%s]", strKey);
                return bRet;
            }
            byte[] val = new byte[nLen];
            System.arraycopy(orgData, nPos, val, 0, nLen);
            if (val.length > 0) {
                if (strKey.equals(tztCosignEx_ServerPubKey)
                        || strKey.equals(tztCosignEx_ClientPubKey)) {
                    val = tztZFDataEncrypt.hexDecrypt(val);
                }
                cosignExMap.setBytes(strKey, val);
                bRet = true;
            } else {
                cosignExMap.removeByKey(strKey);
            }
            nPos += nLen;
        }
        return bRet;
    }

    /*
     Grid    string    多条记录； 单条记录，无Header，按|分隔
     certType_Idx    int    Grid记录中证书类型的索引 = 0
     CAName_idx    int    Ca名称索引值 = 1
     Cert_idx    int    Grid记录中CA证书的索引  = 2
     OU_idx    int    部门名OU的索引 = 3
     O_idx    int    组件O的索引 = 4
     EndDate_idx    int    证书到期日索引值 = 5
     */
    private tztZFDataMapBase<String, Object> dictCAResp(tztZFDataMapBase<String, Object> resp) {
        if (resp == null || resp.getInt("ErrorNo") < 0)
            return null;
        ArrayList<ArrayList<String>> ayGrid = tztZFDataMapBase_AddToReq.getArray(resp, "Grid");
        if (ayGrid == null || ayGrid.size() < 1)
            return null;
        tztZFDataMapBase<String, Object> dictVal = new tztZFDataMapBase<>();
        dictVal.setDataMap(resp);

        dictVal.setObj("Grid", ayGrid);
        return dictCADataMap(dictVal, true);
    }

    private tztZFDataMapBase<String, Object> dictCADataMap(tztZFDataMapBase<String, Object> dictVal, boolean bResp) {
        if (dictVal == null)
            return null;
        ArrayList<Object> ayGrid = dictVal.getObj("Grid");
        if (ayGrid == null || ayGrid.size() < 1)
            return null;

        tztZFDataMapBase<String, Object> dictCert = new tztZFDataMapBase<>();
        if (!bResp) {
            String fileVer = dictVal.getStr(tztCosignEx_CAFileVer);
            if (!tztZFTextUtils.isEmpty(fileVer))
                dictCert.setStr(tztCosignEx_CAFileVer, fileVer);
        }

        int certType_Idx = dictVal.getInt(tztCosignCA_certType_Idx, 0);
        int CAName_idx = dictVal.getInt(tztCosignCA_CAName_idx, 1);
        int Cert_idx = dictVal.getInt(tztCosignCA_Cert_idx, 2);
        int OU_idx = dictVal.getInt(tztCosignCA_OU_idx, 3);
        int O_idx = dictVal.getInt(tztCosignCA_O_idx, 4);
        int EndDate_idx = dictVal.getInt(tztCosignCA_EndDate_idx, 5);
        for (int i = 0; i < ayGrid.size(); i++) {
            ArrayList<String> ayCert = null;
            if (bResp) {
                ayCert = (ArrayList<String>) ayGrid.get(i);
            } else {
                String strGrid = (String) ayGrid.get(i);
                if (strGrid == null || strGrid.length() < 1)
                    continue;
                ayCert = new ArrayList<>();
                tztZFPublicFun.split(strGrid, "|", ayCert);
            }
            if (ayCert.size() < 5)
                continue;
            String certType = ayCert.get(certType_Idx);
            if (certType == null || certType.length() < 1)
                continue;
            String strCACert = ayCert.get(Cert_idx);
            if (strCACert == null || strCACert.length() < 1)
                continue;
            String strName = ayCert.get(CAName_idx);
            String strOU = ayCert.get(OU_idx);
            String strO = ayCert.get(O_idx);
            String strEndDate = "";
            if (ayCert.size() > EndDate_idx) {
                strEndDate = ayCert.get(EndDate_idx);
            }
            tztZFDataMapBase<String, Object> cert = new tztZFDataMapBase<>();
            cert.setObj(tztCosignCA_certType_Idx, certType);
            cert.setObj(tztCosignCA_Cert_idx, strCACert);
            if (strName != null && strName.length() > 0) {
                cert.setObj(tztCosignCA_CAName_idx, strName);
            }
            if (strOU != null && strOU.length() > 0) {
                cert.setObj(tztCosignCA_OU_idx, strOU);
            }
            if (strO != null && strO.length() > 0) {
                cert.setObj(tztCosignCA_O_idx, strO);
            }
            if (strEndDate != null && strEndDate.length() > 0) {
                cert.setObj(tztCosignCA_EndDate_idx, strEndDate);
            }
            dictCert.setObj(certType, cert);
        }
        return dictCert;
    }


    /**
     * int值转byte数组，低位在前
     *
     * @param value int数据
     * @return 转换后的byte数组
     */
    private static byte[] ShortToByteArray(int value) {
        return new byte[]{
                (byte) (value & 0xFF),
                (byte) ((value >> 8) & 0xFF)
        };
    }

    private void setCACertDataWithDict(tztZFDataMapBase<String, Object> dictCA) {
        if (dictCA == null || dictCA.getCount() < 1)
            return;
        tztZFDataMapBase<String, Object> dictVal = new tztZFDataMapBase<>();
        dictVal.setDataMap(dictCA);

        Set<String> allKeys = dictCA.getAllLowKeys();
        ByteArrayOutputStream certdata = new ByteArrayOutputStream();
        for (String strKey : allKeys) {
            if (strKey.length() < 1 || strKey.equalsIgnoreCase("ver"))
                continue;
            Object value = dictCA.getObj(strKey);
            if (value == null || !(value instanceof tztZFDataMapBase))
                continue;
            if (((tztZFDataMapBase<?, ?>) value).getCount() > 0) {
                String strCert = ((tztZFDataMapBase<?, ?>) value).getStr(tztCosignCA_Cert_idx);
                if (strCert == null || strCert.length() < 1)
                    continue;
                byte[] certData = tztZFCosignSSL.decryptCosignCert(strCert.getBytes(), appKey, 1);
                if (certData != null && certData.length > 0) {
                    String certList = new String(certData);
                    if (certList != null && certList.length() > 0) {
                        String[] ayCert = certList.split("-----BEGIN CERTIFICATE-----");
                        if (ayCert != null) {
                            boolean bContinue = false;
                            for (String cert : ayCert) {
                                if (cert.length() < 1)
                                    continue;
                                String val = String.format(Locale.getDefault(), "-----BEGIN CERTIFICATE-----%s", cert);
                                tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Info, "tztZFSSL_Cert", "action:dealCaData ca:%s", val);
                                byte[] valData = val.getBytes();
                                int valLen = valData.length;
                                byte[] cLen = ShortToByteArray(valLen);
                                certdata.write(cLen, 0, 2);
                                certdata.write(valData, 0, valLen);
                                bContinue = true;
                            }
                            if (bContinue) {
                                continue;
                            }
                        }
                    }
                }
            }
            dictVal.removeByKey(strKey);
        }
        byte[] result = certdata.toByteArray();
        if (result.length > 0) {
            cosignExMap.setBytes(tztCosignEx_CACert, result);
        }
        cosignExMap.setObj(tztCosignEx_CACertDict, dictVal);
    }

    private boolean dealCaData(tztZFDataMapBase<String, Object> resp) {
        tztZFDataMapBase<String, Object> dictCA = dictCAResp(resp);
        if (dictCA == null || dictCA.getCount() < 1)
            return true;
        /// 初始值
        tztZFDataMapBase<String, Object> dictCACert = cosignExMap.getDataMap(tztCosignEx_CACertDict);
        if (dictCACert != null && dictCACert.getCount() > 0) {
            Set<String> allKeys = dictCACert.getAllLowKeys();
            for (String strKey : allKeys) {
                if (strKey.equals(tztCosignEx_CAFileVer)) {
                    String fileVer = dictCACert.getStr(strKey);
                    dictCA.setStr(strKey, fileVer);
                    continue;
                }
                tztZFDataMapBase<String, Object> dictVal = dictCACert.getDataMap(strKey);
                if (dictVal == null || dictVal.getCount() < 1)
                    continue;
                tztZFDataMapBase<String, Object> respVal = dictCA.getDataMap(strKey);
                if (respVal != null && respVal.getCount() > 0)
                    continue;
                dictCA.setObj(strKey, dictVal);
            }
        }
        //设置CA数据
        setCACertDataWithDict(dictCA);
        //缓存
        writeCAData();
        return dictCA.getCount() > 0;
    }

    public static boolean reqCosignExInfo(tztZFDataMapBase<String, Object> reqInfo, tztZFNetCosignCallBack callBack) {
        return getShareInstance().reqCosignExInfoIn(reqInfo, callBack);
    }

    private boolean reqCosignExInfoIn(tztZFDataMapBase<String, Object> reqInfo, final tztZFNetCosignCallBack callBack) {
        String strAction = "51980";
        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        int nDebug = 0;
        if (reqInfo != null) {
            nDebug = reqInfo.getInt("Debug");
            tztZFDataMapBase_AddToReq.addToReqObj(reqInfo, reqObj);
        }
        reqObj.setTimeOut(6 * 1000);
        reqObj.setStr("Action", strAction);
        reqObj.setStr("GMVersion", cosignExMap.getStr(tztCosignEx_Ver));
        reqObj.setStr("Len", "32");
        reqObj.setStr("EncryptType", "1");

        tztZFDataMapBase<String, Object> dictCA = cosignExMap.getObj(tztCosignEx_CACertDict);
        if (dictCA != null && dictCA.getCount() > 0 && (nDebug != 1)) {
            Set<String> ayKey = dictCA.getAllLowKeys();
            List<String> keys = new ArrayList<>(ayKey);
            String certtypelist = "";
            for (int i = 0; i < keys.size(); i++) {
                String strType = keys.get(i);
                tztZFDataMapBase<String, Object> dictVal = dictCA.getDataMap(strType);
                if (dictVal != null && dictVal.getCount() > 0) {
                    certtypelist = String.format(Locale.getDefault(), "%s%s", certtypelist, strType);
                    String strData = dictVal.getStr(tztCosignCA_EndDate_idx);
                    if (strData != null && strData.length() > 0) {
                        certtypelist = String.format(Locale.getDefault(), "%s:%s,", certtypelist, strData);
                        continue;
                    }
                }
            }
            if (certtypelist.length() > 0) {
                if (certtypelist.endsWith(","))
                    certtypelist = certtypelist.substring(0, certtypelist.length() - 1);
                reqObj.setStr("CertTypeList", certtypelist);
            }
        }
        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj obj = new tztZFNetCosign.tztZFCosignSendRecvObj();
            obj.setSendData(reqObj.getSendValues());
            obj.setLinkType(tztZFLinkType_Cosign);
            obj.setRecvBlock(new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    //应答数据处理
                    if (receiveData == null)
                        return;
                    boolean bStatus = dealCosignExInfoResponse(receiveData);
                    if (callBack != null) {
                        callBack.netCosignCallBack(bStatus, receiveData);
                    }
                }
            });
            return tztZFNetCosign.getShareInstance().getSendRecvBlock().onCallBack(obj);
        } else {
            return tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    if (callBack != null) {
                        tztZFDataMapBase<String, Object> retDict = new tztZFDataMapBase<>();
                        retDict.setInt("ErrorNo", c_errorNo);
                        retDict.setStr("ErrorMessage", c_errorMsg);
                        retDict.setInt("tztZFRecvType", c_recvType);
                        callBack.netCosignCallBack(false, retDict);
                    }
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    if (c_respObj == null)
                        return;
                    boolean bStatus = dealCosignExInfoResponse(c_respObj.getDataMap());
                    tztZFDataMapBase<String, Object> retDict = c_respObj.getDataMap();
                    if (c_respObj.getRecvType() != null) {
                        retDict.setInt("tztZFRecvType", c_respObj.getRecvType().value());
                    }
                    if (callBack != null) {
                        callBack.netCosignCallBack(bStatus, retDict);
                    }
                }
            });
        }
    }

    private boolean dealCosignExInfoResponse(tztZFDataMapBase<String, Object> c_respObj) {
        if (c_respObj == null)
            return false;
        // 状态
        int nGMStatus = c_respObj.getInt("GMStatus");
        boolean bStatus = (nGMStatus == 1);
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param1 = tztZFRecordProcessLogTools.stringFormat("国密状态=%d", nGMStatus);
            String param2 = tztZFRecordProcessLogTools.stringFormat("51980应答数据：%s", c_respObj.toString());
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
            tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_GMStatus,
                    tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        cosignExMap.setInt(tztCosignEx_Status, nGMStatus);
        //建议CA类别
        int nActiveCertType = c_respObj.getInt("ActiveCertType");
        cosignExMap.setInt(tztCosignEx_ActiveCertType, nActiveCertType);
        //建议协同签名类别
        int nActiveCosignType = c_respObj.getInt("ActiveCosignType", 0);
        cosignExMap.setInt(tztCosignEx_ActiveCosignType, nActiveCosignType);
        //服务器时间
        cosignExMap.setStr(tztCosignEx_CurrentTime, c_respObj.getStr("CurrentTime"));
        //是否验签标志=0验签，=1不验签
        cosignExMap.setInt(tztCosignEx_NoSignFlag, c_respObj.getInt("NoSignFlag", 1));
        //即将过期提醒天数
        cosignExMap.setStr(tztCosignEx_AlertBeforeExpire, c_respObj.getStr("AlertBeforeDays"));
        //随机数(Base64)
        String cosignRand = c_respObj.getStr("RandomData");
        if (!tztZFTextUtils.isEmpty(cosignRand)) {
            byte[] decData = tztZFDataEncrypt.decryptBase64(cosignRand.getBytes());
            tztZFCosignSSL.cosignRandSeed(decData);
        }

        byte[] cosignData = c_respObj.getBytes("data");
        if (cosignData != null && cosignData.length > 0) {
            bStatus = bStatus && dealCosignData(cosignData);
        }
        dealCaData(c_respObj);
        return bStatus;
    }

    public static boolean reqCosignRand(int len, final tztZFNetCosignCallBack block) {
        if (block == null)
            return false;
        final tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
        if (len < 1) {
            ret.setErrorNo(-1);
            ret.setErrorMessage("随机数长度不正确，请重试!");
            block.netCosignCallBack(false, ret);
            return true;
        }

        String strAction = "51901";
        tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
        reqObj.setStr("Action", strAction);
        reqObj.setInt("Len", len);
        boolean bOK = false;
        if (tztZFNetCosign.getShareInstance().getSendRecvBlock() != null) {
            tztZFNetCosign.tztZFCosignSendRecvObj obj = new tztZFNetCosign.tztZFCosignSendRecvObj();
            obj.setSendData(reqObj.getSendValues());
            obj.setLinkType(tztZFLinkType_Cosign);
            obj.setRecvBlock(new tztZFNetCosign.tztZFNetCosignReceiveBlock() {
                @Override
                public void onCallBack(tztZFDataMapBase<String, Object> receiveData) {
                    if (receiveData != null) {
                        block.netCosignCallBack(receiveData.getInt("ErrorNo") >= 0, receiveData);
                        return;
                    } else {
                        ret.setErrorNo(-2);
                        ret.setErrorMessage("获取随机数失败，请重试!");
                        block.netCosignCallBack(false, ret);
                    }
                }
            });
            bOK = tztZFNetCosign.getShareInstance().getSendRecvBlock().onCallBack(obj);
        } else {
            bOK = tztZFNetWork.sendDataAction(tztZFLinkType_Cosign.value(), reqObj, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                    tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
                    ret.setErrorNo(c_errorNo);
                    ret.setErrorMessage(c_errorMsg);
                    block.netCosignCallBack(false, ret);
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    if (c_respObj != null) {
                        block.netCosignCallBack(c_reqObj.getInt("ErrorNo") >= 0, c_respObj.getDataMap());
                    } else {
                        tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
                        ret.setErrorNo(-2);
                        ret.setErrorMessage("获取随机数失败，请重试！");
                        block.netCosignCallBack(false, ret);
                    }
                }
            });
        }

        if (!bOK) {
            ret.setErrorNo(-1);
            ret.setErrorMessage("获取随机数失败，请重试！");
            block.netCosignCallBack(false, ret);
        }
        return true;
    }

    public static tztZFDataMapBase<String, Object> cosignExInfo() {
        return getShareInstance().cosignExInfoIn();
    }

    private tztZFDataMapBase<String, Object> cosignExInfoIn() {
        return cosignExMap;
    }

    public static byte[] getExDataForKey(String key) {
        tztZFDataMapBase<String, Object> map = cosignExInfo();
        if (map != null) {
            return map.getBytes(key);
        }
        return null;
    }

    public static String getExStrForKey(String key) {
        tztZFDataMapBase<String, Object> map = cosignExInfo();
        if (map != null) {
            return map.getStr(key);
        }
        return null;
    }

    public static int getExIntForKey(String key) {
        tztZFDataMapBase<String, Object> map = cosignExInfo();
        if (map != null) {
            return map.getInt(key);
        }
        return 0;
    }

    public static void initCosignStatus() {
        tztZFDataMapBase<String, Object> map = cosignExInfo();
        if (map != null) {
            int cosignStatus = map.getInt(tztCosignEx_Status);
            if (cosignStatus != 2) {
                //非降级状态，恢复非国密
                map.setInt(tztCosignEx_Status, 0);
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param1 = tztZFRecordProcessLogTools.stringFormat("国密状态=%d", 0);
                    String param2 = tztZFRecordProcessLogTools.stringFormat("message=%s", "恢复国密");
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                    tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_GMStatus,
                            tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                }
            }
        }
    }

    public static void didGMDowngrade() {
        tztZFDataMapBase<String, Object> map = cosignExInfo();
        if (map != null) {
            map.setInt(tztCosignEx_Status, 2);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("国密状态=%d", 2);
                String param2 = tztZFRecordProcessLogTools.stringFormat("message=%s", "国密降级");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(tztZFNetWork.keyForLinkType(tztZFLinkType_Cosign.value()), tztZFRecordEventType.tztZFRecordEventType_GMStatus,
                        tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
        }
    }

    public static tztZFDataMapBase<String, Object> encData(byte[] orgData) {
        return getShareInstance().encDataIn(orgData);
    }

    private tztZFDataMapBase<String, Object> encDataIn(byte[] orgData) {
        byte[] key = cosignExMap.getBytes(tztCosignEx_ServerPubKey);
        String keyIndex = cosignExMap.getStr(tztCosignEx_ServerPubKeyIndex);
        String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
        if (key == null || key.length < 1 || keyIndex == null || keyIndex.length() < 1) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Warn, "tztZFNetCosign", "加密错误[%s未加载]!", pinLable);
            return null;
        }

        byte[] enc = tztZFCosignSSL.encData(orgData, key);
        if (enc != null && enc.length > 0) {
            String strEnc64 = new String(enc);
            if (!tztZFTextUtils.isEmpty(strEnc64)) {
                tztZFDataMapBase<String, Object> ret = new tztZFDataMapBase<>();
                ret.setObj(tztCosignEnc_Enc, strEnc64);
                ret.setObj(tztCosignEnc_KeyIndex, keyIndex);
                return ret;
            }
        }
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Warn, "tztZFNetCosign", "加密错误[%s加密错误]!", pinLable);
        return null;
    }

    /**
     * 设置证书sub公共信息
     *
     * @param certPub 信息字典
     */
    public static boolean setCertPub(tztZFDataMapBase<String, Object> certPub) {
        return getShareInstance().setCertPubIn(certPub);
    }

    //C=CN,ST=江苏,L=南京,O=天威诚信测试系统,OU=SM2KMC,
    //设置证书申请Sub相关信息
    private boolean setCertPubIn(tztZFDataMapBase<String, Object> certPub) {
        if (certPub == null || certPub.getCount() < 1) {
            this.cosignPub = String.format(Locale.CHINA, "%s", tztCosignCert_Pub);
        } else {
            final String[] ret = {""};
            certPub.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                @Override
                public boolean enumerate(String lowKey, String key, Object value) {
                    if (lowKey == null || lowKey.length() < 1 || lowKey.equals("cn") || lowKey.equals("o") || lowKey.equals("ou")) {
                    } else {
                        ret[0] = String.format(Locale.CHINA, "%s%s=%s,", ret[0], key, value);
                    }
                    return false;
                }
            });
            this.cosignPub = String.format(Locale.CHINA, "%s", ret[0]);
        }

        return this.cosignPub != null && this.cosignPub.length() > 0;
    }

    /**
     * 根据userid获取证书sub
     *
     * @param userID 客户号
     */
    public static String getCertSub(String userID) {
        return getShareInstance().getCertSubIn(userID);
    }

    private String getCertSubIn(String userID) {
        if (userID == null || userID.length() < 1)
            return "";
        String strO = "";
        String strOU = "";
        String certtype = cosignExMap.getStr(tztCosignEx_ActiveCertType);
        if (!tztZFTextUtils.isEmpty(certtype)) {
            tztZFDataMapBase<String, Object> dictCert = cosignExMap.getObj(tztCosignEx_CACertDict);
            if (dictCert != null && dictCert.getCount() > 0) {
                tztZFDataMapBase<String, Object> certInfo = dictCert.getDataMap(certtype);
                if (certInfo != null) {
                    strO = certInfo.getStr(tztCosignCA_O_idx);
                    strOU = certInfo.getStr(tztCosignCA_OU_idx);
                }
            }
        }

        String cosignPub = tztCosignCert_Pub;
        if (!tztZFTextUtils.isEmpty(this.cosignPub)) {
            cosignPub = this.cosignPub;
        }

        tztZFDataMapBase<String, String> dictPub = splitToMap(cosignPub, ",", "=");
        if (dictPub != null) {
            dictPub.removeByKey("CN");
            dictPub.removeByKey("O");
            dictPub.removeByKey("OU");
        }

        if (tztZFTextUtils.isEmpty(strO)) {
            strO = "";
        }
        if (tztZFTextUtils.isEmpty(strOU)) {
            strOU = "";
        }

        //重新拼接
        final String[] strCertPub = {String.format(Locale.getDefault(), "CN=%s,O=%s,OU=%s,", userID, strO, strOU)};
        if (dictPub != null) {
            dictPub.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                @Override
                public boolean enumerate(String lowKey, String key, Object value) {
                    strCertPub[0] += key + "=" + value + ",";
                    return false;
                }
            });
        }
        if (strCertPub[0].endsWith(",")) {
            strCertPub[0] = strCertPub[0].substring(0, strCertPub[0].length() - 1);
        }
        return strCertPub[0];
    }

    /**
     * 字符串分割成字典
     *
     * @param strValue 源字符串
     * @param regex    分割符
     * @param kvRegex  key-value分割符
     * @return 字典 or null
     */
    private static tztZFDataMapBase<String, String> splitToMap(String strValue, String regex, String kvRegex) {
        if (tztZFTextUtils.isEmpty(strValue) || tztZFTextUtils.isEmpty(regex))
            return null;
        String[] ayParam = strValue.split(regex);
        if (ayParam.length < 1)
            return null;

        if (tztZFTextUtils.isEmpty(kvRegex))
            kvRegex = "=";

        tztZFDataMapBase<String, String> paramvalue = new tztZFDataMapBase<>();
        for (String strParamValue : ayParam) {
            if (tztZFTextUtils.isEmpty(strParamValue))
                continue;

            int pos = strParamValue.indexOf(kvRegex);
            if (pos > 0) {
                String key = strParamValue.substring(0, pos);
                String value = strParamValue.substring(pos + kvRegex.length());
                paramvalue.setObj(key, value);
            } else {
                String key = strParamValue;
                String value = "";
                paramvalue.setObj(key, value);
            }
        }

        return paramvalue;
    }

    /**
     * SM3摘要处理
     *
     * @param org 原文
     */
    public static byte[] SM3(byte[] org) {
        if (org == null || org.length < 1)
            return null;
        return tztZFDataEncrypt.encryptOper(tztZFOperType.tztZFEncrypt, tztZFEncryptType.tztZFSM3, null, org, 0);
    }

    private byte[] _cosignKey(byte[] key) {
        byte[] appKey = this.appKey.getBytes();
        int nAppKeyLen = appKey.length;
        int nKeyLen = 0;
        if (key != null) {
            nKeyLen = key.length;
        }
        byte[] retKey = new byte[nAppKeyLen + nKeyLen];
        System.arraycopy(appKey, 0, retKey, 0, nAppKeyLen);
        if (nKeyLen > 0) {
            System.arraycopy(key, 0, retKey, nAppKeyLen, nKeyLen);
        }
        return SM3(retKey);
    }

    /**
     * 协同key
     *
     * @param key key内容
     */
    public static byte[] cosignKey(byte[] key) {
        return getShareInstance()._cosignKey(key);
    }

    private String _decryptStrSM(String str) {
        if (str == null || str.length() < 1)
            return null;
        byte[] key = _cosignKey(tztCosignEnc_Key.getBytes());
        byte[] hexData = str.getBytes();
        byte[] smData = tztZFDataEncrypt.encryptOper(tztZFDecrypt, tztZFHEX, null, hexData, 0);
        if (smData == null || smData.length < 1)
            return null;
        byte[] data = tztZFDataEncrypt.encryptOper(tztZFDecrypt, tztZFSM4, key, smData, 0);
        if (data == null || data.length < 1)
            return null;
        return new String(data);
    }

    /**
     * 解密字符串
     *
     * @param str 待解密内容
     */
    public static String decryptStrSM(String str) {
        return getShareInstance()._decryptStrSM(str);
    }

    private String _encryptStrSM(String str) {
        if (str == null || str.length() < 1)
            return null;
        byte[] data = str.getBytes();
        byte[] key = _cosignKey(tztCosignEnc_Key.getBytes());
        byte[] smData = tztZFDataEncrypt.encryptOper(tztZFEncrypt, tztZFSM4, key, data, 0);
        if (smData == null || smData.length < 1)
            return null;
        byte[] hexData = tztZFDataEncrypt.encryptOper(tztZFEncrypt, tztZFHEX, null, smData, 0);
        if (hexData == null || hexData.length < 1)
            return null;
        return new String(hexData);
    }

    /**
     * 加密字符串
     *
     * @param str 待加密内容
     */
    public static String encryptStrSM(String str) {
        return getShareInstance()._encryptStrSM(str);
    }
}
