package com.tztzf.network.define;

import java.util.Locale;

/**
 * ****************************************************************
 * 文件名称 : tztZFLinkType
 * 作   者 : Created by wlz
 * 创建时间 : 2020-03-02 14:11
 * 文件描述 :
 * 注意事项 :
 * 版权声明 : Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史 : 2020-03-02 1.00 初始版本
 * ****************************************************************
 */
public enum tztZFLinkType {

    //region 内部私有属性及函数
    /** 未定义 */
    tztZFLinkType_Non(-1),
    /** 行情 */
    tztZFLinkType_HQ(0),
    /** 交易 */
    tztZFLinkType_JY(1),
    /** 资讯 */
    tztZFLinkType_ZX(2),
    /** 均衡 */
    tztZFLinkType_JH(3),
    /** 统计、认证 */
    tztZFLinkType_RZ(4),
    /** 开户 */
    tztZFLinkType_KH(5),
    /** 协同签名 */
    tztZFLinkType_Cosign(6),
    /** 消息推送 */
    tztZFLinkType_MsgPush(19),
    /** >=20短连接通道 */
    tztZFLinkType_Short(20);
    //endregion

    //region 具体实现

    //region 公布方法（允许其他包访问）

    private int value = 0;

    tztZFLinkType(int value) {
        this.value = value;
    }

    /**
     * 获取枚举int值
     *
     * @return 枚举int值
     */
    public int value() {
        return this.value;
    }

    /**
     * int值转tztZFLinkType对象
     *
     * @param value int值
     * @return tztZFLinkType对象
     */
    public static tztZFLinkType valueOf(int value) {
        for (tztZFLinkType c : tztZFLinkType.values()) {
            if (c.value == value)
                return c;
        }
        return null;
    }

    /**
     * 获取链接通道tag
     *
     * @param linktype 链接通道类别
     * @return 链接通道tag
     */
    public static String nameOf(int linktype) {
        if (linktype >= tztZFLinkType_Short.value())
            return String.format(Locale.CHINA, "SHORT_%d", linktype);
        switch (valueOf(linktype)) {
            case tztZFLinkType_Non:
                return "NON";
            case tztZFLinkType_HQ:
                return "HQ";
            case tztZFLinkType_JY:
                return "JY";
            case tztZFLinkType_ZX:
                return "ZX";
            case tztZFLinkType_JH:
                return "JH";
            case tztZFLinkType_RZ:
                return "RZ";
            case tztZFLinkType_KH:
                return "KH";
            case tztZFLinkType_Cosign:
                return "Cosign";
            case tztZFLinkType_MsgPush:
                return "PUSH";
        }
        return String.format(Locale.CHINA, "LINK_%d", linktype);
    }

    /**
     * 获取链接通道tag
     *
     * @param linktype 链接通道类别
     * @param sdktype  sdktype
     * @return 链接通道tag
     */
    public static String linkTagOf(int linktype, int sdktype) {
        String strType = nameOf(linktype);
        if (linktype >= tztZFLinkType_Short.value())
            return strType;
        switch (sdktype) {
            case 0:
                return String.format("APP-%s", strType);
            case 1:
                return String.format("US-%s", strType);
            case 2:
                return String.format("L2-%s", strType);
            case 3:
                return String.format("SDK-%s", strType);
            case 4:
                return String.format("H5SDK-%s", strType);
            default:
                break;
        }
        return strType;
    }

    //endregion

    //region 保护方法（允许子类访问、允许相同包中访问）
    //endregion

    //region 内部方法（只允许相同包中访问）
    //endregion

    //region 私有方法（只允许自己调用）
    //endregion

    //endregion

    //region 开发新增处理代码

    //endregion

}