package com.tztzf.network.gm;

import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.keychain.tztZFKeyChain;
import com.tztzf.network.cosign.tztZFNetCosign;

import java.util.Locale;

/**
 * 协同签名账号类
 */
class tztZFCosignAccount {
    /** 账号 */
    private String account;
    /** 账号类别 */
    private String accountType;
    /** 客户号 */
    private String userCode;
    private Object context;

    private static final String tztCosignAccountCode = "tztZF_userAccount";
    private static final String tztCosignAccountKey = "key";
    private static final String tztCosignAccountUserID = "userid";
    private static final String tztCosignAccountKHBK = "khbh";

    public String getAccount() {
        return account;
    }

    public void setAccount(String account) {
        this.account = account;
    }

    public String getAccountType() {
        return accountType;
    }

    public void setAccountType(String accountType) {
        this.accountType = accountType;
    }

    /**
     * 账户信息是否有效，account和accountType同时有值 = YES
     */
    public boolean isInvalid() {
        return (account == null || account.length() < 1 || accountType == null || accountType.length() < 1);
    }

    private boolean isHKBH() {
        if (isInvalid())
            return false;
        return accountType.equalsIgnoreCase(tztCosignAccountKHBK);
    }

    private tztZFCosignAccount() {

    }

    public tztZFCosignAccount(Object context) {
        this.context = context;
    }

    private String accountKey() {
        if (isInvalid())
            return "";
        String key = String.format(Locale.getDefault(), "%s.%s", accountType, account).toLowerCase();
        return tztZFNetCosign.encryptStrSM(key);
    }

    /**
     * 读取客户号
     * @param account 账号信息对象
     * @param checkKHBH 是否校验客户编号，当为YES，并且accountType=KHBH时，直接返回account
     */
    public static String loadUserID(tztZFCosignAccount account, boolean checkKHBH) {
        if (account == null || account.isInvalid())
            return null;
        ///是客户编号，直接返回
        if (account.isHKBH() && !checkKHBH)
            return account.account;

        tztZFDataMapBase<String, Object> dictAccount = tztZFKeyChain.load(account.context, tztCosignAccountCode);
        if (dictAccount == null || dictAccount.getCount() < 1)
            return null;
        String strAccountKey = account.accountKey();
        tztZFDataMapBase<String, Object> dict = dictAccount.getDataMap(strAccountKey);
        if (dict == null || dict.getCount() < 1)
            return null;
        String strUserID = dict.getStr(tztCosignAccountUserID);
        return tztZFNetCosign.decryptStrSM(strUserID);
    }

    /**
     * 保存账号信息
     * @param account 账号信息对象
     * @param userID 客户号
     */
    public static boolean saveAccount(tztZFCosignAccount account, String userID) {
        //读取原来存储的
        if (account == null)
            return false;

        tztZFDataMapBase<String, Object> dictAccount = tztZFKeyChain.load(account.context, tztCosignAccountCode);
        if (dictAccount == null) {
            dictAccount = new tztZFDataMapBase<>();
        }
        String strAccountKey = account.accountKey();

        tztZFDataMapBase<String, Object> dict = new tztZFDataMapBase<>();
        dict.setStr(tztCosignAccountUserID, tztZFNetCosign.encryptStrSM(userID));
        dictAccount.setObj(strAccountKey, dict);

        tztZFKeyChain.save(account.context, tztCosignAccountCode, dictAccount);
        return false;
    }


}
