package com.tztzf.network.gm;

import com.tztzf.baseos.tztZFIntDef;
import com.tztzf.network.define.tztZFErrorMessageDefine;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.Locale;

import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepApplyCert;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepBindUserID;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepGMDowngrade;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepGMStatus;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepLogin;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepLoginTimeout;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepQueryCert;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepQueryUserID;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepResetPin;
import static com.tztzf.network.gm.tztZFGMStep.tztZFGMStepRevokeCert;

@tztZFIntDef({
        tztZFGMStepGMStatus,
        tztZFGMStepQueryUserID, ///查询客户号
        tztZFGMStepBindUserID, // 绑定客户号
        tztZFGMStepQueryCert, // 查询证书
        tztZFGMStepRevokeCert, //注销证书
        tztZFGMStepResetPin, //重置Pin码
        tztZFGMStepApplyCert, // 申请证书
        tztZFGMStepGMDowngrade, //国密降级
        tztZFGMStepLoginTimeout, //超时
        tztZFGMStepLogin // 登录
})

@Retention(RetentionPolicy.SOURCE)
public @interface tztZFGMStep {
    int tztZFGMStepGMStatus = 0;    // 国密状态
    int tztZFGMStepQueryUserID = 1; //查询客户号
    int tztZFGMStepBindUserID = 2;  // 绑定客户号
    int tztZFGMStepQueryCert = 3;   // 查询证书
    int tztZFGMStepRevokeCert = 4;  //注销证书
    int tztZFGMStepResetPin = 5;    //重置Pin码
    int tztZFGMStepApplyCert = 6;   // 申请证书
    int tztZFGMStepLoginSign = 7;   // 签名
    int tztZFGMStepLogin = 8;       // 登录
    int tztZFGMStepGMDowngrade = 9; //国密降级
    int tztZFGMStepLoginTimeout = 10; //超时
    int tztZFGMStepSSLStep = 11;    //双向通道步骤信息


    class tztZFGMStepTips {
        public static String tips(@tztZFGMStep int step) {
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            switch (step) {
                case tztZFGMStepGMStatus: //国密状态
                    return "交易状态";
                case tztZFGMStepQueryUserID: //查询客户号
                    return "查询客户号";
                case tztZFGMStepBindUserID:  // 绑定客户号
                    return "绑定客户号";
                case tztZFGMStepQueryCert: // 查询证书
                    return String.format(Locale.getDefault(), "查询客户%s", pinLable);
                case tztZFGMStepRevokeCert: //注销证书
                    return String.format(Locale.getDefault(), "注销客户%s", pinLable);
                case tztZFGMStepResetPin:  //重置Pin码
                    return String.format(Locale.getDefault(), "重置%s", pinLable);
                case tztZFGMStepApplyCert: // 申请证书
                    return String.format(Locale.getDefault(), "申请用户%s", pinLable);
                case tztZFGMStepLoginSign:  //国密签名
                    return "信息签名";
                case tztZFGMStepLogin: // 登录
                    return "交易登录";
                case tztZFGMStepGMDowngrade: //国密降级
                    return "交易降级";
                case tztZFGMStepLoginTimeout: //处理超时
                    return "超时";
                case tztZFGMStepSSLStep:    //
                    return "SSL步骤信息";
                default:  // 无效
                    return String.format(Locale.getDefault(), "无效步骤[%d]", step);
            }
        }
    }
}
