package com.tztzf.network.socket.Netty;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;

import java.net.InetSocketAddress;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.ExecutorService;

import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepClose;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepHand;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.handler.codec.MessageToByteEncoder;

/**
 * ****************************************************************
 * 文件名称 : tztZFGCDAsyncSocket
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : socket-nettyClient
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */

public class tztZFGCDAsyncSocket {
    private static final String logTag = "NettyC";
    //连接线程
    private ExecutorService connectthread = null;
    //接口
    private tztZFGCDAsyncSocketDelegate delegate = null;
    //连接超时
    private int connecttimeout = 6000;
    //连接的服务端ip地址
    private int linktype = 0;
    private String host = "";
    private int port = 0;
    //netty
    private static NioEventLoopGroup group = null;
    private Bootstrap bootstrap = null;
    private ChannelFuture channelFuture = null;
    //与服务端的连接通道
    private Channel channel = null;
    //SSL
    private tztZFSocketSSL tztSSL = null;
    //服务器
    private tztZFHostportIpv4Ipv6 userData = null;
    private final Object delegateQueue;

    public tztZFGCDAsyncSocket(ExecutorService connectthread, tztZFGCDAsyncSocketDelegate delegate, Object delegateQueue) {
        this.connectthread = connectthread;
        this.delegate = delegate;
        this.delegateQueue = delegateQueue;
    }
    //=====tztZFGCDAsyncSocket 操作API=======================================================

    /**
     * 连接
     *
     * @return 连接 true 发起连接，false 发起连接失败
     */
    public final boolean Connect(int linktype, String host, int port, int connectTimeout) {
        this.linktype = linktype;
        this.host = host;
        this.port = port;
        if (tztZFPublicFun.isEmpty(this.host) || this.port <= 0 || linktype < 0) {
            return false;
        }
        tztlog("Connect %s:%d", host, port);
        if (connectthread == null)
            return false;
        connecttimeout = connectTimeout;
        ///毫秒
        if (connecttimeout > 0 && connecttimeout < 15)
            connecttimeout = connecttimeout * 1000;
        if (connecttimeout < 3000) {
            connecttimeout = 3000;
        } else if (connecttimeout > 15000) {
            connecttimeout = 15000;
        }
        connectthread.execute(new Runnable() {
            @Override
            public void run() {
                socketConnect();
            }
        });
        return true;
    }

    /**
     * 连接
     */
    protected void socketConnect() {
        tztlog("Connect ing");
        final tztZFGCDAsyncSocket asyncSocket = tztZFGCDAsyncSocket.this;
        try {
            if (group == null) {
                group = new NioEventLoopGroup();
            }
            bootstrap = new Bootstrap();
            // 指定channel类型
            bootstrap.channel(NioSocketChannel.class)
                    // 指定EventLoopGroup
                    .group(group)
                    .option(ChannelOption.TCP_NODELAY, true)
                    .option(ChannelOption.SO_KEEPALIVE, true)
                    .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, connecttimeout);
            bootstrap.handler(new tztZFChannelInitializer());
            // 连接到服务端
            channelFuture = bootstrap.connect(new InetSocketAddress(host, port));
            channelFuture.addListener(new ChannelFutureListener() {
                @Override
                public void operationComplete(ChannelFuture future) throws Exception {
                    channelFuture = null;
                    if (future.isSuccess()) {
                        channel = future.channel();
                        tztlog("channel_Connect OK");
                        synchronized (delegateQueue) {
                            if (delegate != null) {
                                delegate.didConnectToHost(asyncSocket, host, port);
                            }
                        }
                    } else {
                        String errmsg = "连接失败";
                        if (future.cause() != null)
                            errmsg = future.cause().getMessage();
                        tztlog("channel_Connect fail：%s", errmsg);
                        synchronized (delegateQueue) {
                            if (delegate != null) {
                                delegate.socketDidDisconnect(asyncSocket, -101, errmsg);
                            }
                        }
                    }
                }
            });
            //获取连接通道
            //channel = channelFuture.sync().channel();
        } catch (Exception e) {
            synchronized (delegateQueue) {
                if (delegate != null) {
                    delegate.socketDidDisconnect(asyncSocket, -100, e.getMessage());
                }
            }
            tztlog("channel_Connect Exception：%s", e.getMessage());
            e.printStackTrace();
        }
    }

    //disconnect之后不能再调用write
    //disconnect之后如果没有清楚listenner，会回调clientDisconnected
    public void disconnect(String tip) {
        tztlog("disconnect %s", tip);
        if (isUseSSL()) {
            if (tztSSL != null) {
                tztSSL.sslfree();
                tztSSL = null;
            }
        }
        if (channelFuture != null) {
            channelFuture.cancel(true);
        }
        channelFuture = null;
        if (channel != null && channel.isOpen()) {
            channel.close();
        }
        channel = null;
//        if (group != null) {
//            group.shutdownGracefully();
//        }
//        group = null;
        bootstrap = null;
    }

    public boolean isUseSSL() {
        if (tztSSL == null) return false;
        return tztSSL.isUseSSL();
    }

    public boolean isActive() {
        return channel != null && channel.isActive();
    }

    public int socketRead(byte[] data, int len) {
        if (channel == null) {
            return -1;
        }
        if (delegate != null) {
            tztlog("channel_didReadData %d", len);
            delegate.didReadData(tztZFGCDAsyncSocket.this, data, len, data);
        }
        return len;
    }

    public int writeData(byte[] data, final Object tag) {
        if (channel == null) {
            return -1;
        }
        if (isUseSSL()) {
            return tztSSL.sslwrite(data);
        }
        return socketWrite(data, tag);
    }

    public int socketWrite(byte[] data, final Object tag) {
        ChannelFuture future = channel.writeAndFlush(data);
        future.addListener(new ChannelFutureListener() {
            @Override
            public void operationComplete(ChannelFuture future) throws Exception {
                if (delegate != null && future.isSuccess()) {
//                    Throwable cause = future.cause();
//                    String msg = cause != null ? cause.getMessage() : null;
                    tztlog("channel_didWriteDataWithTag");
                    delegate.didWriteDataWithTag(tztZFGCDAsyncSocket.this, tag);
                }
            }
        });
        return data.length;
    }

    public String getHost() {
        return host;
    }

    public int getPort() {
        return port;
    }

    public String GetCertUser() {
        if (isUseSSL()) return tztSSL.getCertUser();
        return "";
    }

    /**
     * 打印日志
     */
    private void tztlog(String format, Object... arg) {
        if (tztZFLog.getLevel() <= tztZFLogLevel.tztZFLogLevel_Debug.value()) {
            String cTag = String.format(Locale.getDefault(), "[%d]tztZFGCDAsyncSocket[%d],host[%s:%d]", linktype, this.hashCode(), host, port);
            String tag = "";
            if (!tztZFPublicFun.isEmpty(logTag)) {
                tag = "[" + logTag + "]";
            }
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(),
                    cTag, tag + format, arg);
        }
    }

    public void setDelegate(tztZFGCDAsyncSocketDelegate delegate) {
        this.delegate = delegate;
    }

    public tztZFHostportIpv4Ipv6 getUserData() {
        return userData;
    }

    public void setUserData(tztZFHostportIpv4Ipv6 obj) {
        userData = obj;
    }

    public void useSocketSSL(tztZFSocketSSL socketSSL) {
        tztSSL = socketSSL;
        final tztZFGCDAsyncSocket asyncSocket = tztZFGCDAsyncSocket.this;
        tztSSL.setDelegate(new tztZFSocketSSLDelegate() {
            @Override
            public int didSSLStepInfo(tztZFSocketSSL socketSSL, tztZFGCDAsyncSocket sock, int step, tztZFSocketSSLInfo info) {
                if (step == tztZFSocketSSLStepClose) {

                } else if (step == tztZFSocketSSLStepHand) {
                    synchronized (delegateQueue) {
                        if (delegate != null) {
                            delegate.didSSLConnectToHost(asyncSocket, host, port);
                        }
                    }
                }
                return 0;
            }
        });
    }

    class tztZFChannelInitializer extends ChannelInitializer<SocketChannel> {

        @Override
        protected void initChannel(SocketChannel ch) throws Exception {
            ch.pipeline().addLast(new tztZFDataEncoder());
            ch.pipeline().addLast(new tztZFDataDecoder());
            ch.pipeline().addLast(new ChannelInboundHandlerAdapter() {
                @Override
                public void channelInactive(ChannelHandlerContext ctx) throws Exception {
                    tztlog("channel_Inactive");
                    synchronized (delegateQueue) {
                        if (delegate != null) {
                            delegate.socketDidDisconnect(tztZFGCDAsyncSocket.this, -1, "连接断开");
                        }
                    }
                    super.channelInactive(ctx);
                }

                @Override
                public void channelActive(ChannelHandlerContext ctx) throws Exception {
                    //连接成功后已经回调过，这里不回调
                    tztlog("channel_Active");
                    super.channelActive(ctx);
                }

                @Override
                public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
                    byte[] data = (byte[]) msg;
                    int datalen = data.length;
                    tztlog("channel_Read:%d", datalen);
                    byte[] readData = data;
                    //是SSL连接
                    if (isUseSSL()) {
                        //将data写入bioin
                        tztSSL.sslread(readData, datalen);
                        return;
                    }
                    if (delegate != null) {
                        tztlog("channel_delegateDidReadData:%d", datalen);
                        delegate.didReadData(tztZFGCDAsyncSocket.this, readData, datalen, readData);
                    }
                }

                @Override
                public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
                    //super.exceptionCaught(ctx, cause);
                    tztlog("channel_exceptionCaught:%s", cause.getMessage());
                    ctx.close();
                }
            });
        }
    }

    //应答解码
    class tztZFDataDecoder extends ByteToMessageDecoder {
        @Override
        protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
            int len = in.readableBytes();
            tztlog("channel_readdecode:%d", len);
            ByteBuf dstAll = ctx.alloc().buffer(len);
            in.readBytes(dstAll, len);
            out.add(dstAll.array());
            dstAll.release();
        }
    }

    //发送编码
    class tztZFDataEncoder extends MessageToByteEncoder<Object> {
        @Override
        protected void encode(ChannelHandlerContext ctx, Object msg, ByteBuf out) throws Exception {
            byte[] dt = (byte[]) msg;
            tztlog("channel_writeencode:%d", dt.length);
            out.writeBytes(dt);
        }
    }
}
