package com.tztzf.network.socket.Netty;

import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeAddress;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeNetType;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeNoSSL;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeSSL;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeSSLUser;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_CheckDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ClientDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Close;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Hand;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_InvalidSSLCert;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoConnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoSSLUser;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_OK;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Recv;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SSLFail;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SendInvalid;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SendMax;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ServerDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SuspendDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_TimeOutDiscon;

import com.tztzf.baseos.tztZFIntDef;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * ****************************************************************
 * 文件名称 : tztZFNetWorkError
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : 网络错误号
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */

@tztZFIntDef({
        tztZFNetWorkError_OK,
        tztZFNetWorkError_Close,
        tztZFNetWorkError_ChangeNetType,
        tztZFNetWorkError_CheckDiscon,
        tztZFNetWorkError_TimeOutDiscon,
        tztZFNetWorkError_SuspendDiscon,
        tztZFNetWorkError_ClientDiscon,
        tztZFNetWorkError_ChangeAddress,
        tztZFNetWorkError_ChangeSSL,
        tztZFNetWorkError_ChangeNoSSL,
        tztZFNetWorkError_ChangeSSLUser,
        tztZFNetWorkError_Disconnect,
        tztZFNetWorkError_SSLFail,
        tztZFNetWorkError_Hand,
        tztZFNetWorkError_ServerDiscon,
        tztZFNetWorkError_Send,
        tztZFNetWorkError_SendInvalid,
        tztZFNetWorkError_SendMax,
        tztZFNetWorkError_Recv,
        tztZFNetWorkError_NoSSLUser,
        tztZFNetWorkError_InvalidSSLCert,
        tztZFNetWorkError_NoConnect
})

@Retention(RetentionPolicy.SOURCE)
public @interface tztZFNetWorkError {
    //region 内部私有属性及函数
    /**
     * 成功
     */
    int tztZFNetWorkError_OK = (0);
    /**
     * 关闭释放连接
     */
    int tztZFNetWorkError_Close = (-1);
    int tztZFNetWorkError_ChangeNetType = (-2); //网络切换主动断开
    int tztZFNetWorkError_CheckDiscon = (-3); //验证失效主动断开
    int tztZFNetWorkError_TimeOutDiscon = (-4); //验证超时主动断开
    int tztZFNetWorkError_SuspendDiscon = (-5); // 主动断开，通讯挂起

    int tztZFNetWorkError_ClientDiscon = (-8); //主动断开（业务层）
    int tztZFNetWorkError_ChangeAddress = (-9); //主动断开，变更连接地址

    ///SSL连接
    int tztZFNetWorkError_ChangeSSL = (-10); //变更为SSL连接
    int tztZFNetWorkError_ChangeNoSSL = (-11); //变更为非SSL连接
    int tztZFNetWorkError_ChangeSSLUser = (-12); //变更SSL连接user

    /**
     * 网络错误
     */
    int tztZFNetWorkError_Disconnect = (-30); //连接超时
    int tztZFNetWorkError_SSLFail = (-31); //SSL 握手失败
    /**
     * 协商错误
     */
    int tztZFNetWorkError_Hand = (-38); //协议握手失败
    int tztZFNetWorkError_ServerDiscon = (-39); //远程网络断开
    /**
     * 请求错误
     */
    int tztZFNetWorkError_Send = (-60); //发送超时
    int tztZFNetWorkError_SendInvalid = (-61); //无效请求
    int tztZFNetWorkError_SendMax = (-62); //请求过快
    /**
     * 应答错误
     */
    int tztZFNetWorkError_Recv = (-70); //应答超时

    /**
     * 网络设置错误，无连接
     */
    int tztZFNetWorkError_NoSSLUser = (-97); //SSL未设置user
    int tztZFNetWorkError_InvalidSSLCert = (-98); //SSL设置证书无效
    int tztZFNetWorkError_NoConnect = (-99); //未注册链接、未设置地址列表
    //endregion

    class tztZFNetWorkErrorTip {
        public static String tips(@tztZFNetWorkError int errorNo) {
            String tip = "";
            switch (errorNo) {
                case tztZFNetWorkError_OK:
                    tip = "tztZFNetWorkError_OK";
                    break;
                case tztZFNetWorkError_Close:
                    tip = "tztZFNetWorkError_Close";
                    break;
                case tztZFNetWorkError_ChangeNetType:
                    tip = "tztZFNetWorkError_ChangeNetType";
                    break;
                case tztZFNetWorkError_CheckDiscon:
                    tip = "tztZFNetWorkError_CheckDiscon";
                    break;
                case tztZFNetWorkError_TimeOutDiscon:
                    tip = "tztZFNetWorkError_TimeOutDiscon";
                    break;
                case tztZFNetWorkError_SuspendDiscon:
                    tip = "tztZFNetWorkError_SuspendDiscon";
                    break;
                case tztZFNetWorkError_ClientDiscon:
                    tip = "tztZFNetWorkError_ClientDiscon";
                    break;
                case tztZFNetWorkError_ChangeAddress:
                    tip = "tztZFNetWorkError_ChangeAddress";
                    break;
                case tztZFNetWorkError_ChangeSSL:
                    tip = "tztZFNetWorkError_ChangeSSL";
                    break;
                case tztZFNetWorkError_ChangeNoSSL:
                    tip = "tztZFNetWorkError_ChangeNoSSL";
                    break;
                case tztZFNetWorkError_ChangeSSLUser:
                    tip = "tztZFNetWorkError_ChangeSSLUser";
                    break;
                case tztZFNetWorkError_Disconnect:
                    tip = "tztZFNetWorkError_Disconnect";
                    break;
                case tztZFNetWorkError_SSLFail:
                    tip = "tztZFNetWorkError_SSLFail";
                    break;
                case tztZFNetWorkError_Hand:
                    tip = "tztZFNetWorkError_Hand";
                    break;
                case tztZFNetWorkError_ServerDiscon:
                    tip = "tztZFNetWorkError_ServerDiscon";
                    break;
                case tztZFNetWorkError_Send:
                    tip = "tztZFNetWorkError_Send";
                    break;
                case tztZFNetWorkError_SendInvalid:
                    tip = "tztZFNetWorkError_SendInvalid";
                    break;
                case tztZFNetWorkError_SendMax:
                    tip = "tztZFNetWorkError_SendMax";
                    break;
                case tztZFNetWorkError_Recv:
                    tip = "tztZFNetWorkError_Recv";
                    break;
                case tztZFNetWorkError_NoSSLUser:
                    tip = "tztZFNetWorkError_NoSSLUser";
                    break;
                case tztZFNetWorkError_InvalidSSLCert:
                    tip = "tztZFNetWorkError_InvalidSSLCert";
                    break;
                case tztZFNetWorkError_NoConnect:
                    tip = "tztZFNetWorkError_NoConnect";
                    break;
                default:
                    tip = String.format("tztZFNetWorkError_Invalid_%d", errorNo);
                    break;
            }
            return tip;
        }
    }
}
