package com.tztzf.network.socket.Netty;

import androidx.annotation.NonNull;

import com.tztzf.base.tztZFEncryptFun;
import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.net.tztZFNetAvailable;
import com.tztzf.baseos.net.tztZFNetChange;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.callback.tztZFNoUserCertCallBack;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkCallBackType;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.define.tztZFSocketFlags;
import com.tztzf.network.gm.tztZFSocketSSLStep;
import com.tztzf.network.socket.tztZFSocketLink;
import com.tztzf.network.socket.tztZFSocketSendData;
import com.tztzf.network.socket.tztZFThreadRespRunnable;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.network.tztZFQuitTradeEvent;
import com.tztzf.protocol.callback.tztZFDataProtocolInterface;
import com.tztzf.protocol.core.tztZFDataProtocolKit;
import com.tztzf.protocol.core.tztZFProtocolCert;
import com.tztzf.protocol.core.tztZFRecvPackage;
import com.tztzf.protocol.core.tztZFSSLCert;
import com.tztzf.protocol.define.tztZFHandshakeStatus;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;
import com.tztzf.tztZFNotificationObj;

import org.greenrobot.eventbus.EventBus;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Locale;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import static com.tztzf.log.tztZFLogLevel.tztZFLogLevel_Info;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrCertDeviceDel;
import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrServerCertUnfind;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFConnectError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_CertInvalid;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_ConnectError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_NoUserCert;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_ReadError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_WriteError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFOutTimeError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFReadError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFWriteError;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_DisConnect;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Non;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_TimeOut;
import static com.tztzf.network.define.tztZFRecvType.tztZFTipOfRecvType;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_ConnFail;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_ConnSucess;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Connect;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Conning;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_End;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_Init;
import static com.tztzf.network.gm.tztZFGMManage.tztZFJYLoginGMProcessStep;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepDowngrade;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepInit;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepSocket;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepSucess;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnClientCert;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnServerCert;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketStepConnect;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketStepSucess;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeAddress;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeNetType;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeNoSSL;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeSSL;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ChangeSSLUser;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_CheckDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ClientDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Close;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Hand;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_InvalidSSLCert;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoSSLUser;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Recv;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SendInvalid;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_SendMax;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_ServerDiscon;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_TimeOutDiscon;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Close;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_OKAndTaken;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Socket;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Error;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Failed;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Handing;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Init;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Rehand;
import static com.tztzf.protocol.define.tztZFHandshakeStatus.tztZFHandshake_Success;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_AutoPush;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Data;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Handshake;
import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Socket;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ActionBusying;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ModuleException;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_NotFindAction;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_ReqListFull;
import static com.tztzf.protocol.define.tztZFProtocolError.tztZFHandshakeError_WaitTimeOut;

/**
 * ****************************************************************
 * 文件名称 : tztZFSocket
 * 作   者 : Created by yangdl
 * 创建时间 : 20221206
 * 文件描述 : tztZFSocket
 * 注意事项 :
 * 版权声明 : Copyright (C) 2022-2027 杭州中焯信息技术股份有限公司
 * 修改历史 : 20221206 1.00 初始版本
 * ****************************************************************
 */

public class tztZFSocket {
    /**
     * 通讯库版本号
     */
    private static final String tztZFSocketVer = "V9.02.003.20230922001";
    private static final String tztZFKeyStatistic = "zzstatistic";
    private static final int tztMinOutTime = 3000; //最小超时时间，设置时判断用
    private static final int tztConnOutTime = 12000; //连接总超时
    private static final int tztSingleConnOutTime = 6000; //Socket单个连接超时时间
    private static final int tztSendOutTime = 5000;  //发送超时
    private static final int tztCheckOutTime = 6000; //通讯验证包超时时间
    private static final int tztCheckTime = 30000; //通讯验证包间隔时间
    private static final int tztRecvOutTime = 20000; //应答超时
    private static final int tztOnlineTime = 60000; //心跳间隔时间
    private static final int tztreqmax = 150;
    ///连接超时
    private static final String tztZFSocketMsgConnErr = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ConnectError, tztZFConnectError);
    ///连接断开
    private static final String tztZFSocketMsgConnDis = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError, tztZFReadError);
    ///请求堆积
    private static final String tztZFSocktMsgReqMax = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError);
    ///请求包非法
    private static final String tztZFSocktMsgReqErr = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError);
    ///请求超时
    private static final String tztZFSocktMsgReqTime = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError);
    ///请求取消
    private static final String tztZFSocktMsgReqCancel = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ConnectError, tztZFConnectError);
    ///应答超时
    private static final String tztZFSocktMsgRecvTime = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError, tztZFOutTimeError);
    ///未设置证书用户
    private static final String tztZFSocktMsgUserCertNo = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_NoUserCert, "未设置用户有效加密安全码");
    ///设置用户证书无效
    private static final String tztZFSocktMsgUserCertInvalid = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_CertInvalid, "请确认申请加密安全码且在有效期内");
    //参数
    //网络事件监听
    private Object context = null;
    //网络事件监听
    private tztZFNetAvailable netAvailable = null;
    //代理
    private boolean isProxy = false;
    private String proxyHost = "";
    private int proxyPort = 0;
    //日志tag
    private String logTag = "";
    //耗时统计
    private String statisticVal = "";

    //线程是否运行
    private boolean bRun = false;
    //线程等待时间(S)
    private int doWait = 1;
    //连接状态
    private @tztZFSocketFlags int socketStep = tztZFSocketFlags_Init;

    //错误号、错误信息
    private int lastErrorNo = 0;
    private String lastErrorMsg = "";
    //最近一次断开连接信息
    private String logdisconnMsg = "";

    //通讯相关参数
    private String sdkID = "0";
    //券商ID
    private int userID = 0;
    //类别
    private int sdkType = 0;
    private String linkkey = "";
    //通道
    private int linkType = tztZFLinkType.tztZFLinkType_HQ.value();
    //通讯通道标示
    private String socketKey = "";

    //包序号
    private int packageIndex = 1;

    //接收数据
    private final ByteArrayOutputStream recvStream = new ByteArrayOutputStream(255);

    //短连接
    private boolean bShortConn = false;

    //地址列表
    private final CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayAddress = new CopyOnWriteArrayList<>();
    //序号
    private int addressIndex = 0;

    //双向地址列表
    private final CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayGMAddress = new CopyOnWriteArrayList<>();
    //序号
    private int gmAddressIndex = 0;

    //网络类型
    private boolean isSSL = false;
    private int connectType = 0;
    //自定义协议类型
    private int busType = 0;
    //网络类型
    private String currentNetType = "";

    //协议数据对象
    private tztZFDataProtocolKit dataProtocolKit = null;
    private boolean bLoadDataProtocol = false;

    //最近请求时间
    private long lastSendTime = 0;
    //最近一次应答后的第一次请求时间
    private long recvAfterFirstSendDataTime = 0;
    //最近应答时间
    private long lastRecvTime = 0;
    //最近连接时间
    private long connectedTime = 0;

    //主线程和读线程
    private tztZFThreadPool sendThread = null;
    //应答处理线程
    private ThreadPoolExecutor respThread = null;
    //
    private static final Object tztSocketQueue = new Object();

    //自定义对象
    private Object userData = null;

    //行情主推
    private tztZFSocketPushHQ pushHQ = null;
    //服务主推
    private tztZFSocketPushCode pushCode = null;

    ///连接成功地址
    private String connectedHostPort = "";
    ///连接地址
    private String connectHostPort = "";
    ///单个连接超时时间(MS)
    private int singleConnOutTime = tztSingleConnOutTime;
    ///连接超时时间(MS)
    private final int connOutTime = tztConnOutTime;
    ///应答超时时间(MS)
    private int recvOutTime = tztRecvOutTime;
    ///待发送队列最大值
    private int maxReqCount = tztreqmax;
    ///请求列表
    private final CopyOnWriteArrayList<tztZFSocketSendData> reqObjList = new CopyOnWriteArrayList<>();
    ///已请求列表
    private final ConcurrentHashMap<Integer, tztZFSocketSendData> sendedReqList = new ConcurrentHashMap<>();
    ///连接回调接口
    private tztZFLinkCallBack linkBlock = null;
    ///当前使用的GcdSocket
    private tztZFGCDAsyncSocket syncSocket = null;
    ///GCDSocket回调接口
    private tztZFGCDAsyncSocketDelegate syncSocketDelegate = null;
    ///并发连接池
    private tztZFSocketPool sockPool = null;
    ///协议握手数据包
    private tztZFSocketSendData handSendData = null;
    ///连接验证包
    private tztZFSocketSendData checkSendData = null;

    //连接国密绑定账号ID
    private String gmUserID = "";
    private tztZFCosignUser gmCosignUser = null;
    private boolean gmHaveCosignCert = false;
    //国密连接超时时间
    private int gmconnOutTime = tztConnOutTime;
    //SSL回调接口
    private tztZFGMSSLStepCallBack SSLStep = null;
    //国密外部回调接口
    private tztZFGMSSLStepCallBack gmSSLStep = null;
    private tztZFNoUserCertCallBack noUserCertCallBack;

    public tztZFSocket() {
        initData();
    }

    private void initHandSendData() {
        if (handSendData != null && handSendData.getPackage() != null) return;
        tztZFNetWorkReqObj reqhand = new tztZFNetWorkReqObj();
        handSendData = new tztZFSocketSendData(reqhand, tztZFPackage_Handshake, socketKey);
        handSendData.setTimeOut(3000);
        handSendData.setResponseCallBack(new tztZFNetWorkResponseCallBack() {
            @Override
            public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                synchronized (tztSocketQueue) {
                    tztlog("[Hand] OnError:[%d]%s", c_errorNo, c_errorMsg);
                    if (isStoping()) return;
                    setSocketFlag(tztZFSocketFlags_ConnFail);
                    dataProtocolKit.onHandshakeErr(c_errorNo);
                }
            }

            @Override
            public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {

            }
        });
    }

    private void initCheckSendData() {
        if (checkSendData != null && checkSendData.getPackage() != null) return;
        tztZFNetWorkReqObj reqCheck = new tztZFNetWorkReqObj();
        checkSendData = new tztZFSocketSendData(reqCheck, tztZFPackage_Socket, socketKey);
        checkSendData.setTimeOut(tztCheckOutTime);
        checkSendData.setResponseCallBack(new tztZFNetWorkResponseCallBack() {
            @Override
            public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                synchronized (tztSocketQueue) {
                    tztlog("[Check] OnError:[%d]%s", c_errorNo, c_errorMsg);
                    if (isStoping()) return;
                    if (c_recvType != tztZFRecvType.tztZFRecv_DataOK.value()) {
                        setSocketFlag(tztZFSocketFlags_ConnFail);
                        logdisconnMsg = "检测包超时主动断开";
                        QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_CheckDiscon, tztZFSocketMsgConnDis);
                    }
                }
            }

            @Override
            public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {

            }
        });
    }

    private void initSocketPool() {
        if (sockPool != null) return;
        sockPool = new tztZFSocketPool(linkType, tztSocketQueue);
        sockPool.setTimeOut(connOutTime);
        sockPool.setFirstTimeOut(gmconnOutTime);
        sockPool.setSingleTimeOut(singleConnOutTime);
        sockPool.setFirstAddresses(ayGMAddress, gmAddressIndex);
        sockPool.setAddresses(ayAddress, addressIndex);
        sockPool.poolFirstUsed(GMisSSL());
    }

    //清除释放线程
    private void queueFree(String tip) {
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("tip=%s", tip));
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_QueueFree, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        synchronized (tztSocketQueue) {
            QueueSocketFree(tip);
        }
    }

    private void QueueSocketFree(String tip) {
        tztlog("[Run] queueFree %s", tip);
        if (dataProtocolKit != null) {
            dataProtocolKit.free();
            dataProtocolKit = null;
        }
        if (sockPool != null) {
            sockPool.poolFree();
            sockPool = null;
        }
        if (sendThread != null) {
            sendThread.setUsed(false);
            sendThread = null;
        }
        if (respThread != null) {
            respThread.shutdown();
            respThread = null;
        }
        if (pushCode != null) {
            pushCode.setDelegate(null);
            pushCode = null;
        }
    }

    private void GCDSocketStop(String tip) {
        synchronized (tztSocketQueue) {
            QueueGCDSocketStop(tip);
        }
    }

    private void QueueGCDSocketStop(String tip) {
        tztlog("[Run] %s", "socketFree");
        if (socketStep == tztZFSocketFlags_End) return;
        socketStep = tztZFSocketFlags_End;
        logdisconnMsg = "链接关闭";
        pushReqClear();
        pushRespClear(tztZFRecvType.tztZFSend_SendCancel);
        ReqlistClear(true, tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocktMsgReqCancel);
        SendedListClear(true, tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocketMsgConnDis);
        QueueGCDSocketClientClose(tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocketMsgConnDis);
        QueueSocketPoolClose(tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocketMsgConnDis);

    }

    private void initData() {
        pushCode = new tztZFSocketPushCode(new tztZFPushRespDelegate() {
            @Override
            public void resp(tztZFSocketSendData push, tztZFRecvType type, tztZFDataMapBase<String, Object> recv) {
                asyncRespQueueReqObj(push, false, type, recv);
            }
        });
        pushHQ = new tztZFSocketPushHQ(new tztZFPushRespDelegate() {
            @Override
            public void resp(tztZFSocketSendData push, tztZFRecvType type, tztZFDataMapBase<String, Object> recv) {
                asyncRespQueueReqObj(push, false, type, recv);
            }
        });

        respThread = new ThreadPoolExecutor(
                10,      //核心线程数设置成1
                10,  //线程池最大线程数为1
                10, TimeUnit.SECONDS, //超过核心线程数量的线程idle 10秒之后会退出
                new ArrayBlockingQueue<Runnable>(50), //队列长度为50
                new tztSocketDiscardOldestPolicy() //任务被拒绝以后的处理类
        );
    }

    //=====tztZFSocket 设置初始属性=======================================================

    /**
     * 设置属性
     *
     * @param tag 上下文
     * @return tztZFSocket
     */
    public tztZFSocket InitWithTag(String tag) {
        this.logTag = tag;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param context 上下文
     * @return tztZFSocket
     */
    public tztZFSocket InitWithContext(Object context) {
        if (isInit())
            this.context = context;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param threadPool 线程池
     * @return tztZFSocket
     */
    public tztZFSocket InitWithThreadPool(tztZFThreadPool threadPool) {
        if (isInit())
            this.sendThread = threadPool;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param userID userID
     * @return tztZFSocket
     */
    public tztZFSocket InitWithUserID(int userID) {
        if (isInit())
            this.userID = userID;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param sdktype sdktype
     * @return tztZFSocket
     */
    public tztZFSocket InitWithSDKType(int sdktype) {
        if (isInit()) this.sdkType = sdktype;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param linktype linktype
     * @return tztZFSocket
     */
    public tztZFSocket InitWithLinkType(int linktype) {
        if (linktype <= tztZFLinkType.tztZFLinkType_Non.value()) {
            linktype = tztZFLinkType.tztZFLinkType_HQ.value();
        }
        if (isInit()) this.linkType = linktype;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param linkKey linkKey
     * @return tztZFSocket
     */
    public tztZFSocket InitWithLinkKey(String linkKey) {
        if (isInit()) this.linkkey = linkKey;
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param connectType connectType
     * @return tztZFSocket
     */
    public tztZFSocket InitWithConnectType(int connectType) {
        if (isInit()) {
            this.connectType = connectType;
            this.isSSL = (connectType > 0);
        }
        return tztZFSocket.this;
    }

    /**
     * 设置属性
     *
     * @param busType busType
     * @return tztZFSocket
     */
    public tztZFSocket InitWithBusType(int busType) {
        if (isInit()) {
            this.busType = busType;
        }
        return tztZFSocket.this;
    }

    public boolean SocketConnect() {
        boolean result = false;
        synchronized (tztSocketQueue) {
            if (isInit())
                result = socketRuning();
        }
        return result;
    }

    /**
     * 通讯开启
     *
     * @param context     上下文
     * @param userID      userID
     * @param sdkType     sdkType
     * @param linkType    链路类型
     * @param connectType 连接类型（0普通、8单向、9双向）
     * @return true：开启 false：失败
     */
    public boolean SocketConnect(Object context, int userID, int sdkType, tztZFLinkType linkType, int connectType, int busType) {
        return InitWithContext(context)
                .InitWithUserID(userID)
                .InitWithSDKType(sdkType)
                .InitWithLinkType(linkType.value())
                .InitWithConnectType(connectType)
                .InitWithBusType(busType)
                .SocketConnect();
    }

    //连接通道userID
    public int GetUserID() {
        synchronized (tztSocketQueue) {
            return userID;
        }
    }

    //连接通道sdktype
    public int GetSDKType() {
        synchronized (tztSocketQueue) {
            return sdkType;
        }
    }

    //连接通道类别linktype
    public int GetLinkType() {
        synchronized (tztSocketQueue) {
            return linkType;
        }
    }

    //连接通道connecttype
    public int GetConnectType() {
        synchronized (tztSocketQueue) {
            return connectType;
        }
    }

    //连接通道bustype
    public int GetBusType() {
        synchronized (tztSocketQueue) {
            return busType;
        }
    }

    //连接状态
    public @tztZFSocketFlags int GetStockStep() {
        synchronized (tztSocketQueue) {
            return socketStep;
        }
    }

    public String GetLinkKey() {
        synchronized (tztSocketQueue) {
            return linkkey;
        }
    }

    public tztZFCosignUser GetCosignUser() {
        return gmCosignUser;
    }

    public boolean GetGmHaveCosignCert() {
        return gmHaveCosignCert;
    }

    public void setNoUserCertCallBack(tztZFNoUserCertCallBack noUserCertCallBack) {
        this.noUserCertCallBack = noUserCertCallBack;
    }

    public Object getUserData() {
        synchronized (tztSocketQueue) {
            return userData;
        }
    }

    public void setUserData(Object userData) {
        synchronized (tztSocketQueue) {
            this.userData = userData;
        }
    }

    //=====tztZFSocketThreadPool 设置、获取属性=======================================================

    /**
     * 设置服务器地址
     *
     * @param addList     服务器地址
     * @param selectIndex 地址索引
     */
    public void SetAddressList(String addList, int selectIndex) {
        SetAddressList(addList, null, selectIndex);
    }

    public void SetAddressList(String addList, String nameList, int selectIndex) {
        QueueSetAddressList(addList, nameList, selectIndex);
    }

    /**
     * 设置连接回调接口
     *
     * @param linkCallBack 连接回调
     */
    public final void SetLinkBlock(tztZFLinkCallBack linkCallBack) {
        synchronized (tztSocketQueue) {
            if (this.linkBlock != linkCallBack) {
                this.linkBlock = linkCallBack;
            }
        }
    }

    /**
     * 获取设置的连接回调接口
     *
     * @return tztZFLinkBlock
     */
    public tztZFLinkCallBack GetLinkBlock() {
        synchronized (tztSocketQueue) {
            return linkBlock;
        }
    }

    /**
     * 设置单个连接超时时间,单位毫秒
     *
     * @param timeOut 超时时间
     */
    public void SetSignleConnectOutTime(int timeOut) {
        synchronized (tztSocketQueue) {
            if (timeOut < tztMinOutTime)
                return;
            singleConnOutTime = timeOut;
            connectedTime = Math.max(tztMinOutTime, timeOut * 2 - 1000);
            if (sockPool != null) {
                sockPool.setTimeOut(connOutTime)
                        .setSingleTimeOut(singleConnOutTime);
            }
        }
    }

    /**
     * 设置应答超时时间,单位毫秒
     *
     * @param timeOut 超时时间
     */
    public void SetRecvOutTime(int timeOut) {
        synchronized (tztSocketQueue) {
            if (timeOut < tztMinOutTime)
                return;
            recvOutTime = timeOut;
        }
    }

    /**
     * 获取应答超时时间,单位毫秒
     *
     * @return 设置的应答超时时间
     */
    public int GetRecvOutTime() {
        synchronized (tztSocketQueue) {
            return recvOutTime;
        }
    }

    /**
     * 设置待发送队列最多有多大，超过了就清空队列
     *
     * @param maxReq 待发送队列最大个数
     */
    public void SetMaxReqCount(int maxReq) {
        synchronized (tztSocketQueue) {
            if (maxReq > 10)
                maxReqCount = maxReq;
        }
    }

    public boolean SocketConnectSuccessed() {
        synchronized (tztSocketQueue) {
            return (bRun && syncSocket != null && syncSocket.isActive() && dataProtocolKit != null && GetStockStep() == tztZFSocketFlags_ConnSucess);
        }
    }

    /**
     * 获取当前连接使用的地址信息(地址:端口)
     *
     * @return 当前使用的地址信息
     */
    public String GetSocketSuccessedHostPort() {
        synchronized (tztSocketQueue) {
            return connectedHostPort;
        }
    }

    /**
     * 获取当前设置的连接地址信息(地址:端口)
     *
     * @return 当前设置的连接地址信息
     */
    public String GetSocketHostPort() {
        synchronized (tztSocketQueue) {
            return connectHostPort;
        }
    }

    /**
     * 是否使用SSL
     *
     * @return 是否使用SSL
     */
    public boolean UsedSSL() {
        synchronized (tztSocketQueue) {
            return connectType > 0;
        }
    }

    /**
     * 获取stocket的hashcode值
     *
     * @return stocket的hashcode值
     */
    public int GetSocketID() {
        synchronized (tztSocketQueue) {
            if (syncSocket != null) {
                return syncSocket.hashCode();
            }
            return 0;
        }
    }

    /**
     * 当前网络是否在连接状态
     */
    public boolean IsConnected() {
        synchronized (tztSocketQueue) {
            if (!bRun)
                return false;
            if (socketStep != tztZFSocketFlags_Connect
                    && socketStep != tztZFSocketFlags_ConnSucess)
                return false;
            if (syncSocket == null)
                return false;
            return syncSocket.isActive();
        }
    }

    public boolean OnSendReqModel(tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        if (callBack == null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "未设置请求回调"), convertRequestParams(reqObj));
                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        synchronized (tztSocketQueue) {
            int type = tztZFPackage_Data;
            tztZFSocketSendData sendData = new tztZFSocketSendData(reqObj, type, socketKey);
            sendData.setResponseCallBack(callBack);
            if (isStoping()) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "bRun=false中断请求"), convertRequestParams(reqObj));
                    tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                asyncRespQueueReqObjError(sendData, tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocktMsgReqCancel);
                return true;
            }
            if (sendData.getReqObj() == null ||
                    sendData.getAction() <= 0) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "请求参数错误"), convertRequestParams(reqObj));
                    tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                tztlog("无效请求:%s", convertRequestParams(sendData));
                asyncRespQueueReqObjError(sendData, tztZFRecvType.tztZFSend_SendError, tztZFNetWorkError_SendInvalid, tztZFSocktMsgReqErr);
                return true;
            }

            int nAction = sendData.getAction();
            int nTimeOut = reqObj.getTimeOut();
            if (nTimeOut <= 0) {
                if (bShortConn) {
                    nTimeOut = 6000;
                } else {
                    nTimeOut = recvOutTime;
                }
                reqObj.setTimeOut(nTimeOut);
            }
            sendData.setTimeOut(nTimeOut);
            tztlog("[run] addAction:%d", nAction);
            ///国密双向
            if (GMisSSL()) {
                sendData.setStr("tztGMSSL", "1");
                if (!gmHaveCosignCert) {
                    GMLoadCosignUser(false);
                    if (gmCosignUser == null) {
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", tztZFSocktMsgUserCertNo);
                            String param2 = tztZFRecordProcessLogTools.stringFormat("localCertMessage=%s", tztZFNetCosign.getLocalCertMessage());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), param1, convertRequestParams(reqObj), param2);
                            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                        }
                        boolean responseError = true;
                        tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
                        resp.setStatusCode(tztZFNetWorkError_NoSSLUser);
                        resp.setMessage(tztZFSocktMsgUserCertNo);
                        resp.setReqObj(reqObj);
                        if (noUserCertCallBack != null) {
                            responseError = noUserCertCallBack.onHandler(resp);
                        }
                        if (responseError) {
                            GMDowngradeConnect(tztZFSocktMsgUserCertNo);
                        }
                    } else if (gmCosignUser.isNeedGMLogin() && !gmCosignUser.isValidity()) {
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param1 = tztZFRecordProcessLogTools.stringFormat("message=%s", tztZFSocktMsgUserCertInvalid);
                            String param2 = tztZFRecordProcessLogTools.stringFormat("localCertMessage=%s", tztZFNetCosign.getLocalCertMessage());
                            String param3 = tztZFRecordProcessLogTools.stringFormat("cosignUser=%s", gmCosignUser.string());
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), param1, convertRequestParams(reqObj), param2, param3);
                            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                        }
                        boolean responseError = true;
                        tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
                        resp.setStatusCode(tztZFNetWorkError_InvalidSSLCert);
                        resp.setMessage(tztZFSocktMsgUserCertInvalid);
                        resp.setReqObj(reqObj);
                        if (noUserCertCallBack != null) {
                            responseError = noUserCertCallBack.onHandler(resp);
                        }
                        if (responseError) {
                            GMDowngradeConnect(tztZFSocktMsgUserCertNo);
                        }
                    } else {
                        gmHaveCosignCert = true;
                    }
                }
            }
            ///主推处理
            if (reqObj.isRetainBlock() || tztZFSocketPushHQ.isHQPushAction(nAction) || tztZFSocketPushCode.isPushAction(nAction)) {
                type = tztZFPackage_AutoPush;
                reqObj.setIsRetainBlock(true);
                sendData.setType(type);
            }
            return ReqlistAddReqObj(sendData);
        }
    }

    public void CloseConnect(int type, String msg) {
        CloseConnect(type, msg, -1);
    }

    public void CloseConnect(int type, String msg, int connectIndex) {
        synchronized (tztSocketQueue) {
            lastErrorNo = type;
            lastErrorMsg = msg;
            logdisconnMsg = "外部主动断开," + msg;
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ClientDiscon, msg);
            ///连接中断开
            QueueSocketPoolClose(tztZFConn_DisConnect, tztZFNetWorkError_ClientDiscon, msg);
        }
    }

    /**
     * 关闭连接
     */
    public void QuitSocket() {
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "结束连接"));
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_FreeLink, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        tztlog("Quit %s", "结束连接!");
        synchronized (tztSocketQueue) {
            socketStoping();
        }
    }

    private void socketStoping() {
        if (isQuit()) return;
        doStop();
    }

    /**
     * 通讯开启
     *
     * @return true：开启 false：失败
     */
    private boolean socketRuning() {
        if (bRun) {
            return true;
        }
        if (context == null || sendThread == null) {
            return false;
        }
        if (socketStep != tztZFSocketFlags_Init) {
            return false;
        }
        if (linkType == tztZFLinkType.tztZFLinkType_Non.value()) {
            return false;
        }
        bShortConn = linkType > tztZFLinkType.tztZFLinkType_Short.value() || linkType == tztZFLinkType.tztZFLinkType_JH.value();
        socketKey = String.format(Locale.getDefault(), "%d.%d.%d", sdkType, linkType, hashCode());
        initHandSendData();
        initCheckSendData();
        //协议对象
        if (dataProtocolKit == null) {
            tztlogi("初始化连接! Ver:%s", tztZFSocketVer);
            setSocketFlag(tztZFSocketFlags_Conning);
            dataProtocolKit = new tztZFDataProtocolKit(context, linkType, new tztZFDataProtocolInterface() {
                @Override
                public void didHandshake(@tztZFHandshakeStatus int status) {
                    if (status != tztZFHandshake_Handing
                            && status != tztZFHandshake_Init) {
                        if (bRun) {
                            tztlog("[Hand] didHandshake status:%s", status + "");
                            doNotify("didHandshake");
                        }
                    }
                }
            });

            bLoadDataProtocol = dataProtocolKit.loadProtocolCert(userID, sdkType);
            if (!bLoadDataProtocol) {
                lastErrorNo = dataProtocolKit.getErrorNo();
                lastErrorMsg = dataProtocolKit.getErrorInfo();
                tztlog("[DataProtocol] loadErr:[%d]%s", lastErrorNo, lastErrorMsg);
                dataProtocolKit.free();
                dataProtocolKit = null;
                return false;
            }
            if (busType > 0) {
                dataProtocolKit.setProtocolBus(busType);
            }
            sdkID = dataProtocolKit.getHandAppID();
            if (syncSocketDelegate == null) {
                if (this.linkType < tztZFLinkType.tztZFLinkType_Short.value()) {
                    netAvailable = new tztZFNetAvailable(context, new tztZFNetChange() {
                        @Override
                        public void netChange(boolean isAvailable, String netType, boolean isPorxy, String proxyAdd, int proxyPort) {
                            if (isAvailable) {
                                doNotify("[Net] netChange isAvailable");
                            }
                            if (!isAvailable || (syncSocket != null && !currentNetType.equals(netType))) {
                                if (tztZFRecordProcessLogTools.isRecord()) {
                                    String param = tztZFRecordProcessLogTools.stringFormat("currentNetType=%s;netType=%smessage=%s", currentNetType, netType, "网络变更");
                                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), param);
                                    tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_NetChange, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                                }
                                netChanged(netType, isPorxy, proxyAdd, proxyPort);
                            }
                        }
                    });
                    //获取上一次网络类型
                    currentNetType = netAvailable.getLastNetWorkType();
                }
                syncSocketDelegate = new tztZFGCDAsyncSocketDelegate() {
                    @Override
                    public void didConnectToHost(tztZFGCDAsyncSocket sock, String host, int port) {
//                        if(sock == syncSocket){
//                            if(socketStep == tztZFSocketFlags_Conning){
//                                setStep(tztZFSocketFlags_Connect);
//                                doNotify("SSLConnectToHost "+host+":"+port);
//                            }
//                        }
                    }

                    @Override
                    public void didSSLConnectToHost(tztZFGCDAsyncSocket sock, String host, int port) {
                        synchronized (tztSocketQueue) {
                            if (sock == syncSocket) {
                                if (socketStep == tztZFSocketFlags_Conning) {
                                    setSocketFlag(tztZFSocketFlags_Connect);
                                    String strTip = String.format(Locale.getDefault(), "[AsyncSocketDelegate]%d didSSLConnectToHost %s:%d",
                                            (sock == null ? 0 : sock.hashCode()), host, port);
                                    doNotify(strTip);
                                }
                            }
                        }
                    }

                    @Override
                    public void socketDidDisconnect(tztZFGCDAsyncSocket sock, int errorno, String errormessage) {
                        synchronized (tztSocketQueue) {
                            String strErr = String.format(Locale.getDefault(), "errorno=%d;errormessage=%s", errorno, errormessage);
                            tztlog("[AsyncSocketDelegate][%d == %d] socket socketDidDisconnect:errorno=%d;errormessage=%s", (sock == null ? "" : sock.hashCode()), GetSocketID(), errorno, errormessage);
                            if (sock == syncSocket) {
                                logdisconnMsg = String.format("远程连接断开,%s", strErr);
                                QueueSocketPoolDisconnect(logdisconnMsg);
                                QueueGCDSocketServer(sock, tztZFConn_DisConnect, tztZFNetWorkError_ServerDiscon, tztZFSocketMsgConnDis);
                                syncSocket = null;
                            } else {
                                sockPool.poolSocketClose(sock, String.format("tztZFSocket.DisConnect Err:%s", strErr));
                            }
                            doNotify(String.format(Locale.getDefault(), "[AsyncSocketDelegate][%d GCDSocketDisconnect", (sock == null ? 0 : sock.hashCode())));
                        }
                    }

                    @Override
                    public void didReadData(tztZFGCDAsyncSocket sock, byte[] data, int len, Object tag) {
                        tztlog("[AsyncSocketDelegate]%d == %d didReadData %d", (sock == null ? "" : sock.hashCode()), GetSocketID(), len);
                        if (sock == syncSocket) {
                            lastRecvTime = System.currentTimeMillis();
                            GCDSocketRecvData(sock, data, len);
                        } else if (sock != null) {
                            sock.setDelegate(null);
                            String strTip = String.format(Locale.getDefault(),
                                    "[%d!=%d]other socket didReadData", sock.hashCode(), GetSocketID());
                            sock.disconnect(strTip);
                        }
                    }

                    @Override
                    public void didWriteDataWithTag(tztZFGCDAsyncSocket sock, Object tag) {
                        tztlog("[AsyncSocketDelegate]%d == %d didWriteDataWithTag", (sock == null ? "" : sock.hashCode()), GetSocketID());
                        if (sock == syncSocket) {

                        } else if (sock != null) {
                            sock.setDelegate(null);
                            String strTip = String.format(Locale.getDefault(),
                                    "[%d!=%d]other socket didWriteDataWithTag", sock.hashCode(), GetSocketID());
                            sock.disconnect(strTip);
                        }
                    }
                };
                if (linkType == tztZFLinkType.tztZFLinkType_JY.value() || linkType == tztZFLinkType.tztZFLinkType_Cosign.value()) {
                    this.SSLStep = new tztZFGMSSLStepCallBack() {
                        @Override
                        public void onCallBack(final @tztZFSocketSSLStep int step, final int code, final String certUser) {
                            asyncRespQueue(new Runnable() {
                                @Override
                                public void run() {
                                    if (gmSSLStep != null) {
                                        gmSSLStep.onCallBack(step, code, certUser);
                                    } else {
                                        //通知上层
                                        tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                        notificationObj.setName(tztZFJYLoginGMProcessStep);
                                        tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                        dataMapBase.setInt("step", step);
                                        dataMapBase.setStr("userID", certUser == null ? "" : certUser);
                                        notificationObj.setData(dataMapBase);
                                        EventBus.getDefault().post(notificationObj);
                                    }
                                }
                            });
                        }
                    };
                }
                initSocketPool();
                bRun = true;
                try {
                    sendThread.useThreadPool().execute(new Runnable() {
                        @Override
                        public void run() {
                            while (!canStop()) {
                                syncRun();
                            }
                            GCDSocketStop("Stop");
                        }
                    });
                } catch (Exception e) {
                    GCDSocketStop("Exception:" + e.getMessage());
                    return false;
                }
                return true;
            }
        }
        return false;
    }

    //#pragma mark -tztSocket status
    ///初始状态
    private boolean isInit() {
        synchronized (tztSocketQueue) {
            return socketStep == tztZFSocketFlags_Init;
        }
    }

    ///停止中...待应答包处理完毕
    private boolean isStoping() {
        synchronized (tztSocketQueue) {
            return !bRun;
        }
    }

    ///已结束
    private boolean isQuit() {
        synchronized (tztSocketQueue) {
            return (socketStep == tztZFSocketFlags_End || dataProtocolKit == null);
        }
    }

    ///运行中
    private boolean isRuning() {
        synchronized (tztSocketQueue) {
            return !isInit() &&
                    !isStoping() &&
                    !isQuit();
        }
    }

    ///停止
    private boolean canStop() {
        ///已经结束
        if (isQuit()) return true;
        ///停止状态且应答包处理完毕
        return isStoping() && !haveSended();
    }

    private boolean setSocketFlag(int step) {
        synchronized (tztSocketQueue) {
            if (socketStep == tztZFSocketFlags_End) {
                return (step == tztZFSocketFlags_End);
            } else {
                socketStep = step;
                return true;
            }
        }
    }

    //当前网络是否在连接状态
    private boolean GCDSocketIsConnected() {
        synchronized (tztSocketQueue) {
            if (isStoping()) {
                return false;
            }
            if (socketStep != tztZFSocketFlags_Connect && socketStep != tztZFSocketFlags_ConnSucess) {
                return false;
            }
            return (syncSocket != null && syncSocket.isActive());
        }
    }

    private boolean haveReq() {
        synchronized (tztSocketQueue) {
            if (isStoping()) {
                return false;
            }
            return reqObjList != null && reqObjList.size() > 0;
        }
    }

    private boolean havePush() {
        synchronized (tztSocketQueue) {
            if (isStoping()) {
                return false;
            }
            if (!checkNetWorkAvailableProxy()) {
                return false;
            }
            return (pushHQ != null && pushHQ.have()) || (pushCode != null && pushCode.have());
        }
    }

    private boolean haveSended() {
        synchronized (tztSocketQueue) {
            if (isQuit()) {
                return false;
            }
            return (sendedReqList != null && sendedReqList.size() > 0);
        }
    }

    private String usedHostPort() {
        tztZFHostport hostport = tztZFSocketPool.usedHostPort(syncSocket);
        if (hostport == null)
            return "";
        return hostport.address();
    }

    private String usedHostPortlog() {
        tztZFHostport hostport = tztZFSocketPool.usedHostPort(syncSocket);
        if (hostport == null)
            return "";
        return hostport.toString();
    }

    private void QueueSetAddressList(String addressList, String nameList, int selectIndex) {
        tztlog("[run] addressList[%d]:%s", selectIndex, addressList);
        boolean bDidDisconn = false;
        ArrayList<tztZFHostportIpv4Ipv6> ayArray = tztZFHostportIpv4Ipv6.withAddresslistNamelist(addressList, nameList, connectType);
        ayAddress.clear();
        if (ayArray != null && ayArray.size() > 0) {
            ayAddress.addAll(ayArray);
        } else {
            tztlog("设置无效的服务器地址列表,list:%s", addressList);
        }
        addressIndex = selectIndex;
        tztZFHostportIpv4Ipv6 ipv4Ipv6 = null;
        if (sockPool != null) {
            sockPool.setAddresses(ayAddress, addressIndex);
            bDidDisconn = SocketPoolChangedAndDidDisconnect(addressIndex);
            ipv4Ipv6 = sockPool.poolAddress(false);
        }
        if (ipv4Ipv6 == null && ayAddress.size() > 0)
            ipv4Ipv6 = ayAddress.get(0);
        connectHostPort = (ipv4Ipv6 != null ? ipv4Ipv6.address() : connectedHostPort);
        if (!bDidDisconn) {
            doNotify("addressList");
        }
    }

    ///#pragma mark -tztSocket socketPool
    ///切换服务器断线重连
    private boolean SocketPoolChangedAndDidDisconnect(int addressIndex) {
        if (addressIndex < 0 || sockPool == null || (ayAddress.size() + ayGMAddress.size() < 1) || syncSocket == null || !syncSocket.isActive())
            return false;
        tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = sockPool.poolAddress(false);
        if (hostportIpv4Ipv6 == null || hostportIpv4Ipv6.isInvalid())
            return false;
        boolean isDidDisconn = false;
        tztZFHostportIpv4Ipv6 useHostport = syncsocketAddress();
        ///连接地址变更，断开重连
        if (useHostport != null && !hostportIpv4Ipv6.isHave(useHostport.usedHostport())) {
            isDidDisconn = true;
            logdisconnMsg = "地址切换主动断开,host:" + useHostport + " setHost:" + hostportIpv4Ipv6.address();
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ChangeAddress, tztZFSocketMsgConnDis);
        }
        return isDidDisconn;
    }

    private tztZFHostportIpv4Ipv6 syncsocketAddress() {
        if (syncSocket == null || !syncSocket.isActive())
            return null;
        return syncSocket.getUserData();
    }

    ///连接成功，设置优先地址
    private void SocketPoolSelectedAddress() {
        if (isStoping()) return;
        tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = syncsocketAddress();
        if (hostportIpv4Ipv6 == null)
            return;
        ///连接成功地址
        connectedHostPort = hostportIpv4Ipv6.address();
        connectHostPort = hostportIpv4Ipv6.address();
        ///设置为优选地址
        if (sockPool != null) {
            sockPool.poolSocketUsed(syncSocket, hostportIpv4Ipv6);
        }
    }

    private void SocketPoolMoveLastAddress() {
        if (isStoping()) return;
        if (sockPool == null || syncSocket == null) return;
        sockPool.poolMoveLastAddress(syncSocket.getUserData());
    }

    private void SocketPoolNexAddress(String tip) {
        if (isStoping()) return;
        if (sockPool == null) return;
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", tip));
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_ChangeAddress, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        sockPool.poolNextAddress(tip);
    }

    private void GCDSocketConnect() {
        synchronized (tztSocketQueue) {
            QueueGCDSocketConnect();
        }
    }

    private void QueueGCDSocketConnect() {
        if (isStoping()) return;
        if (sockPool == null) return;
        if (GCDSocketIsConnected()) return;
        if (connectType == 9) {
            if (!tztZFNetCosign.getCosignStatus()) {
                ///准备连接状态、只需发出降级通知即可
                GMDowngradeUser();
            } else {
                GMLoadCosignUser(true);
                sockPool.poolFirstUsed(GMisSSL());
            }
        }
        if (!sockPool.isVaild()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "连接地址未设置"));
                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            tztlogi("[Pool] connect:连接地址未设置");
            return;
        }
        lastErrorNo = 0;
        lastErrorMsg = "";
        setSocketFlag(tztZFSocketFlags_Conning);
        tztlog("[Pool] sockPool poolConnect");
        sockPool.poolConnect(new tztZFSocketPoolDelegate() {
            @Override
            public int didConnecting(tztZFGCDAsyncSocket socket, tztZFHostportIpv4Ipv6 hostport) {
                synchronized (tztSocketQueue) {
                    tztlogi("[Pool] [AsyncSocket]%d SocketdidConnecting %s",
                            socket.hashCode(), (hostport != null ? hostport.toString() : ""));
                    if (hostport != null) {
                        connectHostPort = hostport.address();
                    }
                }
                return 0;
            }

            @Override
            public int didConnected(tztZFGCDAsyncSocket socket, tztZFHostportIpv4Ipv6 hostport, boolean isSSL) {
                int nRet = 0;
                synchronized (tztSocketQueue) {
                    if (socket != null) {
                        if (!isSSL) {
                            nRet = QueueGCDSocketConnected(socket, hostport);
                        } else {
                            nRet = QueueGCDSocketSSLConnected(socket, hostport);
                        }
                        if (tztZFRecordProcessLogTools.isRecord()) {
                            String param = tztZFRecordProcessLogTools.stringFormat("message=%s;SocketdidConnected %s %s ret:%d", (isSSL ? "SSL" : ""), "连接成功", (hostport != null ? hostport.toString() : ""), nRet);
                            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), param);
                            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SocketConnect, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                        }
                        tztlogi("[Pool] [AsyncSocket]%d SocketdidConnected %s %s ret:%d",
                                socket.hashCode(), (isSSL ? "SSL" : ""), (hostport != null ? hostport.toString() : ""), nRet);
                    } else {
                        nRet = tztZFSocketSSLFlag_Close;
                    }
                    return nRet;
                }
            }

            @Override
            public int didConnectTimeOut(tztZFGCDAsyncSocket socket) {
                synchronized (tztSocketQueue) {
                    tztlogi("[Pool] [AsyncSocket]%d SocketdidConnectTimeOut", (socket != null ? socket.hashCode() : 0));
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "连接超时"));
                        tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SocketConnect, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                    }
                    QueueGCDSocketTimeout();
                }
                return 0;
            }

            @Override
            public int didFirstConnectTimeOut(tztZFGCDAsyncSocket socket) {
                synchronized (tztSocketQueue) {
                    tztlogi("[Pool] [AsyncSocket]%d SocketdidFirstConnectTimeOut", (socket != null ? socket.hashCode() : 0));
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "连接国密超时"));
                        tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SocketConnect, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                    }
                    GMDowngradeConnect("双向连接超时降级");
                }
                return 0;
            }

            @Override
            public int disConnnect(tztZFGCDAsyncSocket socket) {
                synchronized (tztSocketQueue) {
                    tztlogi("[Pool] [AsyncSocket]%d SocketdisConnnect", (socket != null ? socket.hashCode() : 0));
                    if (tztZFRecordProcessLogTools.isRecord()) {
                        ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "所有连接超时，断开连接"));
                        tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_Disconnected, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                    }
                    QueueGCDSocketConnMaxFail();
                }
                return 0;
            }
        });
    }

    //#pragma mark -tztSocket gcdSocket connect
    //启用
    private void QueueGCDSocketUsedSocket(tztZFGCDAsyncSocket sock, tztZFHostportIpv4Ipv6 ipv4Ipv6) {
        ///通讯数据协议
        if (GMisSSL()) {
            dataProtocolLoadHandShakeCert();
        } else {
            tztlog("[dataProtocol] switchToLoadCert");
            dataProtocolKit.switchToLoadCert();
        }
        dataProtocolKit.onProtocolInit();
        ///连接地址
        connectHostPort = ipv4Ipv6 != null ? ipv4Ipv6.address() : "";
        recvStream.reset();
        lastSendTime = System.currentTimeMillis();
        lastRecvTime = System.currentTimeMillis();
        //重置协议包序号
        packageIndex = 1;
        ///接手
        tztlog("[Pool] [AsyncSocket]%d gcdSocketConnected", sock.hashCode());
        syncSocket = sock;
        syncSocket.setDelegate(syncSocketDelegate);
        setSocketFlag(tztZFSocketFlags_Connect);
        if (bShortConn) {
            tztlog("[Hand] 短连接[%d 协商成功", linkType);
            connectedTime = System.currentTimeMillis();
            connectedHostPort = ipv4Ipv6 != null ? ipv4Ipv6.usedAddress() : "";
            setSocketFlag(tztZFSocketFlags_ConnSucess);
            SocketPoolSelectedAddress();
            linkblockResp(tztZFRecvType.tztZFConn_Connected, connectedHostPort, "短连接连接成功");
        }
        doNotify("[AsyncSocket]" + GetSocketID() + " didconn");
    }

    private int QueueGCDSocketConnected(tztZFGCDAsyncSocket sock, tztZFHostportIpv4Ipv6 ipv4Ipv6) {
        if (isStoping()) return tztZFSocketSSLFlag_Close;
        if (syncSocket != null && syncSocket.isActive() && syncSocket != sock)
            return tztZFSocketSSLFlag_Disconnect;
        int hostConnecttype = ipv4Ipv6 != null ? ipv4Ipv6.getConnecttype() : 0;
        if (GMisSSL()) {
            ///双向连接
            if (hostConnecttype == 9) {
                GMLoadCosignUser(true);
                if (gmCosignUser == null) {
                    if (this.SSLStep != null) {
                        this.SSLStep.onCallBack(tztZFSocketSSLStepUnClientCert, 0, GMGetUser());
                    }
                    ReqlistClear(true, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_NoSSLUser, tztZFSocktMsgUserCertNo);
                    tztlog("[SSL] GMDowngradeUser:%s UnClientCert", gmUserID);
                    GMDowngradeUser();
                    return tztZFSocketSSLFlag_Close;
                }
                isSSL = true;
            } else {
                ///待连接状态、只需发出降级通知即可
                tztlog("[SSL] GMDowngradeUser:%s", gmUserID);
                GMDowngradeUser();
            }
        } else {
            ///双向连接无效
            if (hostConnecttype == 9) return tztZFSocketSSLFlag_Disconnect;
            tztlog("[dataProtocol] switchToLoadCert");
            isSSL = connectType > 0 && connectType != 9;
        }
        int nSSLFlag = 0;
        if (isSSL) {
            nSSLFlag = GCDSocketSSLConnect(sock);
            ///SSL连接失败
            if (nSSLFlag < 0) {
                return tztZFSocketSSLFlag_Disconnect;
            }
            if (this.SSLStep != null) {
                this.SSLStep.onCallBack(tztZFSocketStepConnect, 0, GMGetUser());
            }
        }
        ///SSL连接
        if (nSSLFlag > 0) {
            return tztZFSocketSSLFlag_Socket;
        }
        QueueGCDSocketUsedSocket(sock, ipv4Ipv6);
        doNotify(String.format(Locale.getDefault(), "[AsyncSocket]%d GCDSocketConnect", GetSocketID()));
        return tztZFSocketSSLFlag_OKAndTaken;
    }

    private int QueueGCDSocketSSLConnected(tztZFGCDAsyncSocket sock, tztZFHostportIpv4Ipv6 ipv4Ipv6) {
        if (isStoping() || connectType <= 0) return tztZFSocketSSLFlag_Close;
        ///已有连接
        if (syncSocket != null && syncSocket.isActive() && syncSocket != sock) {
            return tztZFSocketSSLFlag_Disconnect;
        }
        ///使用socket
        QueueGCDSocketUsedSocket(sock, ipv4Ipv6);
        doNotify(String.format(Locale.getDefault(), "[AsyncSocket]%d GCDSocketSSLConnect", GetSocketID()));
        return tztZFSocketSSLFlag_OKAndTaken;
    }

    private boolean GCDSocketClientClose(tztZFRecvType recvType, @tztZFNetWorkError int error, String msg) {
        synchronized (tztSocketQueue) {
            return QueueGCDSocketClientClose(recvType, error, msg);
        }
    }

    private void QueueGCDSocketServer(tztZFGCDAsyncSocket sock, tztZFRecvType recvType, @tztZFNetWorkError int error, String msg) {
        boolean isConnMaxFail = false;
        if (sockPool != null) {
            isConnMaxFail = sockPool.isConnMaxFail(msg);
        }
        tztlog("[AsyncSocket]%d [%s]%s", sock.hashCode(), tztZFNetWorkError.tztZFNetWorkErrorTip.tips(error), msg);
        tztZFHostportIpv4Ipv6 ipv4ipv6 = sock.getUserData();
        String connAddress = ipv4ipv6.address();
        //返回网络断开消息
        linkblockResp(recvType, connAddress, msg);
        ///清除已发送请求
        SendedListClear(false, recvType, error, msg);
        ///根据状态清除主推列表
        if (bShortConn || (netAvailable != null && !netAvailable.checkNetworkAvailable()) || isStoping()) {
            pushRespClear(tztZFConn_Non);
            ///短连接改为连接超时返回请求应答
            if (bShortConn || isStoping()) {
                isConnMaxFail = false;
                if (sockPool != null) {
                    sockPool.poolResetConn(0);
                }
                ReqlistClear(true, tztZFConn_Non, tztZFNetWorkError_Close, tztZFSocktMsgReqCancel);
            }
        }
        ///消息
        if (!isQuit()) {
            if (isConnMaxFail) {
                ReqlistFirstReqResp(tztZFSend_TimeOut, tztZFNetWorkError_Disconnect, tztZFSocketMsgConnErr);
            }
        }
    }

    private void QueueSocketPoolClose(tztZFRecvType recvType, @tztZFNetWorkError int error, String msg) {
        ///连接中断开
        if (sockPool != null) {
            String strTip = String.format("[%s][%s]%s", tztZFTipOfRecvType(recvType), tztZFNetWorkError.tztZFNetWorkErrorTip.tips(error), msg);
            sockPool.poolClose(strTip);
        }
    }

    private void QueueSocketPoolDisconnect(String msg) {
        if (sockPool != null) {
            sockPool.poolSocketClose(syncSocket, msg);
        }
        setSocketFlag(tztZFSocketFlags_ConnFail);
    }

    ///客户端主动断开
    private boolean QueueGCDSocketClientClose(tztZFRecvType recvType, @tztZFNetWorkError int error, String msg) {
        boolean bResp = false;
        ///已连接断开
        if (syncSocket != null) {
            tztZFHostportIpv4Ipv6 ipv4ipv6 = syncSocket.getUserData();
            String connAddress = ipv4ipv6 != null ? ipv4ipv6.address() : "";
            syncSocket.setDelegate(null);
            syncSocket.disconnect(msg);
            QueueSocketPoolDisconnect(msg);
            syncSocket = null;
            logdisconnMsg = String.format(Locale.getDefault(), "[%d]%s", error, msg);
            bResp = true;
            //返回网络断开消息
            linkblockResp(recvType, connAddress, msg);
            ///清除已发送请求
            SendedListClear(false, recvType, error, msg);
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", msg));
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_Disconnected, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        doNotify(String.format(Locale.getDefault(), "[run] GCDSocketClose [%d]%s", error, msg));
        return bResp;
    }

    ///连接超时处理
    private void QueueGCDSocketTimeout() {
        if (isStoping()) return;
        if (!QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_Disconnect, tztZFSocketMsgConnErr)) {
            //返回网络断开消息
            linkblockResp(tztZFConn_DisConnect, "", tztZFSocketMsgConnErr);
        }
        ReqlistFirstReqResp(tztZFSend_TimeOut, tztZFNetWorkError_Disconnect, tztZFSocketMsgConnErr);
    }

    ///最大连接失败
    private void QueueGCDSocketConnMaxFail() {
        QueueGCDSocketTimeout();
    }

    private int GCDSocketSendReqObj(tztZFSocketSendData reqObj) {
        if (reqObj == null)
            return -1;
        if (!tztZFPublicFun.isEmpty(statisticVal)) {
            reqObj.setStr(tztZFKeyStatistic, statisticVal);
            statisticVal = "";
        }
        if (linkType == tztZFLinkType_JY.value() && (!tztZFNetCosign.getCosignStatus() || connectType != 9)) {
            //国密状态不正常，则移除tztZFGMSSL参数
            reqObj.removeByKey("tztGMSSL");
        }
        byte[] writeData = transferSendData(reqObj);
        if (writeData == null || writeData.length < 1)
            return -2;
        int reqTag = reqObj.getSendTag();
        if (reqTag < 0)
            return -3;
        if (syncSocket == null) {
            tztlog("gcdSocketSendReqObj err socket is null");
            return -4;
        }
        tztlogi("[AsyncSocket]%d 发送请求 [packageindex:%d] [Package.Action:%d] [SendTag:%d] [Host:%s] Json:%s",
                GetSocketID(), packageIndex, reqObj.getAction(), reqObj.getSendTag(), usedHostPort(), reqObj.getReqMapdata().toString());
        reqObj.setSendTime();
        if (syncSocket.writeData(writeData, reqObj) < 0)
            return -5;
        packageIndex++;
        //应答后的第一次请求时间
        if (recvAfterFirstSendDataTime == 0 || lastRecvTime == 0 || recvAfterFirstSendDataTime - lastRecvTime < 0) {
            recvAfterFirstSendDataTime = System.currentTimeMillis();
        }
        return SendedListAddReqObj(reqObj, reqTag);
    }

    private boolean GCDSocketRecvData(tztZFGCDAsyncSocket sock, byte[] data, int len) {
        tztlogi("[AsyncSocket]%d gcdSocketRecvData", sock.hashCode());
        if (data == null || len < 1 || data.length < len || isQuit())
            return true;
        if (sock != syncSocket) {
            sock.setDelegate(null);
            String strTip = String.format(Locale.getDefault(),
                    "[%d!=%d]other socket gcdSocketRecvData", sock.hashCode(), GetSocketID());
            sock.disconnect(strTip);
            return false;
        }
        if (syncSocket == null)
            return false;
        boolean bRet = true;
        recvStream.write(data, 0, len);
        recvStream();
        return bRet;
    }

    private void recvStream() {
        if (dataProtocolKit == null) {
            return;
        }
        boolean bNotify = false;
        while (recvStream.size() >= 6) {
            int headSize = 6;
            byte[] headData = new byte[headSize];
            byte[] recv_data = recvStream.toByteArray();
            System.arraycopy(recv_data, 0, headData, 0, headSize);
            tztlog("[AsyncSocket]%d checkPackageHeader", GetSocketID());
            int datalen = dataProtocolKit.checkPackageHeader(headData);
            if (datalen >= 0) {
                if (datalen + headSize <= recv_data.length) {
                    if (datalen > 0) {
                        byte[] abyte0 = new byte[datalen];// 具体额数据内容
                        System.arraycopy(recv_data, headSize, abyte0, 0, datalen);
                        dealReceiveData(abyte0);
                    }
                    recvStream.reset();
                    if (recv_data.length > headSize + datalen) {
                        recvStream.write(recv_data, headSize + datalen, recv_data.length - headSize - datalen);
                    }
                    bNotify = true;
                } else {
                    //没接收完，需要继续读取
                    tztlog("[AsyncSocket]%d datalen:%d, recvLen:%d", GetSocketID(), datalen, recv_data.length);
                    break;
                }
            } else {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "数据包头校验错误"));
                    tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_ReceiveData, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
                }
                tztlog("[AsyncSocket]%d 数据包头校验错误！", GetSocketID());
                break;
            }
        }
        if (bNotify) {
            doNotify("[AsyncSocket]" + GetSocketID() + " ReceiveData");//收到一个完整包，唤醒发送线程
        }
    }

    //给第三方做计算耗时用，不要删除
    public final void doWriteInfo(int len) {

    }

    //给第三方做计算耗时用，不要删除
    public final void doReadInfo(int len) {

    }

    //根据握手状态进行处理
    private boolean GCDSocketHandshake() {
        if (socketStep != tztZFSocketFlags_Connect || syncSocket == null) {
            return false;
        }
        lastErrorNo = 0;
        lastErrorMsg = "";
        int handStatus = dataProtocolKit.getCurrentStatus();
        String strUsedHost = usedHostPortlog();
        if (handStatus == tztZFHandshake_Init) {
            tztlog("[Hand] 发起协商...[Status:%d][connecttype:%d][host:%s]", handStatus, connectType, strUsedHost);
            int send = GCDSocketSendReqObj(handSendData);
            if (send != 1) {
                lastErrorNo = tztZFNetWorkError_Hand;
                lastErrorMsg = "host:" + strUsedHost + ",协商请求错误(0)！";
                logdisconnMsg = "协商请求包错误";
                QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_Hand, tztZFSocketMsgConnErr);
            }
            return true;
        }

        if (handStatus == tztZFHandshake_Handing) {
            tztlog("[Hand] 协商中...[Status:%d][connecttype:%d][host:%s]", handStatus, connectType, strUsedHost);
            return false;
        }

        if (handStatus == tztZFHandshake_Success) {
            setSocketFlag(tztZFSocketFlags_ConnSucess);
            SocketPoolSelectedAddress();
            tztlog("[Hand] 协商成功...[Status:%d][connecttype:%d][host:%s]", handStatus, connectType, strUsedHost);
            if (this.SSLStep != null) {
                if (syncSocket != null && syncSocket.isActive()) {
                    this.SSLStep.onCallBack(tztZFSocketSSLStepSucess, 0, syncSocket.GetCertUser());
                } else {
                    this.SSLStep.onCallBack(tztZFSocketStepSucess, 0, "");
                }
            }
            connectedTime = System.currentTimeMillis();
            linkblockResp(tztZFRecvType.tztZFConn_Connected, connectedHostPort, "连接协议握手成功");
            //连接成功，重新发起主推
            pushReSend();
            return true;
        }

        //重新握手状态
        if (handStatus == tztZFHandshake_Rehand || handStatus == tztZFHandshake_Error || handStatus == tztZFHandshake_Failed) {
            lastErrorNo = dataProtocolKit.getErrorNo();
            lastErrorMsg = "host:" + strUsedHost + "," + dataProtocolKit.getErrorCode();
            tztlog("[Hand] 协商失败...[Status:%d][connecttype:%d]%s", handStatus, connectType, lastErrorMsg);
            if (handStatus != tztZFHandshake_Rehand) {
                SocketPoolMoveLastAddress();
            }
            logdisconnMsg = "协商失败" + handStatus + ":" + lastErrorMsg;
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_Hand, tztZFSocketMsgConnErr);
            return true;
        }
        return false;
    }

    private void GCDSocketSendHeartBeat() {
        if (bShortConn || isStoping() || isQuit()) return;
        if (lastSendTime <= 0) return;
        if (!SocketConnectSuccessed()) return;
        //距离上一次发送成功已有30S没收到应答，发送验证包验证连接状态，防止假连接
        if (lastRecvTime > 0 && recvAfterFirstSendDataTime > 0
                && recvAfterFirstSendDataTime - lastRecvTime > 0
                && System.currentTimeMillis() - recvAfterFirstSendDataTime >= tztCheckTime) {
            checkSendData.setAddTime();
            recvAfterFirstSendDataTime = System.currentTimeMillis();
            GCDSocketSendReqObj(checkSendData);
            return;
        }
        if (System.currentTimeMillis() - lastSendTime < tztOnlineTime)
            return;
        byte[] onlineData = dataProtocolKit.transferOnlineData();
        if (onlineData == null || onlineData.length < 1)
            return;
        tztlogi("[AsyncSocket]%d 发送请求 [packageindex:%d] [Package.HeartBeat] [Host:%s]",
                GetSocketID(), packageIndex, usedHostPort());
        if (syncSocket.writeData(onlineData, null) > 0) {
            packageIndex++;
            lastSendTime = System.currentTimeMillis();
        }
    }

    private boolean ReqlistAddArrayReq(CopyOnWriteArrayList<tztZFSocketSendData> ayReq) {
        if (isStoping()) {
            asyncRespQueueArray(ayReq, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_Close, tztZFSocketMsgConnDis);
            doNotify("reqlistAddArrayReq isStoping");
            return true;
        }
        if (reqObjList.size() >= maxReqCount) {
            logdisconnMsg = "请求队列过长:" + reqObjList.size() + ">=" + maxReqCount;
            ReqlistClear(false, tztZFRecvType.tztZFSend_MaxError, tztZFNetWorkError_SendMax, tztZFSocktMsgReqMax);
            SocketPoolNexAddress("ReqMax Err");
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_SendMax, tztZFSocktMsgReqMax);
        }
        boolean bRet = false;
        for (tztZFSocketSendData req : ayReq) {
            if (!reqObjList.contains(req)) {
                reqObjList.add(req);
                bRet = true;
            }
        }
        doNotify("reqlistAddArrayReq");
        return bRet;
    }

    private boolean ReqlistAddReqObj(tztZFSocketSendData send) {
        if (isStoping()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "bRun=false中断请求"), convertRequestParams(send));
                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            asyncRespQueueReqObjError(send, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_Close, tztZFSocketMsgConnDis);
            doNotify("reqlistAddReqObj isStoping");
            return true;
        }
        if (reqObjList.size() >= maxReqCount) {
            logdisconnMsg = "请求队列过长:" + reqObjList.size() + ">=" + maxReqCount;
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", logdisconnMsg), convertRequestParams(send));
                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            ReqlistClear(false, tztZFRecvType.tztZFSend_MaxError, tztZFNetWorkError_SendMax, tztZFSocktMsgReqMax);
            SocketPoolNexAddress("ReqMax Err");
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_SendMax, tztZFSocktMsgReqMax);
        }
        //重置
        send.setAddTime();
        reqObjList.add(send);
        if (send.getReqObj().isRetainBlock()) {
            pushAddReqObj(send);
        }
        doNotify("reqlistAddReqObj");
        return true;
    }

    ///获取请求对象
    private tztZFSocketSendData ReqlistGetFirstObj() {
        synchronized (tztSocketQueue) {
            if (reqObjList.size() < 1) {
                return null;
            }
            tztZFSocketSendData req = reqObjList.get(0);
            reqObjList.remove(req);
            return req;
        }
    }

    ///清空请求列表
    private void ReqlistClear(boolean isClear, tztZFRecvType recvType, int errorNo, String errorMsg) {
        if (reqObjList.size() < 1)
            return;
        CopyOnWriteArrayList<tztZFSocketSendData> ayResp = new CopyOnWriteArrayList<>();
        if (isClear) {
            ayResp.addAll(reqObjList);
            reqObjList.clear();
        } else {
            for (tztZFSocketSendData send : reqObjList) {
                if (send == null)
                    continue;
                if (send.isReSend() && isRuning()) {
                    send.setIsReSend(false);
                    send.setAddTime();
                } else {
                    ayResp.add(send);
                }
            }
            if (!ayResp.isEmpty()) {
                reqObjList.removeAll(ayResp);
            }
        }
        //异步应答
        asyncRespQueueArray(ayResp, recvType, errorNo, errorMsg);
    }

    ///请求列表首包
    private void ReqlistFirstReqResp(tztZFRecvType recvType, int errorNO, String errorMsg) {
        if (reqObjList.size() < 1)
            return;
        if (sockPool != null) {
            sockPool.poolResetConn(0);
        }
        if (reqObjList.size() < 1)
            return;
        tztZFSocketSendData firstReq = reqObjList.get(0);
        if (!firstReq.isReSend() || isStoping()) {
            reqObjList.remove(firstReq);
        } else {
            firstReq.setIsReSend(false);
            firstReq.setAddTime();
            return;
        }
        if (firstReq != null) {
            asyncRespQueueReqObjError(firstReq, recvType, errorNO, errorMsg);
        }
    }

    ///超时处理
    private void ReqlistTimeout() {
        ///短连接不处理请求超时
        if (reqObjList.size() < 1 || bShortConn)
            return;
        CopyOnWriteArrayList<tztZFSocketSendData> ayResp = new CopyOnWriteArrayList<>();
        for (tztZFSocketSendData send : reqObjList) {
            if (isStoping() || send.isSendTimeOut(tztSendOutTime + connOutTime)) {
                if (send.isReSend() && isRuning()) {
                    send.setIsReSend(false); //重发标识
                    send.setAddTime(); //重置
                    continue;
                }
                ayResp.add(send);
                reqObjList.remove(send);
            }
        }
        ///异步应答
        asyncRespQueueArray(ayResp, tztZFSend_TimeOut, tztZFNetWorkError_Send, tztZFSocktMsgReqTime);
    }

    private int SendedListAddReqObj(tztZFSocketSendData reqObj, int reqTag) {
        sendedReqList.put(reqTag, reqObj);
        tztlog("[AsyncSocket]%d 加入待接收列表 [Action:%d][Reqno:%d],Host:%s",
                GetSocketID(), reqObj.getAction(), reqTag, usedHostPortlog());
        return 1;
    }

    ///获取请求对象
    private tztZFSocketSendData SendedListReqObjForKey(int reqTag) {
        if (isQuit()) return null;
        if (reqTag < 0 || sendedReqList.size() < 1) return null;
        return sendedReqList.remove(reqTag);
    }

    ///清空等待应答列表
    private void SendedListClear(boolean free, tztZFRecvType recvType, int errorNO, String errorMsg) {
        if (sendedReqList.size() < 1)
            return;
        CopyOnWriteArrayList<tztZFSocketSendData> ayResp = new CopyOnWriteArrayList<>();
        CopyOnWriteArrayList<tztZFSocketSendData> ayReSend = new CopyOnWriteArrayList<>();
        if (free) {
            ayResp.addAll(sendedReqList.values());
        } else {
            for (tztZFSocketSendData send : sendedReqList.values()) {
                if (send.isReSend() && isRuning()) {
                    send.setIsReSend(false);
                    send.setAddTime();
                    ayReSend.add(send);
                } else {
                    ayResp.add(send);
                }
            }
        }
        sendedReqList.clear();
        ///异步应答
        asyncRespQueueArray(ayResp, recvType, errorNO, errorMsg);
        ///加入请求队列
        if (ayReSend.size() > 0) {
            ReqlistAddArrayReq(ayReSend);
        }
    }

    ///超时处理
    private void SendedListTimeout() {
        if (isQuit()) return;
        if (sendedReqList.size() < 1) return;
        CopyOnWriteArrayList<tztZFSocketSendData> ayResp = new CopyOnWriteArrayList<>();
        CopyOnWriteArrayList<tztZFSocketSendData> ayResend = new CopyOnWriteArrayList<>();
        boolean bDisConnect = false;
        for (tztZFSocketSendData send : sendedReqList.values()) {
            if (send == null) continue;
            if (send.isRecvTimeOut()) {
                if (send.isReSend() && isRuning()) {
                    send.setIsReSend(false);
                    send.setAddTime();
                    ayResend.add(send);
                    sendedReqList.remove(send.getSendTag());
                    continue;
                }
                ayResp.add(send);
                sendedReqList.remove(send.getSendTag());
                ///检测是否需要断线重连
                if (!bDisConnect && isRuning()) {
                    ///握手包或通讯检测包超时
                    if (send.getType() == tztZFPackage_Handshake || send.getType() == tztZFPackage_Socket) {
                        bDisConnect = true;
                    } else if (linkType == tztZFLinkType.tztZFLinkType_HQ.value() && send.isBeforeSendTimeOut(connectedTime)) {
                        ///行情连接应答超时,行情请求时间>连接成功时间。
                        connectedTime = 0;
                        bDisConnect = true;
                    }
                }
            }
        }
        ///异步应答
        asyncRespQueueArray(ayResp, tztZFRecvType.tztZFRecv_TimeOut, tztZFNetWorkError_Recv, tztZFSocktMsgRecvTime);
        ///加入请求队列
        if (ayResend.size() > 0) {
            ReqlistAddArrayReq(ayResend);
        }
        ///断线
        if (bDisConnect) {
            lastErrorNo = tztZFNetWorkError_Recv;
            SocketPoolNexAddress("RecvTimeOut disConn");
            logdisconnMsg = "应答超时主动断开连接";
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_TimeOutDiscon, tztZFSocktMsgRecvTime);
        }
    }

    private boolean GCDSocketConnecting() {
        ///已关闭
        if (!isRuning()) {
            return false;
        }
        ///已挂起
        //if (isSuspending)
        //    return false;
        ///有请求
        if (haveReq()) {
            return true;
        }
        ///有主推
        if (havePush()) {
            return true;
        }
        ///Socket已连接
        if (GCDSocketIsConnected()) {
            return true;
        }
        return false;
    }

    private boolean canFree() {
        return false;
    }

    private void syncQuit() {
    }

    // 建立连接、协商、请求队列处理
    private void GCDSocketConnectAndSend() {
        while (isRuning()) {
            if (!GCDSocketConnecting()) {
                sockPool.poolCooldownTryingEnd();
                break;
            }
            ///socket已连接
            if (GCDSocketIsConnected()) {
                ///已协议握手成功
                if (SocketConnectSuccessed()) {
                    tztZFSocketSendData reqObj = ReqlistGetFirstObj();
                    if (reqObj != null) {
                        int nSend = GCDSocketSendReqObj(reqObj);
                        if (nSend < 0) {
                            if (tztZFRecordProcessLogTools.isRecord()) {
                                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s;nSend=%d", tztZFNetWorkError_SendInvalid, nSend)
                                        , convertRequestParams(reqObj));
                                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
                            }
                            asyncRespQueueReqObjError(reqObj, tztZFRecvType.tztZFSend_SendError, tztZFNetWorkError_SendInvalid, "");
                        } else if (nSend == 0) {
                            ReqlistAddReqObj(reqObj);
                        }
                        continue;
                    } else {
                        break;
                    }
                } else if (GCDSocketHandshake()) {
                    ///协议握手
                    continue;
                }
            } else {
                ///socket连接
                GCDSocketConnect();
            }
            ///发送队列超时处理
            ReqlistTimeout();
            break;
        }
    }

    /**
     * 启动过通讯线线程
     *
     * @throws NullPointerException 空指针异常
     */
    private void syncRun() throws NullPointerException {
        try {
//            synchronized (syncObject) {
            ///runing , 有待请求、有待主推、已连接
            synchronized (tztSocketQueue) {
                ///
                GCDSocketConnectAndSend();
                //
                SendedListTimeout();
                //
                GCDSocketSendHeartBeat();
                //
                QueueGMCosignDisconnect();
            }
//
//            synchronized (sendThread) {
            ///stoping状态 、有待请求 、有待应答
            if (isStoping() || haveReq() || haveSended()) {
                doWait(1000);
            } else if (GCDSocketIsConnected() || !checkNetWorkAvailableProxy()) {
                ///Socket已连接、无网络
                doWait(3000);
            } else if (isRuning()) {
                //doWait 改为递增模式
                doWait((doWait > 59 ? 59000 : doWait * 1000));
            } else {
                ///结束状态？？？
            }
//            }
        } catch (Exception e) {
            tztlog("[run] Exception:%s", tztZFLog.getStackTraceString(e));
        }
    }


    //=====tztZFSocketThreadPool 线程=======================================================
    private void doNotify(String tip) {
        tztlog("%s", tip);
        if (sendThread == null)
            return;
        synchronized (sendThread) {
            doWait = 0;
            sendThread.notify();
        }
    }

    private void doStop() {
        if (bRun) {
            bRun = false;
        }
        doNotify("doStop");
    }

    private void doWait(int Howlong) {
        synchronized (sendThread) {
            try {
                tztlog("[run] doWait %d-%d", Howlong, doWait);
                if (sendThread == null) return;
                if (Howlong > 0 && doWait > 0) {
                    sendThread.wait(Howlong);
                }
                if (doWait < 59) {
                    doWait++;
                }
            } catch (Exception e) {
            }
        }
    }

    //发送数据包协议打包
    private byte[] transferSendData(tztZFSocketSendData req) {
        if (isQuit()) return null;
        req.setPackageIndex(packageIndex);
        return dataProtocolKit.transferSendData(req.getPackage());
    }

    /**
     * 应答数据协议处理
     */
    private void dealReceiveData(byte[] RecData) {
        if (dataProtocolKit == null)
            return;
        tztZFDataMapBase<String, Object> mapData = new tztZFDataMapBase<>();
        tztZFRecvPackage recvPackage = new tztZFRecvPackage(mapData);
        if (!dataProtocolKit.transferPackageData(recvPackage, RecData)) {
            tztloglevel(tztZFLogLevel.tztZFLogLevel_error, "[SSL-%d][%d][AsyncSocket]%d 解析错误 [Package.transfer][data:%d][Host:%s]",
                    connectType, linkType, GetSocketID(), RecData.length, usedHostPort());
            recvPackage = null;
        }
        // ====检查数据长度完整性====
        if (recvPackage == null) {
            lastErrorNo = tztZFNetWorkError_Recv;
            lastErrorMsg = "数据解析错误！";
            if (tztZFRecordProcessLogTools.isRecord()) {
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", lastErrorMsg));
                tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_ReceiveData, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            SocketPoolMoveLastAddress();
            logdisconnMsg = "数据解析错误主动断开连接";
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_Hand, "数据解析错误");
            return;
        }
        lastRecvTime = System.currentTimeMillis();
        dealRecvPackageData(recvPackage);
    }

    //应答协议数据包处理
    private void dealRecvPackageData(tztZFRecvPackage recvPackage) {
        int recvTag = recvPackage.getTag();
        tztZFDataMapBase<String, Object> mapData = recvPackage.getRecvData();
        String strUsedHost = usedHostPortlog();
        tztlogi("[AsyncSocket]%d 接收应答 [Package.Action:%d][recvTag:%d][Host:%s]",
                GetSocketID(), mapData.getAction(), recvTag, strUsedHost);
        tztZFSocketSendData sendData = SendedListReqObjForKey(recvTag);
        int nRecvError = mapData.getErrorNo();
        if (tztZFLog.getLevel() <= tztZFLogLevel.tztZFLogLevel_Info.value())
            mapData.setStr("tztUsedHost", strUsedHost);

        if (sendData != null) {
            long nUsedTime = sendData.getUsedTime();
            int nAction = sendData.getAction();
            String strReqno = sendData.getReqno();
            if (tztZFPublicFun.isEmpty(strReqno)) {
                mapData.removeByKey("Reqno");
            } else {
                mapData.setStr("Reqno", strReqno);
            }
            //是主推请求的应答
            if (sendData.getType() == tztZFPackage_AutoPush) {
                pushReqObjRecv(sendData, mapData);
            } else {
                if (nAction == 2 && mapData.getInt("ErrorNo") > 0) {
                    int percent = mapData.getInt("ZZPercent", 100);
                    tztZFSocketLink.setStatisticPercent(percent);
                }
                ///处理耗时
                if (tztZFSocketLink.getStatisticPercent()) {
                    setStatistic(nAction, nUsedTime);
                }
            }
            //判断是否含主动挑战
            checkInitiativeChallenge(mapData, sendData);
            //在有主动挑战的情况下，判断是是否挑战成功
            checkAnsInitiativeChallenge(mapData);
            tztlog("[AsyncSocket]%d [Recv][Action:%d][Reqno:%d]Host:%s;json:%s",
                    GetSocketID(), nAction, recvTag, strUsedHost, mapData.toString());
            asyncRespQueueReqObj(sendData, true, tztZFRecvType.tztZFRecv_DataOK, mapData);
        } else {
            tztlog("[AsyncSocket]%d [Recv-PushData][Action:%d][Reqno:%d]Host:%s;json:%s",
                    GetSocketID(), mapData.getAction(), recvTag, strUsedHost, mapData.toString());
            sendData = pushReqObjForKey(recvTag, mapData);
            if (sendData != null) {
                asyncRespQueueReqObj(sendData, false, tztZFRecvType.tztZFRecv_DataOK, mapData);
            } else {
                //找不到：主推的应答或对应请求已经被超时、断线等过程清除，都当主推处理（因为没法区分）
                linkblockPush(mapData);
            }
        }
        ///服务器处理忙、业务组件未加载，切换服务器  add 20220622 yangdl
        if (nRecvError == tztZFHandshakeError_ReqListFull ||
                nRecvError == tztZFHandshakeError_WaitTimeOut ||
                nRecvError == tztZFHandshakeError_ActionBusying ||
                nRecvError == tztZFHandshakeError_ModuleException ||
                nRecvError == tztZFHandshakeError_NotFindAction) {
            lastErrorNo = tztZFNetWorkError_Recv;
            lastErrorMsg = String.format(Locale.getDefault(), "[%d]%s", nRecvError, mapData.getErrorMessage());
            SocketPoolNexAddress("recvCode Error," + lastErrorMsg);
            logdisconnMsg = "组件错误主动断开连接," + lastErrorMsg;
            QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ChangeAddress, lastErrorMsg);
        }
    }

    //连接回调
    private void linkblockResp(tztZFRecvType type, String address, String msg) {
        if (linkBlock == null)
            return;
        tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
        resp.setRecvType(type);
        tztZFDataMapBase<String, Object> data = new tztZFDataMapBase<>();
        data.setInt("SDKType", sdkType);
        data.setInt("LinkType", linkType);
        data.setInt("RecvType", type.value());
        data.setStr("LinkKey", linkkey);
        if (type != tztZFRecvType.tztZFConn_Connected) {
            data.setErrorNo(lastErrorNo);
            data.setErrorMessage(msg);
        } else {
            data.setErrorNo(0);
        }
        if (!tztZFPublicFun.isEmpty(address)) {
            data.setStr("Address", address);
        }
        if (tztZFLog.getLevel() <= tztZFLogLevel.tztZFLogLevel_Info.value()) {
            data.setStr("tztUsedHost", usedHostPortlog());
        }
        resp.setResponseData(data);
        final tztZFNetWorkRespObj connResp = resp;
        if (respThread != null && linkBlock != null) {
            respThread.execute(new tztZFThreadRespRunnable(linkType, 0, data) {
                @Override
                public void run() {
                    if (linkBlock != null)
                        linkBlock.onHandler(tztZFLinkCallBackType.tztZFLinkCallBackType_NetState.value(), sdkType, linkType, connResp);
                }
            });
        }
    }

    //数据回调
    private void linkblockPush(tztZFDataMapBase<String, Object> data) {
        if (linkBlock == null)
            return;
        tztZFNetWorkRespObj resp = new tztZFNetWorkRespObj();
        if (data != null) {
            resp.setResponseData(data);
        } else {
            resp.setStatusCode(0);
        }
        resp.setRecvType(tztZFRecvType.tztZFRecv_DataOK);
        final tztZFNetWorkRespObj pushResp = resp;
        if (respThread != null && linkBlock != null) {
            respThread.execute(new tztZFThreadRespRunnable(linkType, 0, data) {
                @Override
                public void run() {
                    if (linkBlock != null)
                        linkBlock.onHandler(tztZFLinkCallBackType.tztZFLinkCallBackType_PostData.value(), sdkType, linkType, pushResp);
                }
            });
        }
    }

    private String logHostInfo() {
        String logHost = "";
        if (syncSocket != null) {
            logHost = "used:" + syncSocket.getUserData().toString();
        } else if (!tztZFPublicFun.isEmpty(connectedHostPort)) {
            logHost = "last:" + connectedHostPort;
        } else if (sockPool != null) {
            logHost = "conn:" + sockPool.poolAddress(false);
        }
        return String.format(Locale.getDefault(), "LinkType:%d,Connecttype:%d,Host:%s,step:%s,lastdisconn:%s",
                linkType, connectType, logHost, tztZFSocketFlags.tztZFSocketFlagTip.tips(socketStep), logdisconnMsg);
    }

    private void asyncRespQueueReqObjError(final tztZFSocketSendData reqObj, final tztZFRecvType type, final int errorNo, final String errorMessage) {
        if (reqObj == null)
            return;
        asyncRespQueue(new Runnable() {
            @Override
            public void run() {
                String errorMsg = convertResponseErrorMsg(errorMessage);
                reqObj.responseError(respThread, linkType, type, errorNo, errorMsg, true);
            }
        });
    }

    private void asyncRespQueueReqObj(final tztZFSocketSendData reqObj, final boolean bUsedTime, final tztZFRecvType type, final tztZFDataMapBase<String, Object> recvMap) {
        if (reqObj == null)
            return;
        if (recvMap != null) {
            String errorMsg = recvMap.getStr("ErrorMessage");
            errorMsg = convertResponseErrorMsg(errorMsg);
            recvMap.setStr("ErrorMessage", errorMsg);
        }
        asyncRespQueue(new Runnable() {
            @Override
            public void run() {
                reqObj.responseData(respThread, linkType, type, bUsedTime, recvMap);
            }
        });
    }


    private void asyncRespQueueArray(final CopyOnWriteArrayList<tztZFSocketSendData> ayReqObj, final tztZFRecvType type, final int errorNo, final String errorMessage) {
        if (ayReqObj == null || ayReqObj.size() < 1)
            return;
        asyncRespQueue(new Runnable() {
            @Override
            public void run() {
                for (tztZFSocketSendData reqObj : ayReqObj) {
                    if (reqObj == null) {
                        continue;
                    }
                    String errorMsg = convertResponseErrorMsg(errorMessage);
                    reqObj.responseError(respThread, linkType, type, errorNo, errorMsg, true);
                }
                ayReqObj.clear();
            }
        });

    }

    /**
     * 异步应答回调线程，多线程处理
     *
     * @param runnable 异步线程
     */
    private void asyncRespQueue(@NonNull Runnable runnable) {
        if (respThread == null || isQuit() || isStoping()) {
            runnable.run();
            return;
        }
        respThread.execute(runnable);
    }

    private String convertResponseErrorMsg(String errorMsg) {
        if (tztZFLog.getLevel() <= tztZFLogLevel_Info.value()) {
            String errorHost = "";
            if (syncSocket != null && syncSocket.getUserData() != null) {
                errorHost = syncSocket.getUserData().toString();
            } else if (!tztZFTextUtils.isEmpty(connectedHostPort)) {
                errorHost = String.format("last:%s", connectedHostPort);
            } else if (sockPool != null) {
                errorHost = String.format("conn:%s", sockPool.poolAddress(false));
            }
            String strFlag = tztZFSocketFlags.tztZFSocketFlagTip.tips(socketStep);
            String strExInfo = String.format(Locale.getDefault(), "LinkType:%d,Connecttype:%d,Host:%s,step:%s,lastdisconn:%s",
                    linkType, connectType, errorHost, strFlag, logdisconnMsg);
            errorMsg = String.format("%s,exInfo:%s", errorMsg != null ? errorMsg : "", strExInfo);
        }
        return errorMsg;
    }

    //主推事件处理
    private void pushReSend() {
        pushHQ.clearRec();
        pushCode.clearRec();
        CopyOnWriteArrayList<tztZFSocketSendData> ayReSend = new CopyOnWriteArrayList<>();
        pushHQ.reSendReq(ayReSend);
        pushCode.reSendReq(ayReSend);
        if (ayReSend.size() > 0) {
            ReqlistAddArrayReq(ayReSend);
        }
    }

    //主推请求加入队列
    private void pushAddReqObj(tztZFSocketSendData reqObj) {
        if (reqObj == null) return;
        int nAction = reqObj.getAction();
        if (nAction < 1)
            return;
        if (tztZFSocketPushHQ.isHQPushAction(nAction)) {
            pushHQ.addReqPush(reqObj);
            return;
        }
        if (tztZFSocketPushCode.isPushAction(nAction)) {
            pushCode.addReqPush(reqObj);
            return;
        }
    }

    //根据应答查找主推请求
    private tztZFSocketSendData pushReqObjForKey(int recvTag, tztZFDataMapBase<String, Object> recvMap) {
        if (recvMap == null || recvMap.getCount() < 1)
            return null;
        int nAction = recvMap.getAction();
        if (tztZFSocketPushHQ.isHQPushAction(nAction)) {
            return pushHQ.findPushWithtag(recvTag);
        }
        if (tztZFSocketPushCode.isPushAction(nAction)) {
            String serviceCode = recvMap.getStr("ServiceCode");
            return pushCode.findPushWithCode(serviceCode);
        }
        return null;
    }

    //#pragma mark -处理主推订阅/退订应答
    private void pushReqObjRecv(tztZFSocketSendData reqObj, tztZFDataMapBase<String, Object> recvMap) {
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        int nAction = recvMap.getAction();
        if (tztZFSocketPushHQ.isHQPushAction(nAction)) {
            pushHQ.dealRecvPush(recvMap, reqObj);
        } else if (tztZFSocketPushCode.isPushAction(nAction)) {
            pushCode.dealRecvPush(recvMap, reqObj);
        }
    }

    //主推队列清空
    private void pushReqClear() {
        pushHQ.clearReq();
        pushCode.clearReq();
    }

    private void pushRespClear(tztZFRecvType type) {
        pushHQ.clearRecWithType(type);
        pushCode.clearRecWithType(type);
    }

    //=====证书=======================================================
    private boolean dataProtocolLoadHandShakeCert() {
        tztZFProtocolCert protocolCert = tztZFNetCosign.getSXProtocolCert();
        if (protocolCert == null || protocolCert.isInvalid()) {
            return dataProtocolKit.switchToLoadCert();
        }
        bLoadDataProtocol = dataProtocolKit.setProtocolCert(userID, sdkType, protocolCert);
        if (!bLoadDataProtocol) {
            lastErrorMsg = dataProtocolKit.getErrorInfo();
        }
        tztlog("[dataProtocol] setProtocolCert");
        return bLoadDataProtocol;
    }

    private int GCDSocketSSLConnect(tztZFGCDAsyncSocket sock) {
        if (connectType < 1 || sock == null) return 0;
        tztlogi("[SSL] socketSSLConnect [AsyncSocket]%d", sock.hashCode());
        tztZFSocketSSL socketSSL = new tztZFSocketSSL(context);
        socketSSL.withSSLLinktype(linkType, connectType);
        if (connectType == 8 || connectType == 9) {
            tztZFSSLCert sslCert = new tztZFSSLCert();
            if (connectType == 8)
                tztZFNetCosign.getDXSSLCert(sslCert);
            else {
                tztZFNetCosign.getSXSSLCert(sslCert);
                if (sslCert.getCertSign() == null || sslCert.getCertSign().length < 1) {
                    if (SSLStep != null) {
                        SSLStep.onCallBack(tztZFSocketSSLStepUnClientCert, 0, sslCert.getCertUser());
                    }
                    ReqlistClear(true, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_InvalidSSLCert, tztZFSocktMsgUserCertInvalid);
                    return -1;
                }
            }
            socketSSL.withSSLCert(sslCert);
        } else {
            socketSSL.loadClientSSLCert();
        }
        if (socketSSL.sslinit() > 0) {
            if (SSLStep != null) {
                SSLStep.onCallBack(tztZFSocketSSLStepInit, 0, socketSSL.getCertUser());
            }
        }
        sock.useSocketSSL(socketSSL);
        int nRet = socketSSL.sslconnect(sock);
        if (nRet == 0)
            return 1;
        return -1;
    }

    /**
     * 设置国密连接超时时间,单位毫秒
     *
     * @param timeOut 超时时间
     */
    public void SetGMConnectOutTime(int timeOut) {
        synchronized (tztSocketQueue) {
            if (timeOut < tztMinOutTime)
                return;
            gmconnOutTime = timeOut;
            if (sockPool != null) {
                sockPool.setFirstTimeOut(gmconnOutTime);
            }
        }
    }

    //=====GM=======================================================
    public void setGMAddressList(String addList, String nameList, int selectIndex) {
        GMAddressList(addList, nameList, selectIndex);
    }

    public void didGMBindUserID(String userID, boolean isSSL, tztZFGMSSLStepCallBack sslStep) {
        if (linkType != tztZFLinkType.tztZFLinkType_JY.value()) return;
        boolean bChangeUser = false;
        boolean bChangeSSL = false;
        //国密登录普通信用、退出普通、删除网络证书，重新登录会出现降级(GMDowngradeuser)方法中同步锁问题
//        synchronized (tztZFSocket.this) {
        if (sslStep != null)
            gmSSLStep = sslStep;
        if (isSSL && tztZFPublicFun.isEmpty(userID)) {
            userID = tztZFNetCosign.getBindUserID();
        }
        if (!tztZFPublicFun.isEmpty(userID)) {
            tztZFCosignUser cosignUser = tztZFNetCosign.cosignUserForID(userID);
            isSSL = isSSL && cosignUser != null && cosignUser.isNeedGMLogin();
        }
        String param = String.format("[SSL] didGMBindUserID 绑定新用户:%s,SSL:%s,当前用户:%s", userID, (isSSL ? "YES" : "NO"), gmUserID);
        tztlogi(param);
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param1 = tztZFRecordProcessLogTools.stringFormat("gmUserID=%s;message=%s", gmUserID, "绑定新用户");
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), param1, param);
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_BindUserID, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        int connecttype = 0;
        if (tztZFPublicFun.isEmpty(userID) || !isSSL || !tztZFNetCosign.getCosignStatus()) {
            gmUserID = "";
            gmCosignUser = null;
            gmHaveCosignCert = false;
        } else {
            connecttype = 9;
        }
        bChangeSSL = (connectType != connecttype);
        connectType = connecttype;
        ///连接状态
        if (syncSocket != null) {
            ///非SSL连接
            if (SocketConnectSuccessed() && !syncSocket.isUseSSL() && connectType == 0) {
                if (this.SSLStep != null) {
                    this.SSLStep.onCallBack(tztZFSocketSSLStepSocket, 0, "");
                }
                doNotify("setGMUserID notSSL");
                return;
            }
            if (SocketConnectSuccessed() && syncSocket.isUseSSL() && connectType == 9 && userID.equalsIgnoreCase(gmUserID)) {
                if (this.SSLStep != null) {
                    this.SSLStep.onCallBack(tztZFSocketSSLStepSucess, 0, gmUserID);
                }
                doNotify("setGMUserID isSSL");
                return;
            }
        }
        if (connectType == 9) {
            if (!userID.equalsIgnoreCase(gmUserID)) {
                gmUserID = String.format("%s", userID);
                bChangeUser = true;
            }
            gmCosignUser = tztZFNetCosign.cosignUserForID(gmUserID);
            gmHaveCosignCert = (gmCosignUser != null && gmCosignUser.isNeedGMLogin() && gmCosignUser.isValidity());
        }
//        }
        if (bChangeUser) {
            ReqlistClear(true, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_ChangeSSLUser, tztZFSocketMsgConnDis);
        }
        boolean bDidDisconn = false;
        if (sockPool != null) {
            sockPool.poolFirstUsed(GMisSSL());
        }
        bDidDisconn = SocketPoolChangedAndDidDisconnect(gmAddressIndex);
        ///切换服务器重连
        if (bDidDisconn) {
            return;
        }
        ///未断线重连、更改连接方式或用户、有连接
        if ((bChangeSSL || bChangeUser) && syncSocket != null) {
            String gmLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable, tztZFErrorMessageDefine.tztZFGMLable);
            logdisconnMsg = String.format("%s切换主动断开连接,%s", gmLable, (bChangeSSL ? "SSL" : "UserID"));
            ///主动断开连接
            GCDSocketClientClose(tztZFConn_DisConnect, (bChangeSSL ? tztZFNetWorkError_ChangeSSL : tztZFNetWorkError_ChangeSSLUser), tztZFSocketMsgConnDis);
        } else {
            doNotify(String.format(Locale.getDefault(), "[GM] setGMUserID:%s,connecttype:%d", gmUserID, connectType));
        }
    }

    public void clearGMSSLStepCallBack() {
        gmSSLStep = null;
    }

    ///国密降级不可恢复
    public void GMDowngradeConnect(String msg) {
        QueueGMDowngrade();
    }

    ///国密降级不可恢复
    public void QueueGMDowngrade() {
        ///降级操作、发出降级通知
        GMDowngradeUser();
        ///有连接断开连接
        String gmLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable, tztZFErrorMessageDefine.tztZFGMLable);
        logdisconnMsg = String.format(Locale.getDefault(), "%s降级主动断开连接", gmLable);
        GCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ChangeNoSSL, tztZFSocketMsgConnDis);
    }

    public void GMSetCosignUserID(String userID, int errorNO, String errorMsg) {
        synchronized (tztSocketQueue) {
            QueueGMCosignUserID(userID, errorNO, errorMsg);
        }
    }

    public void QueueGMCosignUserID(String userID, int errorNO, String errorMsg) {
        tztlogi("[SSL] didGMCosignUserID %s,[%d]%s", userID, errorNO, errorMsg);
        if (tztZFPublicFun.isEmpty(userID) || connectType != 9)
            return;
        //单向签名调用的地方进行了同步处理，此处再同步的话，会导致等待
//        synchronized (tztZFSocket.this) {
        if (!userID.equalsIgnoreCase(gmUserID))
            return;
        if (errorNO == tztZFCosignErrServerCertUnfind || errorNO == tztZFCosignErrCertDeviceDel) {
            gmHaveCosignCert = false;
            if (this.SSLStep != null) {
                SSLStep.onCallBack(tztZFSocketSSLStepUnServerCert, 0, gmUserID);
            }
            ReqlistClear(true, tztZFRecvType.tztZFSend_SendCancel, tztZFNetWorkError_InvalidSSLCert, errorMsg);
            respThread.execute(new tztZFThreadRespRunnable(linkType, 0, null) {
                @Override
                public void run() {
                    if (sockPool != null)
                        sockPool.poolClose("ErrServerCertUnfind");
                }
            });
        }
//        }
    }

    ///国密双向连接
    private boolean GMisSSL() {
        return connectType == 9 && tztZFNetCosign.getCosignStatus() && linkType == tztZFLinkType.tztZFLinkType_JY.value();
    }

    private void GMAddressList(String addList, String nameList, int selectIndex) {
        if (linkType != tztZFLinkType.tztZFLinkType_JY.value()) return;
        tztlog("[SSL] GMAddressList[%d],%s", selectIndex, addList);
        boolean bDidDisconn = false;
        ArrayList<tztZFHostportIpv4Ipv6> ayArray = tztZFHostportIpv4Ipv6.withAddresslistNamelist(addList, nameList, 9);
        ayGMAddress.clear();
        if (ayArray != null && ayArray.size() > 0)
            ayGMAddress.addAll(ayArray);
        gmAddressIndex = selectIndex;
        tztZFHostportIpv4Ipv6 ipv4Ipv6 = null;
        if (sockPool != null) {
            sockPool.setFirstAddresses(ayGMAddress, gmAddressIndex);
            bDidDisconn = SocketPoolChangedAndDidDisconnect(gmAddressIndex);
            ipv4Ipv6 = sockPool.poolAddress(false);
        }
        if (ipv4Ipv6 != null) {
            connectHostPort = ipv4Ipv6.address();
        }
        if (!bDidDisconn) {
            doNotify("GMAddressList");
        }
    }

    private String GMGetUser() {
        if (syncSocket != null && syncSocket.isUseSSL())
            return syncSocket.GetCertUser();
        return gmUserID;
    }

    private void GMClearUser() {
        gmUserID = "";
        gmCosignUser = null;
        gmHaveCosignCert = false;
    }

    private void QueueGMCosignDisconnect() {
        if (connectType != 8)
            return;
        if (!GCDSocketIsConnected())
            return;
        if (!haveReq() && System.currentTimeMillis() - lastRecvTime >= 5 * 60 * 1000) {
            String GMStr = tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_GMLable, "国密");
            logdisconnMsg = String.format("%s%s", GMStr, "单向主动断开连接");
            GCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ClientDiscon, tztZFSocketMsgConnDis);
        }
    }

    private void GMDowngradeUser() {
        synchronized (tztSocketQueue) {
            QueueGMGMDowngradeUser();
        }
    }

    private void QueueGMGMDowngradeUser() {
        if (connectType != 9)
            return;
        if (tztZFRecordProcessLogTools.isRecord()) {
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(getSocketParams(), tztZFRecordProcessLogTools.stringFormat("message=%s", "国密降级"));
            tztZFRecordProcessLogTools.recordLog(linkkey, tztZFRecordEventType.tztZFRecordEventType_GMDemote, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        tztlogi("[SSL] 国密降级 %s", GMGetUser());
        connectType = 0;
        if (this.SSLStep != null) {
            SSLStep.onCallBack(tztZFSocketSSLStepDowngrade, 0, GMGetUser());
        }
        GMClearUser();
        if (sockPool != null)
            sockPool.poolFirstUsed(false);
    }

    private tztZFCosignUser GMLoadCosignUser(boolean bLoad) {
        if (!bLoad && gmCosignUser != null)
            return gmCosignUser;
        gmCosignUser = tztZFNetCosign.getBindCosignUser();
        return gmCosignUser;
    }
    //=====tztZFSocketThreadPool 命中事件处理 BEGIN =======================================================

    /**
     * 命中统计
     *
     * @param nAction 功能号
     * @param time    耗时
     */
    private void setStatistic(int nAction, long time) {
        //组织上报的数据格式: 功能号,耗时,网络类型
        if (time > 0) {
            if (netAvailable != null)
                currentNetType = netAvailable.getNetWorkType();
            statisticVal = String.format(Locale.getDefault(), "%d,%d,%s", nAction, time, currentNetType);
        }
    }

    //=====tztZFSocketThreadPool 主动挑战 BEGIN =======================================================

    /**
     * 判断是否有主动性挑战失败的情况，如果失败，则提出所有账号信息
     *
     * @param ans 数据字典
     */
    private void checkAnsInitiativeChallenge(tztZFDataMapBase<String, Object> ans) {
        int nErrorNo = ans.getInt("ErrorNo");
        if (nErrorNo == -204021 || nErrorNo == -204022) {
            String errorMessage = ans.getStr("ErrorMessage");
            //挑战失败，交易强制退出
            if (netAvailable != null)
                netAvailable.eventBus(new tztZFQuitTradeEvent(errorMessage));
        }
    }

    /**
     * 主动性挑战
     * 底层接收到数据后，发现应答数据中有challenge_question和challenge_formulaid字段，
     * 根据challenge_formulaid对应的值(暂定 1-MD5计算 2-Base64计算 )，
     * 将challenge_question用对应的算法计算后，发送6700给服务器进行校验，
     * 在收到-204021或者-204022的情况下，说明挑战出错失败了，
     * 需要退出交易，其他情况下，继续正常使用
     */
    private void checkInitiativeChallenge(tztZFDataMapBase<String, Object> ans, tztZFSocketSendData sendData) {

        if (ans == null || sendData == null || sendData.getReqMapdata() == null || tztZFPublicFun.isEmpty(sendData.getReqMapdata().getStr("mobileCode"))) {
            return;
        }
        String challengeQuestion = ans.getStr("CHALLENGE_QUESTION");
        String challengeFormulaid = ans.getStr("CHALLENGE_FORMULAID");
        String token = ans.getStr("TOKEN");
        if (!tztZFPublicFun.isEmpty(challengeQuestion)
                && !tztZFPublicFun.isEmpty(challengeFormulaid)
                && !tztZFPublicFun.isEmpty(token)) {
            String challengeAns = "";
            switch (Integer.valueOf(challengeFormulaid)) {
                case 1:
                    challengeAns = tztZFEncryptFun.md5(challengeQuestion.getBytes());
                    break;
                case 2:
                    challengeAns = tztZFEncryptFun.encBase64(challengeQuestion.getBytes());
                    break;
            }
            if (!tztZFPublicFun.isEmpty(challengeAns)) {
                //构造挑战请求
                tztZFNetWorkReqObj reqObj = new tztZFNetWorkReqObj();
                reqObj.setInt("action", 6700);
                reqObj.setStr("mobileCode", sendData.getReqMapdata().getStr("mobileCode"));
                reqObj.setStr("token", token);
                reqObj.setStr("answer", challengeAns);
                tztZFSocketSendData challengesendData = new tztZFSocketSendData(reqObj, tztZFPackage_Data, socketKey);
                challengesendData.setResponseCallBack(sendData.getResponseCallBack());
                ReqlistAddReqObj(challengesendData);
            }
        }
    }

    //=====tztZFSocketThreadPool 网络状态=======================================================
    private boolean checkNetWorkAvailableProxy() {
        // 检测网络状态也是不通的
        if (netAvailable != null && !netAvailable.checkNetworkAvailable()) {
            return false;
        }
        boolean Proxy = netAvailable != null && netAvailable.checkProxy();
        //if (m_nConnNum >= m_nMaxConn && lastRecvTime == 0) {
        //}
        isProxy = Proxy;
        return true;
    }

    private void netChanged(String strNetWorkType, boolean isProxy, String proxyAdd, int proxyPort) {
        tztlog("[Net] 系统网络切换:%s==>%s", currentNetType, strNetWorkType);
        if (!strNetWorkType.equalsIgnoreCase(currentNetType))
            linkblockResp(tztZFRecvType.tztZFConn_Change, connectHostPort, "网络切换");
        logdisconnMsg = "网络切换主动断开连接," + currentNetType + "=>" + strNetWorkType;
        QueueGCDSocketClientClose(tztZFConn_DisConnect, tztZFNetWorkError_ChangeNetType, tztZFSocketMsgConnDis);
        currentNetType = strNetWorkType;
        this.isProxy = isProxy;
        this.proxyHost = proxyAdd;
        this.proxyPort = proxyPort;
        doNotify("[Net] netChange");
    }

    //业务处理线程池
    private class tztSocketDiscardOldestPolicy implements RejectedExecutionHandler {
        @Override
        public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
            // 处理任务被拒绝的情况，记录日志
            //弹出最先加入的任务，空出一个位置，然后再次执行execute方法把任务加入队列。
            if (!executor.isShutdown()) {
                Runnable p = executor.getQueue().poll();
                if (p != null) {
                    tztloglevel(tztZFLogLevel.tztZFLogLevel_error, "%s", p.toString());
                }
                executor.execute(r);
            }
        }
    }

    //=====tztZFSocket log=======================================================

    /**
     * 打印日志
     */
    private void tztlog(String format, Object... args) {
        tztloglevel(tztZFLogLevel.tztZFLogLevel_Debug, format, args);
    }

    private void tztlogi(String format, Object... args) {
        tztloglevel(tztZFLogLevel.tztZFLogLevel_Info, format, args);
    }

    private void tztloglevel(tztZFLogLevel level, String format, Object... args) {
        if (tztZFLog.getLevel() <= level.value()) {
            String cTag = String.format(Locale.getDefault(), "[%d]tztZFSocket[%d][%d-%s]",
                    linkType, this.hashCode(), sdkType, tztZFLinkType.nameOf(linkType));
            String tag = "";
            if (!tztZFPublicFun.isEmpty(logTag)) {
                tag = "[" + logTag + "]";
            }
            tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), cTag, tag + format, args);
        }
    }

    private String getSocketParams() {
        return tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;connnectype=%d;currentaddress=%s;", linkkey, userID, sdkType, linkType, connectType,
                tztZFSocket.this.usedHostPortlog());
    }

    private String convertRequestParams(tztZFNetWorkReqObj reqObj) {
        if (reqObj != null && reqObj.getSendValues() != null) {
            return tztZFRecordProcessLogTools.stringFormat("send json:%s", reqObj.getSendValues().toString());
        }
        return null;
    }

    private String convertRequestParams(tztZFSocketSendData send) {
        if (send != null && send.getReqMapdata() != null) {
            return tztZFRecordProcessLogTools.stringFormat("send json:%s", send.getReqMapdata().toString());
        }
        return null;
    }

}