package com.tztzf.network.socket.Netty;

import com.tztzf.baseos.tztZFIntDef;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.define.tztZFLinkType;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.Locale;
import java.util.concurrent.CopyOnWriteArrayList;

import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepCompoundTrying;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepConnectUsed;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepConnected;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepCooldownTrying;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepInitial;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepSingleConnect;
import static com.tztzf.network.socket.Netty.tztZFSocketPoolStep.tztZFSocketPoolStepSingleTrying;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Close;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_OKAndTaken;
import static com.tztzf.network.socket.Netty.tztZFSocketSSLFlag.tztZFSocketSSLFlag_Socket;

@tztZFIntDef({
        tztZFSocketPoolStepInitial,
        tztZFSocketPoolStepConnected,
        tztZFSocketPoolStepSingleConnect,
        tztZFSocketPoolStepSingleTrying,
        tztZFSocketPoolStepCompoundTrying,
        tztZFSocketPoolStepCooldownTrying,
        tztZFSocketPoolStepConnectUsed
})
@Retention(RetentionPolicy.SOURCE)
@interface tztZFSocketPoolStep {
    //region 内部私有属性及函数
    /**
     * 初始
     */
    int tztZFSocketPoolStepInitial = (0);
    /**
     * 单连
     */
    int tztZFSocketPoolStepSingleConnect = (1);
    /**
     * 单连尝试
     */
    int tztZFSocketPoolStepSingleTrying = (2);
    /**
     * 并连尝试
     */
    int tztZFSocketPoolStepCompoundTrying = (3);
    /**
     * 串连尝试
     */
    int tztZFSocketPoolStepCooldownTrying = (4);
    /**
     * 连接成功
     */
    int tztZFSocketPoolStepConnected = (5);
    /** 连接启用 */
    int tztZFSocketPoolStepConnectUsed = (6);

    //endregion
}

public class tztZFSocketPool {
    // 目前并行3个
    private static final int kParallelNum = 3;
    // 复合阶梯简单
    private static final int kStepGap = 1000;
    // 5S，开始冷静期
    private static final int kFinishTimeout = 5000;
    //1800S 冷静期
    private static final int kCoolDownSecs = 1800000;
    //最长重连间隔
    private static final int kCoolDownMaxDelay = 10000;
    //socket列表
    private CopyOnWriteArrayList<tztZFGCDAsyncSocket> aySock = new CopyOnWriteArrayList<>();
    //地址列表
    private CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayAddress = new CopyOnWriteArrayList<>();
    ///是否启用first地址列表
    private boolean firstUsed = true;
    private CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayFirstAddress = new CopyOnWriteArrayList<>();
    ///优先地址
    private tztZFHostportIpv4Ipv6 firstAddress = null;
    private tztZFHostportIpv4Ipv6 singleAddress = null;

    ///序号
    private int firstIndex = 0;
    private int singleIndex = 0;
    //连接失败次数
    private int connectFail = 0;
    //连接失败最大次数
    private int connMaxFail = 0;
    private boolean nextAddress = false;
    ///链路类别
    private int linkType;
    ///连接类别
    private int connecttype = 0;
    ///超时
    private int timeout = 10 * 1000;
    private int firsttimeout = 10 * 1000;
    ///单连接超时
    private int singletimeout = 3 * 1000;
    ///步骤
    private int step = tztZFSocketPoolStepInitial;
    ///socket （支持iPV4、iPV6双地址）
    private tztZFGCDAsyncSocket singleSock1 = null;
    private tztZFGCDAsyncSocket singleSock2 = null;
    private tztZFGCDAsyncSocket usedSock;
    ///连接timer
    private tztZFTimer connectTimer = null;
    private tztZFTimer firstconnectTimer = null;
    ///是否重新分组
    private boolean iscompound = false;
    ///当前并发组序号
    private int compoundIndex = 0;
    ///并发地址组
    private CopyOnWriteArrayList<CopyOnWriteArrayList<tztZFHostportIpv4Ipv6>> ayCompoundAddrs = new CopyOnWriteArrayList<>();
    ///并发连接所使用的timer
    private tztZFTimer compoundTimer = null;
    ///结束并发，进入冷静期所使用的timer
    private tztZFTimer compoundFinishTimer = null;
    ///冷静期开始时间戳
    private long cooldownStartTimeStamp = 0;
    ///冷静期重连的次数，用于控制重连的间隔。
    private int cooldownConnectTimes = 0;
    ///冷静期延时连接的timer
    private tztZFTimer cooldownTimer = null;
    ///回调接口
    private tztZFSocketPoolDelegate poolBlock = null;
    //连接线程
    private tztZFThreadPool pThreadConnPool = null;
    private tztZFGCDAsyncSocketDelegate socketBlock = null;
    private Object socketQueue;

    public tztZFSocketPool(int linkType, Object socketQueue) {
        this.linkType = linkType;
        this.socketQueue = socketQueue;
        pThreadConnPool = new tztZFThreadPool(2, 30, "tztZFSocketPoolConn_" + linkType);
        pThreadConnPool.setUsed(true);
        socketBlock = new tztZFGCDAsyncSocketDelegate() {
            @Override
            public void didConnectToHost(tztZFGCDAsyncSocket sock, String host, int port) {
                socketConnected(sock, false);
            }

            @Override
            public void didSSLConnectToHost(tztZFGCDAsyncSocket sock, String host, int port) {
                socketConnected(sock, true);
            }

            @Override
            public void socketDidDisconnect(tztZFGCDAsyncSocket sock, int errorno, String errormessage) {
                socketDisConnect(sock, errormessage);
            }

            @Override
            public void didReadData(tztZFGCDAsyncSocket sock, byte[] data, int len, Object tag) {

            }

            @Override
            public void didWriteDataWithTag(tztZFGCDAsyncSocket sock, Object tag) {

            }
        };
    }

    public void poolFree() {
        if (pThreadConnPool != null) {
            pThreadConnPool.shutdownNow();
            pThreadConnPool = null;
        }
    }

    public static tztZFHostport usedHostPort(tztZFGCDAsyncSocket socket) {
        if (socket == null)
            return null;
        tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = socket.getUserData();
        return (hostportIpv4Ipv6 == null ? null : hostportIpv4Ipv6.usedHostport());
    }

    private static String descForStep(int step) {
        switch (step) {
            case tztZFSocketPoolStepInitial:
                return "Initial";
            case tztZFSocketPoolStepConnected:
                return "Connected";
            case tztZFSocketPoolStepSingleConnect:
                return "SingleConnect";
            case tztZFSocketPoolStepSingleTrying:
                return "SingleTrying";
            case tztZFSocketPoolStepCompoundTrying:
                return "CompoundTrying";
            case tztZFSocketPoolStepCooldownTrying:
                return "CooldownTrying";
            case tztZFSocketPoolStepConnectUsed:
                return "tztZFSocketPoolStepConnectUsed";
        }
        return String.format(Locale.CHINA, "Invalid_%d", step);
    }

    public tztZFSocketPool setAddresses(CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> addresses, int addressIndex) {
        tztlog("setAddresses " + addressIndex + "/" + (addresses != null ? addresses.size() : 0));
        ayAddress.clear();
        if (addresses != null && addresses.size() > 0) {
            ayAddress.addAll(addresses);
        }
        if (addressIndex >= 0) {
            singleIndex = addressIndex;
            nextAddress = false;
        }
        iscompound = true;
        return tztZFSocketPool.this;
    }

    public tztZFSocketPool setFirstAddresses(CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> addresses, int addressIndex) {
        tztlog("setFirstAddresses " + addressIndex + "/" + (addresses != null ? addresses.size() : 0));
        ayFirstAddress.clear();
        if (addresses != null && addresses.size() > 0) {
            ayFirstAddress.addAll(addresses);
        }
        if (addressIndex >= 0) {
            firstIndex = addressIndex;
            nextAddress = false;
        }
        iscompound = true;
        return tztZFSocketPool.this;
    }

    public tztZFSocketPool setTimeOut(int timeOut) {
        this.timeout = timeOut;
        return tztZFSocketPool.this;
    }

    public tztZFSocketPool setFirstTimeOut(int timeOut) {
        this.firsttimeout = timeOut;
        return tztZFSocketPool.this;
    }

    public tztZFSocketPool setSingleTimeOut(int singleTimeOut) {
        if (singleTimeOut > 1 && singleTimeOut < 60) //转毫秒
            this.singletimeout = singleTimeOut * 1000;
        else if (singleTimeOut > 1000 && singleTimeOut < 60000)
            this.singletimeout = singleTimeOut;
        return tztZFSocketPool.this;
    }

    private void logStep(String tip) {
        tztlog("%s SocketPoolStep[%s]", tip, descForStep(step));
    }

    private void logHostport(String tip) {
        tztlog("%s,hostport:first:%d/%d,single:%d/%d,next:%s",
                tip, firstIndex, firstAddressCount(),
                singleIndex, ayAddress.size(), (nextAddress ? "true" : "false"));
    }

    private tztZFGCDAsyncSocket createSocket(tztZFHostport hostport) {
        if (pThreadConnPool == null) {
            return null;
        }
        return new tztZFGCDAsyncSocket(pThreadConnPool.useThreadPool(), socketBlock, socketQueue);
    }

    private void cooldownConnectAction() {
        singleConnect(true);
    }

    private boolean singleConnect(boolean bNextadd) {
        logStep("singleConnect" + (bNextadd ? " NextAdd" : "") + (firstUsed ? " isFirst" : ""));
        boolean bStart = false;
        if (poolAddressCount() < 1)
            return bStart;
        tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = poolAddress(bNextadd);
        int socketTimeout = singletimeout;
        if (step == tztZFSocketPoolStepSingleTrying && linkType != tztZFLinkType.tztZFLinkType_JH.value() && linkType < tztZFLinkType.tztZFLinkType_Short.value())
            socketTimeout = 3000;
        tztZFHostport hostport = hostportIpv4Ipv6.getHostport1();
        if (hostport != null && hostport.isValid()) {
            tztZFGCDAsyncSocket sock = createSocket(hostport);
            if (sock != null) {
                tztZFHostportIpv4Ipv6 usedHostPortIpv4ipv6 = hostportIpv4Ipv6.withUse(1);
                sock.setUserData(usedHostPortIpv4ipv6);
                singleSock1 = sock;
                boolean connnect = sock.Connect(linkType, hostport.getHost(), hostport.getPort(), socketTimeout);
                bStart = bStart || connnect;
                if (!connnect) {
                    socketCloseSingleFail(sock);
                    tztlog("[AsyncSocket]%d connect host:%s,fail", sock.hashCode(), hostport.toString());
                } else {
                    synchronized (socketQueue) {
                        tztlog("[AsyncSocket]%d connect host:%s", sock.hashCode(), hostport.toString());
                        if (poolBlock != null) {
                            poolBlock.didConnecting(sock, usedHostPortIpv4ipv6);
                        }
                    }
                }
            }
        }

        hostport = hostportIpv4Ipv6.getHostport2();
        if (hostport != null && hostport.isValid()) {
            tztZFGCDAsyncSocket sock = createSocket(hostport);
            if (sock != null) {
                tztZFHostportIpv4Ipv6 usedHostPortIpv4ipv6 = hostportIpv4Ipv6.withUse(2);
                sock.setUserData(usedHostPortIpv4ipv6);
                singleSock2 = sock;
                boolean connnect = sock.Connect(linkType, hostport.getHost(), hostport.getPort(), socketTimeout);
                bStart = bStart || connnect;
                if (!connnect) {
                    socketCloseSingleFail(sock);
                    tztlog("connect_sock:%d,host:%s,fail", sock.hashCode(), hostport.toString());
                } else {
                    synchronized (socketQueue) {
                        tztlog("connect_sock:%d,host:%s", sock.hashCode(), hostport.toString());
                        if (poolBlock != null) {
                            poolBlock.didConnecting(sock, usedHostPortIpv4ipv6);
                        }
                    }
                }
            }
        }
        return bStart;
    }

    private void singleConnectTry() {
        step = tztZFSocketPoolStepSingleTrying;
        singleConnect(nextAddress);
        nextAddress = false;
    }

    private void compoundConnectTry() {
        step = tztZFSocketPoolStepCompoundTrying;
        compoundAddress();
        compoundConnect();
    }

    private void arrayGroups(CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayList, int each, CopyOnWriteArrayList<CopyOnWriteArrayList<tztZFHostportIpv4Ipv6>> ayGroup) {
        if (ayGroup == null || ayList == null || ayList.size() < 1 || each < 1)
            return;
        if (ayList.size() < each) {
            ayGroup.add(ayList);
            return;
        }
        CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayItem = new CopyOnWriteArrayList<>();
        for (int i = 0; i < ayList.size(); i++) {
            ayItem.add(ayList.get(i));
            if (ayItem.size() == each) {
                ayGroup.add(ayItem);
                ayItem = new CopyOnWriteArrayList<>();
            }
        }
        if (ayItem.size() > 0) {
            ayGroup.add(ayItem);
        }
    }

    private void compoundAddress() {
        if (!iscompound)
            return;
        ayCompoundAddrs.clear();
        if (firstUsed) {
            if (firstAddressCount() > 0)
                arrayGroups(ayFirstAddress, kParallelNum, ayCompoundAddrs);
        } else if (ayAddress.size() > 0) {
            arrayGroups(ayAddress, kParallelNum, ayCompoundAddrs);
        }
        compoundIndex = 0;
        iscompound = false;
    }

    private void compoundConnect() {
        tztlog("CompoundCount:%d", ayCompoundAddrs.size());
        logStep("compoundConnect");
        if (ayCompoundAddrs.size() < 1)
            return;
        cancelAllSocketsWithConnected(null);
        int nCompounds = ayCompoundAddrs.size();
        if (nCompounds > 1 && compoundTimer == null) {
            compoundIndex = 0;
            ///注意start应为0,立即发起第一组连接。
            compoundTimer = new tztZFTimer(0, kStepGap, true, new tztZFTimerDelegate() {
                @Override
                public void fire(tztZFTimer timer) {
                    compoundAction();
                }
            });
        }
        // 进行第一批的连接！
        compoundIndex = 0;
        compoundAction();
    }

    private void compoundAction() {
        logStep("compoundAction");
        int nCount = ayCompoundAddrs.size();
        if (compoundIndex == nCount - 1) {
            if (compoundTimer != null) {
                compoundTimer.invalidate();
                compoundTimer = null;
            }
            if (compoundFinishTimer == null) {
                compoundFinishTimer = new tztZFTimer(kFinishTimeout, kFinishTimeout, false, new tztZFTimerDelegate() {
                    @Override
                    public void fire(tztZFTimer timer) {
                        compoundFinished();
                    }
                });
            }
        }
        if (compoundIndex < 0 || compoundIndex >= nCount)
            return;
        CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayAddList = ayCompoundAddrs.get(compoundIndex);
        startCompound(ayAddList, compoundIndex, nCount);
        compoundIndex++;
    }

    private boolean startCompound(CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayAddress, int index, int count) {
        tztlog("startCompound:index%d/%d addrs:%d", index, count, (ayAddress != null ? ayAddress.size() : 0));
        boolean bStart = false;
        if (ayAddress == null)
            return bStart;
        for (tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 : ayAddress) {
            tztZFHostport hostport = hostportIpv4Ipv6.getHostport1();
            if (hostport != null && hostport.isValid()) {
                tztZFGCDAsyncSocket sock = createSocket(hostport);
                if (sock != null) {
                    tztZFHostportIpv4Ipv6 usedhostport = hostportIpv4Ipv6.withUse(1);
                    sock.setUserData(usedhostport);
                    aySock.add(sock);
                    tztlog("[AsyncSocket]%d compoundconnect host:", sock.hashCode(), hostport.toString());
                    boolean connect = sock.Connect(linkType, hostport.getHost(), hostport.getPort(), 3);
                    bStart = connect || bStart;
                    if (!connect) {
                        poolSocketRemove(sock, String.format("compoundconnect err:%s", hostport.toString()));
                        tztlog("[AsyncSocket]%d compoundconnect host:%s,fail", sock.hashCode(), hostport.toString());
                    } else {
                        synchronized (socketQueue) {
                            if (poolBlock != null) {
                                poolBlock.didConnecting(sock, usedhostport);
                            }
                        }
                    }
                }
            }
            hostport = hostportIpv4Ipv6.getHostport2();
            if (hostport != null && hostport.isValid()) {
                tztZFGCDAsyncSocket sock = createSocket(hostport);
                if (sock != null) {
                    tztZFHostportIpv4Ipv6 usedhostport = hostportIpv4Ipv6.withUse(2);
                    sock.setUserData(usedhostport);
                    aySock.add(sock);
                    tztlog("[AsyncSocket]%d compoundconnect host:", sock.hashCode(), hostport.toString());
                    boolean connect = sock.Connect(linkType, hostport.getHost(), hostport.getPort(), 3);
                    bStart = connect || bStart;
                    if (!connect) {
                        poolSocketRemove(sock, String.format("compoundconnect err:%s", hostport.toString()));
                        tztlog("[AsyncSocket]%d compoundconnect host:%s,fail", sock.hashCode(), hostport.toString());
                    } else {
                        synchronized (socketQueue) {
                            if (poolBlock != null) {
                                poolBlock.didConnecting(sock, usedhostport);
                            }
                        }
                    }
                }
            }
        }
        return bStart;
    }

    private void compoundFinished() {
        if (compoundFinishTimer != null) {
            compoundFinishTimer.invalidate();
            compoundFinishTimer = null;
        }

        if (step != tztZFSocketPoolStepConnected && step != tztZFSocketPoolStepConnectUsed) {
            if (firstUsed) {
                endFirstConnectTimeout();
            } else if (step == tztZFSocketPoolStepCompoundTrying) {
                cooldownConnectTry();
            }
        }
    }

    private void cooldownConnectTry() {
        step = tztZFSocketPoolStepCooldownTrying;
        cooldownConnectTimes = 0;
        cooldownStartTimeStamp = System.currentTimeMillis();
        if (poolAddressCount() < 1)
            return;
        cooldownConnectAction();
    }

    private void cooldownReconnectWithStrategy() {
        if (isConnMaxFail("cooldownConnect")) {
            synchronized (socketQueue) {
                if (poolBlock != null) {
                    if (firstUsed) {
                        poolBlock.didFirstConnectTimeOut(null);
                    } else {
                        poolBlock.didConnectTimeOut(null);
                    }
                }
                poolSocketInit("cooldownReconnectWithStrategy");
            }
            return;
        }
        int delaySeconds = 0;
        if (cooldownConnectTimes == 0) {
            delaySeconds = 0;
        } else if (cooldownConnectTimes < 5) {
            delaySeconds = (int) (Math.pow(2.0, cooldownConnectTimes - 1)) * 1000;
        } else {
            delaySeconds = kCoolDownMaxDelay;
        }
        cooldownConnectTimes++;
        if (cooldownTimer == null) {
            cooldownTimer = new tztZFTimer(delaySeconds, delaySeconds, false, new tztZFTimerDelegate() {
                @Override
                public void fire(tztZFTimer timer) {
                    cooldownConnectAction();
                }
            });
        }
    }

    private void cancelAllSocketsWithConnected(tztZFGCDAsyncSocket sock) {
        if (aySock.size() < 1)
            return;
        if (sock != null)
            aySock.remove(sock);
        for (tztZFGCDAsyncSocket socket : aySock) {
            tztlog("[AsyncSocket]%d cancelCompoundSocket cancel host:%s", socket.hashCode(), socket.getUserData().toString());
            socket.setDelegate(null);
            socket.disconnect("other socket");
        }
        aySock.clear();
    }

    private void startFirstConnectTimeout() {
        if (firsttimeout > 0 && firstconnectTimer == null) {
            tztlog("startFirstConnectTimeout timeout:%d", firsttimeout);
            firstconnectTimer = new tztZFTimer(firsttimeout, firsttimeout, false, new tztZFTimerDelegate() {
                @Override
                public void fire(tztZFTimer timer) {
                    endFirstConnectTimeout();
                }
            });
        }
    }

    private void endFirstConnectTimeout() {
        if (firstconnectTimer != null) {
            firstconnectTimer.invalidate();
            firstconnectTimer = null;
            if (step == tztZFSocketPoolStepConnectUsed || !firstUsed)
                return;
            synchronized (socketQueue) {
                if (step == tztZFSocketPoolStepConnected || !firstUsed)
                    return;
                if (poolBlock != null) {
                    poolBlock.didFirstConnectTimeOut(null);
                    poolBlock = null;
                }
                poolSocketInit("endFirstConnectTimeout");
            }
        }
    }

    private void startConnectTimeout() {
        if (timeout > 0 && connectTimer == null) {
            tztlog("startConnectTimeout timeout:%d", timeout);
            connectTimer = new tztZFTimer(timeout, timeout, false, new tztZFTimerDelegate() {
                @Override
                public void fire(tztZFTimer timer) {
                    endConnectTimeout();
                }
            });
        }
    }

    private void endConnectTimeout() {
        if (connectTimer != null) {
            connectTimer.invalidate();
            connectTimer = null;
        }
        if (step == tztZFSocketPoolStepConnectUsed || firstUsed)
            return;
        synchronized (socketQueue) {
            if (step == tztZFSocketPoolStepConnected)
                return;
            if (poolBlock != null) {
                poolBlock.didConnectTimeOut(null);
                poolBlock = null;
            }
            poolSocketInit("endConnectTimeout");
        }
    }

    public boolean isVaild() {
        return poolAddressCount() > 0;
    }

    private int poolAddressCount() {
        if (firstUsed) {
            return firstAddressCount();
        }
        return ayAddress.size();
    }

    private int firstAddressCount() {
        return (firstUsed ? ayFirstAddress.size() : 0);
    }

    public void poolConnect(tztZFSocketPoolDelegate delegate) {
        poolBlock = delegate;
        if (poolAddressCount() < 1)
            return;
        if (!firstUsed && poolAddressCount() == 1) {
            poolSingleConnect(delegate);
            return;
        }
        logStep("poolConnect");
        if (step == tztZFSocketPoolStepInitial || step == tztZFSocketPoolStepCooldownTrying) {
            if (cooldownTimer != null) {
                cooldownTimer.invalidate();
                cooldownTimer = null;
            }
            if (firstUsed) {
                startFirstConnectTimeout();
            } else {
                startConnectTimeout();
            }
            singleConnectTry();
        }
    }

    public void poolSingleConnect(tztZFSocketPoolDelegate delegate) {
        poolBlock = delegate;
        if (poolAddressCount() < 1)
            return;
        if (step == tztZFSocketPoolStepInitial || step == tztZFSocketPoolStepCooldownTrying) {
            step = tztZFSocketPoolStepSingleConnect;
            if (cooldownTimer != null) {
                cooldownTimer.invalidate();
                cooldownTimer = null;
            }
            singleConnect(false);
            return;
        }
        ///单连状态、不存在连接
        if (step == tztZFSocketPoolStepSingleConnect && singleSock1 == null && singleSock2 == null) {
            singleConnect(false);
        }
        ///连接状态、不存在连接
        if (isTakeStep()) {
            if (usedSock == null) {
                step = tztZFSocketPoolStepSingleConnect;
                singleConnect(false);
                return;
            }
        }
    }

    public void poolConnectWithHost(tztZFHostportIpv4Ipv6 hostport, tztZFSocketPoolDelegate delegate) {
        if (hostport != null && hostport.isValid()) {
            if (!ayAddress.contains(hostport)) {
                ayAddress.add(0, hostport);
                singleIndex = 0;
                nextAddress = false;
                iscompound = true;
                compoundAddress();
            } else {
                singleIndex = ayAddress.indexOf(hostport);
            }
        }
        poolConnect(delegate);
    }

    private boolean isFirstOfObject(tztZFHostportIpv4Ipv6 hostport) {
        if (firstAddressCount() < 1 || hostport == null || hostport.isInvalid())
            return false;
        tztZFHostportIpv4Ipv6 firstObj = ayFirstAddress.get(0);
        if (firstObj.getConnecttype() == hostport.getConnecttype())
            return true;
        return false;
    }

    private int indexOfObjectForList(tztZFHostportIpv4Ipv6 hostport, CopyOnWriteArrayList<tztZFHostportIpv4Ipv6> ayHost) {
        if (hostport == null || hostport.isInvalid() || ayHost == null || ayHost.size() < 1)
            return -1;
        tztZFHostport usedhost = hostport.usedHostport();
        if (usedhost == null)
            return -1;
        int findIndex = 0;
        for (tztZFHostportIpv4Ipv6 obj : ayHost
        ) {
            if (obj.isEqualHost(hostport)) {
                return findIndex;
            }
            findIndex++;
        }
        return -1;
    }

    public boolean isConnMaxFail(String tip) {
        int failMax = connMaxFail;
        if (failMax <= 0) {
            int addCount = poolAddressCount();
            failMax = ((addCount > 2) ? addCount : addCount * 2);
        }
        boolean bRet = connectFail > failMax;
        tztlog("connectFail:%d/%d %s", connectFail, failMax, tip);
        if (bRet) {
            connectFail = 0;
        }
        return bRet;
    }

    public void poolResetConn(int maxFail) {
        if (maxFail > 0)
            connMaxFail = maxFail;
        connectFail = 0;
    }

    public void poolSelectedAddress(tztZFHostportIpv4Ipv6 hostport) {
        if (hostport == null || hostport.isInvalid())
            return;
        tztZFHostport usedhost = hostport.usedHostport();
        if (usedhost == null)
            return;
        nextAddress = false;
        if (isFirstOfObject(hostport)) {
            firstAddress = hostport;
            firstIndex = indexOfObjectForList(hostport, ayFirstAddress);
        } else {
            singleAddress = hostport;
            singleIndex = indexOfObjectForList(hostport, ayAddress);
        }
        connectFail = 0;
    }

    public void poolMoveLastAddress(tztZFHostportIpv4Ipv6 hostport) {
        if (hostport == null || hostport.isInvalid())
            return;
        tztZFHostport usedhost = hostport.usedHostport();
        if (usedhost == null)
            return;
        int moveIndex = -1;
        if (isFirstOfObject(hostport)) {
            moveIndex = indexOfObjectForList(hostport, ayFirstAddress);
            if (moveIndex >= 0 && moveIndex < ayFirstAddress.size()) {
                ayFirstAddress.remove(hostport);
                ayFirstAddress.add(hostport);
                if (firstIndex > moveIndex)
                    firstIndex--;
                iscompound = true;
            }
        } else {
            moveIndex = indexOfObjectForList(hostport, ayAddress);
            if (moveIndex >= 0 && moveIndex < ayAddress.size()) {
                ayAddress.remove(hostport);
                ayAddress.add(hostport);
                if (singleIndex > moveIndex)
                    singleIndex--;
                iscompound = true;
            }
        }
        connectFail++;
        tztlog("poolMoveLastAddress moveIndex:%d", moveIndex);
    }

    public void poolFirstUsed(boolean isUsed) {
        if (!isUsed) {
            firstIndex = 0;
        }
        if (firstUsed != isUsed) {
            firstUsed = isUsed;
            iscompound = true;
            connectFail = 0;
            poolClose("change firstused");
        }
        compoundAddress();
    }

    public void poolNextAddress(String tip) {
        nextAddress = true;
        tztlog("poolNextAddress :%s", tip);
    }

    private tztZFHostportIpv4Ipv6 poolAddressFirst(boolean bnextAdd) {
        logStep("poolAddressFirst");
        if (ayFirstAddress.size() < 1)
            return null;
        if (bnextAdd) {
            firstIndex++;
        }
        if (firstIndex < 0 || firstIndex >= ayFirstAddress.size()) {
            firstIndex = 0;
        }
        return ayFirstAddress.get(firstIndex);
    }

    private tztZFHostportIpv4Ipv6 poolAddressSingle(boolean bnextAdd) {
        logStep("poolAddressSingle");
        if (ayAddress.size() < 1)
            return null;
        if (bnextAdd) {
            singleIndex++;
        }
        if (singleIndex < 0 || singleIndex >= ayAddress.size()) {
            singleIndex = 0;
        }
        return ayAddress.get(singleIndex);
    }

    public tztZFHostportIpv4Ipv6 poolAddress(boolean bnextAdd) {
        logStep("poolAddress");
        tztZFHostportIpv4Ipv6 host = null;
        if (firstUsed) {
            host = poolAddressFirst(bnextAdd);
        } else {
            host = poolAddressSingle(bnextAdd);
        }
        tztlog("get_hostport:%s", (host != null ? host.toString() : ""));
        return host;
    }

    /**
     * 关闭所有连接
     */
    public boolean poolClose(String msg) {
        tztlog("poolClose %s", msg);
        poolSocketInit(msg);
        usedSock = null;

        socketReset();

        for (tztZFGCDAsyncSocket sock : aySock) {
            sock.setDelegate(null);
            sock.disconnect("poolClose:" + msg);
        }
        aySock.clear();

        int failmax = connMaxFail;
        if (failmax <= 0) {
            int addCount = poolAddressCount();
            failmax = (addCount > 2 ? addCount : addCount * 2);
        }
        boolean bConnectFail = connectFail > failmax;
        tztlog("connectFail:%d/%d %s", connectFail, failmax, msg);
        if (bConnectFail) {
            connectFail = 0;
        }
        return bConnectFail;
    }

    /**
     * 结束尝试
     */
    public void poolCooldownTryingEnd() {
        if (step == tztZFSocketPoolStepCooldownTrying) {
            poolSocketInit("");
        }
    }

    /**
     * 断开除Socket以外所有其他连接
     *
     * @param socket socket
     */
    private void socketCancelWith(tztZFGCDAsyncSocket socket) {
        if (aySock == null || aySock.size() <= 0 || socket == null) {
            return;
        }
        for (tztZFGCDAsyncSocket sock : aySock) {
            if (sock == socket) return;
            tztlog("[AsyncSocket]%d disconnect host:%s", sock.hashCode(), sock.getUserData());
            sock.disconnect("socketCancelWith");
        }
    }

    /**
     * 重置并关闭所有定时器
     */
    private void socketReset() {
        if (singleSock1 != null) {
            singleSock1 = null;
        }

        if (singleSock2 != null) {
            singleSock2 = null;
        }

        if (connectTimer != null) {
            connectTimer.invalidate();
            connectTimer = null;
        }

        if (firstconnectTimer != null) {
            firstconnectTimer.invalidate();
            firstconnectTimer = null;
        }

        if (compoundTimer != null) {
            compoundTimer.invalidate();
            compoundTimer = null;
        }

        if (compoundFinishTimer != null) {
            compoundFinishTimer.invalidate();
            compoundFinishTimer = null;
        }

        if (cooldownTimer != null) {
            cooldownTimer.invalidate();
            cooldownTimer = null;
        }
    }

    private void socketCloseSingleFail(tztZFGCDAsyncSocket sock) {
        if (sock == singleSock1) {
            singleSock1 = null;
        } else if (sock == singleSock2) {
            singleSock2 = null;
        }
    }

    //#pragma mark - tztZFSocketPool poolSocket
    public void poolSocketUsed(tztZFGCDAsyncSocket sock, tztZFHostportIpv4Ipv6 hostport) {
        if (sock != null) {
            tztlog("[AsyncSocket]%d used host:%s", sock.hashCode(), sock.getUserData());
        }
        if (sock == null || hostport == null) return;
        tztZFHostport usedhost = hostport.usedHostport();
        if (usedhost == null) return;
        connectFail = 0;
        usedSock = sock;
        step = tztZFSocketPoolStepConnectUsed;
        nextAddress = false;
        socketCancelWith(sock);
        if (isFirstOfObject(hostport)) {
            firstAddress = hostport;
            firstIndex = ayFirstAddress.indexOf(hostport);
        } else {
            singleAddress = hostport;
            singleIndex = ayAddress.indexOf(hostport);
        }
        socketReset();
    }

    private boolean isConnectedStep() {
        return (step == tztZFSocketPoolStepConnected);
    }

    private boolean isUsedStep() {
        return (step == tztZFSocketPoolStepConnectUsed);
    }

    private boolean isTakeStep() {
        return (isConnectedStep() || isUsedStep());
    }

    private void poolSocketInit(String msg) {
        logStep(msg);
        step = tztZFSocketPoolStepInitial;
        compoundIndex = 0;
    }

    public void poolSocketClose(tztZFGCDAsyncSocket sock, String msg) {
        if (sock != null) {
            tztlog("[AsyncSocket]%d close host:%s msg:%s", sock.hashCode(), sock.getUserData(), msg);
        }
        logStep(String.format("%s %s", "poolSocketClose", msg));
        boolean isConnected = isConnectedStep();
        boolean isUsedSock = (sock != null && sock == usedSock) || usedSock == null;
        poolSocketRemove(sock, msg);
        if (isUsedSock) {
            poolSocketInit(msg);
            ///连接过程
            if (isConnected) {
                ///判断最大连接失败
                connectFail++;
                if (isConnMaxFail(msg)) {
                    if (firstUsed) {
                        endFirstConnectTimeout();
                        return;
                    } else {
                        endConnectTimeout();
                        return;
                    }
                }
                nextAddress = true;
            }
        }
    }

    public void poolSocketRemove(tztZFGCDAsyncSocket socket, String msg) {

        if (socket == null)
            return;
        socket.setDelegate(null);
        socketCloseSingleFail(socket);
        if (socket == usedSock) {
            usedSock = null;
        }
        aySock.remove(socket);
    }

    //#pragma mark - tztZFGCDAsyncSocket delegates
    private void socketConnected(tztZFGCDAsyncSocket sock, boolean isSSL) {
        synchronized (socketQueue) {
            ///已连接成功
            if (step == tztZFSocketPoolStepConnected || step == tztZFSocketPoolStepConnectUsed) {
                if (sock != usedSock) {
                    tztlog("[AsyncSocket]%d disconnect other socket,[usedAsyncSocket]%d", sock.hashCode(), usedSock.hashCode());
                    sock.disconnect("disconnect other socket");
                }
                return;
            }
            if (singleSock1 != null && singleSock1 != sock) {
                singleSock1.setDelegate(null);
                singleSock1 = null;
            }
            if (singleSock2 != null && singleSock2 != sock) {
                singleSock2.setDelegate(null);
                singleSock2 = null;
            }
            tztZFHostportIpv4Ipv6 hostportIpv4Ipv6 = sock.getUserData();
            tztlog("[AsyncSocket]%d didConnect %s", sock.hashCode(), hostportIpv4Ipv6.toString());
            if (poolBlock != null) {
                int nFlag = poolBlock.didConnected(sock, hostportIpv4Ipv6, isSSL);
                if (nFlag == tztZFSocketSSLFlag_Socket) {
                    return;
                }
                if (nFlag == tztZFSocketSSLFlag_Disconnect ||
                        nFlag == tztZFSocketSSLFlag_Close) {
                    tztlog("[AsyncSocket]%d disconnect poolBlock %d", sock.hashCode(), nFlag);
                    sock.disconnect("poolBlock:" + nFlag);
                    return;
                }
                ///tztZFSocket接管
                if (nFlag == tztZFSocketSSLFlag_OKAndTaken) {
                    if (step != tztZFSocketPoolStepConnectUsed) {
                        usedSock = sock;
                        step = tztZFSocketPoolStepConnected;
                    } else if (usedSock != sock) {
                        tztlog("[AsyncSocket]%d disconnect poolBlock %d,usedSocket[AsyncSocket]%d", sock.hashCode(), nFlag, usedSock.hashCode());
                        sock.disconnect("poolBlock:" + nFlag);
                        return;
                    }
                }
            }
            singleSock1 = null;
            singleSock2 = null;
        }
    }

    private boolean isSingleFail(tztZFGCDAsyncSocket sock) {
        if (sock == singleSock1 && singleSock2 == null)
            return true;
        if (sock == singleSock2 && singleSock1 == null)
            return true;
        return false;
    }

    private void socketDisConnect(tztZFGCDAsyncSocket sock, String msg) {
        tztlog("[AsyncSocket]%d disconnect host:%s,msg:%s", sock.hashCode(), sock.getUserData().toString(), msg);
        ///启用状态
        if (isTakeStep() && sock != null && sock == usedSock) {
            poolSocketClose(sock, msg);
            return;
        }
        boolean bSingleFail = isSingleFail(sock);
        poolSocketRemove(sock, String.format("tztZFSocketPool.DisConnect Err:%s", msg));
        if (bSingleFail && step == tztZFSocketPoolStepCooldownTrying) {
            long timestamp = System.currentTimeMillis();
            connectFail++;
            //仍处在冷静期内
            if (timestamp - cooldownStartTimeStamp < kCoolDownSecs) {
                cooldownReconnectWithStrategy();
                return;
            } else {
                singleConnectTry();
            }
        }
        //单连失败
        if (bSingleFail && step == tztZFSocketPoolStepSingleTrying) {
            connectFail++;
            poolNextAddress("singleConnectTry fail");
            compoundConnectTry();
            return;
        }
        //单连
        if (bSingleFail && step == tztZFSocketPoolStepSingleConnect) {
            if (firstUsed) {
                endFirstConnectTimeout();
            } else {
                endConnectTimeout();
            }
            return;
        }
        //并发连接失败，不需要处理，并发结束时会启动冷静连
        if (step == tztZFSocketPoolStepCompoundTrying) {
            sock.setDelegate(null);
            aySock.remove(sock);
            if (compoundIndex == ayCompoundAddrs.size() && aySock.size() < 1) {
                compoundFinished();
            }
            return;
        }
    }

    private void tztlog(String format, Object... args) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Debug, format, args);
    }

    private void tztlogI(String format, Object... args) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Info, format, args);
    }

    private void tztlog(tztZFLogLevel level, String format, Object... args) {
        if (tztZFLog.getLevel() <= level.value()) {
            String cTag = String.format(Locale.getDefault(), "[%d]tztZFSocketPool[%d]",
                    linkType, hashCode());
            tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), cTag, format, args);
        }
    }
}
