package com.tztzf.network.socket.Netty;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.socket.tztZFSocketSendData;

import java.util.concurrent.CopyOnWriteArrayList;

import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;

public class tztZFSocketPushHQ {
    private static final String DEFGrid = "Grid";
    private Object userData = null;
    private tztZFPushRespDelegate delegate;
    private CopyOnWriteArrayList<tztZFSocketSendData> reqList = new CopyOnWriteArrayList<>();
    private tztZFDataMapBase<String, tztZFSocketSendData> recList = new tztZFDataMapBase<>();

    public tztZFSocketPushHQ(tztZFPushRespDelegate delegate) {
        initData();
        this.delegate = delegate;
    }

    private void initData() {

    }

    public boolean have() {
        return reqList.size() > 0;
    }

    public void clearReq() {
        reqList.clear();
    }

    public void clearRec() {
        recList.removeAllObjects();
    }

    public void clearRecWithType(final tztZFRecvType type) {
        recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                tztZFDataMapBase resp = new tztZFDataMapBase();
                resp.setInt("ErrorNo", tztZFNetWorkError_Send);
                String strMsg = String.format("行情主推Group[%s]订阅已被取消", key);
                resp.setStr("ErrorMessage", strMsg);
                responseTypePush(resp, type, (tztZFSocketSendData) value);
                return false;
            }
        });
    }

    public void reSendReq(CopyOnWriteArrayList<tztZFSocketSendData> ayReSend) {
        if (ayReSend == null || ayReSend.size() < 1)
            return;
        for (tztZFSocketSendData push : reqList
        ) {
            if (push != null) {
                push.setAddTime();
                ayReSend.add(push);
            }
        }
    }

    private void clear() {
        reqList.clear();
        recList.removeAllObjects();
    }

    public Object getUserData() {
        return userData;
    }

    public void setUserData(Object userData) {
        this.userData = userData;
    }

    //20253 行情主推 订阅和取消
    public static boolean isHQPushAction(int nAction) {
        return nAction == 20253;
    }

    //是行情主推订阅请求
    private static boolean isHQPushTake(tztZFSocketSendData push) {
        if (push == null)
            return false;
        if (!isHQPushAction(push.getAction()))
            return false;
        String strGrid = push.getStr(DEFGrid);
        return !tztZFPublicFun.isEmpty(strGrid);
    }

    //是行情主推取消请求
    private static boolean isHQPushRemove(tztZFSocketSendData push) {
        if (push == null)
            return false;
        if (!isHQPushAction(push.getAction()))
            return false;
        String strGrid = push.getStr(DEFGrid);
        return tztZFPublicFun.isEmpty(strGrid);
    }

    //#pragma mark -主推请求加入队列
    public void addReqPush(tztZFSocketSendData push) {
        if (push == null || !isHQPushAction(push.getAction()))
            return;
        if (isHQPushRemove(push)) {
            removeGroup(push.getStr("Group"));
        }
        if (isHQPushTake(push)) {
            String strGroup = push.getStr("Group");
            int group = tztZFPublicFun.parseInt(strGroup, -1);
            //兼容2.0和重构版本 之前老版本发送没有带上group参数 导致使用新的底层库后 没有group直接移除了 应答回来也回调不了
            if (group == -1) {
                push.setStr("Group", "0");
            }
        }
    }

    private void removeGroup(String strGroup) {
        removeGroupPushTake(strGroup, null, false);
    }

    // -清除行情主推
    private void removeGroupPushTake(String strGroup, tztZFSocketSendData push, boolean bTake) {
        removeGroupRec(strGroup, push, bTake);
        removeGroupReq(strGroup, push, bTake);
    }

    //清除行情主推应答队列
    private void removeGroupRec(String group, final tztZFSocketSendData push, boolean bTake) {
        boolean bClear = tztZFPublicFun.isEmpty(group);
        //group为空，取消所有行情主推订阅
        if (bClear) {
            recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                @Override
                public boolean enumerate(String lowKey, String key, Object value) {
                    tztZFSocketSendData sendData = (tztZFSocketSendData) value;
                    if (sendData.equals(push))
                        return false;
                    respGroupRemovePush(key, sendData);
                    return false;
                }
            });
            recList.removeAllObjects();
        } else {
            //取消对应的Group订阅
            tztZFSocketSendData req = recList.getObj(group);
            if (req != null && !req.equals(push)) {
                respGroupRemovePush(group, req);
            }
            recList.removeByKey(group);
        }
        if (bTake && push != null && !tztZFPublicFun.isEmpty(group)) {
            push.setPushKey(group);
            recList.setObj(group, push);
        }
    }

    //清除行情主推请求队列
    private void removeGroupReq(String group, tztZFSocketSendData push, boolean bTake) {
        if (tztZFPublicFun.isEmpty(group)) {
            reqList.clear();
        } else {
            for (tztZFSocketSendData req : reqList) {
                if (req != null && group.equalsIgnoreCase(req.getStr("Group"))) {
                    reqList.remove(req);
                }
            }
        }
        if (bTake && push != null) {
            reqList.add(push);
        }
    }

    //根据tag查找主推请求
    public tztZFSocketSendData findPushWithtag(final int recvTag) {
        final tztZFSocketSendData[] push = {null};
        recList.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                tztZFSocketSendData sendData = (tztZFSocketSendData) value;
                if (sendData != null && sendData.getSendTag() == recvTag) {
                    push[0] = sendData;
                    return true;
                }
                return false;
            }
        });
        return push[0];
    }

    //    #pragma mark -根据应答查找主推请求
    private tztZFSocketSendData findPushWithRecv(tztZFDataMapBase recvMap, int recvTag) {
        if (recvMap == null || recvMap.getCount() < 1)
            return null;
        int nAction = recvMap.getAction();
        if (!isHQPushAction(nAction))
            return null;
        return findPushWithtag(recvTag);
    }

    //serviceCode服务主推订阅/退订应答处理
    public void dealRecvPush(tztZFDataMapBase recvMap, tztZFSocketSendData push) {
        if (push == null)
            return;
        if (recvMap == null || recvMap.getInt("ErrorNo", -1) < 0)
            return;
        int nAtcion = recvMap.getAction();
        if (!isHQPushAction(nAtcion))
            return;
        String strGroup = push.getStr("Group");
        if (isHQPushRemove(push)) {
            removeGroup(strGroup);
        } else if (isHQPushTake(push)) {
            if (tztZFPublicFun.isEmpty(strGroup))
                return;
            push.setPushKey(strGroup);
            removeGroupPushTake(strGroup, push, true);
        }
    }

    private void respGroupRemovePush(String group, tztZFSocketSendData push) {
        if (push == null || tztZFPublicFun.isEmpty(group))
            return;
        tztZFDataMapBase resp = new tztZFDataMapBase();
        resp.setInt("ErrorNo", -1);
        String strMsg = String.format("行情主推Group[%s]订阅已被取消", group);
        resp.setStr("ErrorMessage", strMsg);
        responseTypePush(resp, tztZFRecvType.tztZFRecv_DataOK, push);
    }

    private void responseTypePush(tztZFDataMapBase resp, tztZFRecvType type, tztZFSocketSendData push) {
        if (delegate != null) {
            delegate.resp(push, type, resp);
        }
    }
}
