package com.tztzf.network.socket.gmhttp;

import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Locale;

public class tztZFGMRequestItem {
    private int socketID;
    private Request request;
    private Response response;

    private ByteArrayOutputStream recv_stream = new ByteArrayOutputStream(255);
    private int headLen;
    private int contentLen;

    private tztZFGMURLSessionCompletion completionHandler;

    private long timeOut;
    private long sendDate;
    private boolean isLoginRequest;

    public int getSocketID() {
        return socketID;
    }

    public void setSocketID(int socketID) {
        this.socketID = socketID;
    }

    public Request getRequest() {
        return request;
    }

    public void setRequest(Request request) {
        this.request = request;
    }

    public int getHeadLen() {
        return headLen;
    }

    public void setHeadLen(int headLen) {
        this.headLen = headLen;
    }

    public int getContentLen() {
        return contentLen;
    }

    public void setContentLen(int contentLen) {
        this.contentLen = contentLen;
    }

    public tztZFGMURLSessionCompletion getCompletionHandler() {
        return completionHandler;
    }

    public void setCompletionHandler(tztZFGMURLSessionCompletion completionHandler) {
        this.completionHandler = completionHandler;
    }

    public tztZFGMRequestItem() {
        sendDate = System.currentTimeMillis();
    }

    public long getTimeOut() {
        return timeOut;
    }

    public void setTimeOut(long timeOut) {
        this.timeOut = timeOut;
    }

    public long getSendDate() {
        return sendDate;
    }

    public void setSendDate(long sendDate) {
        this.sendDate = sendDate;
    }

    public boolean isLoginRequest() {
        return isLoginRequest;
    }

    public void setIsLoginRequest(boolean loginRequest) {
        isLoginRequest = loginRequest;
    }

    public void appendRecvData(byte[] data) {
        if (data == null)
            return;
        recv_stream.write(data, 0, data.length);
    }

    public long getRecvDataLen() {
        return recv_stream.toByteArray().length;
    }

    public void appendHTTPBytes(byte[] data, int len) {

    }

    public void addSendTime() {
        sendDate = System.currentTimeMillis();
    }

    public boolean isTimeOut() {
        if (timeOut > 0 && (System.currentTimeMillis() - sendDate) > timeOut) {
            return true;
        }
        return false;
    }


    private int findHeaderEnd(final byte[] buf, int rlen) {
        int splitbyte = 0;
        while (splitbyte + 1 < rlen) {

            // RFC2616
            if (buf[splitbyte] == '\r' && buf[splitbyte + 1] == '\n' && splitbyte + 3 < rlen && buf[splitbyte + 2] == '\r' && buf[splitbyte + 3] == '\n') {
                return splitbyte + 4;
            }

            // tolerance
            if (buf[splitbyte] == '\n' && buf[splitbyte + 1] == '\n') {
                return splitbyte + 2;
            }
            splitbyte++;
        }
        return 0;
    }

    private int decodeHeader(BufferedReader in, tztZFDataMapBase<String, String> headers) {
        int responseCode = -1;
        try {
            // Read the request line
            String line = in.readLine();
            String responseMessage = "";
            if (line != null && line.startsWith("HTTP/1.")) {
                int codePos = line.indexOf(' ');
                if (codePos > 0) {
                    int phrasePos = line.indexOf(' ', codePos + 1);
                    if (phrasePos > 0 && phrasePos < line.length()) {
                        responseMessage = line.substring(phrasePos + 1);
                    }
                    // deviation from RFC 2616 - don't reject status line
                    // if SP Reason-Phrase is not included.
                    if (phrasePos < 0)
                        phrasePos = line.length();
                    try {
                        responseCode = Integer.parseInt
                                (line.substring(codePos + 1, phrasePos));
                    } catch (NumberFormatException e) {
                        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "GMHTTP", "%s", tztZFLog.getStackTraceString(e));
                    }
                }
            }
            while (line != null && !line.trim().isEmpty()) {
                int p = line.indexOf(':');
                if (p >= 0) {
                    headers.setStr(line.substring(0, p).trim().toLowerCase(Locale.US), line.substring(p + 1).trim());
                }
                line = in.readLine();
            }
        } catch (IOException ioe) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "GMHTTP", "%s", tztZFLog.getStackTraceString(ioe));
        }
        return responseCode;
    }

    private void analyseHeadFields() {
        if (headLen <= 0)
            return;
        BufferedReader hin = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(recv_stream.toByteArray(), 0, headLen)));
        tztZFDataMapBase<String, String> headers = new tztZFDataMapBase<>();
        int statusCode = decodeHeader(hin, headers);
        contentLen = headers.getInt("Content-Length", 0);
        response = new Response();
        response.setCode(statusCode);
        response.setRequest(request);
        response.setHeaders(headers);
    }

    public int isHeadComplete() {
        //
        byte[] data = recv_stream.toByteArray();
        int splitbyte = 0;
        if (data.length > 0) {
            splitbyte = findHeaderEnd(data, data.length);
        }
        if (splitbyte > 0) {
            if (splitbyte > data.length)
                return 0;
            //头长度
            headLen = splitbyte;
            //解析头
            analyseHeadFields();
            //
            return headLen;
        }

        return 0;
    }

    public void finish(Error error) {
        if (completionHandler == null)
            return;

        byte[] recvData = recv_stream.toByteArray();
        byte[] body = null;
        if (response != null && contentLen > 0) {
            body = new byte[contentLen];
            System.arraycopy(recvData, headLen, body, 0, contentLen);
            response.setBody(body);
        }

        completionHandler.completion(body, response, error);
    }

    public void finish(Error error, int errorNo) {
        if (completionHandler == null)
            return;
        Response response = new Response();
        response.setCode(errorNo);
        completionHandler.completion(null, response, error);
    }
}
