package com.tztzf.network.socket;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.cosign.tztZFNetCosignEx;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.socket.Netty.tztZFNetWorkError;
import com.tztzf.network.socket.gmhttp.Request;
import com.tztzf.network.socket.gmhttp.tztZFGMRequestItem;
import com.tztzf.network.socket.gmhttp.tztZFGMURLSessionCompletion;
import com.tztzf.network.tztZFNetWork;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

import static com.tztzf.network.cosign.tztZFNetCosignEx.tztCosignEx_CACert;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_ReadError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_TimeOutError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFReadError;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepConnect;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepDowngrade;

public class tztZFGMURLSession {
    private Object context;
    private CopyOnWriteArrayList<tztZFGMGCDSocket> aySockets;
    private CopyOnWriteArrayList<tztZFGMRequestItem> reqItems;
    private ConcurrentHashMap<Integer, tztZFGMRequestItem> sendedItems;
    private static tztZFGMURLSession g_GMURLSessionIns;
    private CopyOnWriteArrayList<String> addList;
    private int addressIndex;
    //写线程
    private tztZFThreadPool m_pThreadPool = null;
    private final Object m_pSyncObject = new Object();
    private boolean isRun;
    private int m_nOnlineTime = 60 * 1000; //心跳间隔时间
    private boolean bNext;

    private static synchronized tztZFGMURLSession getShareInstance() {
        if (g_GMURLSessionIns == null) {
            g_GMURLSessionIns = new tztZFGMURLSession();
        }
        return g_GMURLSessionIns;
    }

    private tztZFGMURLSession() {
        aySockets = new CopyOnWriteArrayList<>();
        reqItems = new CopyOnWriteArrayList<>();
        addList = new CopyOnWriteArrayList<>();
        sendedItems = new ConcurrentHashMap<>();

        bNext = true;
        isRun = true;
        m_pThreadPool = new tztZFThreadPool(2, 2, "tztZFSocket");
        m_pThreadPool.setUsed(true);

        m_pThreadPool.useThreadPool().execute(new Runnable() {
            @Override
            public void run() {
                while (isRun) {
                    tztZFGMURLSession.this.syncRun();
                }
            }
        });
    }

    private tztZFGMGCDSocket getIdleSocket(int socketID, boolean isLoginRequest) {
        tztZFGMGCDSocket retSocket = null;
        if (aySockets.size() < 1) {
            //直接创建
            retSocket = createSocket();
            aySockets.add(retSocket);
        }
        //有存在的，判断当前是否空闲
        for (tztZFGMGCDSocket sock : aySockets) {
            if (socketID > 0 && socketID == sock.hashCode()) {
                return sock;
            }
            if (!isLoginRequest) {
                boolean canUse = isCanUse(sock);
                if (canUse) return sock;
            }
        }

        if (retSocket == null) {
            //判断是否超过10个
            if (aySockets.size() < 10) {
                retSocket = createSocket();
                aySockets.add(retSocket);
            }
        }

        return retSocket;
    }

    private boolean isCanUse(tztZFGMGCDSocket sock) {
        if (sock == null)
            return false;

        for (tztZFGMRequestItem item : reqItems) {
            if (item.getSocketID() == sock.hashCode()) {
                return false;
            }
        }
        for (Map.Entry<Integer, tztZFGMRequestItem> item : sendedItems.entrySet()) {
            if (item == null) {
                continue;
            }
            tztZFGMRequestItem itemValue = item.getValue();
            if (itemValue.getSocketID() == sock.hashCode()) {
                return false;
            }
        }
        return true;
    }

    private tztZFGMGCDSocket createSocket() {
        tztZFGMGCDSocket socket = new tztZFGMGCDSocket(this.context, new tztZFGMGCDSocket.tztZFGMSocketInterface() {
            @Override
            public void onConnect(tztZFGMGCDSocket socket) {
                //链接成功，发起请求
                doNotify("onConnect");
            }

            @Override
            public void onDisConnect(tztZFGMGCDSocket socket) {
                //失败，
                if (socket == null) {
                    return;
                }
                if (!bNext) {
                    return;
                }
                addressIndex++;
                if (addressIndex >= addList.size()) {
                    //已经切完了，但是还是失败，要去做降级处理了
                    addressIndex = 0;
                    tztZFNetCosign.didGMDowngrade();
                    tztZFNetWork.setLinkTypeGMDowngrade(tztZFLinkType.tztZFLinkType_JY.value());
                    tztZFGMRequestItem item = getReqItemWithSock(socket.hashCode());
                    if (item == null) {
                        return;
                    }
                    reqItems.remove(item);
                    String gmLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_GMLable, tztZFErrorMessageDefine.tztZFGMLable);
                    Error error = new Error(String.format(Locale.getDefault(), "%s降级!", gmLable));
                    item.finish(error, -tztZFSocketSSLStepDowngrade);
                }
                if (socket != null) {
                    removeDisConnectedSocket(socket.getSocketID());
                }
            }

            @Override
            public void onRecvData(tztZFGMGCDSocket socket, byte[] recvData) {
                //获取数据完成
                if (socket == null) {
                    return;
                }
                tztZFGMRequestItem item = getSendedReqItemWithSock(socket.hashCode());
                if (item == null)
                    return;

                item.appendRecvData(recvData);
                long length = item.getRecvDataLen();
                if (item.isHeadComplete() <= 0)
                    return;
                int headLen = item.getHeadLen();
                int contentLen = item.getContentLen();

                if (headLen > 0 && contentLen >= 0) {
                    long subLen = length - headLen - contentLen;
                    if (subLen >= 0) {
                        //接收完成
                        if (socket.isShortConnect() == 1) {
                            //清除登录socket
                            bNext = false;
                            removeDisConnectedSocket(socket.hashCode());
                        }

                        item.finish(null);
                        removeSendedReq(socket.hashCode());
                    }
                }
            }
        });
        return socket;
    }

    private void loginWithRequestIn(Request req, tztZFGMURLSessionCompletion completion) {
        tztZFGMGCDSocket sock = createSocket();
        sock.setIsShortConnect(1);
        bNext = true;
        //记录请求对象
        //在连接失败、应答返回等情况下，需要处理清除
        tztZFGMRequestItem reqItem = new tztZFGMRequestItem();
        reqItem.setRequest(req);
        reqItem.setCompletionHandler(completion);
        reqItem.setTimeOut(req.getTimeOut());
        reqItem.addSendTime();
        reqItem.setSocketID(sock.hashCode());
        reqItem.setIsLoginRequest(true);
        reqItems.add(reqItem);
        aySockets.add(sock);
        doNotify("addAction");
    }

    private void dataTaskWithRequestIn(Request req, tztZFGMURLSessionCompletion completion) {
        bNext = true;
        //记录请求对象
        //在链接失败、应答返回等情况下，需要处理清除
        tztZFGMRequestItem reqItem = new tztZFGMRequestItem();
        reqItem.setRequest(req);
        reqItem.setCompletionHandler(completion);
        reqItem.setTimeOut(req.getTimeOut());
        reqItem.addSendTime();
        this.reqItems.add(reqItem);
        doNotify("addAction");
    }

    private boolean sendHTTPData(tztZFGMRequestItem reqItem, tztZFGMGCDSocket sock) {
        if (reqItem == null || sock == null)
            return false;
        Request request = reqItem.getRequest();
        if (request == null)
            return false;
        if (!sendHTTPData(request, sock)) {
            //发送失败，返回上层错误
            Error error = new Error(tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError, "发送请求失败!"));
            reqItem.finish(error);
            return false;
        }
        return true;
    }

    private boolean sendHTTPData(Request request, tztZFGMGCDSocket sock) {
        if (request == null || sock == null)
            return false;
        String host = request.getHost();
        int port = request.getPort();

        String method = request.getMethod();
        if (method == null || method.length() < 1)
            method = "";
        String path = request.getPath();
        if (path == null || path.length() < 1)
            path = "";
        String suffix = "?";
        if (path.length() < 1) {
            path = "";
            suffix = "";
        }
        String strQuery = request.getQuery();
        if (strQuery == null || strQuery.length() < 1) {
            strQuery = "";
            suffix = "";
        }

        String send = String.format(Locale.CHINA, "%s %s%s%s HTTP/1.1\r\n", method, path, suffix, strQuery);
        if (port > 0) {
            send = send.concat(String.format(Locale.CHINA, "Host: %s:%d\r\n", host, port));
        } else {
            send = send.concat(String.format(Locale.CHINA, "Host: %s\r\n", host));
        }

        String nsContentType = "application/x-www-form-urlencoded; charset=UTF-8";
        String nsConnection = "keep-alive";
        String nsAccept = "*/*";
        String nsAcceptEncoding = "gzip, deflate";
        String nsUserAgent = "tztSSLTransfer";

        tztZFDataMapBase<String, String> dictHeader = new tztZFDataMapBase<>();
        dictHeader.setStr("Content-Type", nsContentType);
        dictHeader.setStr("Connection", nsConnection);
        dictHeader.setStr("Accept", nsAccept);
        dictHeader.setStr("Accept-Encoding", nsAcceptEncoding);
        dictHeader.setStr("User-Agent", nsUserAgent);
//        send = send.concat("Content-Type: application/x-www-form-urlencoded; charset=UTF-8\r\n");
//        send = send.concat("Connection: keep-alive\r\n");
//        send = send.concat("Accept: */*\r\n");
//        send = send.concat("Accept-Encoding: gzip, deflate\r\n");
//        send = send.concat("User-Agent: Mozilla/5.0 (Linux; Android 5.0; SM-G900P Build/LRX21T) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Mobile Safari/537.36\r\n");

        if (request.getHeaders() != null) {
            for (String headField : request.getHeaders().getAllLowKeys()) {
                if (headField.equalsIgnoreCase("Host"))
                    continue;
                //
                String value = request.getHeader(headField);
                if (headField.equalsIgnoreCase("Content-Type")) {
                    if (value == null)
                        value = nsContentType;
                } else if (headField.equalsIgnoreCase("Connection")) {
                    if (value == null)
                        value = nsConnection;
                } else if (headField.equalsIgnoreCase("Accept")) {
                    if (value == null)
                        value = nsAccept;
                } else if (headField.equalsIgnoreCase("Accept-Encoding")) {
                    if (value == null)
                        value = nsAcceptEncoding;
                } else if (headField.equalsIgnoreCase("User-Agent")) {
                    if (value == null)
                        value = nsUserAgent;
                }
                if (value != null) {
                    dictHeader.setStr(headField, value);
                }
            }
        }

        for (String lowKey : dictHeader.getAllLowKeys()) {
            String key = dictHeader.getKey(lowKey);
            String value = dictHeader.getStr(key);
            if (key == null || value == null)
                continue;
            send = send.concat(String.format(Locale.getDefault(), "%s: %s\r\n", key, value));
        }

        byte[] body = request.getBody();
        int bodylen = 0;
        if (body != null && body.length > 0) {
            bodylen = body.length;
            send = send.concat(String.format(Locale.CHINA, "Content-Length: %d\r\n", body.length));
        }
        send = send.concat("\r\n");
        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "tztZFGMSocket", "HTTPHeader: %s", send);

        byte[] header = send.getBytes();
        int totalSend = header.length + bodylen;
        byte[] sendData = new byte[totalSend];
        System.arraycopy(header, 0, sendData, 0, header.length);
        if (bodylen > 0)
            System.arraycopy(body, 0, sendData, header.length, bodylen);

        return sock.sendData(sendData);
    }


    private void syncRun() {
        try {
            while (isRun && haveReqAction()) {
                if (!sendRequestData())
                    break;
            }
            checkReqItemTimeOut();// 处理应答超时
            //发送心跳
            doSendHeartBeat();
            if (isRun && m_pSyncObject != null) {
                synchronized (m_pSyncObject) {
                    if (haveReqAction() || getSendedReqCount() > 0) {
                        doWait(Thread.currentThread(), 1 * 1000);
                    } else {
                        doWait(Thread.currentThread(), 59 * 1000);
                    }
                }
            }
        } catch (Exception e) {

        }
    }

    private synchronized boolean sendRequestData() {
        tztZFGMRequestItem reqItem = getAction(0);
        if (reqItem == null) {
            return false;
        }
        //获取可用socket
        final tztZFGMGCDSocket sock = getIdleSocket(reqItem.getSocketID(), reqItem.isLoginRequest());
        if (sock == null) {
            //当前没有可用
            return false;
        }
        //获取数据
        reqItems.get(0).setSocketID(sock.hashCode());
        reqItem.setSocketID(sock.hashCode());
        if (sock.isConnect()) {
            if (sock.isShortConnect() == 1 && !sock.isSslConnect()) {
                //登录需ssl成功之后才允许发送
                return false;
            }
            boolean sended = sendHTTPData(reqItem, sock);
            if (sended) {
                addSendedReq(reqItem.getSocketID(), reqItem);
            }
            removeReqList(reqItem);
            sock.readDoNotify();
            return sended;
        } else {
            //
            String host = null;
            int port = -1;
            if (addList.size() > addressIndex) {
                String address = addList.get(addressIndex);
                String[] hostport = address.split(":");
                if (hostport.length > 0) {
                    host = hostport[0];
                }
                if (hostport.length > 1) {
                    try {
                        port = Integer.parseInt(hostport[1]);
                    } catch (Exception e) {
                        tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, "GMURLSession", "%s", tztZFLog.getStackTraceString(e));
                    }
                }
            }

            final String finalHost = host;
            final int finalPort = port;
            m_pThreadPool.useThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    sock.connectToHost(finalHost, finalPort);
                }
            });
            return false;
        }
    }

    /**
     * 获取某个待请求数据
     *
     * @return 请求
     */
    private tztZFGMRequestItem getAction(int id) {
        if (reqItems == null || id < 0)
            return null;
        if (id < reqItems.size()) {
            return reqItems.get(id);// 得到请求数据
        } else {
            return null;
        }
    }

    /**
     * 有待发请求
     *
     * @return true：有 false：无
     */
    private boolean haveReqAction() {
        if (!isRun)
            return false;
        if (reqItems != null && reqItems.size() > 0)
            return true;

        return false;
    }

    /**
     * 超时处理
     */
    private void checkReqItemTimeOut() {
        synchronized (reqItems) {
            if (reqItems.size() > 0) {
                for (int i = reqItems.size() - 1; i >= 0; i--) {
                    tztZFGMRequestItem requestItem = reqItems.get(i);
                    if (requestItem == null) {
                        reqItems.remove(i);
                        continue;
                    }
                    if (requestItem.isTimeOut()) {
                        //返回超时错误应答
                        Error error = new Error(tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, "应答超时，请检查网络通讯状态是否良好"));
                        requestItem.finish(error);
                        reqItems.remove(i);
                        //清除登录socket
                        tztZFGMGCDSocket gmGCDSocket = getIdleSocket(requestItem.getSocketID(), requestItem.isLoginRequest());
                        if (gmGCDSocket != null && gmGCDSocket.isShortConnect() == 1) {
                            removeDisConnectedSocket(gmGCDSocket.hashCode());
                        }
                    }
                }
            }
        }
        for (Iterator<Map.Entry<Integer, tztZFGMRequestItem>> it = sendedItems.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry<Integer, tztZFGMRequestItem> item = it.next();
            if (item == null) {
                it.remove();
                continue;
            }
            tztZFGMRequestItem itemValue = item.getValue();
            if (itemValue.isTimeOut()) {
                //返回超时错误应答
                Error error = new Error(tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, "应答超时，请检查网络通讯状态是否良好"));
                it.remove();
                itemValue.finish(error);
                //清除登录socket
                tztZFGMGCDSocket gmGCDSocket = getIdleSocket(itemValue.getSocketID(), itemValue.isLoginRequest());
                if (gmGCDSocket != null && gmGCDSocket.isShortConnect() == 1) {
                    removeDisConnectedSocket(gmGCDSocket.hashCode());
                }
            }
        }
    }

    /**
     * 获取待应答队列请求数
     *
     * @return 获取待应答队列数量
     */
    private int getSendedReqCount() {
        if (sendedItems == null)
            return 0;
        return sendedItems.size();
    }

    private void doWait(Thread thread, int Howlong) {
        try {
            if (m_pSyncObject == null)
                return;
            m_pSyncObject.wait(Howlong);
        } catch (Exception e) {
        }
    }

    private void doNotify(String tag) {
        if (m_pSyncObject == null)
            return;
        synchronized (m_pSyncObject) {
            m_pSyncObject.notify();
        }
    }

    /**
     * 从列表移除请求对象
     */
    private void removeReqList(tztZFGMRequestItem req) {
        if (reqItems == null || req == null)
            return;
        synchronized (reqItems) {
            reqItems.remove(req);
        }
    }

    /**
     * 加入到已发送列表
     *
     * @param req 已发送数据对象
     */
    private void addSendedReq(int socketID, tztZFGMRequestItem req) {
        if (sendedItems == null)
            return;
        sendedItems.put(socketID, req);
    }

    /**
     * 根据socketid返回请求对象
     *
     * @return 请求对象
     */
    private tztZFGMRequestItem getReqItemWithSock(Integer recvTag) {
        if (recvTag < 0)
            return null;
        if (reqItems == null || reqItems.size() < 1)
            return null;
        for (int i = reqItems.size() - 1; i >= 0; i--) {
            tztZFGMRequestItem requestItem = reqItems.get(i);
            if (requestItem == null) {
                continue;
            }
            if (requestItem.getSocketID() == recvTag) {
                return requestItem;
            }
        }
        return null;
    }

    /**
     * 根据socketid返回的已发送请求对象
     *
     * @return 请求对象
     */
    private tztZFGMRequestItem getSendedReqItemWithSock(Integer recvTag) {
        if (recvTag < 0)
            return null;
        if (sendedItems == null || !sendedItems.containsKey(recvTag))
            return null;
        tztZFGMRequestItem requestItem = sendedItems.get(recvTag);
        return requestItem;
    }

    /**
     * 根据请求号，从待应答队列中移除，返回移除的请求对象
     */
    private void removeSendedReq(Integer recvTag) {
        if (recvTag < 0)
            return;
        if (sendedItems == null || !sendedItems.containsKey(recvTag))
            return;
        sendedItems.remove(recvTag);
    }

    private void doSendHeartBeat() {
        //
        if (aySockets == null) {
            return;
        }
        for (int i = 0; i < aySockets.size(); i++) {
            tztZFGMGCDSocket socket = aySockets.get(i);
            if (socket == null || !socket.isConnect()) {
                continue;
            }
            //连接状态下需要处理心跳
            String address = socket.getAddress();
            long lastRecvTime = socket.getLastRecvTime();
            if (lastRecvTime > 0 && (System.currentTimeMillis() - lastRecvTime) > m_nOnlineTime && !tztZFTextUtils.isEmpty(address)) {
                //判断socket是否被占用
                if (!isCanUse(socket)) {
                    continue;
                }
                //长时间未收到应答则请求心跳
                String strUrl = String.format("http://%s/reqxml?action=46", address);
                Request request = new Request(strUrl);
                request.setTimeOut(6 * 1000);//10超时时间
//                if(sendHTTPData(request, socket)) {
//                    socket.readDoNotify();
//                }

                tztZFGMRequestItem reqItem = new tztZFGMRequestItem();
                reqItem.setSocketID(socket.hashCode());
                reqItem.setRequest(request);
                reqItem.setCompletionHandler(null);
                reqItem.setTimeOut(10 * 1000);
                reqItem.addSendTime();
                boolean sended = sendHTTPData(reqItem, socket);
                if (sended) {
                    addSendedReq(reqItem.getSocketID(), reqItem);
                }
            }
        }
    }

    private synchronized void removeDisConnectedSocket(int socketID) {
        for (int i = 0; i < aySockets.size(); i++) {
            if (aySockets.get(i).getSocketID() == socketID) {
                doClearrequest(socketID);
                aySockets.remove(i);
                break;
            }
        }
    }

    /**
     * 清除请求处理
     */
    private void doClearrequest(int socketID) {
        synchronized (reqItems) {
            if (reqItems.size() > 0) {
                for (int i = reqItems.size() - 1; i >= 0; i--) {
                    tztZFGMRequestItem requestItem = reqItems.get(i);
                    if (requestItem == null) {
                        reqItems.remove(i);
                        continue;
                    }
                    if (socketID == requestItem.getSocketID()) {
                        //返回超时错误应答
                        Error error = new Error(tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, "应答超时，请检查网络通讯状态是否良好"));
                        requestItem.finish(error);
                        reqItems.remove(i);
                    }
                }
            }
        }
        for (Iterator<Map.Entry<Integer, tztZFGMRequestItem>> it = sendedItems.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry<Integer, tztZFGMRequestItem> item = it.next();
            if (item == null) {
                it.remove();
                continue;
            }
            tztZFGMRequestItem itemValue = item.getValue();
            if (socketID == itemValue.getSocketID()) {
                //返回超时错误应答
                Error error = new Error(tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_TimeOutError, "应答超时，请检查网络通讯状态是否良好"));
                it.remove();
                itemValue.finish(error);
            }
        }
    }

    //

    //public APIs

    /**
     * 初始化处理
     *
     * @param context context
     */
    public static void init(Object context) {
        getShareInstance().context = context;
    }

    /**
     * 设置国密透传地址
     *
     * @param addList 地址列表，以&分隔
     */
    public static void setGMTransferAddress(String addList) {
        getShareInstance().addList.clear();
        if (addList == null || addList.length() < 1)
            return;
        String[] items = addList.split("&");
        List<String> list = Arrays.asList(items);
        getShareInstance().addList.addAll(list);
    }

    /**
     * 调用国密透传通道进行数据请求
     *
     * @param connection 请求
     * @param completion 结果应答回调
     */
    public static void dataTaskWithRequest(Request connection, tztZFGMURLSessionCompletion completion) {
        getShareInstance().dataTaskWithRequestIn(connection, completion);
    }

    public static void loginWithRequest(Request request, tztZFGMURLSessionCompletion completion) {
        getShareInstance().loginWithRequestIn(request, completion);
    }
}

class tztZFGMGCDSocket {
    private tztZFThreadPoolGCDAsyncSocket socket;
    private tztZFThreadPoolGCDAsyncSocketEventsInterface socketEventsInterface;
    //主线程和读线程
    private tztZFThreadPool m_pReadThreadPool = null;
    private final Object m_pReadSyncObject = new Object();
    private boolean isRun;
    private tztZFThreadPool m_pConnectThreadPool = null;
    private byte[] recvData = new byte[4096];

    private tztZFGMSocketInterface callBack;
    private long m_nLastRecvTime;//上一次接收时间
    private String address = "";//连接地址
    private Object context;
    private int isShortConnect;
    private boolean sslConnect;

    public tztZFGMGCDSocket(Object context, final tztZFGMSocketInterface callBack) {
        this.callBack = callBack;
        this.context = context;
        try {
            socketEventsInterface = new tztZFThreadPoolGCDAsyncSocketEventsInterface() {
                @Override
                public void didConnectToHost(tztZFThreadPoolGCDAsyncSocket socket, String host, int port) {
                    //链接成功，唤起发送
                    isRun = true;
                    //通知上层链接成功，发送请求
                    if (callBack != null) {
                        callBack.onConnect(tztZFGMGCDSocket.this);
                    }
                }

                @Override
                public void socketDidDisconnect(tztZFThreadPoolGCDAsyncSocket socket, int errorNo, String errorMsg) {
                    //断开链接，返回上层错误
                    if (callBack != null) {
                        callBack.onDisConnect(tztZFGMGCDSocket.this);
                    }
                    CloseConnect(socket, errorNo, errorMsg);
                }

                @Override
                public int socketReadData(tztZFThreadPoolGCDAsyncSocket socket, byte[] data, int len) {
                    return len;
                }
            };

            m_pReadThreadPool = new tztZFThreadPool(1, 1, "tztZFSocket");
            m_pReadThreadPool.setUsed(true);

            m_pConnectThreadPool = new tztZFThreadPool(1, 1, "tztZFSocket");
            m_pConnectThreadPool.setUsed(true);
            socket = new tztZFThreadPoolGCDAsyncSocket();
            socket.InitWithContext(context)
                    .InitWithThreadPool(m_pConnectThreadPool.useThreadPool())
                    .InitWithSocketKey("com.tzt.gmhttp")
                    .InitWithIndex("1")
                    .InitWithLinktype(99999999)
                    .InitWithIsSSL(true)
                    .InitWithEvents(socketEventsInterface);

            isRun = true;
            m_pReadThreadPool.useThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    while (isRun) {
                        try {
                            tztZFGMGCDSocket.this.readSyncRun();
                        } catch (Exception e) {
                            tztZFGMGCDSocket.this.CloseConnect(socket, tztZFNetWorkError.tztZFNetWorkError_Disconnect,
                                    tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_ReadError, tztZFReadError));
                        }
                    }
                }
            });

            loadCert(socket);
            socket.InitWithSocketKey("tztSSL-Transfer");
            socket.tztZF_SetConnecttype(9);
            m_nLastRecvTime = 0;
        } catch (Exception e) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_error, "GMURLSession", tztZFLog.getStackTraceString(e));
        }
    }

    public int getSocketID() {
        if (socket == null)
            return 0;
        return socket.hashCode();
    }

    private void loadCert(tztZFThreadPoolGCDAsyncSocket socket) {
        byte[] ca = tztZFNetCosignEx.getExDataForKey(tztCosignEx_CACert);
        if (ca == null || ca.length < 1)
            return;

        byte[] sign = null;
        byte[] signKey = null;
        byte[] enc = null;
        byte[] encKey = null;

        int signType = 1;
        int encType = 1;
        int certType = 1;

        tztZFCosignUser cosignUser = tztZFNetCosign.getBindCosignUser();
        if (cosignUser != null && cosignUser.haveCosignCert()) {
            socket.withZFSSL_LinkCosignUserID(tztZFLinkType.tztZFLinkType_JY.value(), cosignUser.userID());
            sign = cosignUser.getLoginCosignCert().getSignCert();
            signType = cosignUser.getLoginCosignCert().getSignType();

            enc = cosignUser.getLoginCosignCert().getEncCert();
            encKey = cosignUser.getLoginCosignCert().getEncKey();
            encType = cosignUser.getLoginCosignCert().getEncType();
            certType = cosignUser.getLoginCosignCert().getCertType();
        }
        signKey = null;

        socket.withZFSSL_UserID(tztZFNetCosign.getBindUserID())
                .withZFSSL_CA(ca, 1)
                .withZFSSL_Enc(enc, encKey, encType, certType)
                .withZFSSL_Sign(sign, signKey, signType);
        socket.setCosignUserInfo(cosignUser);
    }

    /////-------------------------
    public final synchronized void CloseConnect(tztZFThreadPoolGCDAsyncSocket sock, int type, String msg) {
        if (sock != null) {
            sock.setEvents(null);
            sock.Close(msg);
            this.socket = null;
        }
        if (m_pConnectThreadPool != null) {
            m_pConnectThreadPool.shutdown();
            m_pConnectThreadPool = null;
        }
//        if(m_pReadThreadPool != null){
//            m_pReadThreadPool.shutdown();
//            m_pReadThreadPool = null;
//        }
        m_nLastRecvTime = 0;
//        isRun = false;
//        if (callBack != null) {
//            callBack.onDisConnect(tztZFGMGCDSocket.this);
//        }
    }

    public void readDoNotify() {
        synchronized (m_pReadSyncObject) {
            m_pReadSyncObject.notify();
        }
    }

    private void readSyncRun() throws Exception {
        if (isRun && !isConnect()) {
            synchronized (m_pReadSyncObject) {
                m_pReadSyncObject.wait();
            }
        } else {
            //读取数据
            doRecvData();
        }
    }

    public boolean isConnect() {
        if (socket == null)
            return false;

        return socket.IsConnected();
    }

    public void connectToHost(String host, int port) {
        if (socket == null) {
            if (m_pConnectThreadPool != null) {
                m_pConnectThreadPool.shutdownNow();
                m_pConnectThreadPool = null;
            }
            m_pConnectThreadPool = new tztZFThreadPool(1, 1, "tztZFSocket");
            m_pConnectThreadPool.setUsed(true);
            socket = new tztZFThreadPoolGCDAsyncSocket();
            socket.InitWithContext(context)
                    .InitWithThreadPool(m_pConnectThreadPool.useThreadPool())
                    .InitWithSocketKey("com.tzt.gmhttp")
                    .InitWithIndex("1")
                    .InitWithLinktype(99999999)
                    .InitWithIsSSL(true)
                    .InitWithEvents(socketEventsInterface);
            loadCert(socket);
            socket.InitWithSocketKey("tztSSL-Transfer");
            socket.tztZF_SetConnecttype(9);
        }

        socket.setGMSSLStepCallBack(new tztZFGMSSLStepCallBack() {
            @Override
            public void onCallBack(int step, int code, String certUser) {
                if (step == tztZFSocketSSLStepConnect) {
                    //ssl连接
                    sslConnect = true;
                }
            }
        });

        socket.InitWithAddress(host)
                .InitWithPort(port)
                .socketConnect();
        address = host + ":" + port;
    }

    public boolean sendData(byte[] data) {
        boolean ret = false;
        if (socket == null || !isConnect())
            return ret;
        try {
            ret = socket.Write(data);
        } catch (Exception e) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_error, "tztZFGMSocket", "%s", tztZFLog.getStackTraceString(e));
        }
        return ret;
    }

    private void doRecvData() throws Exception {
        //读取数据
        int size = recvData.length;
        int reads = 0;
        try {
            if (socket == null || (reads = socket.ReadData(recvData, size)) < 0) {
                //错误
                throw new Exception("Read Head End(-1)!");
            } else if (reads == 0) {
                return;
            } else if (reads > 0) {
            }
        } catch (Exception e) {
            //错误
            throw new Exception("通讯错误!");
        }
        m_nLastRecvTime = System.currentTimeMillis();
        if (callBack != null) {
            byte[] bytes = new byte[reads];
            System.arraycopy(recvData, 0, bytes, 0, reads);
            callBack.onRecvData(this, bytes);
        }

    }

    long getLastRecvTime() {
        return m_nLastRecvTime;
    }

    String getAddress() {
        return address;
    }

    public int isShortConnect() {
        return isShortConnect;
    }

    public void setIsShortConnect(int isShortConnect) {
        this.isShortConnect = isShortConnect;
    }

    public boolean isSslConnect() {
        return sslConnect;
    }

    interface tztZFGMSocketInterface {
        void onConnect(tztZFGMGCDSocket socket);

        void onDisConnect(tztZFGMGCDSocket socket);

        void onRecvData(tztZFGMGCDSocket socket, byte[] recvData);

    }
}