package com.tztzf.network.socket;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.callback.tztZFNoUserCertCallBack;
import com.tztzf.network.callback.tztZFRegisterCallBack;
import com.tztzf.network.callback.tztZFSocketLinkDelegate;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFSocketFlags;
import com.tztzf.network.socket.Netty.tztZFSocket;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.callback.tztZFRegisterInterface;
import com.tztzf.protocol.core.tztZFDataProtocolKit;
import com.tztzf.protocol.core.tztZFRegister;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;
import com.tztzf.tztZFPrivate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Random;
import java.util.Set;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFNetWorkErrorKey_WriteError;
import static com.tztzf.network.define.tztZFErrorMessageDefine.tztZFWriteError;
import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Non;
import static com.tztzf.network.define.tztZFSocketFlags.tztZFSocketFlags_No;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoConnect;

/**
 * ****************************************************************
 * 文件名称 : tztZFSocketLink
 * 作   者 : Created by wlz
 * 创建时间 : 2020-02-28 14:00
 * 文件描述 :
 * 注意事项 :
 * 版权声明 : Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史 : 2020-02-28 1.00 初始版本
 * ****************************************************************
 */
@tztZFPrivate(name = "内部类，外层禁止使用")
public final class tztZFSocketLink {
    /** 通道列表 */
    private int poolsMax = 20;
    private ArrayList<tztZFThreadPool> ayThreadPool = null;
    private tztZFDataMapBase<String, Object> linkMap;
    private ArrayList<tztZFSDKObj> aySDKObj = null;
    static tztZFSocketLink linkObj;

    private tztZFThreadPool pushThreadPool = null;
    private tztZFSocketObj pushSocketObj = null;

    private tztZFSocketLinkDelegate linkDelegate;
    private int useRandom;
    private int usePercent;
    private Lock linkLock;
    //endregion

    //region 具体实现

    //region 公布方法（允许其他包访问）

    private tztZFSocketLink() {
        init();
    }

    private void init() {
        if (linkMap == null) {
            linkMap = new tztZFDataMapBase<>();
        }
        if (aySDKObj == null) {
            aySDKObj = new ArrayList<>();
            //兼容老版本，设置App默认sdkid:"0",默认sdktype为0，可不注册。
            tztZFSDKObj appObj = new tztZFSDKObj();
            aySDKObj.add(appObj);
        }
        if (ayThreadPool == null) {
            ayThreadPool = new ArrayList<>();
        }
        linkLock = new ReentrantLock();
        usePercent = 100;
        Random r = new Random(System.currentTimeMillis());
        useRandom = r.nextInt(10000);
    }

    /**
     * 获取tztZFSocketLink实例
     *
     * @return tztZFSocketLink实例
     */
    private static synchronized tztZFSocketLink getShareInstance() {
        if (linkObj == null) {
            linkObj = new tztZFSocketLink();
            tztZFDataProtocolKit.setLogLevel(tztZFLog.getLevel());
        }
        return linkObj;
    }

    private tztZFThreadPool getThreadPool() throws Exception {
        synchronized (ayThreadPool) {
            for (int i = 0; i < ayThreadPool.size(); i++) {
                tztZFThreadPool threadPool = ayThreadPool.get(i);
                if (!threadPool.isUsed()) {
                    threadPool.setUsed(true);
                    return threadPool;
                }
            }
            if (ayThreadPool.size() >= poolsMax) {
                throw new Exception("通讯错误，建立了太多的连接！" + poolsMax);
            }
            tztZFThreadPool threadPool = new tztZFThreadPool(2, 2, "tztZFSocket");
            threadPool.setUsed(true);
            ayThreadPool.add(threadPool);
            return threadPool;
        }
    }

    private static void linkLock() {
        tztZFSocketLink.getShareInstance().linkLock.lock();
    }

    private static void linkUnlock() {
        tztZFSocketLink.getShareInstance().linkLock.unlock();
    }

    private tztZFSDKObj getSDKObjForUserIDKit(int userID, int sdkType) {
        for (int i = 0; i < aySDKObj.size(); i++) {
            tztZFSDKObj sdkObj = aySDKObj.get(i);
            if (sdkObj.userID == userID && sdkObj.sdkType == sdkType) {
                return sdkObj;
            }
        }
        tztZFSDKObj obj = new tztZFSDKObj();
        obj.userID = userID;
        obj.sdkType = sdkType;
        aySDKObj.add(obj);
        return obj;
    }

    private static tztZFSDKObj getSDKObjForUserID(int userID, int sdkType) {
        try {
            linkLock();
            return tztZFSocketLink.getShareInstance().getSDKObjForUserIDKit(userID, sdkType);
        } finally {
            linkUnlock();
        }
    }

    //查找对应的socketobj
    private tztZFSocketObj getSocketObjForKeyKit(String linkKey) {
        if (linkKey == null || linkKey.length() < 1)
            return null;
        if (isPushSocket(linkKey))
            return pushSocketObj;
        return (tztZFSocketObj) linkMap.getObj(linkKey);
    }

    private static tztZFSocketObj getSocketObjForKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            return null;
        }
        try {
            linkLock();
            return tztZFSocketLink.getShareInstance().getSocketObjForKeyKit(linkKey);
        } finally {
            linkUnlock();
        }
    }

    private static tztZFSocket getZFSocketForKey(String linkKey) {
        tztZFSocketObj tztSocketObj = tztZFSocketLink.getSocketObjForKey(linkKey);
        if (tztSocketObj == null)
            return null;
        return tztSocketObj.poolSocket;
    }

    //-内部统一处理函数
    //是push使用的socket
    private boolean isPushSocket(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return false;
        if (pushSocketObj == null)
            return false;
        return linkKey.equals(pushSocketObj.linkKey);
    }

    //推送通道Socket
    private boolean createPushSocket(final Object context, String linkKey, int userID, int sdkType, int linkType) {
        try {
            if (pushThreadPool == null) {
                pushThreadPool = new tztZFThreadPool(2, 2, "tztZFPushSocket");
                pushThreadPool.setUsed(true);
            }

            if (pushSocketObj == null) {
                tztZFSocket newSocket = new tztZFSocket();
                newSocket.InitWithContext(context)
                        .InitWithUserID(userID)
                        .InitWithSDKType(sdkType)
                        .InitWithLinkType(linkType)
                        .InitWithLinkKey(linkKey)
                        .InitWithThreadPool(pushThreadPool)
                        .SocketConnect();
                pushSocketObj = new tztZFSocketObj();
                pushSocketObj.poolSocket = newSocket;
                pushSocketObj.linkKey = linkKey;
                pushSocketObj.userID = userID;
                pushSocketObj.sdkType = sdkType;
                pushSocketObj.linkType = linkType;
                return true;
            } else {
                tztZFSocket poolSocket = pushSocketObj.poolSocket;
                if (poolSocket != null) {
                    return true;
                }
            }
        } catch (Exception e) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;message=%s", linkKey, userID, sdkType, linkType, "注册通道异常");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Error, params,
                        tztZFLog.getStackTraceString(e));
            }
            tztlog(tztZFLogLevel.tztZFLogLevel_Warn, "createSocket", "[%s]Error:%s", linkKey, tztZFLog.getStackTraceString(e));
            return false;
        }
        return false;
    }

    private boolean createSocket(final Object context, String linkKey, int userID, int sdkType, int linkType, int connectType) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;connectType=%d;message=%s", linkKey, userID, sdkType, linkType, connectType, "linkKey为空");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        boolean bIsPushSocket = (linkType == tztZFLinkType.tztZFLinkType_MsgPush.value());
        if (bIsPushSocket) {
            return createPushSocket(context, linkKey, userID, sdkType, linkType);
        }

        tztZFSocketObj socketObj = linkMap.getObj(linkKey);
        if (socketObj == null) {
            try {
                tztZFThreadPool threadPool = getThreadPool();
                tztZFSocket newSocket = new tztZFSocket();
                newSocket.InitWithContext(context)
                        .InitWithUserID(userID)
                        .InitWithSDKType(sdkType)
                        .InitWithLinkType(linkType)
                        .InitWithLinkKey(linkKey)
                        .InitWithConnectType(connectType)
                        .InitWithThreadPool(threadPool)
                        .SocketConnect();
                socketObj = new tztZFSocketObj();
                socketObj.poolSocket = newSocket;
                socketObj.linkKey = linkKey;
                socketObj.userID = userID;
                socketObj.sdkType = sdkType;
                socketObj.linkType = linkType;

                linkMap.setObj(linkKey, socketObj);
                return true;
            } catch (Exception e) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;message=%s", linkKey, userID, sdkType, linkType, "注册通道异常");
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                    tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Error, params,
                            tztZFLog.getStackTraceString(e));
                }
                tztlog(tztZFLogLevel.tztZFLogLevel_Warn, "createSocket", "[%s]Error:%s", linkKey, tztZFLog.getStackTraceString(e));
                return false;
            }
        }
        if (userID > 0 && socketObj.userID > 0 && socketObj.userID != userID)
            return false;
        if (linkType != socketObj.linkType)
            return false;
        if (sdkType > 0 && socketObj.sdkType > 0 && socketObj.sdkType != sdkType)
            return false;
        tztZFSocket poolSocket = socketObj.poolSocket;
        if (poolSocket == null) {
            try {
                tztZFThreadPool threadPool = getThreadPool();
                poolSocket = new tztZFSocket();
                poolSocket.InitWithContext(context)
                        .InitWithUserID(userID)
                        .InitWithSDKType(sdkType)
                        .InitWithLinkType(linkType)
                        .InitWithLinkKey(linkKey)
                        .InitWithConnectType(connectType)
                        .InitWithThreadPool(threadPool)
                        .SocketConnect();
                return true;
            } catch (Exception e) {
                if (tztZFRecordProcessLogTools.isRecord()) {
                    String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;message=%s", linkKey, userID, sdkType, linkType, "注册通道异常");
                    ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                    tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Error, params,
                            tztZFLog.getStackTraceString(e));
                }
                tztlog(tztZFLogLevel.tztZFLogLevel_Warn, "createSocket", "[%s]Error:%s", linkKey, tztZFLog.getStackTraceString(e));
                return false;
            }
        } else {
            return true;
        }
    }

    private int doPushLink(boolean bfree) {
        if (pushSocketObj == null)
            return 0;
        tztZFSocket poolSocket = pushSocketObj.poolSocket;
        if (poolSocket == null)
            return 0;
        if (bfree) {
            poolSocket.QuitSocket();
            pushSocketObj.poolSocket = null;
            pushSocketObj = null;
            if (pushThreadPool != null) {
                pushThreadPool.shutdown();
                pushThreadPool = null;
            }
        } else {
            poolSocket.CloseConnect(tztZFNetWorkError_Disconnect, "断开连接");
        }
        return 1;
    }

    private int doLinkForKey(String linkKey, boolean bfree) {
        return doLinkForKey(linkKey, bfree, -1);
    }

    private int doLinkForKey(String linkKey, boolean bfree, int connectIndex) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return 0;
        if (isPushSocket(linkKey)) {
            return doPushLink(bfree);
        }

        tztZFSocketObj socketObj = (tztZFSocketObj) linkMap.getObj(linkKey);
        if (socketObj == null)
            return 0;
        int num = 0;
        tztZFSocket poolSocket = socketObj.poolSocket;
        if (poolSocket != null) {
            //先清除，再断开连接，否则连接断开后会自动重连
            if (bfree) {
                poolSocket.QuitSocket();
                socketObj.poolSocket = null;
            } else {
                poolSocket.CloseConnect(tztZFNetWorkError_Disconnect, "断开连接", connectIndex);
            }
            num++;
        }
        if (bfree) {
            linkMap.removeByKey(linkKey);
        }
        return num;
    }

    private int doLinkAll(final boolean bfree) {
        final List<String> removeKeys = new ArrayList<>();
        linkMap.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
            @Override
            public boolean enumerate(String lowKey, String key, Object value) {
                if (value instanceof tztZFSocketObj) {
                    tztZFSocketObj socketObj = (tztZFSocketObj) value;
                    tztZFSocket poolSocket = socketObj.poolSocket;
                    if (poolSocket != null) {
                        if (bfree) {
                            poolSocket.QuitSocket();
                            socketObj.poolSocket = null;
                        } else {
                            poolSocket.CloseConnect(tztZFNetWorkError_Disconnect, "断开所有连接");
                        }
                        removeKeys.add(lowKey);
                    }
                }
                return false;
            }
        });

        if (bfree) {
            linkMap.removeAllObjects();
            synchronized (ayThreadPool) {
                for (tztZFThreadPool threadpool : ayThreadPool) {
                    threadpool.shutdown();
                }
                ayThreadPool.clear();
            }
        }
        return removeKeys.size();
    }

    /**
     * SDK证书注册
     *
     * @param application Application
     * @param sdkCert     SDK注册证书:分配的证书
     * @param sdkType     SDK注册类别:（0:默认普通APP客户端（注册时不校验）; 1：美股 SDK;2：深证通Level2;3：通用SDK ; 4 :H5认证SDK ）
     * @param sdkID       SDK注册ID：分配的appID
     * @param sdkKey      SDK注册Key:分配的appKey
     * @param callBack    SDK注册结果应答回调，注意：该回调异步执行返回！
     */
    public static boolean loadSDKCert(Object application, int sdkType, String sdkID, String sdkKey, byte[] sdkCert, String appPackage, String appName, final tztZFRegisterCallBack callBack) {
        return tztZFRegister.onRegisterSDKType(application, sdkType, sdkID, sdkKey, sdkCert, appPackage, appName, new tztZFRegisterInterface() {
            @Override
            public void onHandler(String respSDKID, final tztZFDataMapBase<String, Object> respData) {
                int nBlockType = respData.getInt("BlockType", 0);
                int nErrorNo = respData.getInt("ErrorNo", -1);
                String strErrorMessage = respData.getStr("ErrorMessage");
                int nSdkType = respData.getInt("SDKType", -1);
                int nDebug = respData.getInt("Debug", 0);
                int nUserID = respData.getInt("userID", 0);
                if (nBlockType == 1) {
                    if (nUserID >= 0 && nSdkType >= 0) {
                        tztZFSDKObj sdkObj = getSDKObjForUserID(nUserID, nSdkType);
                        sdkObj.debug = nDebug;
                        sdkObj.sdkID = respSDKID;
                        sdkObj.errorNo = nErrorNo;
                        sdkObj.errorMessage = strErrorMessage;
                    }
                }
                if (callBack != null) {
                    final HashMap<String, String> mapData = new HashMap<>();
                    respData.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                        @Override
                        public boolean enumerate(String lowKey, String key, Object value) {
                            if (value == null)
                                return false;
                            String strValue = tztZFDataMapBase.getStringUsingEncoding(value, respData.getEncodeType());
                            if (strValue != null)
                                mapData.put(lowKey, strValue);
                            return false;
                        }
                    });
                    callBack.onHandler(respSDKID, nErrorNo, mapData);
                }
            }
        });
    }

    //-注册通讯链接

    /**
     * 创建通讯链接
     *
     * @param linkKey     链接通道Key(使用时根据key获取对应链接)
     * @param userID      SDK的用户编号:userID
     * @param sdkType     SDK的类别
     * @param linkType    链接通道类别
     * @param connectType 连接类别(0:非SSL，1:SSL)，默认0
     */
    public static boolean registSocket(final Object context, String linkKey, int userID, int sdkType, int linkType, int connectType) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;connectType=%d;message=%s", linkKey, userID, sdkType, linkType, connectType, "linkKey为空");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        try {
            linkLock();
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;userID=%d;sdkType=%d;linkType=%d;connectType=%d;message=%s", linkKey, userID, sdkType, linkType, connectType, "注册通道");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_RegistLink, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
            }
            return getShareInstance().createSocket(context, linkKey, userID, sdkType, linkType, connectType);
        } finally {
            linkUnlock();
        }
    }

    public static boolean registSocket(final Object context, String linkKey, int userID, int sdkType, int linkType) {
        return registSocket(context, linkKey, userID, sdkType, linkType, 0);
    }

    //- 发送请求

    /**
     * 发送请求
     *
     * @param reqObj   请求
     * @param linkKey  linkKey 链接Key
     * @param callBack 回调接口
     */
    public static boolean requestLinkKey(String linkKey, tztZFNetWorkReqObj reqObj, final tztZFNetWorkResponseCallBack callBack) {
        String errMsg = "";
        int errorNo = -1;
        int reqObjTag = 0;
        if (reqObj != null)
            reqObjTag = reqObj.getReqObjTag();
        if (tztZFTextUtils.isEmpty(linkKey)) {
            errMsg = String.format("设置linkKey参数错误[%s]", linkKey);
            errorNo = tztZFNetWorkError_NoConnect;
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;message=%s", linkKey, errMsg);
                String param2 = (reqObj != null && reqObj.getSendValues() != null) ? tztZFRecordProcessLogTools.stringFormat("send json:%s", reqObj.getSendValues().toString()) : null;
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return responseErr(reqObj, reqObjTag, tztZFConn_Non.value(), errorNo, errMsg, callBack);
        }
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null) {
            errorNo = tztZFNetWorkError_NoConnect;
            errMsg = String.format("请先设置[%s]连接", linkKey);
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;message=%s", linkKey, errMsg);
                String param2 = (reqObj != null && reqObj.getSendValues() != null) ? tztZFRecordProcessLogTools.stringFormat("send json:%s", reqObj.getSendValues().toString()) : null;
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return responseErr(reqObj, reqObjTag, tztZFConn_Non.value(), errorNo, errMsg, callBack);
        }
        return poolSocket.OnSendReqModel(reqObj, callBack);
    }

    // - Setter
    public static void setLinkTag(String linkKey, String tag) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return;
        poolSocket.InitWithTag(tag);
    }

    /**
     * 设置服务器地址
     *
     * @param linkKey   linkKey 链接Key
     * @param linkBlock 地址列表
     */
    public static boolean setLinkBlock(String linkKey, tztZFLinkCallBack linkBlock) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return false;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return false;
        poolSocket.SetLinkBlock(linkBlock);
        return true;
    }

    /**
     * 设置服务器地址
     *
     * @param linkKey     linkKey 链接Key
     * @param addresslist 地址列表
     * @param selectindex 地址列表序号
     */
    public static boolean setLinkAddressList(String linkKey, String addresslist, int selectindex) {
        return setLinkAddressList(linkKey, addresslist, "", selectindex);
    }

    /**
     * 设置服务器地址
     *
     * @param linkKey     inkKey 链接Key
     * @param addressList 地址列表
     * @param nameList    地址名称列表
     * @param selectIndex 地址列表序号
     * @return 是否设置成功
     */
    public static boolean setLinkAddressList(String linkKey, String addressList, String nameList, int selectIndex) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "linkKey为空");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "未注册通道");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "设置地址");
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
            tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        poolSocket.SetAddressList(addressList, nameList, selectIndex);
        return true;
    }

    /**
     * 设置国密服务器地址
     *
     * @param linkKey     inkKey 链接Key
     * @param addressList 地址列表
     * @param nameList    地址名称列表
     * @param selectIndex 地址列表序号
     * @return 是否设置成功
     */
    public static boolean setLinkGMAddressList(String linkKey, String addressList, String nameList, int selectIndex) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "国密linkKey为空");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "未注册国密通道");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }
        if (tztZFRecordProcessLogTools.isRecord()) {
            String param = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;addressList=%s;nameList=%s;selectIndex=%d;message=%s", linkKey, addressList, nameList, selectIndex, "设置国密地址");
            ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param);
            tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Debug, params, "");
        }
        poolSocket.setGMAddressList(addressList, nameList, selectIndex);
        return true;
    }

    /**
     * 设置国密降级处理（不可恢复）
     *
     * @param linkKey 通道key
     */
    public static void setLinkKeyGMDowngrade(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket socket = getZFSocketForKey(linkKey);
        if (socket == null)
            return;
        socket.GMDowngradeConnect("业务调用降级");
    }

    /**
     * 设置国密绑定账号
     *
     * @param linkKey 通道key
     * @param userID  客户号
     * @param sslStep ssl连接回调
     */
    public static void setGMBindUserID(String linkKey, String userID, tztZFGMSSLStepCallBack sslStep) {
        setGMBindUserID(linkKey, userID, true, -1, sslStep);
    }

    public static void setGMBindUserID(String linkKey, String userID, boolean isSSL, int addressIndex, tztZFGMSSLStepCallBack sslStep) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket socket = getZFSocketForKey(linkKey);
        if (socket == null)
            return;
        socket.didGMBindUserID(userID, isSSL, sslStep);
    }

    /**
     * 国密协同签名错误处理
     *
     * @param linkKey      通道key
     * @param userID       客户号
     * @param errorNo      错误号
     * @param errorMessage 错误信息
     */
    public static void setGMCosignUserID(String linkKey, String userID, int errorNo, String errorMessage) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket socket = getZFSocketForKey(linkKey);
        if (socket == null)
            return;
        socket.GMSetCosignUserID(userID, errorNo, errorMessage);
    }

    public static void clearGMSSLStepCallback(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket socket = getZFSocketForKey(linkKey);
        if (socket == null)
            return;
        socket.clearGMSSLStepCallBack();
    }

    /**
     * 设置服务器连接超时时间
     *
     * @param timeOut 连接超时时间
     * @param linkKey linkKey 链接Key
     */
    public static void setConnectTimeOut(String linkKey, int timeOut) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return;
        if (timeOut < 100)
            timeOut = timeOut * 1000;
        poolSocket.SetSignleConnectOutTime(timeOut);
    }

    /**
     * 设置国密服务器连接超时时间
     *
     * @param timeOut 连接超时时间
     * @param linkKey linkKey 链接Key
     */
    public static void setGMConnectTimeOut(String linkKey, int timeOut) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return;
        if (timeOut < 100)
            timeOut = timeOut * 1000;
        poolSocket.SetGMConnectOutTime(timeOut);
    }

    /**
     * 设置服务器连接超时时间
     *
     * @param nMaxReq 连接超时时间
     * @param linkKey linkKey 链接Key
     */
    public static void setMaxReqCount(String linkKey, int nMaxReq) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return;
        poolSocket.SetMaxReqCount(nMaxReq);
    }

    // - Getter

    /**
     * 获取链接连接地址
     *
     * @param linkKey linkKey 链接Key
     */
    public static String getHostPortLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return "";
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return "";
        return poolSocket.GetSocketHostPort();//connectedAddress
    }

//    public static String getHostPortLinkKeyGM(String linkKey) {
//        if (tztZFTextUtils.isEmpty(linkKey))
//            return "";
//        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
//        if (poolSocket == null)
//            return "";
//        return poolSocket.GetSocketHostPortGM();//connectedAddress
//    }
//
//    public static String getHostPortLinkKeyPT(String linkKey) {
//        if (tztZFTextUtils.isEmpty(linkKey))
//            return "";
//        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
//        if (poolSocket == null)
//            return "";
//        return poolSocket.GetSocketHostPortPT();//connectedAddress
//    }

    /**
     * 获取链接成功地址
     *
     * @param linkKey linkKey 链接Key
     */
    public static String getSuccessedHostPortLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return "";
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return "";
        return poolSocket.GetSocketSuccessedHostPort();//connectedAddress
    }

    public static int getConnectTypeKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return 0;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return 0;
        return poolSocket.GetConnectType();
    }

    public static tztZFCosignUser getCosignUserKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return null;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return null;
        return poolSocket.GetCosignUser();
    }

    public static boolean getGmHaveCosignCertKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return false;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return false;
        return poolSocket.GetGmHaveCosignCert();
    }


    /**
     * 获取连接状态
     *
     * @param linkKey linkKey 链接Key
     */
    @tztZFSocketFlags
    public static int getConnectStateLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return tztZFSocketFlags_No;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return tztZFSocketFlags_No;
        return poolSocket.GetStockStep();
    }

    /**
     * 获取连接通道sdktype
     *
     * @param linkKey linkKey 链接Key
     */
    public static int getSDKTypeLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return -1;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return -1;
        return poolSocket.GetSDKType();
    }

    /**
     * 获取连接通道userID
     *
     * @param linkKey linkKey 链接Key
     */
    public static int getUserIDLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return -1;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return -1;
        return poolSocket.GetUserID();
    }

    /**
     * 获取连接通道类别linktype
     *
     * @param linkKey linkKey 链接Key
     */
    public static int getLinkTypeLinkKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return tztZFLinkType.tztZFLinkType_Non.value();
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return tztZFLinkType.tztZFLinkType_Non.value();
        return poolSocket.GetLinkType();
    }

    //- Free

    /**
     * 断开Key链接连接
     *
     * @param linkKey 链接Key
     */
    public static int disConnectionForKey(String linkKey) {
        return disConnectionForKey(linkKey, -1);
    }

    public static int disConnectionForKey(String linkKey, int connectIndex) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return 0;
        try {
            linkLock();
            return getShareInstance().doLinkForKey(linkKey, false, connectIndex);
        } finally {
            linkUnlock();
        }
    }

    public static int disConnectionALL() {
        try {
            linkLock();
            return getShareInstance().doLinkAll(false);
        } finally {
            linkUnlock();
        }
    }

    /**
     * 释放Key链接连接对象
     *
     * @param linkKey linkKey 链接Key
     */
    public static int freeLinkForKey(String linkKey) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return 0;
        try {
            linkLock();
            return getShareInstance().doLinkForKey(linkKey, true);
        } finally {
            linkUnlock();
        }
    }

    public static int freeLinkAll() {
        try {
            linkLock();
            return getShareInstance().doLinkAll(true);
        } finally {
            linkUnlock();
        }
    }

    //- Comm
    public static Set<String> allLinkKeys() {
        tztZFSocketLink netLink = getShareInstance();
        return netLink.linkMap.getAllLowKeys();
    }

    public static String linkKeyforUserID(int userID, int sdkType, int linkType) {
        if (userID < 0 || sdkType < 0 || linkType <= tztZFLinkType.tztZFLinkType_Non.value())
            return "";
        return String.format(Locale.CHINA, "tztLinkKey-%d-%d-%d", userID, sdkType, linkType);
    }

    //- 无连接应答
    public static boolean responseNoLink(tztZFNetWorkReqObj reqObj, final tztZFNetWorkResponseCallBack callBack) {
        int reqObjTag = 0;
        if (reqObj != null) {
            reqObjTag = reqObj.getReqObjTag();
        }
        return tztZFSocketLink.responseErr(reqObj, reqObjTag,
                tztZFConn_Non.value(), tztZFNetWorkError_NoConnect,
                tztZFErrorMessageDefine.getUserDefineError(tztZFNetWorkErrorKey_WriteError, tztZFWriteError), callBack);
    }

    //- 错误应答
    private static boolean responseErr(tztZFNetWorkReqObj reqObj, int reqObjTag, int recvType, int errorNo, String errorMessage, final tztZFNetWorkResponseCallBack callBack) {
        if (callBack != null) {
            callBack.OnError(reqObj, reqObjTag, recvType, errorNo, errorMessage);
            return true;
        }
        return false;
    }

    public static String setActionForErrorMessage(int errorNo, String action, String errorMessage) {
        return "[" + errorNo + "-" + action + "]" + errorMessage;
    }

    public static void setActionForErrorMessage(tztZFNetWorkRespObj respObj) {
        if (respObj == null)
            return;
        tztZFDataMapBase<String, Object> dataMap = respObj.getDataMap();
        if (dataMap == null)
            return;
        int errorNo = dataMap.getErrorNo();
        if (errorNo >= 0)
            return;
        String strAction = dataMap.getStr("Action");
        String strErrMsg = dataMap.getStr("ErrorMessage");
        strErrMsg = "[" + errorNo + "-" + strAction + "]" + strErrMsg;
        dataMap.setStr("ErrorMessage", strErrMsg);
        respObj.setMessage(strErrMsg);
    }

    public static void setStatisticPercent(int percent) {
        tztZFSocketLink.getShareInstance().usePercent = percent;
    }

    public static boolean getStatisticPercent() {
        return tztZFSocketLink.getShareInstance().useRandom < tztZFSocketLink.getShareInstance().usePercent;
    }

    public static void setSocketLink(String linkKey, String key, Object obj) {
        if (tztZFTextUtils.isEmpty(linkKey) || tztZFTextUtils.isEmpty(key))
            return;
        tztZFSocketObj socketObj = getSocketObjForKey(linkKey);
        if (socketObj == null)
            return;
        socketObj.setExValue(key, obj);
    }

    public static Object getSocketLink(String linkKey, String key) {
        if (tztZFTextUtils.isEmpty(linkKey) || tztZFTextUtils.isEmpty(key))
            return null;
        tztZFSocketObj socketObj = getSocketObjForKey(linkKey);
        if (socketObj == null)
            return null;
        return socketObj.getExValueforKey(key);
    }

    private static void tztlog(tztZFLogLevel level, String tag, String format, Object... args) {
        tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), "ZFSocketLink[" + tag + "]", format, args);
    }

    public static boolean setNoUserCertCallBackKey(String linkKey, tztZFNoUserCertCallBack noUserCertCallBack) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return false;
        tztZFSocket poolSocket = getZFSocketForKey(linkKey);
        if (poolSocket == null)
            return false;
        poolSocket.setNoUserCertCallBack(noUserCertCallBack);
        return true;
    }

    /**
     * ****************************************************************
     * 文件名称 : tztZFSocketObj
     * 作   者 : Created by wlz
     * 创建时间 : 12/30/20 10:05
     * 文件描述 :
     * 注意事项 :
     * 版权声明 : Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
     * 修改历史 : 12/30/20 1.00 初始版本
     * ****************************************************************
     */

    class tztZFSDKObj {
        String sdkID = "0";
        int userID = 0;
        int sdkType = 0;
        int debug = 0;
        int errorNo = 1;
        String errorMessage = "";

        tztZFSDKObj() {
        }
    }

    class tztZFSocketObj {
        String linkKey;
        tztZFSocket poolSocket;
        String sdkID;
        int userID;
        int linkType;
        int sdkType;
        tztZFDataMapBase<String, Object> exValueMap;

        tztZFSocketObj() {
            exValueMap = new tztZFDataMapBase<>();
        }

        void setExValue(String key, Object obj) {
            exValueMap.setObj(key, obj);
        }

        Object getExValueforKey(String key) {
            return exValueMap.getObj(key);
        }
    }
}