package com.tztzf.network.socket;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFErrorMessageDefine;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.tztZFNetWorkReqObj;
import com.tztzf.network.tztZFNetWorkRespObj;
import com.tztzf.protocol.core.tztZFSendPackage;
import com.tztzf.protocol.define.tztZFPackageType;

import java.util.Locale;
import java.util.concurrent.ThreadPoolExecutor;

import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoConnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Recv;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Send;

/*
 * ****************************************************************
 * 文件名称 : tztZFSocketSendData
 * 作   者 : Created by yangdl
 * 创建时间 : 202100309 17:06
 * 文件描述 :
 * 注意事项 :
 * 版权声明 : Copyright (C) 2021-2030 杭州中焯信息技术股份有限公司
 * 修改历史 : 202100309 1.00 初始版本
 * ****************************************************************
 */

/**
 * 发送数据包对象
 */
public class tztZFSocketSendData {
    //region 内部私有属性及函数
    //请求原始包
    tztZFNetWorkReqObj reqObj;
    //回调接口
    tztZFNetWorkResponseCallBack responseCallBack = null;
    //加入队列时间
    long addTime = 0;
    //发送的时间
    long sendTime = 0;
    //超时时间
    int recvTimeOut = 0;
    //发送对象标示，应答时原样返回
    int reqObjTag = 0;
    //请求数据包
    tztZFSendPackage sendPackage;
    //请求参数reqNo值
    String reqNo = "";
    //主推标记 暂不使用
    String pushKey = "";
    // 是否允许失败重发（重发一次）
    boolean isReSend = false;
    //是否未主推请求标志
    private boolean mIsRetainBlock = false;
    //endregion

    //region 具体实现

    /**
     * 构造函数
     *
     * @param reqObj      请求对象
     * @param packageType 数据包类型
     * @param socketKey   通道标示
     */
    public tztZFSocketSendData(tztZFNetWorkReqObj reqObj, @tztZFPackageType int packageType, String socketKey) {
        this.reqObj = reqObj;
        if (reqObj == null)
            return;
        setReqObjTag(reqObj.getReqObjTag());
        tztZFDataMapBase<String, Object> sendMap = reqObj.getSendValues();
        if (sendMap == null)
            return;
        this.reqNo = sendMap.getStr("Reqno");
        sendPackage = new tztZFSendPackage(sendMap, packageType, socketKey);
        setIsReSend(reqObj.isReSend());
        setIsRetainBlock(reqObj.isRetainBlock());
    }

    /**
     * 设置加入请求队列时间
     */
    public void setAddTime() {
        this.addTime = System.currentTimeMillis();
    }

    /**
     * 设置请求发送时间
     */
    public void setSendTime() {
        this.sendTime = System.currentTimeMillis();
    }

    public boolean isSendTimeOut(int timeout) {
        if (addTime > 0)
            return System.currentTimeMillis() - addTime >= timeout;
        return false;
    }

    public boolean isRecvTimeOut() {
        if (recvTimeOut <= 0) {
            recvTimeOut = 20 * 1000;
        }
        return System.currentTimeMillis() - sendTime >= recvTimeOut;
    }

    public boolean isBeforeSendTimeOut(long times) {
        if (times <= 0)
            return false;
        if (recvTimeOut <= 0) {
            recvTimeOut = 20 * 1000;
        }
        return sendTime - times >= recvTimeOut;
    }

    /// 获取请求应答耗时(ms)
    public long getUsedTime() {
        if (sendTime > 0) {
            long usedTime = System.currentTimeMillis() - sendTime;
            if (reqObj != null) {
                reqObj.setNativeStr("UsedTime", String.valueOf(usedTime));
            }
            return usedTime;
        }
        return 0;
    }

    /**
     * 设置请求标示
     */
    public void setReqObjTag(int reqObjTag) {
        this.reqObjTag = reqObjTag;
    }

    public int getReqObjTag() {
        return this.reqObjTag;
    }

    /**
     * 设置超时时间
     *
     * @param nTimeOut 超时时间
     */
    public void setTimeOut(int nTimeOut) {
        this.recvTimeOut = nTimeOut;
    }

    public int getTimeOut() {
        return this.recvTimeOut;
    }

    public boolean isReSend() {
        return isReSend;
    }

    public void setIsReSend(boolean reSend) {
        isReSend = reSend;
    }

    /**
     * 设置主推请求标志
     *
     * @param bRetainBlock 主推请求标志
     */
    public void setIsRetainBlock(boolean bRetainBlock) {
        mIsRetainBlock = bRetainBlock;
    }

    /**
     * 获取主推请求标志
     *
     * @return 主推请求标志
     */
    public boolean isRetainBlock() {
        return mIsRetainBlock;
    }

    /**
     * 设置回调接口
     *
     * @param callBack 回调接口
     */
    public void setResponseCallBack(tztZFNetWorkResponseCallBack callBack) {
        responseCallBack = callBack;
    }

    public tztZFNetWorkResponseCallBack getResponseCallBack() {
        return responseCallBack;
    }

    ///设置发送包序号
    public void setPackageIndex(int packageIndex) {
        sendPackage.setPackageIndex(packageIndex);
    }

    public void setType(@tztZFPackageType int type) {
        sendPackage.setPackageType(type);
    }

    @tztZFPackageType
    public int getType() {
        return sendPackage.getPackageType();
    }

    /// 获取发送包
    public tztZFSendPackage getPackage() {
        return sendPackage;
    }

    /// 获取发送的数据字典
    public tztZFDataMapBase<String, Object> getReqMapdata() {
        return sendPackage.getSendData();
    }

    /// 获取请求的req对象
    public tztZFNetWorkReqObj getReqObj() {
        return reqObj;
    }

    public int getAction() {
        return sendPackage.getAction();
    }

    public int getInt(String key, int defValue) {
        tztZFDataMapBase<String, Object> sendData = getReqMapdata();
        if (sendData != null) {
            return sendData.getInt(key, defValue);
        }
        return defValue;
    }

    public int getInt(String key) {
        return getInt(key, 0);
    }

    public void setInt(String key, int value) {
        tztZFDataMapBase<String, Object> sendData = getReqMapdata();
        if (sendData != null) {
            sendData.setInt(key, value);
        }
    }

    public String getStr(String key) {
        tztZFDataMapBase<String, Object> sendData = getReqMapdata();
        if (sendData != null) {
            return sendData.getStr(key);
        }
        return null;
    }

    public void setStr(String key, String value) {
        tztZFDataMapBase<String, Object> sendData = getReqMapdata();
        if (sendData != null) {
            sendData.setStr(key, value);
        }
    }

    public Integer getSendTag() {
        return sendPackage.getTag();
    }

    public String getReqno() {
        return reqNo;
    }

    //设置主推标示
    public void setPushKey(String pushKey) {
        this.pushKey = pushKey;
    }

    //获取主推标示
    public String getPushKey() {
        return pushKey;
    }

    /**
     * 删除请求参数
     *
     * @param key key
     */
    public void removeByKey(String key) {
        tztZFDataMapBase<String, Object> sendData = getReqMapdata();
        if (sendData != null) {
            sendData.removeByKey(key);
        }
    }

    public boolean isEqualReq(tztZFSocketSendData sendData) {
        if (sendData == null || sendData.getReqObj() == null)
            return false;
        return (sendData.getReqObj() == reqObj);
    }

    public String getErrorCode(tztZFRecvType type, int erroNo) {
        int errLevel = 1;
        int errPlace = 4;
        switch (type) {
            case tztZFConn_Connected:
            case tztZFRecv_DataOK:
                return "";
            case tztZFConn_Change:
            case tztZFConn_Non:
            case tztZFConn_DisConnect:
                errPlace = 4;
                break;
            case tztZFRecv_Error:
            case tztZFRecv_TimeOut:
                errPlace = 3;
                break;
            case tztZFSend_SendError:
            case tztZFSend_MaxError:
            case tztZFSend_TimeOut:
            case tztZFSend_SendCancel:
                errPlace = 1;
                break;
            default:
                break;
        }
        //ErrCode共4 位，分成三个部分：错误级别(ErrLevel)+错误产生的位置(ErrPlace)+两位编号(ErrNo)
        //ErrorTag+ErrorCode
        return String.format(Locale.CHINA, "N%d%d%02d", errLevel, errPlace, Math.abs(erroNo % 100));
    }

    private int getErrorNoOfRecvType(tztZFRecvType type) {
        switch (type) {
            case tztZFConn_Non:
                return tztZFNetWorkError_NoConnect;
            case tztZFConn_Change:
            case tztZFConn_DisConnect:
                return tztZFNetWorkError_Disconnect;
            case tztZFSend_MaxError:
            case tztZFSend_SendCancel:
            case tztZFSend_SendError:
            case tztZFSend_TimeOut:
                return tztZFNetWorkError_Send;
            case tztZFRecv_TimeOut:
            case tztZFRecv_Error:
                return tztZFNetWorkError_Recv;
            case tztZFConn_Connected:
            case tztZFRecv_DataOK:
                break;
        }
        return 0;
    }

    // 返回应答错误
    public boolean responseError(ThreadPoolExecutor respThread, final int linkType, final tztZFRecvType type, final int errorNo, String errorMessage, boolean bLocalError) {
        if (responseCallBack != null) {
            final int error = getErrorNoOfRecvType(type);
            String errorCode = getErrorCode(type, error);
            if (!tztZFTextUtils.isEmpty(errorMessage) && !tztZFTextUtils.isEmpty(errorCode) && tztZFErrorMessageDefine.tztZFShowCustomErrorCode) {
                errorMessage = String.format("%s[%s]", errorMessage, errorCode);
            }
            tztZFDataMapBase<String, Object> dataMap = new tztZFDataMapBase<>();
            dataMap.setInt("ErrorNo", error);
            dataMap.setStr("ErrorMessage", errorMessage);
            dataMap.setInt("Action", getAction());
            reqObj.setLocalError(bLocalError);
            final String respMsg = errorMessage;
            if (respThread != null) {
                respThread.execute(new tztZFThreadRespRunnable(linkType, 1, dataMap) {
                    @Override
                    public void run() {
                        responseCallBack.OnError(reqObj, reqObjTag, type.value(), error, respMsg);
                    }
                });
            } else {
                responseCallBack.OnError(reqObj, reqObjTag, type.value(), error, respMsg);
            }
            return true;
        }
        return false;
    }

    // 返回应答数据流
    public boolean responseData(ThreadPoolExecutor respThread, final int linkType, tztZFRecvType type, boolean bUsedTime, tztZFDataMapBase<String, Object> dataMap) {
        if (responseCallBack != null) {
            if (dataMap == null) {
                dataMap = new tztZFDataMapBase<>();
                dataMap.setInt("ErrorNo", 0);
                dataMap.setStr("ErrorMessage", "");
                dataMap.setInt("Action", getAction());
            } else if (dataMap.getInt("Action") == 0) {
                dataMap.setInt("Action", getAction());
            }
            if (!tztZFTextUtils.isEmpty(reqNo)) {
                dataMap.setStr("ReqNo", reqNo);
            }
            if (bUsedTime)
                dataMap.setStr("tztUsedTime", reqObj.getNativeStr("UsedTime"));
            tztZFNetWorkRespObj respObj = reqObj.convertToResponse(dataMap);
            try {
                respObj.setRecvType(type);
                respObj.setReqObj(reqObj);
                respObj.setReqObjTag(reqObjTag);
                final tztZFNetWorkRespObj resp = respObj;
                if (respThread != null) {
                    respThread.execute(new tztZFThreadRespRunnable(linkType, 1, dataMap) {
                        @Override
                        public void run() {
                            responseCallBack.OnAns(reqObj, reqObjTag, resp);
                        }
                    });
                } else {
                    responseCallBack.OnAns(reqObj, reqObjTag, resp);
                }
                return true;
            } catch (Exception e) {
                e.printStackTrace();
                tztZFLog.log(tztZFLogLevel.tztZFLogLevel_error, "tztZFSocketSendData", "%s", tztZFLog.getStackTraceString(e));
            }
        }
        return false;
    }
    //endregion

    //region 开发新增处理代码

    //endregion

}