package com.tztzf.network.socket;

import android.util.Log;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.cosign.tztZFCosignCert;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.cosign.tztZFNetCosign;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.protocol.callback.tztZFBlocktztSSL;
import com.tztzf.protocol.core.tztZFSSLCert;
import com.tztzf.protocol.core.tztZFVPNSSL;
import com.tztzf.protocol.define.tztZFSSLCallbackType;
import com.tztzf.tztZFNotificationObj;
import com.tztzf.tztZFPrivate;

import org.greenrobot.eventbus.EventBus;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.Locale;
import java.util.concurrent.ExecutorService;

import static com.tztzf.network.gm.tztZFGMManage.tztZFJYLoginGMProcessStep;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepClose;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepConnect;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepHand;
import static com.tztzf.network.gm.tztZFSocketSSLStep.tztZFSocketSSLStepUnServerCert;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_Disconnect;
import static com.tztzf.network.socket.Netty.tztZFNetWorkError.tztZFNetWorkError_NoConnect;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Hand;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Read;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_Write;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_close;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_d2iSession;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_err;
import static com.tztzf.protocol.define.tztZFSSLCallbackType.tztZFSSLCallback_i2dSession;

/**
 * 作者: yangdl 2021.02.23
 * 用途: 基础的请求应答类，封装了网络通讯，调用者不需要关心网络通讯与业务层协议
 * 修改记录如下:
 * <p>
 * 版本号: 3.00.001 2021.02.26
 * 更新说明: 第一版正式发布
 */

class tztZFThreadPoolGCDAsyncSocket {
    private volatile boolean m_bRun = false; // 通讯线程是否运行
    //连接关闭状态
    private int m_nSocketFlag = 0;
    private long m_nStartTime = 0;
    private Object context = null;
    //线程池
    private ExecutorService m_pThreadPool = null;

    private String m_sAddress = ""; // 连接地址
    private int m_nPort = 7778; // 连接端口
    private boolean m_isSSL = false;

    //proxy
    private boolean m_isProxy = false; //是否wap代理,现在的wap一般也可以建立socket连接的,所以先尝试socket连接。
    private String m_sProxyAdd = "10.0.0.172";
    private int m_nProxyPort = 0;

    private String m_sIndex = "0";
    //通讯相关入参
    private String m_sSocketKey = "";
    //证书ID
    private String m_sSdkID = "0";
    //证书用户标示
    private int m_nUserID = 0;
    //证书类别
    private int m_nSdkType = 0;
    //连接通道
    private int m_nLinkType = tztZFLinkType.tztZFLinkType_HQ.value();
    private int m_nLastError = 0;
    private String m_sLastErrorMsg = "";

    //Socket
    private Socket m_pSocket = null;
    private InputStream m_pMyInputStream = null;
    private OutputStream m_pMyOutputStream = null;

    /** 时间设置,单位毫秒 */
    private int m_nOneConnectOutTime = 3 * 1000; // 连接超时时间,单位毫秒;m_Socket.connect的超时

    //最近应答时间
    private long m_nLastRecTime = 0;
    //最近请求时间
    private long m_nLastSendTime = 0;
    //连接状态
    private boolean m_bConnected = false;
    //回调接口
    private tztZFThreadPoolGCDAsyncSocketEventsInterface m_pEvents = null;

    //新增ssl
    private tztZFSSLCert mSSLCert = new tztZFSSLCert();

    int tztZF_Connecttype;

    private boolean tztZF_usessl;
    private boolean isSSLSucc;

    private tztZFVPNSSL tztZF_vpnssl;
    private String tztZF_CosignUserID;
    private int tztZF_LinkType;

    private boolean isAjaxServer;
    Object userData;
    private tztZFGMSSLStepCallBack GMSSLStep;
    private tztZFCosignUser cosignUser;
    private String sslGMUserID;
    //
    public static final int tztZFStep_init = 1 << 1; //初始
    public static final int tztZFStep_Connect = 1 << 2;//连接开始
    public static final int tztZFStep_Connected = 1 << 3;//连接成功
    public static final int tztZFStep_ConnectFail = 1 << 4;//连接失败
    public static final int tztZFStep_Closing = 1 << 5;//准备关闭连接
    public static final int tztZFStep_Close = 1 << 6;//关闭连接
    public static final int tztZFStep_Closed = 1 << 7;//关闭完成

    //=====tztZFGCDAsyncSocketThreadPool 构造=======================================================

    /**
     * 初始化构造 tztZFGCDAsyncSocketThreadPool
     */
    tztZFThreadPoolGCDAsyncSocket() {
    }

    //=====tztZFGCDAsyncSocketThreadPool 设置属性=======================================================

    /**
     * 设置属性
     *
     * @param context 上下文
     * @return tztZFGCDAsyncSocketThreadPool
     */
    tztZFThreadPoolGCDAsyncSocket InitWithContext(Object context) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.context = context;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    tztZFThreadPoolGCDAsyncSocket InitWithIndex(String index) {
        this.m_sIndex = index;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param socketKey 上下文
     * @return tztZFGCDAsyncSocketThreadPool
     */
    tztZFThreadPoolGCDAsyncSocket InitWithSocketKey(String socketKey) {
        this.m_sSocketKey = socketKey;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param threadPool 线程池
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithThreadPool(ExecutorService threadPool) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_pThreadPool = threadPool;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param userID userID
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithUserID(int userID) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_nUserID = userID;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param sdktype sdktype
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithSdktype(int sdktype) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_nSdkType = sdktype;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param linktype linktype
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithLinktype(int linktype) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_nLinkType = linktype;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param address address
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithAddress(String address) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_sAddress = address;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param port port
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithPort(int port) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_nPort = port;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param isSSL isSSL
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithIsSSL(boolean isSSL) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_isSSL = isSSL;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param isProxy isProxy
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithIsProxy(boolean isProxy) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_isProxy = isProxy;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param proxyAdd proxyAdd
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithProxyAdd(String proxyAdd, int proxyPort) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_sProxyAdd = proxyAdd;
        this.m_nProxyPort = proxyPort;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param outtime outtime
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithConnectOutTime(int outtime) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        if (outtime < 1 * 1000)
            return tztZFThreadPoolGCDAsyncSocket.this;

        this.m_nOneConnectOutTime = outtime;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    /**
     * 设置属性
     *
     * @param events events
     * @return tztZFGCDAsyncSocketThreadPool
     */
    public tztZFThreadPoolGCDAsyncSocket InitWithEvents(tztZFThreadPoolGCDAsyncSocketEventsInterface events) {
        if (m_bRun)
            return tztZFThreadPoolGCDAsyncSocket.this;

        setEvents(events);
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    void setEvents(tztZFThreadPoolGCDAsyncSocketEventsInterface events) {
        this.m_pEvents = events;
    }
    //=====tztZFGCDAsyncSocketThreadPool 获取属性=======================================================

    /**
     * 获取socket的hashcode值
     *
     * @return socket的hashcode值
     */
    public final int GetSocketID() {
        if (m_pSocket == null) {
            return 0;
        } else {
            return m_pSocket.hashCode();
        }
    }

    /**
     * 获取当前地址端口
     *
     * @return 地址端口
     */
    public final String GetAddressPort() {
        if (!tztZFTextUtils.isEmpty(m_sAddress) && m_nPort > 0) {
            return m_sAddress + ":" + m_nPort;
        }
        return "";
    }

    /**
     * 获取当前连接状态
     *
     * @return 连接状态 true 已连接成功，false 未连接成功
     */
    public final boolean IsConnected() {
        return (haveStep(tztZFStep_Connected)) && (!IsClosing());
    }

    public final boolean IsClosing() {
        return (haveStep(tztZFStep_Closing) || haveStep(tztZFStep_Close) || haveStep(tztZFStep_Closed));
    }

    public final boolean IsClosed() {
        return (m_nSocketFlag & tztZFStep_Closed) == tztZFStep_Closed;
    }

    public final boolean IsTimeOut() {
        if (m_nStartTime <= 0 || m_nOneConnectOutTime <= 0)
            return false;
        return System.currentTimeMillis() - m_nStartTime > m_nOneConnectOutTime;
    }
    //=====tztZFGCDAsyncSocket 操作API=======================================================

    /**
     * 连接
     *
     * @return 连接 true 发起连接，false 发起连接失败
     */
    public final boolean Connect() {
        tztlog("Connect", "%s", " Connect");
        if (context == null || m_pThreadPool == null) {
            m_nLastError = tztZFNetWorkError_NoConnect;
            m_sLastErrorMsg = "请设置连接";
            closeAsyncSocket(m_sLastErrorMsg);
            return false;
        }

        if (tztZFTextUtils.isEmpty(m_sAddress) || m_nPort <= 0) {
            m_nLastError = tztZFNetWorkError_NoConnect;
            m_sLastErrorMsg = "请设置连接服务器地址";
            closeAsyncSocket(m_sLastErrorMsg);
            return false;
        }

        if (m_nLinkType <= tztZFLinkType.tztZFLinkType_Non.value()) {
            m_nLastError = tztZFNetWorkError_NoConnect;
            m_sLastErrorMsg = "请设置连接通道";
            closeAsyncSocket(m_sLastErrorMsg);
            return false;
        }
        m_pThreadPool.execute(new Runnable() {
            @Override
            public void run() {
                socketConnect();
            }
        });
        return true;
    }

    /**
     * 连接
     *
     * @return 连接及握手处理
     */
    protected boolean socketConnect() {
        try {
            tztlog("Connect", "%s", " SyncRun");
            m_bRun = true;
            if (haveStep(tztZFStep_Closing)) {
                return false;
            }
            m_nSocketFlag |= tztZFStep_init;
            m_nStartTime = System.currentTimeMillis();
            if (m_pSocket == null || !m_pSocket.isConnected() || m_pSocket.isOutputShutdown() || m_pSocket.isInputShutdown()) {
                if (m_pSocket == null) {
                    m_pSocket = new Socket();
//                    if(false == m_isSSL){
//                        m_pSocket = new Socket();
//                    }
//                    else {
//                        try {
//                            m_pSocket = new tztZFEasySSLSocketFactory().createSocket();
//                        } catch (IOException e) {
//                            tztlog("SSLConnect",  " Error=" + e.getMessage());
//                        }
//                    }
                }
                m_nSocketFlag |= tztZFStep_Connect;
                tztlog("Connect", "%s", " Get_InetSocketAddress");
                InetSocketAddress add;
                if (!m_isProxy) {
                    add = new InetSocketAddress(m_sAddress, m_nPort);
                    tztlog("Connect", " 发起服务器连接,超时时间:%d", m_nOneConnectOutTime);
                } else {
                    add = new InetSocketAddress(m_sProxyAdd, m_nProxyPort);
                    tztlog("Connect", " 发起Proxy服务器连接,超时时间:%d", m_nOneConnectOutTime);
                }
                //已经断开连接，不需要再去做连接
                if (haveStep(tztZFStep_Closing)) {
                    tztlog("Connect", " Closing,超时时间:%d", m_nOneConnectOutTime);
                    m_nSocketFlag |= tztZFStep_ConnectFail;
                    m_nStartTime = 0;
                    return false;
                }
                //开始连接
                if (!m_isProxy) {
                    m_pSocket.setTcpNoDelay(true);
                }
                m_pSocket.connect(add, m_nOneConnectOutTime);
                m_nStartTime = 0;
                try {
                    m_pMyInputStream = m_pSocket.getInputStream();
                } catch (IOException e) {
                    tztlog("Connect", " InIOExceptError=%s", tztZFLog.getStackTraceString(e));
                }
                try {
                    m_pMyOutputStream = m_pSocket.getOutputStream();
                } catch (IOException e) {
                    tztlog("Connect", " OutIOExceptError=%s", tztZFLog.getStackTraceString(e));
                }
                if (tztZF_usessl && tztZF_vpnssl != null) {
                    try {
                        int result = tztZF_vpnssl.ssl_connect(m_pSocket);
                        if (result < 0) {
                            //错误，释放
                            m_nSocketFlag |= tztZFStep_init;
                            m_nStartTime = 0;
                            tztZF_freeZFSSL();
                            return false;
                        }
                    } catch (Exception e) {
                        tztlog("SSLConnect", " ExceptError=%s", tztZFLog.getStackTraceString(e));
                    }
                }
                tztlog("Connect", " OK=%s", (System.currentTimeMillis() - m_nStartTime) + "");
                //连接成功
                m_nSocketFlag |= tztZFStep_Connected;
                if (m_pEvents != null) {
                    m_pEvents.didConnectToHost(tztZFThreadPoolGCDAsyncSocket.this, m_sAddress, m_nPort);
                }
                return true;
            }
        } catch (Exception e) {
            //置为连接失败
            m_nSocketFlag |= tztZFStep_ConnectFail;
            m_nLastError = tztZFNetWorkError_Disconnect;
            m_sLastErrorMsg = "连接失败:" + e;
            tztlog("Connect", " ExceptError=%s", tztZFLog.getStackTraceString(e));
            closeAsyncSocket(m_sLastErrorMsg);
        } finally {
            if (haveStep(tztZFStep_Closing)) { //处于关闭状态
                closeAsyncSocket(m_sLastErrorMsg);
            } else if (haveStep(tztZFStep_Connected)) { //连接成功
                return true;
            }
            return false;
        }
    }

    private final boolean haveStep(int step) {
        return (m_nSocketFlag & step) == step;
    }

    //外部开放接口，关闭连接
    public synchronized void Close(String msg) {
        tztlog("Close", " 关闭连接(%s)", msg);
        closeAsyncSocket(msg);
    }

    //内部接口
    private void closeAsyncSocket(String msg) {
        //断开连接回调
        disconnSocket(msg);
        //断开Socket
        closeSocket(msg);
    }

    private void disconnSocket(String msg) {
        m_nStartTime = 0;
        m_sLastErrorMsg = msg;
        if (m_pEvents != null) {
            m_pEvents.socketDidDisconnect(tztZFThreadPoolGCDAsyncSocket.this, m_nLastError, m_sLastErrorMsg);
            m_pEvents = null;
        }
        m_nSocketFlag |= tztZFStep_Closing;
    }

    private void closeSocket(String msg) {
        //已经关闭，不再处理
        if (haveStep(tztZFStep_Close)) {
            return;
        }
        //准备关闭连接
        tztlog("Close", " Closing!(%s)", msg);
        //已经初始化，连接未结束,等待连接结束
        if (haveStep(tztZFStep_init) && !haveStep(tztZFStep_ConnectFail) && !haveStep(tztZFStep_Connected)) {
            return;
        }
        //开始关闭连接
        try {
            tztlog("Close", " Close begin!(%s)", msg);
            m_nSocketFlag |= tztZFStep_Close;
            if (m_pSocket != null) {
                try {
                    m_pSocket.shutdownInput();
                } catch (Exception e) {
                }
                try {
                    m_pSocket.shutdownOutput();
                } catch (Exception e) {
                }
            }

            if (m_pMyInputStream != null) {
                try {
                    m_pMyInputStream.close();
                } catch (Exception e) {
                }
                m_pMyInputStream = null;
            }

            if (m_pMyOutputStream != null) {
                try {
                    m_pMyOutputStream.close();
                } catch (Exception e) {
                }
                m_pMyOutputStream = null;
            }

            if (m_pSocket != null) {
                try {
                    m_pSocket.close();
                } catch (Exception e) {
                }
                tztlog("Close", " Closed OK!(%s)", msg);
            } else {
                tztlog("Close", " Closed!(%s)", msg);
            }

            if (tztZF_vpnssl != null) {
                tztZF_freeZFSSL();
            }
            m_pSocket = null;
        } finally {
            m_nSocketFlag |= tztZFStep_Closed;
        }
    }

    /**
     * 发送请求
     *
     * @return true 发送成功，false 发送失败
     */
    public final boolean Write(byte[] data) throws IOException {
        if (data == null || data.length < 6)
            return false;

        if (!IsConnected()) {
            return false;
        }

        if (tztZF_vpnssl != null && tztZF_usessl) {
            if (tztZF_vpnssl.ssl_write(data) <= 0) {
                return false;
            }
        } else {
            if (m_isProxy) {
                String Header = addHttpHeader(data.length, m_sAddress, Integer.toString(m_nPort));
                m_pMyOutputStream.write(Header.getBytes());
            }
            m_pMyOutputStream.write(data);
            m_pMyOutputStream.flush();
            tztlog("Write", " len:" + data.length);
        }
        return true;
    }

//    public final int Read() throws IOException {
//        if (tztZF_usessl && tztZF_vpnssl != null) {
//            //FIXME: 暂时取1024的长度
//            int size = 4096;
//            byte[] temp = new byte[size];
//            int result = tztZF_vpnssl.ssl_read(temp, size);
//            return result;
//        } else {
//            int nRead = m_pMyInputStream.read();
//            tztlog("Read", " len:" + nRead);
//            return nRead;
//        }
//    }

    public final int ReadData(byte[] data, int len) throws IOException {
        if (tztZF_usessl && tztZF_vpnssl != null) {
            int nRead = m_pMyInputStream.read(data, 0, len);
            if (nRead < 1) return nRead;
            int ret = tztZF_vpnssl.ssl_read(data, nRead);
            if (ret >= 0) return 0;
            return -1;
        } else {
            int nRead = m_pMyInputStream.read(data, 0, len);
            tztlog("Read", " len:" + nRead);
            return nRead;
        }
    }

//    public final int Read(byte[] data, int index, int len) throws IOException {
//        if (tztZF_usessl && tztZF_vpnssl != null) {
//            int nResult = 0;
//            int totalSize = Math.max(len, 4096);
//            while (totalSize > 0) {
//                int size = Math.min(totalSize, 4096);
//                byte[] temp = new byte[size];
//                int result = tztZF_vpnssl.ssl_read(temp, size);
//                if (result > 0) {
//                    System.arraycopy(result, 0, data, index + nResult, Math.min(len, result));
//                    nResult += result;
//                }
//                totalSize -= size;
//            }
//            tztlog("TZTSSL", "vpnssl_read len:" + nResult);
//            return nResult;
//        } else {
//            int nRead = m_pMyInputStream.read(data, index, len);
//            tztlog("Read", " len:%d", nRead);
//            return nRead;
//        }
//    }

    //=====Http代理请求========
    private final String addHttpHeader(int len, String addr, String port) {
        String User_Agent = "Nokia6610/1.0 (5.52) Profile/MIDP-1.0 Configuration/CLDC-1.0";
        return "POST / HTTP/1.1\r\n" + "Connection: Keep-Alive\r\n" + "Accept: application/octet-stream, */*\r\n" + "Content-Type: application/octet-stream\r\n" + "User-Agent: " + User_Agent + "\r\n"
                + "X-Online-Host: " + addr + ":" + port + "\r\n" + "Host: " + addr + ":" + port + "\r\n" + "Content-Length: " + len + "\r\n\r\n";
    }

    /**
     * 打印日志
     */
    private void tztlog(String tag, String format, Object... arg) {
        if (tztZFLog.getLevel() <= tztZFLogLevel.tztZFLogLevel_Debug.value()) {
            String cTag = "AsyncSocket[" + this.hashCode() + "]" + m_nSdkType + "-" + tztZFLinkType.nameOf(m_nLinkType);
            String addFormat = "[%s][%s][%s][%s:%d]";
            String str = String.format(Locale.CHINA, addFormat, tag, Thread.currentThread().getName(), m_sIndex, m_sAddress, m_nPort);
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Debug, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(),
                    cTag, str + format, arg);
        }
    }


    ///region
    public tztZFThreadPoolGCDAsyncSocket withZFSSL_UserID(String userID) {
        this.sslGMUserID = userID;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    public tztZFThreadPoolGCDAsyncSocket withZFSSL_CA(byte[] ca, int type) {
        mSSLCert.setCertCA(ca);
        mSSLCert.setCertCAType(type);
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    public tztZFThreadPoolGCDAsyncSocket withZFSSL_Enc(byte[] enc, byte[] encKey, int type, int certType) {
        mSSLCert.setCertEnc(enc);
        mSSLCert.setCertEncKey(encKey);
        mSSLCert.setCertEncType(type);
        mSSLCert.setCertType(certType);
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    public tztZFThreadPoolGCDAsyncSocket withZFSSL_Sign(byte[] sign, byte[] signKey, int type) {
        mSSLCert.setCertSign(sign);
        mSSLCert.setCertSignKey(signKey);
        mSSLCert.setCertSignType(type);
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    public tztZFThreadPoolGCDAsyncSocket withZFSSL_LinkCosignUserID(int linkType, String cosignUserID) {
        this.tztZF_CosignUserID = cosignUserID;
        this.tztZF_LinkType = linkType;
        return tztZFThreadPoolGCDAsyncSocket.this;
    }

    private boolean tztZF_setClientCertSignFile() {
        return false;
    }

    private boolean tztZF_setClientCertEncFile() {
        return false;
    }

    private boolean tztZF_setClientCertSign() {
        if (mSSLCert.getCertSignType() == 0)
            return tztZF_setClientCertSignFile();
        return false;
    }

    private boolean tztZF_setClientCertEnc() {
        if (mSSLCert.getCertEncType() == 0)
            return tztZF_setClientCertEncFile();
        return false;
    }

    private boolean tztZF_setClientCert() {
        return false;
    }

    public boolean tztZF_SetConnecttype(int connectType) {
        tztZF_Connecttype = connectType;
        tztZF_usessl = (connectType > 0);
        if (!tztZF_usessl)
            return true;

        //读取，加载证书

        if (!isAjaxServer) {
//            if (tztZF_Connecttype == 9) {
////                tztZF_loadCertWithSSLVPN();
//            } else if (tztZF_Connecttype == 8) {
////                tztZF_loadCertWithNetCosign();
//            }
            return tztZF_setSSLVPN(connectType);
        }
        return true;
    }

    public void setGMSSLStepCallBack(tztZFGMSSLStepCallBack callBack) {
        this.GMSSLStep = callBack;
    }

    private boolean tztZF_setSSLVPN(final int connectType) {
        if (isAjaxServer || !tztZF_usessl)
            return false;
        if (tztZF_vpnssl == null) {
            tztZF_vpnssl = new tztZFVPNSSL();
            tztZF_vpnssl.initWithConnectType(connectType, new tztZFBlocktztSSL() {
                @Override
                public int onHandler(Object socket, @tztZFSSLCallbackType int type, int code, byte[] data, int len) {
                    //处理数据的收发
                    //和当前使用的socket进行比较
                    if (socket != m_pSocket)
                        return 0;
                    int nRet = len;
                    String logtype = String.format(Locale.getDefault(), "%d", type);
                    switch (type) {
                        case tztZFSSLCallback_Hand: {
                            if (GMSSLStep != null) {
                                GMSSLStep.onCallBack(tztZFSocketSSLStepHand, code, getCertUser());
                            } else {
                                tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                notificationObj.setName(tztZFJYLoginGMProcessStep);
                                tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                dataMapBase.setInt("step", tztZFSocketSSLStepHand);
                                String userID = getCertUser();
                                if (userID == null)
                                    userID = "";
                                dataMapBase.setStr("userID", userID);
                                notificationObj.setData(dataMapBase);
                                EventBus.getDefault().post(notificationObj);
                            }
                            logtype = "Hand";
                            nRet = 0;
                        }
                        break;
                        case tztZFSSLCallback_Write: {
                            logtype = "Write";
                            try {
                                m_pMyOutputStream.write(data);
                                m_pMyOutputStream.flush();
                            } catch (Exception e) {
                                nRet = -1;
                                m_sLastErrorMsg = "连接失败:" + e;
                                tztlog("TZTSSL", "Write wrong, ExceptError=%s", tztZFLog.getStackTraceString(e));
                            }
                        }
                        break;
                        case tztZFSSLCallback_Read: {
                            logtype = "Read";
                            try {
                                if (m_pEvents != null) {
                                    nRet = m_pEvents.socketReadData(tztZFThreadPoolGCDAsyncSocket.this, data, len);
                                }
                            } catch (Exception e) {
                                nRet = -1;
                                m_sLastErrorMsg = "连接失败:" + e;
                                tztlog("TZTSSL", "Read wrong, ExceptError=%s", tztZFLog.getStackTraceString(e));
                            }
                        }
                        break;
                        case tztZFSSLCallback_i2dSession: {
                            logtype = "SaveSession";
                            if (data != null && len > 0) {
                                tztZF_SaveSSLSession(data, len);
                            } else {
                                tztlog("TZTSSL", "socket[%s]->saveSession, data is empty!", socket.toString());
                            }
                        }
                        break;
                        case tztZFSSLCallback_d2iSession: {
                            logtype = "RestoreSession";
                            byte[] sslSession = tztZF_RestoreSSLSession();
                            if (sslSession != null) {
                                int dataLen = sslSession.length;
                                if (dataLen > 0 && dataLen < len) {
                                    System.arraycopy(sslSession, 0, data, 0, dataLen);
                                    nRet = dataLen;
                                }
                            } else {
                                tztlog("TZTSSL", "socket[%s]->restoreSession, data is empty", socket.toString());
                            }
                            if (GMSSLStep != null) {
                                GMSSLStep.onCallBack(tztZFSocketSSLStepConnect, code, getCertUser());
                            } else {
                                tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                notificationObj.setName(tztZFJYLoginGMProcessStep);
                                tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                dataMapBase.setInt("step", tztZFSocketSSLStepConnect);
                                String userID = getCertUser();
                                if (userID == null)
                                    userID = "";
                                dataMapBase.setStr("userID", userID);
                                notificationObj.setData(dataMapBase);
                                EventBus.getDefault().post(notificationObj);
                            }
                        }
                        break;
                        case tztZFSSLCallback_err: {
                            logtype = "ERR";
                            String error = "";
                            if (data != null && data.length > 0) {
                                try {
                                    error = new String(data);
                                } catch (Exception e) {
                                    error = "";
                                }
                            }
                            /**
                             * SSL3_AD_CERTIFICATE_REVOKED = 44 证书注销
                             * SSL3_AD_CERTIFICATE_EXPIRED = 45 证书过期
                             */
                            int errCode = (code & 0xFF);
                            if (errCode == 44 || errCode == 45) {
                                //删除本地证书
                                if (sslGMUserID != null && sslGMUserID.length() > 0) {
                                    int nError = -1;
                                    if (errCode == 44) {
                                        nError = -3;
                                    } else if (errCode == 45) {
                                        nError = -2;
                                    }
                                    tztZFNetCosign.expiredCert(sslGMUserID, nError);
                                }
                                if (GMSSLStep != null) {
                                    GMSSLStep.onCallBack(tztZFSocketSSLStepUnServerCert, code, tztZFThreadPoolGCDAsyncSocket.this.getCertUser());
                                } else {
                                    tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                    notificationObj.setName(tztZFJYLoginGMProcessStep);
                                    tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                    dataMapBase.setInt("step", tztZFSocketSSLStepUnServerCert);
                                    String userID = getCertUser();
                                    if (userID == null)
                                        userID = "";
                                    dataMapBase.setStr("userID", userID);
                                    notificationObj.setData(dataMapBase);
                                    EventBus.getDefault().post(notificationObj);
                                }
                            }
                            nRet = -1;
                        }
                        break;
                        case tztZFSSLCallback_close: {
                            logtype = "CLose";
                            debugPrintCosignUserInfo();
                            if (GMSSLStep != null) {
                                GMSSLStep.onCallBack(tztZFSocketSSLStepClose, code, getCertUser());
                            } else {
                                tztZFNotificationObj notificationObj = new tztZFNotificationObj();
                                notificationObj.setName(tztZFJYLoginGMProcessStep);
                                tztZFDataMapBase<String, Object> dataMapBase = new tztZFDataMapBase<>();
                                dataMapBase.setInt("step", tztZFSocketSSLStepClose);
                                String userID = getCertUser();
                                if (userID == null)
                                    userID = "";
                                dataMapBase.setStr("userID", userID);
                                notificationObj.setData(dataMapBase);
                                EventBus.getDefault().post(notificationObj);
                            }
                            nRet = -1;
                        }
                        break;
                        default:
                            break;
                    }

                    tztlog("tztZFSSL", "[%s][%d][tztZFSSL_%s]socket[%d]->vpnssl_%s[len:%d][code:%d][ret:%d]",
                            m_sSocketKey != null ? m_sSocketKey : "", connectType, logtype, socket.hashCode(), logtype, len, code, nRet);
                    return nRet;
                }
            });
            tztZF_vpnssl.ssl_setSSLCert(mSSLCert);
        }
        return tztZF_vpnssl.ssl_init() > 0;
    }

    /**
     * 测试使用，外层禁止调用
     *
     * @param cosignUser 用户对象
     */
    @tztZFPrivate(name = "测试使用")
    public void setCosignUserInfo(tztZFCosignUser cosignUser) {
        this.cosignUser = cosignUser;
    }

    public String getCertUser() {
        return this.sslGMUserID;
    }

    /**
     * 连接失败时打印用户证书信息，主要用于测试排查问题使用，外层禁止调用
     */
    private void debugPrintCosignUserInfo() {
        if (tztZFLog.getLevel() > tztZFLogLevel.tztZFLogLevel_Debug.value())
            return;
        //失败时，打印下证书相关信息，其中证书内容做base64处理
        tztZFCosignCert cert = null;
        if (this.cosignUser != null) {
            cert = this.cosignUser.getLoginCosignCert();
        }

        String strLog = String.format(Locale.getDefault(), "\n" +
                        "------------连接使用安全码相关信息----------------\n" +
                        "%s\n" +
                        "---------------------------------------\n\n",
                mSSLCert.toString());
        Log.d("安全码信息", strLog);
        Log.d("安全码相关信息：", cert != null ? cert.toString() : "null");
    }

    public boolean tztZF_GetUseSSL() {
        return tztZF_usessl;
    }

    private void tztZF_freeZFSSL() {
        if (tztZF_vpnssl != null) {
            tztZF_vpnssl.ssl_close();
            tztZF_vpnssl = null;
        }
    }

    public void tztZF_SSLFree() {
        tztZF_freeZFSSL();
    }

//    private String tztZF_SSLSessionFile() {
//        String fileName = String.format(Locale.CHINA, "SSLSESSION_%d_%s.data", tztZF_Connecttype, "");
//        return tztZFPublicFun.getFileDocumentPath(context, fileName);
//    }

    private int tztZF_SaveSSLSession(byte[] session, int len) {
        if (session == null || session.length < 1)
            return 0;
        return tztZFNetCosign.saveSSLSession(session, tztZF_CosignUserID, tztZF_Connecttype, tztZF_LinkType);
    }

    private byte[] tztZF_RestoreSSLSession() {
        return tztZFNetCosign.restoreSSLSession(tztZF_CosignUserID, tztZF_Connecttype, tztZF_LinkType);
    }
    ///endregion
}
