package com.tztzf.network;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.callback.tztZFEncryptCallBack;
import com.tztzf.network.callback.tztZFGMLinkDelegate;
import com.tztzf.network.callback.tztZFGMSSLStepCallBack;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.callback.tztZFNoUserCertCallBack;
import com.tztzf.network.callback.tztZFRegisterCallBack;
import com.tztzf.network.callback.tztZFSocketLinkDelegate;
import com.tztzf.network.cosign.tztZFCosignUser;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.define.tztZFSocketFlags;
import com.tztzf.network.socket.tztZFSocketLink;
import com.tztzf.recordprocesslog.tztZFRecordEventType;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogCallback;
import com.tztzf.recordprocesslog.tztZFRecordProcessLogTools;
import com.tztzf.recordprocesslog.tztZFReocordLogType;
import com.tztzf.tztZFPrivate;

import java.util.ArrayList;
import java.util.HashMap;

import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_JY;
import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Non;

/**
 * ****************************************************************
 * 文件名称:TztNetWork.tztZZNetWork.java
 * 作    者:Created by pjliang
 * 创建时间:2018/9/21 11:08
 * 文件描述:通讯对外公布的接口
 * 注意事项:
 * 版权声明:Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史:2018/9/21 1.00 初始版本
 * ****************************************************************
 */

public class tztZFNetWork {
    /** tztNetWork的单例 */
    protected static tztZFNetWork ins;

    private tztZFSocketLinkDelegate linkDelegate;
    /** 错误信息是否显示功能号, 默认显示 */
    private boolean showActionForErrorMessage = true;

    private int tztUserID;
    private int tztSDKType;
    private String tztSDKID;
    private tztZFGMLinkDelegate gmLinkDelegate;
    private tztZFEncryptCallBack encryptCallBack;

    /**
     * 构造函数，并获取保存的数据
     */
    public tztZFNetWork() {
        initData();
    }

    private void initData() {
        tztSDKID = "0";
        tztUserID = 0;
        tztSDKType = 0;
    }

    /**
     * 获取sdkid
     *
     * @return sdkid
     */
    public static String linkSDKID() {
        return getShareInstance().tztSDKID;
    }

    /**
     * 获取userid
     *
     * @return userid
     */
    public static int linkUserID() {
        return getShareInstance().tztUserID;
    }

    /**
     * 获取sdktype
     *
     * @return sdktype
     */
    public static int linkSDKType() {
        return getShareInstance().tztSDKType;
    }

    /**
     * 根据链接通道类别获取链接通道key
     *
     * @param linkType 链接通道类别
     * @return 链接通道key
     */
    public static String keyForLinkType(int linkType) {
        return tztZFSocketLink.linkKeyforUserID(linkUserID(), linkSDKType(), linkType);
    }

    /**
     * 获取tztZFNetWork实例
     *
     * @return tztZFNetWork实例
     */
    private static tztZFNetWork getShareInstance() {
        if (ins == null) {
            ins = new tztZFNetWork();
        }
        return ins;
    }

    // - registerApp

    /**
     * APP证书注册，该函数需要在程序开始就调用
     *
     * @param appID      注册申请分配的appID
     * @param appKey     注册申请分配的appkey
     * @param certData   注册申请分配的证书
     * @param appPackage 程序包名
     * @param appName    程序应用名
     * @param options    其他信息
     * @param callBack   注册结果应答回调，注意：该回调异步执行返回！
     */
    public static boolean registerApp(Object application, String appID, String appKey, byte[] certData, String appPackage, String appName, tztZFDataMapBase<String, Object> options, final tztZFRegisterCallBack callBack) {
        int sdkType = 0;
        getShareInstance().tztSDKType = sdkType;
        getShareInstance().tztSDKID = appID;
        return tztZFSocketLink.loadSDKCert(application, sdkType, appID, appKey, certData, appPackage, appName, new tztZFRegisterCallBack() {
            @Override
            public void onHandler(String respAppid, int respError, HashMap<String, String> options) {
                String strBlocktype = options.get("blocktype");
                int nBlocktype = tztZFPublicFun.parseInt(strBlocktype, -1);
                if (nBlocktype == 1) {
                    String strErrorNo = options.get("errorno");
                    String strSdkType = options.get("sdktype");
                    String strUserID = options.get("userid");
                    int nErrorNo = tztZFPublicFun.parseInt(strErrorNo, 0);
                    int nSdkType = tztZFPublicFun.parseInt(strSdkType, 0);
                    int nUserID = tztZFPublicFun.parseInt(strUserID, 0);
                    if (nErrorNo > 0 && nUserID >= 0 && nSdkType >= 0) {
                        getShareInstance().tztSDKID = respAppid;
                        getShareInstance().tztUserID = nUserID;
                        getShareInstance().tztSDKType = nSdkType;
                    } else {
                        getShareInstance().tztUserID = 0;
                        getShareInstance().tztSDKType = 0;
                    }
                }
                if (callBack != null) {
                    callBack.onHandler(respAppid, respError, options);
                }
            }
        });
    }

    /**
     * 设置发送前应答后回调
     *
     * @param linkDelegate 发送前应答后回调
     */
    public static void setSocketLinkDelegate(tztZFSocketLinkDelegate linkDelegate) {
        getShareInstance().linkDelegate = linkDelegate;
    }

    /**
     * 获取发送前应答后回调（Android主要在兼容tztZZNetWork使用）
     *
     * @return 发送前应答后回调
     */
    public static tztZFSocketLinkDelegate getSocketLinkDelegate() {
        return getShareInstance().linkDelegate;
    }

    /**
     * 设置错误信息是否显示功能号
     *
     * @param showActionForErrorMessage 错误信息是否显示功能号
     */
    public static void setShowActionForErrorMessage(boolean showActionForErrorMessage) {
        getShareInstance().showActionForErrorMessage = showActionForErrorMessage;
    }

    /**
     * 获取错误信息是否显示功能号
     */
    public static boolean getShowActionForErrorMessage() {
        return getShareInstance().showActionForErrorMessage;
    }

    public static tztZFGMLinkDelegate getGmLinkDelegate() {
        return getShareInstance().gmLinkDelegate;
    }

    public static void setGmLinkDelegate(tztZFGMLinkDelegate gmLinkDelegate) {
        getShareInstance().gmLinkDelegate = gmLinkDelegate;
    }

    /**
     * 注册链接通道
     *
     * @param context  上下文
     * @param linkType 链接通道类别
     * @return 是否注册成功
     */
    public static boolean registLink(Object context, int linkType) {
        return registLink(context, linkType, 0);
    }

    public static boolean registLink(Object context, int linkType, int connectType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.registSocket(context, linkKey, linkUserID(), linkSDKType(), linkType, connectType);
    }

    /**
     * 注册链接通道
     *
     * @param context  上下文
     * @param linkKey  链接通道key
     * @param linkType 链接通道类别
     * @return 注册后的链接通道key
     */
    public static String registLinkKey(Object context, String linkKey, int linkType) {
        return registLinkKey(context, linkKey, linkType, 0);
    }

    public static String registLinkKey(Object context, String linkKey, int linkType, int connectType) {
        String strKey = linkKey;
        if (tztZFTextUtils.isEmpty(linkKey)) {
            strKey = keyForLinkType(linkType);
        }
        boolean bOk = tztZFSocketLink.registSocket(context, strKey, linkUserID(), linkSDKType(), linkType, connectType);
        if (bOk)
            return strKey;
        return "";
    }

    /**
     * 发送数据请求
     *
     * @param linkType 链接通道类别,注意是linktype
     * @param reqObj   请求具体参数数据
     * @param callBack 发送请求的回调
     * @return 是否加入待发送队列
     */
    public static boolean sendDataAction(int linkType, tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        String linkKey = keyForLinkType(linkType);
        return requestLinkKey(linkKey, reqObj, callBack);
    }

    /**
     * 发送数据请求
     *
     * @param linkKey  链接通道key
     * @param reqObj   请求具体参数数据
     * @param callBack 发送请求的回调
     * @return 是否加入待发送队列
     */
    public static boolean sendDataActionLinkKey(String linkKey, tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        return requestLinkKey(linkKey, reqObj, callBack);
    }

    /**
     * 设置socket 日志 Tag
     *
     * @param linkType 链接通道类别
     * @param tag      日志tag
     */
    public static void setLinkTag(int linkType, String tag) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkTag(linkKey, tag);
    }

    /**
     * 设置socket 日志 Tag
     *
     * @param linkKey 链接通道key
     * @param tag     日志key
     */
    public static void setLinkTagLinkKey(String linkKey, String tag) {
        tztZFSocketLink.setLinkTag(linkKey, tag);
    }

    /**
     * 设置链接通道回调
     *
     * @param linkType  链接通道类别
     * @param linkBlock 通道回调
     */
    public static void setLinkBlock(int linkType, tztZFLinkCallBack linkBlock) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkBlock(linkKey, linkBlock);
    }

    /**
     * 设置链接通道回调
     *
     * @param linkKey   链接通道key
     * @param linkBlock 通道回调
     */
    public static void setLinkBlockLinkKey(String linkKey, tztZFLinkCallBack linkBlock) {
        tztZFSocketLink.setLinkBlock(linkKey, linkBlock);
    }

    /**
     * 设置链接通道服务器地址
     *
     * @param linkType    链接通道类别
     * @param addList     服务器地址列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressList(int linkType, String addList, int selectIndex) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkAddressList(linkKey, addList, selectIndex);
    }

    /**
     * 设置链接通道服务器地址
     *
     * @param linkType    链接通道类别
     * @param addressList 服务器地址列表
     * @param nameList    服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressList(int linkType, String addressList, String nameList, int selectIndex) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkAddressList(linkKey, addressList, nameList, selectIndex);
    }

    /**
     * 设置国密服务器地址
     *
     * @param linkType    链接通道类别
     * @param addressList 服务器地址列表
     * @param nameList    服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setGMAddressList(int linkType, String addressList, String nameList, int selectIndex) {
        if (linkType != tztZFLinkType.tztZFLinkType_JY.value()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linktype=%d;message=%s", linkType, "设置国密地址时通道错误");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(keyForLinkType(linkType), tztZFRecordEventType.tztZFRecordEventType_SetAddressList, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return;
        }
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkGMAddressList(linkKey, addressList, nameList, selectIndex);
    }

    /**
     * 设置链接通道服务器地址
     *
     * @param linkKey     链接通道key
     * @param addList     服务器地址列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressListLinkKey(String linkKey, String addList, int selectIndex) {
        tztZFSocketLink.setLinkAddressList(linkKey, addList, selectIndex);
    }

    /**
     * 设置链接通道服务器地址
     *
     * @param linkKey     链接通道key
     * @param addressList 服务器地址列表
     * @param nameList    服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressListLinkKey(String linkKey, String addressList, String nameList, int selectIndex) {
        tztZFSocketLink.setLinkAddressList(linkKey, addressList, nameList, selectIndex);
    }

    /**
     * 设置国密服务器地址
     *
     * @param linkKey     链接通道key
     * @param addressList 服务器地址列表
     * @param nameList    服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setGMAddressListLinkKey(String linkKey, String addressList, String nameList, int selectIndex) {
        tztZFSocketLink.setLinkGMAddressList(linkKey, addressList, nameList, selectIndex);
    }

    public static void setLinkTypeGMDowngrade(int linkType) {
        if (linkType != tztZFLinkType_JY.value()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linktype=%d;message=%s", linkType, "设置国密降级通道错误");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(keyForLinkType(linkType), tztZFRecordEventType.tztZFRecordEventType_GMDemote, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return;
        }
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkKeyGMDowngrade(linkKey);
    }

    public static void setGMBindUserID(int linkType, String userID, tztZFGMSSLStepCallBack sslStep) {
        setGMBindUserID(linkType, userID, true, -1, sslStep);
    }

    public static void setGMBindUserID(int linkType, String userID, boolean isSSL, int addressIndex, tztZFGMSSLStepCallBack sslStep) {
        if (linkType != tztZFLinkType_JY.value()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linktype=%d;message=%s", linkType, "设置国密绑定账号通道错误");
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1);
                tztZFRecordProcessLogTools.recordLog(keyForLinkType(linkType), tztZFRecordEventType.tztZFRecordEventType_BindUserID, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return;
        }
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setGMBindUserID(linkKey, userID, isSSL, addressIndex, sslStep);
    }
    
    public static void clearGMSSLStepCallback(int linkType) {
        tztZFSocketLink.clearGMSSLStepCallback(keyForLinkType(linkType));
    }

    public static void setGMCosignUserID(int linkType, String userID, int errorNo, String errorMessage) {
        if (linkType != tztZFLinkType_JY.value())
            return;
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setGMCosignUserID(linkKey, userID, errorNo, errorMessage);
    }

    /**
     * 设置待发送队列最大个数
     *
     * @param linkType 链接通道类别
     * @param nMaxReq  待发送队列最大个数
     */
    @tztZFPrivate
    public static void setMaxReqCount(int linkType, int nMaxReq) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setMaxReqCount(linkKey, nMaxReq);
    }

    /**
     * 设置待发送队列最大个数
     *
     * @param linkKey 链接通道key
     * @param nMaxReq 待发送队列最大个数
     */
    @tztZFPrivate
    public static void setMaxReqCountLinkKey(String linkKey, int nMaxReq) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocketLink.setMaxReqCount(linkKey, nMaxReq);
    }

    /**
     * 设置socket连接超时时间，超时时间必须大于1才有效，默认3s
     *
     * @param linkType 链接通道类别
     * @param nTimeOut 超时时间
     */
    public static void setConnectTimeOut(int linkType, int nTimeOut) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 设置socket连接超时时间，超时时间必须大于1才有效，默认3s
     *
     * @param linkKey  链接通道key
     * @param nTimeOut 超时时间
     */
    public static void setConnectTimeOutLinkKey(String linkKey, int nTimeOut) {
        tztZFSocketLink.setConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 设置国密双向socket连接超时时间，超时时间必须大于1才有效，默认3s
     *
     * @param linkType 链接通道类别
     * @param nTimeOut 超时时间
     */
    public static void setGMConnectTimeOut(int linkType, int nTimeOut) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setGMConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 设置国密双向socket连接超时时间，超时时间必须大于1才有效，默认3s
     *
     * @param linkKey  链接通道key
     * @param nTimeOut 超时时间
     */
    public static void setGMConnectTimeOutLinkKey(String linkKey, int nTimeOut) {
        tztZFSocketLink.setGMConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 获取用户证书
     * @param linkType 通道类型
     * @return 用户证书
     */
    public static tztZFCosignUser getCosignUser(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return getCosignUserLinkKey(linkKey);
    }

    /**
     * 获取用户证书
     * @param linkKey 通道类型
     * @return 用户证书
     */
    public static tztZFCosignUser getCosignUserLinkKey(String linkKey) {
        return tztZFSocketLink.getCosignUserKey(linkKey);
    }

    /**
     * 获取是否有国密证书
     * @param linkType 通道类型
     * @return 是否有国密证书
     */
    public static boolean getGmHaveCosignCert(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return getGmHaveCosignCertLinkKey(linkKey);
    }

    /**
     * 获取是否有国密证书
     * @param linkKey 通道类型
     * @return 是否有国密证书
     */
    public static boolean getGmHaveCosignCertLinkKey(String linkKey) {
        return tztZFSocketLink.getGmHaveCosignCertKey(linkKey);
    }

    /**
     * 获取通道当前链接服务器地址
     *
     * @param linkType 链接通道类别
     * @return 当前链接服务器地址
     */
    public static String getLinkHostPort(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getHostPortLinkKey(linkKey);
    }

//    /**
//     * 获取普通通道当前链接服务器地址
//     * @param linkType 链接通道类别
//     * @return 当前链接服务器地址
//     */
//    public static String getLinkHostPortPT(int linkType) {
//        String linkKey = keyForLinkType(linkType);
//        return tztZFSocketLink.getHostPortLinkKeyPT(linkKey);
//    }
//
//    /**
//     * 获取国密通道当前链接服务器地址
//     * @param linkType 链接通道类别
//     * @return 当前链接服务器地址
//     */
//    public static String getLinkHostPortGM(int linkType) {
//        String linkKey = keyForLinkType(linkType);
//        return tztZFSocketLink.getHostPortLinkKeyGM(linkKey);
//    }

    /**
     * 获取通道当前链接成功服务器地址
     *
     * @param linkType 链接通道类别
     * @return 当前链接成功服务器地址
     */
    public static String getLinkSuccessedHostPort(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getSuccessedHostPortLinkKey(linkKey);
    }

    /**
     * 获取通道的连接类型，仅交易通道有效
     *
     * @param linkType 通道类型
     * @return 通道对应连接类型，仅交易有效双向返回9，单向返回0
     */
    public static int getConnectType(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return getConnectTypeLinkKey(linkKey);
    }

    /**
     * 获取通道的连接类型，仅交易通道有效
     *
     * @param linkKey 通道类型
     * @return 通道对应连接类型，仅交易有效双向返回9，单向返回0
     */
    public static int getConnectTypeLinkKey(String linkKey) {
        return tztZFSocketLink.getConnectTypeKey(linkKey);
    }


    /**
     * 获取通道当前链接地址
     *
     * @param linkKey 链接通道key
     * @return 当前链接服务器地址
     */
    public static String getLinkHostPortLinkKey(String linkKey) {
        return tztZFSocketLink.getHostPortLinkKey(linkKey);
    }

//    /**
//     * 获取普通通道当前链接服务器地址
//     *
//     * @param linkKey 链接通道key
//     * @return 当前链接服务器地址
//     */
//    public static String getLinkHostPortPTLinkKey(String linkKey) {
//        return tztZFSocketLink.getHostPortLinkKeyPT(linkKey);
//    }
//
//    /**
//     * 获取国密通道当前链接服务器地址
//     *
//     * @param linkKey 链接通道key
//     * @return 当前链接服务器地址
//     */
//    public static String getLinkHostPortGMLinkKey(String linkKey) {
//        return tztZFSocketLink.getHostPortLinkKeyGM(linkKey);
//    }

    /**
     * 获取通道当前链接成功服务器地址
     *
     * @param linkKey 链接通道key
     * @return 当前链接成功服务器地址
     */
    public static String getLinkSuccessedHostPortLinkKey(String linkKey) {
        return tztZFSocketLink.getSuccessedHostPortLinkKey(linkKey);
    }

    /**
     * 获取链接状态
     *
     * @param linkType 链接通道类别
     * @return 链接状态
     */
    @tztZFSocketFlags
    public static int getSocketConnectState(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getConnectStateLinkKey(linkKey);
    }

    /**
     * 获取链接状态
     *
     * @param linkKey 链接通道key
     * @return 链接状态
     */
    @tztZFSocketFlags
    public static int getSocketConnectStateLinkKey(String linkKey) {
        return tztZFSocketLink.getConnectStateLinkKey(linkKey);
    }

    /**
     * 断开通道连接
     *
     * @param linkType 链接通道类别
     */
    public static void disConnection(int linkType) {
        disConnection(linkType, -1);
    }

    /**
     * 断开通道连接
     *
     * @param linkType     链接通道类别
     * @param connectIndex 下次连接地址索引
     */
    public static void disConnection(int linkType, int connectIndex) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.disConnectionForKey(linkKey, connectIndex);
    }

    /**
     * 断开通道链接
     *
     * @param linkKey 通道key
     */
    public static void disConnectionLinkKey(String linkKey) {
        disConnectionLinkKey(linkKey, -1);
    }

    /**
     * 断开通道连接
     *
     * @param linkKey      通道key
     * @param connectIndex 下次连接地址索引
     */
    public static void disConnectionLinkKey(String linkKey, int connectIndex) {
        tztZFSocketLink.disConnectionForKey(linkKey, connectIndex);
    }

    /**
     * 断开所有通道链接
     */
    public static void disConnectAllLink() {
        tztZFSocketLink.disConnectionALL();
    }

    /**
     * 释放指定的通讯对象
     *
     * @param linkType 链接通道类别
     */
    public static void freeLink(int linkType) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.freeLinkForKey(linkKey);
    }

    /**
     * 释放指定的通讯对象
     *
     * @param linkKey 链接类别
     */
    public static void freeLinkKey(String linkKey) {
        tztZFSocketLink.freeLinkForKey(linkKey);
    }

    /**
     * 释放所有链接对象，包括断开连接
     */
    public static void freeAllLink() {
        tztZFSocketLink.freeLinkAll();
    }

    /**
     * 设置用户未设置证书回调
     * @param linkType 连接类型
     * @param noUserCertCallBack 回调
     * @return 是否设置成功
     */
    public static boolean setNoUserCertCallBack(int linkType, tztZFNoUserCertCallBack noUserCertCallBack) {
        String linkKey = keyForLinkType(linkType);
        return setNoUserCertCallBackLinkKey(linkKey, noUserCertCallBack);
    }

    /**
     * 设置用户未设置证书回调
     * @param linkKey 连接类型
     * @param noUserCertCallBack 回调
     * @return 是否设置成功
     */
    public static boolean setNoUserCertCallBackLinkKey(String linkKey, tztZFNoUserCertCallBack noUserCertCallBack) {
        return tztZFSocketLink.setNoUserCertCallBackKey(linkKey, noUserCertCallBack);
    }

    /**
     * 获取加解密回调
     *
     * @return 加解密回调
     */
    public static tztZFEncryptCallBack getEncryptCallBack() {
        return getShareInstance().encryptCallBack;
    }

    /**
     * 设置加解密回调
     *
     * @param encryptCallBack 加解密回调
     */
    public static void setEncryptCallBack(tztZFEncryptCallBack encryptCallBack) {
        getShareInstance().encryptCallBack = encryptCallBack;
    }

    /**
     * 设置是否记录通讯流程日志
     * @param context Context上下文
     * @param bSwitch 是否记录 true=记录；false=不记录
     * @param callback 回调
     */
    public static void setRecordSwitch(Object context, boolean bSwitch, tztZFRecordProcessLogCallback callback) {
        tztZFRecordProcessLogTools.setRecordSwitch(context, bSwitch, callback);
    }

    // -内部统一处理函数
    private static boolean requestLinkKey(final String linkKey, tztZFNetWorkReqObj reqObj, final tztZFNetWorkResponseCallBack callBack) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;message=%s", linkKey, "未设置linkKey");
                String param2 = convertRequestParams(reqObj);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return tztZFSocketLink.responseNoLink(reqObj, callBack);
        }

        final int linkType = tztZFSocketLink.getLinkTypeLinkKey(linkKey);
        if (linkType <= tztZFLinkType_Non.value()) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;message=%s", linkKey, "未设置通道");
                String param2 = convertRequestParams(reqObj);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return tztZFSocketLink.responseNoLink(reqObj, callBack);
        }

        //发送请求前，统一通过代理回调上层，可以进行业务数据的统一补充处理
        boolean continueSend = true;//是否继续发送请求
        final tztZFSocketLinkDelegate linkDelegate = getShareInstance().linkDelegate;
        if (linkDelegate != null) {
            continueSend = linkDelegate.beforeSendData(reqObj, callBack, linkType, linkKey);
        }
        if (!continueSend) {
            if (tztZFRecordProcessLogTools.isRecord()) {
                String param1 = tztZFRecordProcessLogTools.stringFormat("linkKey=%s;linkType=%d;message=%s", linkKey, linkType, "上层中断请求");
                String param2 = convertRequestParams(reqObj);
                ArrayList<Object> params = tztZFRecordProcessLogTools.covertParams(param1, param2);
                tztZFRecordProcessLogTools.recordLog(linkKey, tztZFRecordEventType.tztZFRecordEventType_SendDataAction, tztZFReocordLogType.tztZFReocordLogType_Error, params, "");
            }
            return false;
        }

        return tztZFSocketLink.requestLinkKey(linkKey, reqObj, new tztZFNetWorkResponseCallBack() {
            @Override
            public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                if (c_errorNo < 0 && c_reqObj != null && getShowActionForErrorMessage()) {
                    c_errorMsg = tztZFSocketLink.setActionForErrorMessage(c_errorNo, c_reqObj.getString("Action"), c_errorMsg);
                }

                if (linkDelegate != null) {
                    tztZFNetWorkRespObj respObj = new tztZFNetWorkRespObj();
                    respObj.setMessage(c_errorMsg);
                    respObj.setRecvType(tztZFRecvType.valueOf(c_recvType));
                    respObj.setStatusCode(c_errorNo);
                    if (c_reqObj != null) {
                        respObj.setReqObj(c_reqObj);
                    }
                    //应答接收后，统一回调上层进行处理，可以用于特殊业务的错误判断等逻辑
                    linkDelegate.afterReceiveData(respObj, linkType, linkKey);
                }
                if (callBack != null) {
                    callBack.OnError(c_reqObj, c_reqObjTag, c_recvType, c_errorNo, c_errorMsg);
                }
            }

            @Override
            public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                //处理错误应答逻辑并打印应答数据
                if (tztZFNetWork.getShowActionForErrorMessage()) {
                    tztZFSocketLink.setActionForErrorMessage(c_respObj);
                }

                if (linkDelegate != null) {
                    linkDelegate.afterReceiveData(c_respObj, linkType, linkKey);
                }
                if (callBack != null) {
                    callBack.OnAns(c_reqObj, c_reqObjTag, c_respObj);
                }
            }
        });
    }

    private static String convertRequestParams(tztZFNetWorkReqObj reqObj) {
        if (reqObj != null && reqObj.getSendValues() != null) {
            return String.format("send json:%s", reqObj.getSendValues().toString());
        }
        return null;
    }
}