package com.tztzf.network;

import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFLinkType;

/**
 * ****************************************************************
 * 文件名称 : tztZFNetWorkReqObj
 * 作   者 : Created by wlz
 * 创建时间 : 2019-12-18 15:53
 * 文件描述 : 请求
 * 注意事项 :
 * 版权声明 : Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史 : 2019-12-18 1.00 初始版本
 * ****************************************************************
 */
public class tztZFNetWorkReqObj {

    //region 内部私有属性及函数

    //请求内容
    private tztZFDataMapBase<String, Object> mData = new tztZFDataMapBase<>();
    //设置参数
    private final tztZFDataMapBase<String, Object> mSet = new tztZFDataMapBase<>();
    //请求对象标志
    private int mReqObjTag;
    private int mTimeOut = 0; //增加每个请求的独立超时时间, 0表示使用link的缺省值
    private boolean mIsRetainBlock = false;
    // 是否允许失败重发（重发一次）
    private boolean isReSend = false;
    //报错数据是否为app层报错，非服务端报错
    private boolean bLocalError;
    //endregion

    //region 具体实现

    //region 公布方法（允许其他包访问）

    //region native函数用于本地设置标识，内部内容不发往服务器端

    /**
     * 设置本地参数值
     * @param key 参数
     * @param value 值
     */
    public void setNativeInt(String key, int value) {
        mSet.setInt(key, value);
    }

    /**
     * 获取本地参数值
     * @param key 参数
     */
    public int getNativeInt(String key) {
        return mSet.getInt(key);
    }

    /**
     * 设置参数值
     * @param key 参数
     * @param value 值
     */
    public void setNativeStr(String key, String value) {
        mSet.setStr(key, value);
    }

    /**
     * 获取设置参数值
     * @param key 参数
     */
    public String getNativeStr(String key) {
        return mSet.getStr(key);
    }

    /**
     * 设置参数值
     * @param key 参数
     * @param object 值
     */
    public void setNativeObj(String key, Object object) {
        if (key != null && object != null) {
            mSet.setObj(key, object);
        }
    }

    /**
     * 获取设置参数值
     * @param key 参数
     */
    public Object getNativeObj(String key) {
        return mSet.getObj(key);
    }

    //endregion

    /**
     * 设置超时时间
     * @param nTimeOut 超时时间
     */
    public void setTimeOut(int nTimeOut) {
        mTimeOut = nTimeOut;
    }

    /**
     * 设置主推请求标志
     * @param bRetainBlock 主推请求标志
     */
    public void setIsRetainBlock(boolean bRetainBlock) {
        mIsRetainBlock = bRetainBlock;
    }

    /**
     * 获取主推请求标志
     * @return 主推请求标志
     */
    public boolean isRetainBlock() {
        return mIsRetainBlock;
    }

    /**
     * 设置请求对象标志
     * @param nReqObjTag 请求对象标志
     */

    public void setReqObjTag(int nReqObjTag) {
        mReqObjTag = nReqObjTag;
    }

    /**
     * 获取请求标志
     * @return 请求对象标志
     */
    public int getReqObjTag() {
        return mReqObjTag;
    }

    /**
     * 设置请求参数
     * @param sendValue 请求参数
     */
    public void setSendMap(tztZFDataMapBase<String, Object> sendValue) {
        //避免上层设置空时，获取值异常
        if (sendValue == null) {
            mData = new tztZFDataMapBase<>();
        } else {
            mData = sendValue;
        }
    }

    /**
     * 设置字典
     * @param key key
     * @param dataMapBase 字典
     */
    public void setDataMap(String key, tztZFDataMapBase<String, Object> dataMapBase) {
        mData.setDataMap(key, dataMapBase);
    }

    /**
     * 设置字符串数据
     * @param key key值
     * @param value 字符串数据
     */
    public void setStr(String key, String value) {
        mData.setStr(key, value);
    }

    /**
     * 设置数据流数据
     * @param key key值
     * @param value 数据流数据
     */
    public void setBytes(String key, byte[] value) {
        mData.setBytes(key, value);
    }

    /**
     * 设置整型数据
     * @param key key值
     * @param value 整型数据
     */
    public void setInt(String key, int value) {
        mData.setInt(key, value);
    }

    /**
     * 是否允许重发
     * @return 是否允许重发（true=允许；false=不允许）
     */
    public boolean isReSend() {
        return isReSend;
    }

    /**
     * 设置是否允许重发
     * @param reSend 是否允许重发（true=允许；false=不允许）
     */
    public void setIsReSend(boolean reSend) {
        isReSend = reSend;
    }

    public boolean isLocalError() {
        return bLocalError;
    }

    public void setLocalError(boolean bLocalError) {
        this.bLocalError = bLocalError;
    }

    //endregion

    //region 保护方法（允许子类访问、允许相同包中访问）
    //endregion

    //region 内部方法（只允许相同包中访问）

    /**
     * 获取字符串数据
     * @param key key值
     * @return 字符串数据
     */
    public String getString(String key) {
        return mData.getStr(key);
    }

    /**
     * 获取数据流数据
     * @param key key值
     * @return 数据流数据
     */
    public byte[] getBytes(String key) {
        return mData.getBytes(key);
    }

    /**
     * 获取整型数据
     * @param key key值
     * @return 整型数据
     */
    public int getInt(String key) {
        return mData.getInt(key);
    }

    /**
     * 获取超时时间
     * @return 超时时间
     */
    public int getTimeOut() {
        return mTimeOut;
    }

    /**
     * 获取请求数据
     * @return 请求数据
     */
    public tztZFDataMapBase<String, Object> getSendValues() {
        return mData;
    }

    /**
     * 请求数据
     * @param linkType 链接通道类别
     * @param callBack 应答回调
     */
    public void sendRequest(tztZFLinkType linkType, tztZFNetWorkResponseCallBack callBack) {
        tztZFNetWork.sendDataAction(linkType.value(), this, callBack);
    }

    /**
     * 请求数据并且返回发送结果
     *
     * @param linkType 链接通道类别
     * @param callBack 应答回调
     * @return true 是否成功发送
     */
    public boolean sendRequestWithResult(int linkType, tztZFNetWorkResponseCallBack callBack) {
        return tztZFNetWork.sendDataAction(linkType, this, callBack);
    }

    /**
     * 转换应答数据结构
     * @param recvData 应答数据
     * @return 应答数据结构
     */
    public tztZFNetWorkRespObj convertToResponse(tztZFDataMapBase<String, Object> recvData) {
        tztZFNetWorkRespObj respObj = new tztZFNetWorkRespObj();
        respObj.setReqObj(this);
        if (recvData != null) {
            respObj.setStatusCode(recvData.getInt("errorno"));
            respObj.setMessage(recvData.getStr("errormessage"));
        }
        respObj.setResponseData(recvData);
        return respObj;
    }

    //endregion

    //region 私有方法（只允许自己调用）
    //endregion

    //endregion

    //region 开发新增处理代码

    //endregion

}