package com.tztzf.network;

import static com.tztzf.network.define.tztZFRecvType.tztZFConn_Non;
import static com.tztzf.network.define.tztZFRecvType.tztZFSend_SendCancel;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFThreadPool;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.log.tztZFLogPrintType;
import com.tztzf.network.callback.tztZFNetPoolsCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.define.tztZFLinkType;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.socket.tztZFThreadPoolSocket;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class tztZFPoolsShortNet {
    private Object context;
    private String m_sAddlist;
    private String[] m_ayList;

    private int m_nLinkType;
    private int m_nNum;
    private int m_nTimeOut;
    private int m_nConnTimeOut = 3 * 1000;
    private tztZFNetWorkReqObj m_pReqObj;
    private tztZFNetPoolsCallBack m_pCallback;

    private boolean m_bRun;
    private boolean m_bFinish;
    private int m_nSize;
    private int m_nGroup;
    private int m_nSocket;
    private int m_nRecv;
    private int m_nDoPools;

    private String m_strErr;

    private ScheduledFuture m_pFuture = null;
    private ScheduledExecutorService m_pTimer = null;
    private Lock m_recvlock;

    private ArrayList<tztZFThreadPool> ayThreadPool = null;

    private ConcurrentHashMap<Integer, tztZFThreadPoolSocket> mapSocket = null;


    private String poolName = "tztZFShortNet";
    tztZFThreadPool syncThreadPool;

    enum tztZFPoolsStep {
        Continue(0),// 继续执行
        Stop(1),  // 停止
        Finish(2), //完成
        End(3); //结束
        private int value = 0;

        tztZFPoolsStep(int value) {
            this.value = value;
        }
    }

    tztZFPoolsShortNet(String poolName) {
        if (!tztZFTextUtils.isEmpty(poolName))
            this.poolName = poolName;
        initdata();
    }

    private void initdata() {
        m_sAddlist = "";
        m_nLinkType = tztZFLinkType.tztZFLinkType_Short.value();
        m_nNum = 5;
        m_nTimeOut = 0;
        m_pReqObj = null;
        m_pCallback = null;
        m_bRun = false;
        m_bFinish = false;
        m_nSize = 0;
        m_nGroup = 0;
        m_nSocket = 0;
        m_nRecv = 0;
        m_nDoPools = 0;
        m_strErr = "";
        m_pFuture = null;
        m_pTimer = null;
        m_recvlock = new ReentrantLock();
    }

    tztZFPoolsShortNet InitWithContext(Object context) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.context = context;
        }
        return tztZFPoolsShortNet.this;
    }

    tztZFPoolsShortNet InitWithAddlist(String strAddList) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.m_sAddlist = strAddList;
        }
        return tztZFPoolsShortNet.this;
    }

    tztZFPoolsShortNet InitWithNum(int num) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.m_nNum = num;
        }
        return tztZFPoolsShortNet.this;
    }

    tztZFPoolsShortNet InitWithTimeOut(int timeOut) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.m_nTimeOut = timeOut;
        }
        return tztZFPoolsShortNet.this;
    }

    tztZFPoolsShortNet InitWithConnTimeOut(int timeOut) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.m_nConnTimeOut = timeOut;
        }
        return tztZFPoolsShortNet.this;
    }

    tztZFPoolsShortNet InitWithLinktype(int linktype) {
        if (m_bRun)
            return tztZFPoolsShortNet.this;
        synchronized (this) {
            this.m_nLinkType = linktype;
        }
        return tztZFPoolsShortNet.this;
    }

    boolean DoPoolsAction(tztZFNetWorkReqObj reqObj, tztZFNetPoolsCallBack callBack) {
        if (m_bRun)
            return false;
        synchronized (this) {
            if (tztZFTextUtils.isEmpty(m_sAddlist))
                return false;
            if (callBack == null)
                return false;
            this.m_pCallback = callBack;
            if (reqObj != null) {
                this.m_pReqObj = reqObj;
            }
            poolSendReqObj();
        }
        return true;
    }

    private void poolSendReqObj() {
        m_bRun = true;
        m_bFinish = false;
        String strAddlist = m_sAddlist;
        if (!strAddlist.contains("&")) {
            strAddlist += "&";
        }
        if (!strAddlist.endsWith("&"))
            strAddlist = strAddlist + "&";
        m_ayList = tztZFPublicFun.getArray(strAddlist, '&');
        if (m_ayList.length < 1) {
            m_strErr = "地址列表不能为空";
            onFinshPools(tztZFRecvType.tztZFConn_Non, -1);
            return;
        }

        if (ayThreadPool == null) {
            ayThreadPool = new ArrayList<>();
        }

        if (mapSocket == null) {
            mapSocket = new ConcurrentHashMap<>();
        }

        m_nSize = Math.max(1, m_nNum);

        if (syncThreadPool == null) {
            syncThreadPool = new tztZFThreadPool(2, m_nSize + 1, poolName + "_Sync");
            syncThreadPool.setUsed(true);
        }

        m_nDoPools = 0;
        m_nGroup = 0;
        m_strErr = "";

        poolSendTimeOut();
        poolSendReqObjGroup(m_nGroup);
    }

    private void poolSendTimeOut() {
        if (m_nTimeOut <= 0)
            return;
        final long nStartTime = System.currentTimeMillis();
        m_pTimer = Executors.newScheduledThreadPool(1);
        m_pFuture = m_pTimer.schedule(new Runnable() {
            @Override
            public void run() {
                try {
                    if (m_nTimeOut > 0 && System.currentTimeMillis() - nStartTime >= m_nTimeOut) {
                        doTimeout();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }, m_nTimeOut, TimeUnit.MILLISECONDS);
    }

    private void doTimeout() {
        tztlog("onFinshPools", m_nDoPools + "超时结束");
        m_strErr = m_strErr + m_nDoPools + "处理超时[" + m_nTimeOut + "]，结束处理!\\r\\n";
        onFinshPools(tztZFRecvType.tztZFRecv_TimeOut, -1);
    }

    private void endTimeout() {
        tztlog("endTimeout", "endTimeout");

        if (m_pFuture != null) {
            m_pFuture.cancel(true);
            m_pFuture = null;
        }

        if (m_pTimer != null) {
            m_pTimer.shutdownNow();
            m_pTimer = null;
        }
    }

    private void poolSendReqObjGroup(int nGroup) {
        int nCount = m_ayList.length;
        m_nSocket = Math.min(m_nSize, nCount - nGroup * m_nSize);
        m_nRecv = 0;
        for (int i = 0; i < m_nSize && nGroup * m_nSize + i < nCount; i++) {
            final int nPoolIndex = nGroup * m_nSize + i;
            if (nPoolIndex < 0 || nPoolIndex > m_ayList.length) {
                break;
            }
            final String strPoolAdd = m_ayList[nPoolIndex];
            Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    tztZFPoolsShortNet.this.poolSend(nPoolIndex, strPoolAdd);
                }
            };
            try {
                syncThreadPool.useThreadPool().execute(runnable);
            } catch (Exception e) {
                tztZFNetWorkRespObj respObj = getRespObj(tztZFConn_Non, -1, e.getMessage());
                respObj.setReqObj(m_pReqObj);
                poolRecv(nPoolIndex, strPoolAdd, true, 0, m_pReqObj, respObj);
                e.printStackTrace();
            }
        }
    }

    private void poolSend(Integer nPoolIndex, String strPoolAdd) {
        try {
            tztZFThreadPool threadPool = getThreadPool();
            String strLinkKey = "tztZFShortNet." + poolName + "." + m_nLinkType + "." + nPoolIndex + "." + strPoolAdd;
            final tztZFNetWorkReqObj request = new tztZFNetWorkReqObj();
            if (m_pReqObj != null)
                request.setSendMap(m_pReqObj.getSendValues());
            request.setNativeStr("pool_hostport", strPoolAdd);
            request.setReqObjTag(nPoolIndex);

            final tztZFThreadPoolSocket newSocket = new tztZFThreadPoolSocket();
            mapSocket.put(nPoolIndex, newSocket);
            int nLinktype = this.m_nLinkType;
//            if(this.m_nLinkType == tztZFLinkType.tztZFLinkType_Short.value()){
//                nLinktype = this.m_nLinkType + nPoolIndex;
//            }
            newSocket.InitWithContext(context)
                    .InitWithTag(poolName)
                    .InitWithLinkkey(strLinkKey)
                    .InitWithUserID(0)
                    .InitWithSdktype(0)
                    .InitWithLinktype(nLinktype)
                    .InitWithThreadPool(threadPool)
                    .SocketConnect();
            newSocket.SetConnectOutTime(m_nConnTimeOut);
            newSocket.SetRecvOutTime(10 * 1000);
            newSocket.SetAddressList(strPoolAdd, 0);
            request.setNativeStr("pool_time", System.currentTimeMillis() + "");
            boolean bSend = newSocket.OnSendReqModel(request, new tztZFNetWorkResponseCallBack() {
                @Override
                public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvtype, int c_errorNo, String c_errorMsg) {
                    tztZFNetWorkRespObj respObj = getRespObj(tztZFRecvType.valueOf(c_recvtype), c_errorNo, c_errorMsg);
                    respObj.setReqObj(c_reqObj);
                    long times = getTztUsedTime(c_reqObj);
                    boolean bQuit = poolRecv(c_reqObjTag, c_reqObj.getNativeStr("pool_hostport"), true, times, c_reqObj, respObj);
                    if (!bQuit) //recv中未关闭，该连接单独关闭
                        newSocket.Quit();
                }

                @Override
                public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                    long times = getTztUsedTime(c_reqObj);
                    boolean bQuit = poolRecv(c_reqObjTag, c_reqObj.getNativeStr("pool_hostport"), false, times, c_reqObj, c_respObj);
                    if (!bQuit)//recv中未关闭，该连接单独关闭
                        newSocket.Quit();
                }
            });
            if (!bSend) {
                tztZFNetWorkRespObj respObj = getRespObj(tztZFRecvType.tztZFSend_SendCancel, -1, "请求失败");
                respObj.setReqObj(request);
                boolean bQuit = poolRecv(nPoolIndex, strPoolAdd, true, 0, request, respObj);
                if (!bQuit) //recv中未关闭，该连接单独关闭
                    newSocket.Quit();
            }
        } catch (Exception e) {
            tztZFNetWorkRespObj respObj = getRespObj(tztZFConn_Non, -1, e.getMessage());
            respObj.setReqObj(m_pReqObj);
            poolRecv(nPoolIndex, strPoolAdd, true, 0, m_pReqObj, respObj);
        }
    }

    private tztZFPoolsStep doPoolsStep(int nPool, String strPoolAdd, boolean isError, long tztUsedTime, tztZFNetWorkReqObj reqObj, tztZFNetWorkRespObj respObj) {
        if (!m_bRun || m_recvlock == null || m_pCallback == null)
            return tztZFPoolsStep.End;
        m_nRecv++;
        m_nDoPools++;
        boolean bStop = m_pCallback.onPools(nPool, strPoolAdd, reqObj, respObj);
        if (bStop) {
            tztlog("onFinshPools", m_nDoPools + "接口返回结束");
            return tztZFPoolsStep.Stop;
        }
        String str = respObj.getMessage();
        int nError = respObj.getInt("ErrorNo", 0);
        m_strErr = m_strErr + "[" + strPoolAdd + "][" + nError + "]" + (str == null ? "" : str) + "\\r\\n";
        //已经全部均衡返回
        if (m_bRun && m_ayList != null && m_nDoPools == m_ayList.length) {
            tztlog("onFinshPools", m_nDoPools + "全部结束");
            return tztZFPoolsStep.Finish;
        }
        //本轮均衡全部返回
        if (m_bRun && m_ayList != null && m_ayList.length > 0 && m_nRecv == m_nSocket) {
            m_nGroup++;
            poolSendReqObjGroup(m_nGroup);
        }
        return tztZFPoolsStep.Continue;
    }


    private synchronized boolean poolRecv(int nPool, String strPoolAdd, boolean isError, long tztUsedTime, tztZFNetWorkReqObj reqObj, tztZFNetWorkRespObj respObj) {
        if (!m_bRun || m_recvlock == null || m_pCallback == null)
            return false;
        tztZFPoolsStep step = tztZFPoolsStep.Continue;
        try {
            m_recvlock.lock();
            step = doPoolsStep(nPool, strPoolAdd, isError, tztUsedTime, reqObj, respObj);
            if (step == tztZFPoolsStep.Stop || step == tztZFPoolsStep.Finish) {
                m_bFinish = true;
            }
        } finally {
            m_recvlock.unlock();
            if (step == tztZFPoolsStep.End)
                return false;
            if (step == tztZFPoolsStep.Stop) {
                onFinshPools(tztZFRecvType.tztZFRecv_DataOK, nPool);
                return true;
            }
            if (step == tztZFPoolsStep.Finish) {
                onFinshPools(tztZFRecvType.tztZFSend_MaxError, -1);
                return true;
            }
            return false;
        }
    }

    long getTztUsedTime(tztZFNetWorkReqObj reqObj) {
        if (reqObj == null)
            return 0;
        String strTime = reqObj.getNativeStr("UsedTime");
        if (tztZFTextUtils.isEmpty(strTime))
            return 0;
        try {
            return Long.parseLong(strTime);
        } catch (Exception e) {
        }
        return 0;
    }

    private tztZFNetWorkRespObj getRespObj(tztZFRecvType type, int nError, String strErr) {
        tztZFNetWorkRespObj respObj = new tztZFNetWorkRespObj();
        tztZFDataMapBase<String, Object> dataMap = new tztZFDataMapBase<>();
        dataMap.setInt("ErrorNo", nError);
        if (strErr != null)
            dataMap.setStr("ErrorMessage", strErr);
        respObj.setRecvType(type);
        respObj.setResponseData(dataMap);
        return respObj;
    }

    void onFinshPools(tztZFRecvType type, int errorNo) {
        if (m_recvlock == null)
            return;
        try {
            m_recvlock.lock();
            m_bFinish = true;
            endTimeout();
            tztlog("onFinshPools", "onFinshPools");
            if (m_pCallback != null) {
                tztZFNetWorkRespObj respObj = getRespObj(type, errorNo, m_strErr);
                respObj.setReqObj(m_pReqObj);
                m_pCallback.onFinish(m_nDoPools, m_sAddlist, m_pReqObj, respObj);
                m_pCallback = null;
                tztlog("onFinshPools", "onFinshPools end");
            }
        } finally {
            m_recvlock.unlock();
            if (!m_bRun)
                m_recvlock = null;
        }
    }


    synchronized void QuitPools() {
        if (!m_bRun)
            return;
        tztlog("QuitPools", "QuitPools");
        m_bRun = false;
        try {
            if (m_recvlock != null) {
                if (!m_bFinish) {
                    m_bFinish = true;
                    onFinshPools(tztZFSend_SendCancel, -1);
                }
            }

            if (syncThreadPool != null) {
                syncThreadPool.shutdown();
                syncThreadPool = null;
            }

            if (mapSocket != null) {
                for (Iterator<Map.Entry<Integer, tztZFThreadPoolSocket>> it = mapSocket.entrySet().iterator(); it.hasNext(); ) {
                    Map.Entry<Integer, tztZFThreadPoolSocket> item = it.next();
                    if (item != null) {
                        tztZFThreadPoolSocket tztSocket = item.getValue();
                        it.remove();
                        if (tztSocket != null) {
                            tztSocket.Quit();
                        }
                    }
                }
                mapSocket.clear();
                mapSocket = null;
            }

            if (ayThreadPool != null) {
                synchronized (ayThreadPool) {
                    for (tztZFThreadPool pool : ayThreadPool) {
                        if (pool == null) {
                            tztlog("QuitPools", "pool is null");
                        } else {
                            pool.shutdown();
                        }
                    }
                    ayThreadPool.clear();
                    ayThreadPool = null;
                }
            }

            if (m_ayList != null)
                m_ayList = null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        tztlog("QuitPools", "QuitPools End");
    }


    private tztZFThreadPool getThreadPool() throws Exception {
        if (ayThreadPool == null)
            throw new Exception("事件已结束！");

        synchronized (ayThreadPool) {
            for (int i = 0; i < ayThreadPool.size(); i++) {
                tztZFThreadPool threadPool = ayThreadPool.get(i);
                if (!threadPool.isUsed()) {
                    threadPool.setUsed(true);
                    return threadPool;
                }
            }
//            if (ayThreadPool.size() > 10) {
//                throw new Exception("通讯错误，建立了太多的连接！");
//            }
            tztZFThreadPool threadPool = new tztZFThreadPool(3, 4, poolName);
            threadPool.setUsed(true);
            ayThreadPool.add(threadPool);
            return threadPool;
        }
    }

    private void tztlog(String tag, String msg) {
        tztlog(tztZFLogLevel.tztZFLogLevel_Verbose, tag, msg);
    }

    private void tztlog(tztZFLogLevel level, String tag, String msg) {
        String cTag = "PoolsShort[" + this.hashCode() + "]";
        tztZFLog.log(level, tztZFLogPrintType.tztZFLogPrintType_tztSocket.value(), cTag, "[%s]%s", tag, msg);
    }


}
