package com.tztzf.network;

import static com.tztzf.network.define.tztZFLinkType.tztZFLinkType_Non;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.network.callback.tztZFLinkCallBack;
import com.tztzf.network.callback.tztZFNetWorkResponseCallBack;
import com.tztzf.network.callback.tztZFRegisterCallBack;
import com.tztzf.network.callback.tztZFSocketLinkDelegate;
import com.tztzf.network.define.tztZFRecvType;
import com.tztzf.network.define.tztZFSocketFlags;
import com.tztzf.network.socket.tztZFSocketLink;
import com.tztzf.tztZFPrivate;

import java.util.HashMap;

/**
 * ****************************************************************
 * 文件名称:TztNetWork.tztZFSDKWork.java
 * 作    者:Created by pjliang
 * 创建时间:2018/9/21 11:08
 * 文件描述:通讯对外公布的接口
 * 注意事项:
 * 版权声明:Copyright (C) 2015-2025 杭州中焯信息技术股份有限公司
 * 修改历史:2018/9/21 1.00 初始版本
 * ****************************************************************
 */

public class tztZFSDKLink {
    protected Object mContext;
    private tztZFSocketLinkDelegate linkDelegate;
    /** 错误信息是否显示功能号, 默认显示 */
    private boolean showActionForErrorMessage = true;

    private int tztUserID;
    private int tztSDKType;
    private String tztSDKID;

    /**
     * 构造函数，并获取保存的数据
     */
    public tztZFSDKLink(int sdkType) {
        tztSDKType = sdkType;
        initData();
    }

    private void initData() {

    }

    /**
     * SDK证书注册，该函数需要在程序开始就调用
     * @param sdkID 注册申请分配的appID
     * @param sdkKey 注册申请分配的appkey
     * @param sdkCert 注册申请分配的证书
     * @param options 其他信息
     * @param callBack 注册结果应答回调，注意：该回调异步执行返回！
     */
    public boolean registerSDK(Object application, String sdkID, String sdkKey, byte[] sdkCert, String appPackage, String appName, tztZFDataMapBase<String, Object> options, final tztZFRegisterCallBack callBack) {
        mContext = application;
        tztSDKID = sdkID;
        return tztZFSocketLink.loadSDKCert(application, tztSDKType, sdkID, sdkKey, sdkCert, appPackage, appName, new tztZFRegisterCallBack() {
            @Override
            public void onHandler(String respAppid, int respError, HashMap<String, String> options) {
                String strBlocktype = options.get("blocktype");
                int nBlocktype = tztZFPublicFun.parseInt(strBlocktype, -1);
                if (nBlocktype == 1) {
                    String strErrorNo = options.get("errorno");
                    String strSdkType = options.get("sdktype");
                    String strUserID = options.get("userid");
                    int nErrorNo = tztZFPublicFun.parseInt(strErrorNo, 0);
                    int nSdkType = tztZFPublicFun.parseInt(strSdkType, 0);
                    int nUserID = tztZFPublicFun.parseInt(strUserID, 0);
                    if (nErrorNo > 0 && nUserID >= 0 && nSdkType >= 0) {
                        tztUserID = nUserID;
                        tztSDKID = respAppid;
                        tztSDKType = nSdkType;
                    } else {
                        tztUserID = 0;
                    }
                }
                if (callBack != null) {
                    callBack.onHandler(respAppid, respError, options);
                }
            }
        });
    }

    /**
     * 获取sdkid
     * @return sdkid
     */
    public String getLinkSDKID() {
        return tztSDKID;
    }

    /**
     * 获取userid
     * @return userid
     */
    public int getLinkUserID() {
        return tztUserID;
    }

    /**
     * 获取sdktype
     * @return sdktype
     */
    public int getLinkSDKType() {
        return tztSDKType;
    }

    /**
     * 根据链接通道类型获取链接通道key
     * @param linkType 链接通道类别
     * @return 链接通道key
     */
    public String keyForLinkType(int linkType) {
        return tztZFSocketLink.linkKeyforUserID(tztUserID, tztSDKType, linkType);
    }

    /**
     * 设置发送前应答后回调
     * @param linkDelegate 发送前应答后回调
     */
    public void setSocketLinkDelegate(tztZFSocketLinkDelegate linkDelegate) {
        this.linkDelegate = linkDelegate;
    }

    /**
     * 获取发送前应答后回调（Android主要在兼容tztZZNetWork使用）
     * @return 发送前应答后回调
     */
    public tztZFSocketLinkDelegate getSocketLinkDelegate() {
        return linkDelegate;
    }

    /**
     * 设置错误信息是否显示功能号
     * @param showActionForErrorMessage 错误信息是否显示功能号
     */
    public void setShowActionForErrorMessage(boolean showActionForErrorMessage) {
        this.showActionForErrorMessage = showActionForErrorMessage;
    }

    /**
     * 获取错误信息是否显示功能号
     */
    public boolean getShowActionForErrorMessage() {
        return showActionForErrorMessage;
    }

    /**
     * 根据链接通道类型注册链接
     * @param context 上下文
     * @param linkType 链接通道类别
     * @return 注册是否成功
     */
    public boolean registLink(Object context, int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.registSocket(context, linkKey, tztUserID, tztSDKType, linkType);
    }

    /**
     * 根据链接通道key、通链接道类型注册通道
     * @param context 上下文
     * @param linkKey 链接通道key
     * @param linkType 链接通道类别
     * @return 注册成功后的链接通道key
     */
    public String registLinkKey(Object context, String linkKey, int linkType) {
        String strKey = linkKey;
        if (tztZFTextUtils.isEmpty(linkKey)) {
            strKey = keyForLinkType(linkType);
        }
        boolean bOk = tztZFSocketLink.registSocket(context, strKey, tztUserID, tztSDKType, linkType);
        if (bOk)
            return strKey;
        return "";
    }

    /**
     * 发送数据请求
     * @param linkType 链接通道类别,注意是linktype
     * @param reqObj 请求具体参数数据
     * @param callBack 发送请求的回调
     * @return 是否添加到通讯
     */
    public boolean requestLinkType(int linkType, tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        String linkKey = keyForLinkType(linkType);
        return sendLinkKey(linkKey, reqObj, callBack);
    }

    /**
     * 发送数据请求
     * @param linkKey 链接通道key
     * @param reqObj 链接通道类别
     * @param callBack 发送请求的回调
     * @return 是否添加到通讯
     */
    public boolean requestLinkKey(String linkKey, tztZFNetWorkReqObj reqObj, tztZFNetWorkResponseCallBack callBack) {
        return sendLinkKey(linkKey, reqObj, callBack);
    }

    /**
     * 设置socket 日志 Tag
     * @param linkType 链接通道类别
     * @param tag 日志tag
     */
    public void setLinkTag(int linkType, String tag) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkTag(linkKey, tag);
    }

    /**
     * 设置socket 日志 Tag
     * @param linkKey 链接通道key
     * @param tag 日志tag
     */
    public static void setLinkTagLinkKey(String linkKey, String tag) {
        tztZFSocketLink.setLinkTag(linkKey, tag);
    }

    /**
     * 设置通讯回调
     * @param linkType 链接通道类别
     * @param linkBlock 回调
     */
    public void setLinkBlock(int linkType, tztZFLinkCallBack linkBlock) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkBlock(linkKey, linkBlock);
    }

    /**
     * 设置通讯回调
     * @param linkKey 链接通道key
     * @param linkBlock 回调
     */
    public static void setLinkBlockLinkKey(String linkKey, tztZFLinkCallBack linkBlock) {
        tztZFSocketLink.setLinkBlock(linkKey, linkBlock);
    }

    /**
     * 设置通讯服务器地址
     * @param linkType 链接通道类别
     * @param addList 服务器地址列表
     * @param selectIndex 连接地址索引值
     */
    public void setAddressList(int linkType, String addList, int selectIndex) {
        setAddressList(linkType, addList, "", selectIndex);
    }

    /**
     * 设置通讯服务器地址
     * @param linkType 链接通道类别
     * @param addList 服务器地址列表
     * @param nameList 服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public void setAddressList(int linkType, String addList, String nameList, int selectIndex) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkAddressList(linkKey, addList, nameList, selectIndex);
    }

    /**
     * 设置通讯国密服务器地址
     * @param linkType 链接通道类别
     * @param addList 服务器地址列表
     * @param nameList 服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public void setGMAddressList(int linkType, String addList, String nameList, int selectIndex) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setLinkGMAddressList(linkKey, addList, nameList, selectIndex);
    }

    /**
     * 设置通讯服务器地址
     * @param linkKey 链接通道key
     * @param addList 服务器地址列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressListLinkKey(String linkKey, String addList, int selectIndex) {
        setAddressListLinkKey(linkKey, addList, "", selectIndex);
    }

    /**
     * 设置通讯服务器地址
     * @param linkKey 链接通道key
     * @param addList 服务器地址列表
     * @param nameList 服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setAddressListLinkKey(String linkKey, String addList, String nameList, int selectIndex) {
        tztZFSocketLink.setLinkAddressList(linkKey, addList, nameList, selectIndex);
    }

    /**
     * 设置通讯国密服务器地址
     * @param linkKey 链接通道key
     * @param addList 服务器地址列表
     * @param nameList 服务器名称列表
     * @param selectIndex 连接地址索引值
     */
    public static void setGMAddressListLinkKey(String linkKey, String addList, String nameList, int selectIndex) {
        tztZFSocketLink.setLinkGMAddressList(linkKey, addList, nameList, selectIndex);
    }

    /**
     * 设置待发送队列最大个数
     * @param linkType 链接通道类别
     * @param nMaxReq 待发送队列最大个数
     */
    @tztZFPrivate
    public void setMaxReqCount(int linkType, int nMaxReq) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setMaxReqCount(linkKey, nMaxReq);
    }

    /**
     * 设置待发送队列最大个数
     * @param linkKey 链接通道key
     * @param nMaxReq 待发送队列最大个数
     */
    @tztZFPrivate
    public static void setMaxReqCountLinkKey(String linkKey, int nMaxReq) {
        if (tztZFTextUtils.isEmpty(linkKey))
            return;
        tztZFSocketLink.setMaxReqCount(linkKey, nMaxReq);
    }

    /**
     * 设置socket连接超时时间，超时时间必须大于1才有效，默认3s
     * @param linkType 链接通道类别
     * @param nTimeOut 超时时间
     */
    public void setConnectTimeOut(int linkType, int nTimeOut) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.setConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 设置socket连接超时时间，超时时间必须大于1才有效，默认3s
     * @param linkKey 链接通道key
     * @param nTimeOut 超时时间
     */
    public static void setConnectTimeOutLinkKey(String linkKey, int nTimeOut) {
        tztZFSocketLink.setConnectTimeOut(linkKey, nTimeOut);
    }

    /**
     * 获取通道当前链接服务器地址
     * @param linkType 链接通道类别
     * @return 当前链接服务器地址
     */
    public String getLinkHostPort(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getHostPortLinkKey(linkKey);
    }

    /**
     * 获取通道当前链接成功服务器地址
     * @param linkType 链接通道类别
     * @return 当前链接服务器地址
     */
    public String getLinkSuccessedHostPort(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getSuccessedHostPortLinkKey(linkKey);
    }

    /**
     * 获取通道当前链接地址
     * @param linkKey 链接通道key
     * @return 当前链接服务器地址
     */
    public static String getLinkHostPortLinkKey(String linkKey) {
        return tztZFSocketLink.getHostPortLinkKey(linkKey);
    }

    /**
     * 获取通道当前链接成功服务器地址
     * @param linkKey 链接通道类别
     * @return 当前链接服务器地址
     */
    public static String getLinkSuccessedHostPortLinkKey(String linkKey) {
        return tztZFSocketLink.getSuccessedHostPortLinkKey(linkKey);
    }

    /**
     * 获取链接状态
     * @param linkType 链接通道类别
     * @return 链接状态
     */
    @tztZFSocketFlags
    public int getSocketConnectState(int linkType) {
        String linkKey = keyForLinkType(linkType);
        return tztZFSocketLink.getConnectStateLinkKey(linkKey);
    }

    /**
     * 获取链接状态
     * @param linkKey 链接通道key
     * @return 链接状态
     */
    @tztZFSocketFlags
    public static int getSocketConnectStateLinkKey(String linkKey) {
        return tztZFSocketLink.getConnectStateLinkKey(linkKey);
    }

    /**
     * 断开通道链接
     * @param linkType 链接通道类别
     */
    public void disConnection(int linkType) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.disConnectionForKey(linkKey);
    }

    /**
     * 断开通道链接
     * @param linkKey 通道key
     */
    public static void disConnectionLinkKey(String linkKey) {
        tztZFSocketLink.disConnectionForKey(linkKey);
    }

    /**
     * 释放指定的通讯对象
     * @param linkType 链接通道类别
     */
    public void freeLink(int linkType) {
        String linkKey = keyForLinkType(linkType);
        tztZFSocketLink.freeLinkForKey(linkKey);
    }

    /**
     * 释放指定的通讯对象
     * @param linkKey 链接类别
     */
    public static void freeLinkKey(String linkKey) {
        tztZFSocketLink.freeLinkForKey(linkKey);
    }

    //内部统一请求函数
    private boolean sendLinkKey(final String linkKey, tztZFNetWorkReqObj reqObj, final tztZFNetWorkResponseCallBack callBack) {
        if (tztZFTextUtils.isEmpty(linkKey)) {
            return tztZFSocketLink.responseNoLink(reqObj, callBack);
        }
        final int linkType = tztZFSocketLink.getLinkTypeLinkKey(linkKey);
        if (linkType <= tztZFLinkType_Non.value()) {
            return tztZFSocketLink.responseNoLink(reqObj, callBack);
        }

        //发送请求前，统一通过代理回调上层，可以进行业务数据的统一补充处理
        boolean continueSend = true;//是否继续发送请求
        final tztZFSocketLinkDelegate linkDelegate = this.linkDelegate;
        if (linkDelegate != null) {
            continueSend = linkDelegate.beforeSendData(reqObj, callBack, linkType, linkKey);
        }
        if (!continueSend) {
            return false;
        }

        return tztZFSocketLink.requestLinkKey(linkKey, reqObj, new tztZFNetWorkResponseCallBack() {
            @Override
            public void OnError(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, int c_recvType, int c_errorNo, String c_errorMsg) {
                if (c_errorNo < 0 && c_reqObj != null && getShowActionForErrorMessage()) {
                    c_errorMsg = tztZFSocketLink.setActionForErrorMessage(c_errorNo, c_reqObj.getString("Action"), c_errorMsg);
                }
                if (linkDelegate != null) {
                    tztZFNetWorkRespObj respObj = new tztZFNetWorkRespObj();
                    respObj.setRecvType(tztZFRecvType.valueOf(c_recvType));
                    respObj.setMessage(c_errorMsg);
                    respObj.setStatusCode(-1);
                    if (c_reqObj != null) {
                        respObj.setReqObj(c_reqObj);
                    }
                    //应答接收后，统一回调上层进行处理，可以用于特殊业务的错误判断等逻辑
                    linkDelegate.afterReceiveData(respObj, linkType, linkKey);
                }
                if (callBack != null) {
                    callBack.OnError(c_reqObj, c_reqObjTag, c_recvType, c_errorNo, c_errorMsg);
                }
            }

            @Override
            public void OnAns(tztZFNetWorkReqObj c_reqObj, int c_reqObjTag, tztZFNetWorkRespObj c_respObj) {
                //处理错误应答逻辑并打印应答数据
                if (getShowActionForErrorMessage()) {
                    tztZFSocketLink.setActionForErrorMessage(c_respObj);
                }
                if (linkDelegate != null) {
                    linkDelegate.afterReceiveData(c_respObj, linkType, linkKey);
                }
                if (callBack != null) {
                    callBack.OnAns(c_reqObj, c_reqObjTag, c_respObj);
                }
            }
        });
    }
}