package com.tztzf.protocol.core;

import com.tztzf.base.tztZFPublicFun;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.log.tztZFLog;
import com.tztzf.log.tztZFLogLevel;
import com.tztzf.protocol.callback.tztZFBlocktztCosign;
import com.tztzf.protocol.callback.tztZFBlocktztRand;
import com.tztzf.protocol.tztnative.tztNativeCosignSSL;
import com.tztzf.protocol.tztnative.tztNativePubFun;
import com.tztzf.tztZFPrivate;

import java.util.ArrayList;

import static com.tztzf.protocol.define.tztZFEncryptType.tztZFSM2;
import static com.tztzf.protocol.define.tztZFOperType.tztZFEncrypt;

@tztZFPrivate(name = "内部使用，外层禁止调用")
public class tztZFCosignSSL {
    private static tztZFCosignSSL cosignSSL = new tztZFCosignSSL();
    private int connectType;
    private tztZFBlocktztCosign cosignblock;
    private tztZFBlocktztRand randblock;
    private byte[] cosignSeed;

    private tztZFCosignSSL() {
        tztNativeCosignSSL.loadCosignSSLFun();
    }

    private static synchronized tztZFCosignSSL getShareInstance() {
        if (cosignSSL == null) {
            cosignSSL = new tztZFCosignSSL();
        }
        return cosignSSL;
    }

    public static tztZFCosignSSL withConnectType(int connectType, final tztZFBlocktztCosign block) {
        return withConnectType(connectType, block, null);
    }

    public static tztZFCosignSSL withConnectType(int connectType, final tztZFBlocktztCosign block, final tztZFBlocktztRand randBlock) {
        getShareInstance().connectType = connectType;
        getShareInstance().cosignblock = block;
        getShareInstance().randblock = randBlock;
        return getShareInstance();
    }

    public static byte[] cosignRand(int type, int len) {
        return getShareInstance().cosignRandIn(type, len);
    }

    public static byte[] rand(int type, int len) {
        return getShareInstance().randIn(type, len);
    }

    public static void cosignRandSeed(byte[] data) {
        getShareInstance().cosignRandSeedIn(data);
    }

    public static byte[] cosignData(int type, byte[] data) {
        return getShareInstance().cosignDataIn(type, data);
    }

    public static byte[] decryptCosignCert(byte[] cert, String appKey) {
        return decryptCosignCert(cert, appKey, 0);
    }

    public static byte[] decryptCosignCert(byte[] cert, String appKey, int decryptType) {
        if (cert == null || cert.length < 1 || tztZFTextUtils.isEmpty(appKey))
            return null;
        byte[] key = appKey.getBytes();
        return tztNativePubFun.decryptCertDataWithKeyWithType(key, cert, decryptType);
    }

    public static byte[] encData(byte[] data, byte[] key) {
        byte[] sm2 = tztZFDataEncrypt.encryptOper(tztZFEncrypt, tztZFSM2, key, data, 0);
        if (sm2 == null || sm2.length < 1) {
            tztZFLog.log(tztZFLogLevel.tztZFLogLevel_Warn, "tztZFCosign", "SM2加解密错误");
            return null;
        }
        return tztZFDataEncrypt.encryptBase64(sm2);
    }

    public static ArrayList<byte[]> cosignInit() {
        byte[] result = tztNativeCosignSSL.cosignInit();
        if (result == null || result.length < 8)
            return null;
        int ckeylen = tztZFPublicFun.getInt(result, 0, 4);
        int skeylen = tztZFPublicFun.getInt(result, 4 + ckeylen, 4);
        if (4 + ckeylen + 4 + skeylen != result.length)
            return null;
        byte[] ckey = new byte[ckeylen];
        byte[] skey = new byte[skeylen];
        System.arraycopy(result, 4, ckey, 0, ckeylen);
        System.arraycopy(result, 4 + ckeylen + 4, skey, 0, skeylen);
        ArrayList<byte[]> ayData = new ArrayList<>();
        ayData.add(skey);
        ayData.add(ckey);
        return ayData;
    }

    public static ArrayList<byte[]> cosignStep1() {
        byte[] result = tztNativeCosignSSL.cosignStep1();
        if (result == null || result.length < 8)
            return null;
        int k1len = tztZFPublicFun.getInt(result, 0, 4);
        int q1len = tztZFPublicFun.getInt(result, 4 + k1len, 4);
        if (4 + k1len + 4 + q1len != result.length)
            return null;
        byte[] k1 = new byte[k1len];
        byte[] q1 = new byte[q1len];
        System.arraycopy(result, 4, k1, 0, k1len);
        System.arraycopy(result, 4 + k1len + 4, q1, 0, q1len);
        ArrayList<byte[]> ayData = new ArrayList<>();
        ayData.add(k1);
        ayData.add(q1);
        return ayData;
    }

    public static byte[] cosignStep3(byte[] clientKey, byte[] k1, byte[] r, byte[] s2, byte[] s3) {
        return tztNativeCosignSSL.cosignStep3(clientKey, k1, r, s2, s3);
    }

    public static tztZFResponse createPKCS10String(byte[] pub, String sub, String pin) {
        tztZFResponse response = new tztZFResponse();
        if (pub == null
                || sub == null || sub.length() < 1
                || pin == null || pin.length() < 1) {
            response.retCode = -1;
            return response;
        }
        byte[] datasub = sub.getBytes();
        byte[] datapin = pin.getBytes();
        byte[] result = tztNativeCosignSSL.createPKCS10String(pub, datasub, datapin);
        if (result == null || result.length < 1) {
            response.retCode = -2;
            return response;
        }
        response.retCode = tztZFPublicFun.getInt(result, 0, 4);
        int nLen = result.length - 4;
        if (nLen > 0) {
            byte[] data = new byte[nLen];
            System.arraycopy(result, 4, data, 0, nLen);
            String pkcs10 = new String(data);
            if (pkcs10.startsWith("-----BEGIN CERTIFICATE REQUEST-----")) {
                pkcs10 = pkcs10.replace("-----BEGIN CERTIFICATE REQUEST-----", "");
                pkcs10 = pkcs10.replace("-----END CERTIFICATE REQUEST-----", "");
                pkcs10 = pkcs10.replace("\n", "");
            }
            response.retData = data;
            response.retValue = pkcs10;
        }
        return response;
    }

    /**
     * 签名数据转换成标准ECDSA_SIG格式
     *
     * @param sig      签名数据
     * @param signtype 签名类型
     * @return 标准格式数据流
     */
    public static byte[] cosign_ECDSA_SIG(byte[] sig, int signtype) {
        if (sig == null || sig.length < 1)
            return null;
        return tztNativeCosignSSL.tztZF_i2d_ECDSA_SIG(signtype, sig);
    }

    private byte[] cosignRandIn(int type, int len) {
        if (randblock == null) {
            return getShareInstance().randIn(type, len);
        } else {
            byte[] rand = randblock.onHandler(type, len);
            if (rand == null || rand.length < 1) {
                rand = getShareInstance().randIn(type, len);
            }
            return rand;
        }
    }

    private byte[] randIn(int type, int len) {
        return tztNativeCosignSSL.rand(type, len, cosignSeed);
    }

    private void cosignRandSeedIn(byte[] data) {
        if (data != null && data.length > 0) {
            cosignSeed = data;
        }
    }

    private byte[] cosignDataIn(int type, byte[] data) {
        if (this.cosignblock == null)
            return null;
        tztZFDataMapBase<String, Object> sign = this.cosignblock.onHandler(type, data);
        if (sign == null || sign.getCount() < 1)
            return null;
        int sigtype = sign.getInt("signType", 0);
        byte[] sigdata = sign.getBytes("signData");
        byte[] signbyte = new byte[4 + sigdata.length];
        System.arraycopy(tztZFPublicFun.intToByteArray(sigtype), 0, signbyte, 0, 4);
        System.arraycopy(sigdata, 0, signbyte, 4, sigdata.length);
        return signbyte;
    }

    public static class tztZFResponse {
        /**
         * 返回码
         */
        private int retCode;
        /**
         * 返回信息
         */
        private String retMessage;
        /**
         * 返回data
         */
        private byte[] retData;
        /**
         * 返回value
         */
        private String retValue;
        /**
         * 扩展字典
         */
        private tztZFDataMapBase<String, Object> retEx;

        public int getRetCode() {
            return retCode;
        }

        public void setRetCode(int retCode) {
            this.retCode = retCode;
        }

        public String getRetMessage() {
            return retMessage;

        }


        public void setRetMessage(String retMessage) {
            this.retMessage = retMessage;
        }

        public byte[] getRetData() {
            return retData;
        }

        public void setRetData(byte[] retData) {
            this.retData = retData;
        }

        public String getRetValue() {
            return retValue;
        }

        public void setRetValue(String retValue) {
            this.retValue = retValue;
        }

        public tztZFDataMapBase<String, Object> getRetEx() {
            return retEx;
        }

        public void setRetEx(tztZFDataMapBase<String, Object> retEx) {
            this.retEx = retEx;
        }

        private tztZFResponse() {
            retCode = -1;
            retMessage = "";
            retData = null;
            retEx = new tztZFDataMapBase<>();
        }

        public tztZFResponse(int code, String msg) {
            retCode = code;
            retMessage = msg != null ? msg : "";
            retData = null;
            retEx = new tztZFDataMapBase<>();
        }

        public static tztZFResponse withCode(int code, String msg) {
            return new tztZFResponse(code, msg);
        }
    }
}
