package com.tztzf.protocol.core;

import static com.tztzf.protocol.define.tztZFEncryptType.tztZFBASE64;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFHEX;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFMD5_16;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFMD5_32;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFRC4;
import static com.tztzf.protocol.define.tztZFEncryptType.tztZFSM4;
import static com.tztzf.protocol.define.tztZFOperType.tztZFDecrypt;
import static com.tztzf.protocol.define.tztZFOperType.tztZFEncrypt;

import com.tztzf.protocol.tztnative.tztNativeCosignSSL;
import com.tztzf.protocol.tztnative.tztNativeEncryptFun;
import com.tztzf.protocol.tztnative.tztNativePubFun;

/**
 * 作者: yangdl 2019.06
 * 用途: tztProtocol.so:NDK数据编解码处理
 * 修改记录如下:
 *
 * @版本号: 1.00.001 2019.06.10
 * 更新说明: 第一版正式发布
 */

public class tztZFDataEncrypt {

    private static tztZFDataEncrypt encryptKit = new tztZFDataEncrypt();

    private static byte[] byteseNull = null;

    private static tztZFDataEncrypt getShareInstance() {
        if (encryptKit == null)
            encryptKit = new tztZFDataEncrypt();
        return encryptKit;
    }

    public static byte[] encryptOper(int oper, int type, byte[] key, byte[] data, int len) {
        return tztNativeEncryptFun.encryptOper(oper, type, key, data, len);
    }

    public static byte[] encryptResource(byte[] data) {
        byte[] key = tztNativeEncryptFun.tztGetAjaxRC4Key();
        return rc4Encrypt(key, data);
    }

    //使用固定密钥解密资源数据
    public static byte[] descryptResource(byte[] srcData) {
        byte[] key = tztNativeEncryptFun.tztGetAjaxRC4Key();
        return rc4Descrypt(key, srcData);
    }

    //使用指定密钥加密数据
    public static byte[] encryptResource(byte[] key, byte[] srcData) {
        return rc4Encrypt(key, srcData);
    }

    //使用指定密钥解密数据
    public static byte[] descryptResource(byte[] key, byte[] srcData) {
        return rc4Descrypt(key, srcData);
    }

    //使用rc4加密
    public static byte[] rc4Encrypt(byte[] key, byte[] srcData) {
        return encryptOper(tztZFEncrypt, tztZFRC4, key, srcData, 0);
    }

    //使用rc4解密
    public static byte[] rc4Descrypt(byte[] key, byte[] srcData) {
        return encryptOper(tztZFDecrypt, tztZFRC4, key, srcData, 0);
    }

    //MD5-16位
    public static byte[] encryptMD5_16(byte[] bytes) {
        return encryptOper(tztZFEncrypt, tztZFMD5_16, null, bytes, 0);
    }

    //MD5-32位
    public static byte[] encryptMD5_32(byte[] bytes) {
        return encryptOper(tztZFEncrypt, tztZFMD5_32, null, bytes, 0);
    }

    public static String encryptMD532Str(byte[] bytes) {
        byte[] data = encryptOper(tztZFEncrypt, tztZFMD5_32, null, bytes, 0);
        try {
            return new String(data);
        } catch (Exception e) {
            return null;
        }
    }

    //Base64 Encrypt
    public static byte[] encryptBase64(byte[] bytes) {
        return encryptOper(tztZFEncrypt, tztZFBASE64, null, bytes, 0);
    }

    public static String encryptBase64Str(byte[] bytes) {
        byte[] data = encryptBase64(bytes);
        try {
            return new String(data);
        } catch (Exception e) {
            return null;
        }
    }

    //Base64 Decrypt
    public static byte[] decryptBase64(byte[] bytes) {
        return encryptOper(tztZFDecrypt, tztZFBASE64, null, bytes, 0);
    }

    public static byte[] sm4Encrypt(byte[] data, byte[] key) {
        return encryptOper(tztZFEncrypt, tztZFSM4, key, data, 0);
    }

    public static byte[] sm4Decrypt(byte[] data, byte[] key) {
        return encryptOper(tztZFDecrypt, tztZFSM4, key, data, 0);
    }

    public static byte[] hexEncrypt(byte[] data) {
        return encryptOper(tztZFEncrypt, tztZFHEX, null, data, 0);
    }

    public static byte[] hexDecrypt(byte[] data) {
        return encryptOper(tztZFDecrypt, tztZFHEX, null, data, 0);
    }

    //SDK认证使用
    public static byte[] encryptDataWithCert(byte[] bytes, int userID, int linkType) {
        if (bytes == null)
            return null;
        return tztNativePubFun.operDataWithCert(0, userID, 4, linkType, bytes);
    }

    public static byte[] decryptDataWithCert(byte[] bytes, int userID, int linkType) {
        if (bytes == null)
            return null;
        return tztNativePubFun.operDataWithCert(1, userID, 4, linkType, bytes);
    }

    public static byte[] signDataWithCert(byte[] data, int userID, int linkType) {
        if (data == null)
            return null;
        return tztNativePubFun.operDataWithCert(2, userID, 4, linkType, data);
    }

    public static byte[] encryptRSA(long Exponent, byte[] modulus, byte[] srcData) {
        if (srcData == null || modulus == null)
            return null;
        return tztNativeEncryptFun.encryptRSA(Exponent, modulus, srcData);
    }

    public static int tztSM2_verify(byte[] cert, byte[] org, byte[] sign) {
        if (cert == null || cert.length < 1 || org == null || org.length < 1 || sign == null || sign.length < 1)
            return 0;
        return tztNativeCosignSSL.tztZFSM2_verify(1, cert, org, sign);
    }

    public static int tztSM2_verify(byte[] cert, byte[] org, byte[] kx, byte[] ky) {
        if (cert == null || cert.length < 1 || org == null || org.length < 1 || kx == null || kx.length < 1 || ky == null || ky.length < 1)
            return 0;
        return tztNativeCosignSSL.tztZFSM2_verifyXY(1, cert, org, kx, ky);
    }

    public static byte[] makeRSA(int bit) {
        return tztNativeEncryptFun.tztMakeTempRSA(bit);
    }

    public static byte[] makeRandom(int bit) {
        return tztNativeEncryptFun.tztZFGetRandom(bit);
    }
}
