package com.tztzf.protocol.core;

import com.tztzf.commbase.tztZFDataMapBase;
import com.tztzf.commbase.tztZFDataMapCallBack;
import com.tztzf.protocol.define.tztZFPackageType;

import static com.tztzf.protocol.define.tztZFPackageType.tztZFPackage_Non;

public final class tztZFSendPackage {
    private tztZFDataMapBase<String, Object> sendData = null;
    private @tztZFPackageType int packageType = tztZFPackage_Non;
    private int packageIndex;
    private String socketKey;
    private Integer sendTag;
    private int action;

    public tztZFSendPackage(final tztZFDataMapBase<String, Object> mapData, @tztZFPackageType int type, String socketKey) {
        sendData = new tztZFDataMapBase<>();
        if (mapData != null) {
            sendData.setKlenBit(mapData.getKlenBit());
            sendData.setEncodeType(mapData.getEncodeType());
            mapData.enumerateKeysAndObjectsUsingBlock(new tztZFDataMapCallBack() {
                @Override
                public boolean enumerate(String lowKey, String key, Object value) {
                    sendData.setObj(key, value);
                    return false;
                }
            });
        }
        this.packageType = type;
        this.socketKey = socketKey;
    }

    public tztZFDataMapBase<String, Object> getSendData() {
        return this.sendData;
    }

    public int getAction() {
        if (sendData == null)
            return 0;
        int nAction = sendData.getInt("Action");
        if (nAction != 0) {
            action = nAction & 0xFFFF; // 0~65535，防止溢出
        }
        sendData.setInt("Action", action);
        return action;
    }

    public Integer getTag() {
        return this.sendTag;
    }

    public @tztZFPackageType int getPackageType() {
        return this.packageType;
    }

    public void setPackageType(@tztZFPackageType int type) {
        this.packageType = type;
    }

    int getPackageIndex() {
        return this.packageIndex;
    }

    public void setPackageIndex(int packageIndex) {
        this.packageIndex = packageIndex;
    }

    boolean withProtocol(int protocolType, int reqNo) {
        if (sendData == null)
            return false;
        sendTag = reqNo;
        sendData.setInt("Reqno", sendTag);
        String strTag;
        strTag = String.format("%1$s=%2$s=%3$s=%4$s=%5$s", this.hashCode(), packageIndex, protocolType, socketKey, System.currentTimeMillis());
        sendData.setStr("ReqTag", strTag);
        return true;
    }

    void setIntactToServer(byte[] intactToServer) {
        if (sendData == null)
            return;
        if (intactToServer != null && intactToServer.length > 0) {
            sendData.setBytes("IntactToServer", intactToServer);
        } else {
            sendData.removeByKey("IntactToServer");
        }
    }
}
