package com.tztzf.protocol.tztnative;

import com.tztzf.soloader.tztZFSoLoader;

/**
 * 作者: yangdl 2019.06
 * 用途: tztProtocol.so:NDK数据编解码处理
 * 修改记录如下:
 *
 * @版本号: 1.00.001 2019.06.10
 * 更新说明: 第一版正式发布
 */

public class tztNativeBioSSL {

    private static tztNativeBioSSL biosslKit = new tztNativeBioSSL();

    private static byte[] byteseNull = null;

    static {
        tztZFSoLoader.loadLibrary("tztZFProtocol");
//        System.loadLibrary("tztZFProtocol");
    }

    private static tztNativeBioSSL getShareInstance() {
        if (biosslKit == null)
            biosslKit = new tztNativeBioSSL();
        return biosslKit;
    }

    private static int getlen(byte[] data) {
        if (data != null)
            return data.length;
        return 0;
    }

    public static long createBioSSL(int connecttype) {
        return getShareInstance().createObjNative(connecttype);
    }

    public static boolean freeBioSSL(long biosslObj) {
        return getShareInstance().freeObjNative(biosslObj);
    }

    public static int bio_cacert(long biosslObj, byte[] ca, int type) {
        return getShareInstance().biocacertNative(biosslObj, ca, getlen(ca), type);
    }

    public static int bio_sign(long biosslObj, byte[] sign, byte[] signkey, byte[] signpass, int type) {
        return getShareInstance().biosignNative(biosslObj,
                sign, getlen(sign),
                signkey, getlen(signkey),
                signpass, getlen(signpass),
                type);
    }

    public static int bio_enc(long biosslObj, byte[] enc, byte[] enckey, byte[] encpass, int type, int certtype) {
        return getShareInstance().bioencNative(biosslObj,
                enc, getlen(enc),
                enckey, getlen(enckey),
                encpass, getlen(encpass),
                type, certtype);
    }

    public static int bio_init(long biosslObj) {
        return getShareInstance().bioinitNative(biosslObj);
    }

    public static int bio_connect(long biosslObj, Object sockObj) {
        return getShareInstance().bioconnectNative(biosslObj, sockObj);
    }

    public static int bio_read(long biosslObj, byte[] read, int len) {
        return getShareInstance().bioreadNative(biosslObj, read, len);
    }

    public static int bio_write(long biosslObj, byte[] write) {
        return getShareInstance().biowriteNative(biosslObj, write, getlen(write));
    }

    public static int bio_close(long biosslObj) {
        return getShareInstance().biocloseNative(biosslObj);
    }

    public static int bio_free(long biosslObj) {
        return getShareInstance().biofreeNative(biosslObj);
    }

    public static int bio_verify(long biosslObj) {
        return getShareInstance().bioverifyNative(biosslObj);
    }

    public static int bio_error(long biosslObj, int error, byte[] msg) {
        return getShareInstance().bioerrorNative(biosslObj, error, msg, getlen(msg));
    }

    ///校验证书 cacerttype、certtype 详见 tztZFCertType
    public static int ssl_verifyCert(int cacerttype, byte[] ca, int certtype, byte[] cert) {
        return getShareInstance().sslverifyCertNative(cacerttype, ca, getlen(ca), certtype, cert, getlen(cert));
    }

    public static int ssl_setCA(Object ctx, int type, byte[] data) {
        return getShareInstance().sslsetCANative(ctx, type, data, getlen(data));
    }

    public static int ssl_setCert(Object ctx, int sig, int type, byte[] data, byte[] key, byte[] pass) {
        return getShareInstance().sslsetCertNative(ctx, sig, type, data, getlen(data), key, getlen(key), pass, getlen(pass));
    }

    public static boolean ssl_CheckTimeCert(int type, byte[] data) {
        return getShareInstance().sslCheckTimeCertNative(type, data, getlen(data));
    }

    public static Object ssl_getX509Cert(int type, byte[] data) {
        return getShareInstance().sslgetX509CertNative(type, data, getlen(data));
    }

    public static boolean ssl_isGMCert(int type, byte[] data) {
        return getShareInstance().sslisGMCertNative(type, data, getlen(data));
    }
    //endregion


    //region NDK 函数
//------------------------------NDK 函数----------------------//
    //创建dataobj
    private native long createObjNative(int connecttype);

    //释放dataobj
    private native boolean freeObjNative(long biosslObj);

    private native int biocacertNative(long biosslObj,
                                       byte[] ca, int calen,
                                       int type);

    private native int biosignNative(long biosslObj,
                                     byte[] sign, int signlen,
                                     byte[] signkey, int signkeylen,
                                     byte[] signpass, int signpasslen,
                                     int type);

    private native int bioencNative(long biosslObj,
                                    byte[] enc, int enclen,
                                    byte[] enckey, int enckeylen,
                                    byte[] encpass, int encpasslen,
                                    int type, int certtype);

    private native int bioinitNative(long biosslObj);

    private native int bioconnectNative(long biosslObj, Object sockObj);

    private native int bioreadNative(long biosslObj, byte[] read, int readlen);

    private native int biowriteNative(long biosslObj, byte[] write, int writelen);

    private native int biocloseNative(long biosslObj);

    private native int biofreeNative(long biosslObj);

    private native int bioverifyNative(long biosslObj);

    private native int bioerrorNative(long biosslObj, int error, byte[] msg, int msglen);

    ///校验证书 cacerttype、certtype 详见 tztZFCertType
    private native int sslverifyCertNative(int cacerttype, byte[] ca, int calen, int certtype, byte[] cert, int certlen);

    private native int sslsetCANative(Object ctx, int type, byte[] data, int len);

    private native int sslsetCertNative(Object ctx, int sig, int type, byte[] data, int len, byte[] key, int keylen, byte[] pass, int passlen);

    private native boolean sslCheckTimeCertNative(int type, byte[] data, int len);

    private native Object sslgetX509CertNative(int type, byte[] data, int len);

    private native boolean sslisGMCertNative(int type, byte[] data, int len);
    //endregion
}
