package com.tztzf.recordprocesslog;

import com.tztzf.baseos.tztZFIntDef;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_BindUserID;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_ChangeAddress;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_CreateCert;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_DeleteCert;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_Disconnected;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_FreeLink;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_GMDemote;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_GMStatus;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_NetChange;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_QueryUserWithAccount;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_QueueFree;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_ReceiveData;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_RegistLink;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_ResetCert;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_SendDataAction;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_SetAddressList;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_SocketConnect;
import static com.tztzf.recordprocesslog.tztZFRecordEventType.tztZFRecordEventType_TradeLogin;

/**
 * ****************************************************************
 * 文件名称 : tztZFRecordEventType.java
 * 作   者 : Created by 王路子
 * 创建时间 : 2023-06-28 14:26
 * 文件描述 : 记录事件类型
 * 注意事项 :
 * 版权声明 : Copyright (C) 2020-2030 杭州中焯信息技术股份有限公司
 * 修改历史 : 2023-06-28 1.00 初始版本
 * ****************************************************************
 */

@tztZFIntDef({
        tztZFRecordEventType_RegistLink,
        tztZFRecordEventType_SetAddressList,
        tztZFRecordEventType_SendDataAction,
        tztZFRecordEventType_ReceiveData,
        tztZFRecordEventType_SocketConnect,
        tztZFRecordEventType_Disconnected,
        tztZFRecordEventType_ChangeAddress,
        tztZFRecordEventType_FreeLink,
        tztZFRecordEventType_GMStatus,
        tztZFRecordEventType_CreateCert,
        tztZFRecordEventType_ResetCert,
        tztZFRecordEventType_DeleteCert,
        tztZFRecordEventType_BindUserID,
        tztZFRecordEventType_TradeLogin,
        tztZFRecordEventType_GMDemote,
        tztZFRecordEventType_NetChange,
        tztZFRecordEventType_QueueFree,
        tztZFRecordEventType_QueryUserWithAccount
})

@Retention(RetentionPolicy.SOURCE)
public @interface tztZFRecordEventType {
    int tztZFRecordEventType_RegistLink = 0;//注册通道
    int tztZFRecordEventType_SetAddressList = 1;//设置地址
    int tztZFRecordEventType_SendDataAction = 2;//请求数据
    int tztZFRecordEventType_ReceiveData = 3;//应答
    int tztZFRecordEventType_SocketConnect = 4;//连接服务器
    int tztZFRecordEventType_Disconnected = 5;//断开连接
    int tztZFRecordEventType_ChangeAddress = 6;//切换服务器
    int tztZFRecordEventType_FreeLink = 7;//释放通道
    int tztZFRecordEventType_GMStatus = 8;//国密状态
    int tztZFRecordEventType_CreateCert = 9;//申请证书
    int tztZFRecordEventType_ResetCert = 10;//重置证书
    int tztZFRecordEventType_DeleteCert = 11;//删除证书
    int tztZFRecordEventType_BindUserID = 12;//国密绑定UserID
    int tztZFRecordEventType_TradeLogin = 13;//交易登录
    int tztZFRecordEventType_GMDemote = 14;//国密降级
    int tztZFRecordEventType_NetChange = 15;//网络变更
    int tztZFRecordEventType_QueueFree = 16;//释放线程
    int tztZFRecordEventType_QueryUserWithAccount = 17;//单向查询客户号

    class tztZFRecordEventTypeTips {
        public static String[] tips(@tztZFRecordEventType int eventType) {
            String[] arrayValue = new String[2];
            switch (eventType) {
                case tztZFRecordEventType_RegistLink:
                    arrayValue[0] = "RegistLinK";
                    arrayValue[1] = "注册通道";
                    break;
                case tztZFRecordEventType_SetAddressList:
                    arrayValue[0] = "SetAddressList";
                    arrayValue[1] = "设置地址";
                    break;
                case tztZFRecordEventType_SendDataAction:
                    arrayValue[0] = "SendDataAction";
                    arrayValue[1] = "请求数据";
                    break;
                case tztZFRecordEventType_ReceiveData:
                    arrayValue[0] = "ReceiveData";
                    arrayValue[1] = "应答";
                    break;
                case tztZFRecordEventType_SocketConnect:
                    arrayValue[0] = "SocketConnect";
                    arrayValue[1] = "连接服务器";
                    break;
                case tztZFRecordEventType_Disconnected:
                    arrayValue[0] = "Disconnected";
                    arrayValue[1] = "断开连接";
                    break;
                case tztZFRecordEventType_ChangeAddress:
                    arrayValue[0] = "ChangeAddress";
                    arrayValue[1] = "切换地址";
                    break;
                case tztZFRecordEventType_FreeLink:
                    arrayValue[0] = "FreeLink";
                    arrayValue[1] = "释放通道";
                    break;
                case tztZFRecordEventType_GMStatus:
                    arrayValue[0] = "GMStatus";
                    arrayValue[1] = "国密状态";
                    break;
                case tztZFRecordEventType_CreateCert:
                    arrayValue[0] = "CreateCert";
                    arrayValue[1] = "申请证书";
                    break;
                case tztZFRecordEventType_ResetCert:
                    arrayValue[0] = "ResetCert";
                    arrayValue[1] = "重置证书";
                    break;
                case tztZFRecordEventType_DeleteCert:
                    arrayValue[0] = "DeleteCert";
                    arrayValue[1] = "删除证书";
                    break;
                case tztZFRecordEventType_BindUserID:
                    arrayValue[0] = "BindUserID";
                    arrayValue[1] = "绑定UserID";
                    break;
                case tztZFRecordEventType_TradeLogin:
                    arrayValue[0] = "TradeLogin";
                    arrayValue[1] = "交易登录";
                    break;
                case tztZFRecordEventType_GMDemote:
                    arrayValue[0] = "GMDemote";
                    arrayValue[1] = "国密降级";
                    break;
                case tztZFRecordEventType_NetChange:
                    arrayValue[0] = "NetChange";
                    arrayValue[1] = "网络变更";
                    break;
                case tztZFRecordEventType_QueueFree:
                    arrayValue[0] = "QueueFree";
                    arrayValue[1] = "释放线程";
                    break;
                case tztZFRecordEventType_QueryUserWithAccount:
                    arrayValue[0] = "QueryUserWithAccount";
                    arrayValue[1] = "单向查询客户号";
                    break;
                default:
                    break;
            }
            return arrayValue;
        }
    }
}